;; @(#) tinyad.el -- Colletion of (ad)viced functions

;; @(#) $Id: tinyad.el,v 1.44 1997/06/27 21:25:31 jaalto NotReleased $
;; @(#) $Keywords: extensions $
;; $KnownCompatibility: 19.28,30 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs


;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Apr 21 1996
;;
;; To get information on this program use ident(1) or do M-x tiad-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyad|Jari Aalto|jari.aalto@poboxes.com|
;; Emacs enchancements. Collection of adviced functions|
;; 28-Jun-1997|$Revision: 1.44 $|~/misc/tinyad.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file.
;;
;;     (require 'tinyad)
;;
;; But because this takes lot of time, you should preferebly use
;; following autolaod method.
;;
;;*   (require 'tinylib)
;;*   (cond
;;*    ((emacs-p)				;Don't load for XE
;;*     (if (fboundp 'run-with-idle-timer)      ;emacs 19.34 way
;;* 	    (run-with-idle-time (* 4 60) nil '(lambda () (require 'tinyad)))
;;*       (run-at-time "4 min" nil '(lambda () (require 'tinyad))))
;;*     ))
;;* _
;;
;;
;; But before you leap into this, make sure you want to do it.
;;
;;	CHECK IF YOUR EMACS IS SUPPORTED
;;	change tiad-:re to try advices in non-supported emacs versions
;;
;; This file modifies original Emacs functions, so read the docs
;; carefully to tailor this package for you (enabling/disabling advices)
;;
;; If you have any questions, use this function
;;
;;      M-x tiad-submit-bug-report      ,send bug report or feedback
;;

;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tiad-" in front of
;;   every function & variable. It stands for '(ti)ny (ad)vices'
;; - variable names contain letters 'tiad-:'.

;;  Preface, apr 1996
;;
;;	What you see here is a selection of adviced functions
;;	that I have found extremely usefull. Some are written by me
;;	(if there is not author written) and some of them I have seen in
;;	the emacs newsgroups.
;;
;;	I also have also included extra advices which I think, are not so
;;	essential, but which may interest user of this package.
;;
;;	Those advices are put to the end of file and they are all disabled
;;	by default. You have to manually enable them if you want to use those.
;;	These advices are put under package name "my" and not to "tiad".
;;
;;	This file does not include all the advices that I'm currenly
;;	using, many of them are not included, because I considered them
;;	too personal; their benefit to someone else was questionable.
;;
;;
;;  Can these advices be included with emacs -- Mail RMS
;;
;;	Many of these enhancements could have shipped with the Emacs itself.
;;	And I have suggested many of them to be added to the next
;;	Emacs release, with no luck.
;;
;;	If you think some feature here is usefull, Please, please,
;;	mail <rms@gnu.mit.ai.edu> and say,
;;
;;	    "I would like to have this XXX feature, as described in tinyad.el,
;;	    to be included in next Emacs release."
;;
;;	This because, RMS isn't convinced that a feature should be added to
;;	Emacs if only I ask for it. We need group power. The more mail RMS
;;	gets, the more likely feature will is added. I'd rather not have
;;	these adviced functions, but for now it's the only way to get
;;	these extensions with Emacs.
;;
;;  How to use this package
;;
;;	I would suggest just loading this package right away and then
;;	printing out the whole file and reading the comments about
;;	individual functions and how they have changed.
;;
;;  File advice note:
;;
;;	If you set variable tiad-:file-compress-support to no-nil
;;	then you can compress files on disk at wish and emacs packages
;;	that read/save data from/to files doesn't even notice that
;;	you have compressed them.
;;
;;	This is very valuable feature if you're low of quota, eg.
;;	you can now compress GNUS 5 .newsrc.eld and gnus will read
;;	the .newsrc.eld.gz trasparently. It also saves to
;;	.newsrc.eld.gz too.
;;
;;	See write-file's and find-file's fucntion documentation
;;	for full explanation of this feature.
;;
;;  Overview of features -- not all features are presented here!
;;
;;	o  Enchancements to existing emacs functions
;;	o  Advices that make your day in emacs more easier.
;;	o  Includes best advices found from the net or written by the Author.
;;
;;	o   gud highlights full line
;;	o   no dialogs in X for y-or-n-p styled questions.
;;	o   mouse window delete confrmation (pointing on the mode line)
;;	o   diables dired mouse properties --> dired executes faster.
;;
;;	o   debugger-eval-expression,  Backtrace buffer's
;;			    "e" offers current word for prompt
;;	o   dired-man	    , make sure variables are initalized.
;;	o   find-file	    ; can load .gz and .Z file
;;	o   goto-line and imenu now widens automatically before executing
;;	o   rename-buffer   , offers old buffer name for editing
;;	o   recover-fil	    , offers buffer filename by default
;;	o   switch-to-buffer-other-frame , selects some non existig frame
;;	o   setenv	    , getenv offers completion
;;	o   write-file	    , confirmation upon overwrite, VAX support too
;;	o   write-region    , confirmation upon ovwrt, VAX support
;;
;;	o   C-x ;  , indent-for-comment's negative arg deletes comment.
;;	o   C-x =  , what-cursor-position shows the line number too
;;	o   C-x i  , insert buffer offers other window
;;	o   C-x C-c, save-buffers-kill-emacs asks confirmation
;;		     to prevent accidents
;;	o   C-x b  , swich-to-buffer ask cfrm for non-existing buffers.
;;	o   C-x C-b, list-buffers puts you to "*Buffer List*
;;
;;	o   compilation: buffer auto scroll
;;	o   compilation: smart save feature (only .cc .h files, not
;;	    all emacs files)
;;	o   compilation: find-file is done in non dedicated frame
;;
;;	o   completion:  case sensitive filename completion
;;
;;	o   ediff:  list only files that have diff over directories.
;;
;;	o   grep:   filename completion via TAB key!
;;
;;	o   vc:     vc-print-log, puts cursor on the buffer's revision number.
;;	o   vc:     smarter vc-mode-line, shows "b" if ver is in the middle.
;;	o   vc:     vc-register creates RCS directory if does not exist
;;	o   vc:     user to set the initial comment when doing 1st CI.
;;
;;
;;  Handling advices
;;
;;	If you have some other emacs version that is not supported in the
;;
;;	    tiad-:advice-table
;;
;;	you can modify the regexps in the list and try if the advice works
;;	in your emacs. If it does, please drop me a mail immediately and I
;;	update the regexp.
;;
;;	If some advice annoys you, there is simple method how you disable
;;	advice(s). Following code can be ripped with tinylib.el's function
;;	ti::m-pkg-rip-magic
;;
;;* _
;;* (setq tiad-load-hook '(tiad-install my-tiad-load-hook))
;;* _
;;* (defun my-tiad-load-hook ()
;;*   "Configure 'tiny tool's advices' to my taste."
;;*   (interactive)                      ;; For user call
;;* _
;;*   ;; This diables two advices
;;*   (tiad-advice 'disable '(switch-to-buffer mouse-delete-other-windows))
;;*   )
;;*
;;* (require 'tinyad)
;;
;;
;;
;;  Disabling disturbing advice by hand
;;
;;	If some piece of advice disturbs or causes trouble in your current
;;	emacs session. You can deactivate it immediately. First
;;	you have to know the function name that generates problems.
;;	Say you used
;;
;;	    C-x C-b
;;
;;	Which runs function (see C-h k C-x C-b)
;;
;;	    switch-to-buffer
;;
;;	and you don't like the confirmation for non-existent buffers.
;;	You can disable this behavior immedistely by calling:
;;
;;	    C-u M-x tiad-advice
;;
;;	and giving the function name switch-to-buffer to it. To permanently
;;	turn it off in your emacs sessions, see previous section's lisp code.
;;
;;
;;  About functions and variables
;;
;;	Mostly this package has only standard emacs functions that are
;;	adviced in some way. However it may be necessary that
;;	particular advice needs some other function's help or uses some other
;;	external variables.
;;
;;	These extra functions, outside of advide, are named starting with the
;;	prefix "tiad-" for functions and "tiad-:" for variables.
;;
;;
;;  Code note
;;
;;	You see this in the code:
;;
;;	    (when (tiad-activate-p)
;;		(defadvice ..
;;
;;	If emacs version is wrong, the advice is _never_ actually assembled.
;;	You can't activate or deactivate this function with tiad-advice.
;;
;;
;;  Many thanks to, in no particular order:
;;
;;	Vladimir Alexiev	<vladimir@cs.ualberta.ca>
;;	Kevin    Rodgers	<kevinr@ihs.com>
;;	Ilya     Zakharevich	<ilya@math.ohio-state.edu>
;;	Peter			<pbreton@i-kinetics.com>
;;	T. V. Raman		<raman@adobe.com>

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun 27	1997	[jari]		19.28	1.42-1.44	NotReleased
;; - Removed RMAIL advices and moved them to TinyRmail.el
;; - Removed mail send and exit and moved to tinyMail.el
;;
;; Jun 3	1997	[jari]		19.28	1.39-1.41	NotReleased
;; - Changed all non standard boolean tests to use when and unless
;; - Removed some functions from 'm' lib --> needed update here.
;; - Checkdoc 1.33 clear.
;;
;; Mar 27	1997	[jari]		19.28	1.36-1.38	NotReleased
;; - mouse-drag-region removed. The secondary selection does the same
;; - Internal changes.
;;
;; Mar 19	1997	[jari]		19.28	1.34-1.35	NotReleased
;; - You can now complete filenames in M-x Grep propmt
;; - Corrected recover-file advice. It didn't work at all!
;; - debugger-eval-expression; the "e" nowoffers current word at point
;;   for evaling.
;; - compilation-find-file Now finds non dedicated frame.
;; - Added compressed file load support (find-file)
;; - corrected write-file functions (serious errors in arg handling)
;; - occur is now adviced. Removed overlay autoload from 'xe' lib, because
;;   they do not work in XEmacs.
;;
;; Feb 16	1997	[jari]		19.28	1.31-1.33	NotReleased
;; - New advices: mouse-drag-region; Put to class 'my' and is disabled
;;   by default due to problems.
;; - New advice switch-to-buffer-other-frame
;; - Corrected all file advices that choked when there was no filename(nil).
;;
;; Jan 15	1997	[jari]		19.28	1.28-1.30	NotReleased
;; - Added advice by Kevin Rodgers.
;;   tiad-:ediff-directories-different-files	:+
;;   ediff-intersect-directories		:+
;;
;; Oct 18	1996	[jari]		19.28	1.27		NotReleased
;; - Added advice to fill-paragraph so that it fills C comments right.
;; - Reordered all advices so that their default state is "dis", the
;;   load hook function tiad-install now activates all advices. If
;;   user wants to keep only 10 advices and ignore others, this should
;;   speed up loading time of this package.
;; - Shuffled loading the libraries a bit, so that they load faster.
;; - Now vc-mode-line can better find the correct RCS version number.
;;   Uses new function from the main lib, which looks at delta file
;;   if file is checked out. This can cope with files checked out
;;   with -k option (RCS strings are not updated in file
;;
;; Sep 9	1996	[jari]		19.28	1.26		NotReleased
;; - Added code to tiad-rcs-initial-comment so that the new comment setting
;;   is really reflected in Emacs buffer. The initial chekin a bit
;;   slow now, but can't help that, because there is no other way to
;;   make the comment setting visible than doing extra "co".
;;
;; Sep 4	1996	[jari]		19.28	1.24-1.25	NotReleased
;; - Programmed more smarter advice to vc-register.
;; - Corrected save-buffers-kill-emacs, diddn't quit if called from lisp.
;;
;; Aug	30	1996	[jari]		19.28	1.22-1.23	NotReleased
;; - Error found from vc-do-command, thanks to "Zach Leber"
;;   <zach@radionics.com>. Corrected.
;; - Error in save-buffers-kill-emacs, still killed the emacs although
;;   user answered "no". Thanks to Zach. Corrected.
;;
;; Aug	27	1996	[jari]		19.28	1.21		NotReleased
;; - vc-mode-line corrected. Bug reported by Zach Leber <zach@radionics.com>
;; - Added vc-do-command advice to add RCS comment after initial checkin.
;; - Added vc-rcs-status status advice to show right version number.
;;   The original VC implementation didn't handle middle brach revisions
;;   right.
;;
;; Aug	23	1996	[jari]		19.28	1.81-20		NotReleased
;; - All ByteCompiler errors wiped away. The XEmacs compiler is great!
;; - Corrected some XEmacs 19.14 compatibility problems. Package not tested.
;;
;; Aug	19	1996	[jari]		19.28	1.17		NotReleased
;; - Added save-excursion around PC-complete, so that the point
;;   doesn't move.
;;
;; Jun	19	1996	[jari]		19.28	v1.14-1.15	NotReleased
;; - PC-complete is now case sensitive if current word is filename.
;;
;; Jun	12	1996	[jari]		19.28	v1.13	        NotReleased
;; - I started using PGP and noticed that I couldn't do automatic mail
;;   detection with just one hook: rmail-get-new-mail-hook. I needed
;;   the states "before" and "after" the mail was fetched.
;;   tiad-:rmail-get-new-mail-before-hook :+
;; - VC version string (modeline) display corrected.
;;
;; May	31	1996	[jari]		19.28	v1.12	        NotReleased
;; - Corrected example and rewrote tiad-advice, now uses tinylib library
;;   function for advice control.
;;
;; May	31	1996	[jari]		19.28	v1.11	        NotReleased
;; - Started using igep.el and noticed that I needed new
;;   advices :-)
;;   tiad-compile-internal-buffer	:+
;;   tiad-:compile-internal-hook		:+
;;   tiad-:find-alternate-file-flag	:+
;;   compile-internal			:+ tiad-tap-buffer
;;   compile-internal			:+ tiad-run-hook
;;   find-alternate-file		:+
;;   my-compile-font-lock		:+ new example
;;   setenv				:! editing previous value
;;
;; May	25	1996	[jari]		19.28	v1.10	        NotReleased
;; - Moved the 19.30 "completion" error fixes to appropriate module,
;;   tinypatch.el.
;;
;; May	24	1996	[jari]		19.28	v1.9	        NotReleased
;; - Added more completion fixes to shut up annoying error in 19.30
;; - goto-line now has 2 widen commands.
;;
;; May	16	1996	[jari]		19.28	v1.8	        NotReleased
;; - Corrected the vc-mode-line: it showed "b" branch indicator in
;;   wrong situations.
;;   tiad-version					:!
;;   completion-separator-self-insert-autofilling	:!
;;   tiad-vc-version-now				:+
;;
;; May	8	1996	[jari]		19.28	v1.7	        NotReleased
;; - Corrected irratating completion.el error in 19.30 with advice.
;; - Wrote the 'overview' section.
;;
;; May	6	1996	[jari]		19.28	v1.6	        NotReleased
;; - Jerome Santini <santini@chambord.univ-orleans.fr> suggested
;;   switch-to-buffer to use find-file too. Now added new variable
;;   tiad-:switch-to-buffer-find-file to enable this behavior.
;; - pbreton@i-kinetics.com (Peter) send me some handy advices. Thanks!
;;
;; May	2	1996	[jari]		19.28	v1.5	        NotReleased
;; - Added advices sent by Vladimir. Rewrote most of the XEmacs and Emacs
;;   function table code. Now same function can be enabled both in XE
;;   and Emacs with flag set to "t" in the table.
;; - Rewrote all the tiad support functions.
;;
;; Apr	29	1996	[jari]		19.28	v1.4	        NotReleased
;; - Pre-anounncement to g.e newsgroups
;;
;; Apr	24	1996	[jari]		19.28	v1.3	        NotReleased
;; - Added "code note" section. Errors in ENV advices
;;   tiad-:write-file-no-confirm	:+
;;   tiad-ad-function-list			:+
;;   tiad-get-table			:+
;;   after-find-file			:+
;;   write-file				:+
;;   write-region			:+
;;   tiad-advice			:! adjustment
;;
;;
;; Apr	24	1996	[jari]		19.28	v1.2	        NotReleased
;; - Massive changes, many more advices added. User interface functions
;;   rewritten. Advices not fully tested yet [just moved here from my
;;   own emacs ini files]
;;
;; Apr	21	1996	[jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'advice)
(require 'tinylib)

(eval-and-compile
  (defvar vc-parent-buffer nil)		;Emacs vc.el
  (when (xemacs-p)
    (message "tinyad: You must configure this package manually to XEmacs")
    (message "tinyad: In general, don not use this packaage on XEmacs.")
    (load "overlay" 'noerr)		;19.15+
    )
  )


;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defvar tiad-load-hook '(tiad-install)
  "Hook that is run when package is loaded.")


;;; ... private variables ................................. &v-private ...

(defconst tiad-:advice-re  "^tiad"
  "Prefix name used in advices for TIAD package.")

(defconst tiad-:tmp-buffer  "*tiad*"
  "Temporary working buffer.")



(defvar tiad-:compile-internal-buffer  nil
  "The compilation buffer created by compile-internal.")

(defvar tiad-:vc-p nil
  "Variable indicating if file in `vc-do-command' is version controlled.")


;;; ... user configurable .................................. &v-config ...



(defvar tiad-:compile-internal-hook nil
  "*Hook run after `compile-internal' funtion.
You can peek variable `tiad-:compile-internal-buffer' too.")


(defvar tiad-:compile-save-re  "\\(\\.hh?\\|\\.cc?\\|\\.C?\\)$"
  "*Regexp. Case sensitive. Which buffers to save when compiling.")



(defvar tiad-:gud-overlay
  (let* ((ov (make-overlay (point-min) (point-min))))
    (overlay-put ov 'face 'highlight)
    ov)
  "*Gud. Current line overlay")


(defvar tiad-:find-alternate-file-flag  t
  "*Non-nil means : `buffer-name' in \\[find-file] if no `buffer-file-name'.")


;;  My temprory files start with T, P, ignore /tmp/ dir files
;;  - like ~/T  ~/TT ~/T1 ~/T2 ~/T.test ~/T1.xx ...
;;
(defconst tiad-:write-file-no-confirm "^/tmp\\|/[PT][PTt0-9]?\\.?\\|/[Tt]$"
  "*Do not verify overwriting these files. See advice write-file.")


(defconst tiad-:file-compress-support  nil
  "Non-nil add .Z .gz support to `find-file', `write-file' and write-region.
If this variable is non-nil the FILE.txt is written
out/read as FILE.txt.gz if such file exists.

See `write-file' and find-files function description for Full
explanation.")


(defvar tiad-:switch-to-buffer-find-file  t
  "*Suggest `find-file' for non-existing buffers in `switch-to-buffer'.")

(defvar tiad-:vc-main-rcs-dir  "~/RCS"
  "Main RCS repository. See advice of function vc-register.")


;;; ........................................................ &v-advice ...


;;; 19.39 ?? -- Okay, you can turn the advice off if it doesn't work :-)
;;;
(defvar tiad-:re "19\\.2[7-9]\\|19\\.3[0-9]"
  "General regexp for advices that work in variety of (X)Emacs versions.")


;;; - Change the REGEXP is you know the advice works in your emacs ver.
;;;   Drop me mail if you change any of these, so that I can update list
;;;
;;; - Functions that have ".", almost always get advice, see the code.
;;;   In those rows the regexp value is almost always ignored.
;;;
;;; - If it says ";; always on", then the regexp has no effect,
;;;   you have to disable feature by hand, if you don't want it.
;;;
(defconst tiad-:advice-table		;alphabetically ordered
  (list
   (list 'after-find-file		".")			;;always on
   (list 'ange-ftp-dired-run-shell-command ".")			;;always on

   (list 'compile			".")
   (list 'compile-internal		"2[7-9]")		;;fixed 19.30+
   (list 'compilation-find-file         ".")

   (list 'debugger-eval-expression	".")
   (list 'dired-insert-set-properties	".")			;;always on
   (list 'dired-man			".")			;;always
   (list 'display-time-process-this-message "19" 'xe)

   (list 'ediff-intersect-directories	".")			;;always
   (list 'exchange-point-and-mark       ".")
   (list 'find-file			".")

   (list 'grep				".")

   (list 'find-alternate-file		".")
   (list 'fill-paragraph                "2[0-8]")

   (list 'getenv			".")			;;always on
   (list 'goto-line			".")
   (list 'gud-display-line		".")			;;always

   (list 'hkey-help-show		".")			;;hyberbole

   (list 'imenu				".")			;; always
   (list 'indent-for-comment		".")
   (list 'insert-buffer			tiad-:re)
   (list 'list-buffers			".")
   (list 'line-move			".")

   (list 'map-y-or-n-p			tiad-:re)
   (list 'mouse-delete-other-windows	tiad-:re)
   (list 'mouse-delete-window		tiad-:re)

   (list 'occur				".")
   (list 'PC-complete			".")                    ;;always on

   (list 'recompile			".")
   (list 'recover-file                  ".")
   (list 'rename-buffer			tiad-:re)

   (list 'save-buffers-kill-emacs	".")
   (list 'sendmail-pre-abbrev-expand-hook tiad-:re)
   (list 'setenv			".")			;;always on
   (list 'set-mark-command		".")			;;always on
   (list 'switch-to-buffer		tiad-:re)
   (list 'switch-to-buffer-other-frame	".")

   (list 'tq-process-buffer		".")

   (list 'vc-do-command                 tiad-:re)
   (list 'vc-mode-line			tiad-:re)
   (list 'vc-print-log			"2[89]\\|3[01]")	;;fixed 19.32
   (list 'vc-register			tiad-:re)
   (list 'vc-rcs-status                 tiad-:re)

   (list 'what-cursor-position		tiad-:re)
   (list 'write-file			".")
   (list 'write-region			".")

   (list 'y-or-n-p			tiad-:re)
   )
  "*Flag table of enabled advices.
It is consulted if particular advice can be used in current emacs. Format is

  ((FUNCTION ALLOW-ADVICE-REGEXP [FLAG])
   (FUNCTION ALLOW-ADVICE-REGEXP)
   ..
   )

The FLAG is optional and values can be:
  nil   ,or missing: Only works in Emacs
  'xe   ,only works in Xemacs
  t     ,works both Emacs and XEmacs")

;;}}}
;;{{{ version

;;; ....................................................... &v-version ...


(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyad.el"
 "tiad"
 tiad-:version-id
 "$Id: tinyad.el,v 1.44 1997/06/27 21:25:31 jaalto NotReleased $"
 '(tiad-version-id
   tiad-:compile-save-re
   tiad-:write-file-no-confirm
   tiad-:file-compress-support
   tiad-:re
   )))

;;}}}

;;; ########################################################### &Funcs ###

;;{{{ tiad: misc

;;; ----------------------------------------------------------------------
;;;
(defmacro tiad-elts (elt func re type)
  "Decode ELT to variables FUNC RE TYPE"
  (`
   (setq (, func) (nth 0 (, elt))
	 (, re)   (nth 1 (, elt))
	 (, type) (if (eq 3 (length (, elt)))
		      (nth 0 (, elt))
		    nil))
   ))



;;; ----------------------------------------------------------------------
;;;
(defun tiad-match  (re &optional type)
"Check if RE match emacs version according to TYPE.
TYPE :
  nil = Emacs
  t   = XEmacs and Emacs
  'xe = XEmacs"
(let* ((ver   (emacs-version))
	ret
       )
  (cond
   ((and (eq type 'xe)
	 (xemacs-p)
	 (string-match re ver))
    (setq ret 1))

   ((and (eq type nil)
	 (emacs-p)
	 (string-match re ver))
    (setq ret 2))

   ((and (eq type t)
	 (string-match re ver))
    (setq ret 3)
    ))
  ret
  ))



;;; ----------------------------------------------------------------------
;;; Testing... (tiad-activate-p 'compile-internal)
;;;
(defun tiad-activate-p (func-sym)
  "Determine if we can advice FUNC-SYM."
  (let* ((elt   (assoc func-sym tiad-:advice-table))
 	 func re type
 	 )
    (when elt
      (tiad-elts elt func re type)
      ;;  XEmacs 19.14 ByteComp, Shut up "bound but not referenced"
      (if func (setq func 'ignore))
      (tiad-match re type)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun tiad-install  ()
  "Activates advices that are listed in `tiad-:advice-table'"
  (interactive)
  (tiad-advice nil (tiad-ad-function-list)))

;;; ----------------------------------------------------------------------
;;;
(defun tiad-ad-function-list  (&optional string-format)
  "Return list of tiad ad-functions for current emacs.
Notice: all functions may not be adviced; this merely
return entries in the table. See source file's \"Code note\"

If STRING-FORMAT is non nil, then return string list.

Return:
  '(func    func ..)
  '(\"func\" \"func\" ..)"
 (let* ((table tiad-:advice-table)
	func re type member
	list
	)
   (list-loop table member
     (tiad-elts member func re type)
     (cond
      ((tiad-match re type)
       (if string-format
	   (push (symbol-name func) list)
	 (push func list)
	 )))
     )
   list
   ))


;;; ----------------------------------------------------------------------
;;;
;;; This is slow, but returns only tiad adviced functions...
;;;
;;; (ad-do-advised-functions (func)
;;;  (if (ad-find-some-advice func 'any tiad-:advice-re)
;;;      (push func list)))
;;;
;;;
(defun tiad-advice (&optional disable func-or-list)
  "Activate or optionally DISABLE tiad advice for FUNC-OR-LIST."
  (interactive
   (list
    current-prefix-arg

    (let* (var)
      (setq var (completing-read
		 (concat
		  (if current-prefix-arg "un"  "")
		  "advice function: ")
		  (list-to-assoc-menu (tiad-ad-function-list 'strings))
		  nil t))
      (intern-soft var)
      )))

  ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ...  args end . .
  ;; This is in fact cheating a little; we check against full advice list,
  ;; not just "tiad" owned functions.
  ;;
  (cond
   ((and (symbolp func-or-list)
	 (not (member (list (symbol-name func-or-list))
		      ad-advised-functions )))

    ;; This makes the call to 'ti::' after this if, unefective
    ;;
    (setq func-or-list nil)

    (if (interactive-p)
	;; more accurate: "No advice found..." but since we deal with
	;; tiad ones only the following is better.
	(message "Sorry, the function is not advice controlled by TIAD."))
    ))

  ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. do the job . .
  (ti::m-ad-control func-or-list tiad-:advice-re disable (interactive-p))
  )



;;; ----------------------------------------------------------------------
;;;
(defun tiad-advice-control  (&optional disable verb)
  "Acivate all TIAD advices. Use extra argument to DISABLE all. VERB."
  (interactive "P")
  (let* ((verb	(or verb (interactive-p)))
	 (re	tiad-:advice-re)
	 (doit  t)
	 msg
	 )
    ;;  This is rough! Be sure...
    (if verb
	(setq
	 doit
	 (y-or-n-p (format "advices %s: No mistake here? "
			   (if disable "off" "on")))))

    (when doit
      (cond
       (disable
	(ad-disable-regexp re)		;only sets flag
	(setq msg "All TIAD advices activated")
	)
       (t
	(ad-enable-regexp re)		;only sets flag
	(setq msg "All TIAD advices deactivated")
	))
      (ad-update-regexp re)
      (if verb  (message msg))
      )
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tiad-convert-filename  (file &optional cautious)
  "Return normal or compressed filename.

Input:

 FILE	    ,full filename
 CAUTIOUS   ,if non-nil then when in doubt do not change the filename.
	     (eg. in clash situation, where there is bot un/compressed  file)

Return:
 string	    ,possibly modified."
  (interactive)
  (unless (string-match "\\.Z$\\|\\.gz$"  file)
    (cond
     ((and (file-exists-p file)
	   (or (file-exists-p (concat file ".gz"))
	       (file-exists-p (concat file ".Z"))))
      (message-flash 2 nil "\
tiad: clash, both un/compressed file found. %s " file)

      (if (and
	   (null cautious)		;only if no cautious mode
	   (setq
	    file
	    (or (ti::f-newer-exist file (concat file ".gz"))
		(ti::f-newer-exist file (concat file ".Z"))))
	   )
	  ;;  We must load this package too to enable compress support.
	  ;;
	  (require 'jka-compr))
      )))
  file
  )

;;}}}

;;{{{ ange-ftp

;;; ----------------------------------------------------------------------
;;; 24 May 1996, rdong@printrak.com Rui-Tao Dong,
;;; in efs-help-request@cuckoo.hpl.hp.com mailing list
;;;
;;; I am still using the good & old ange-ftp 4.20 most of the time.  I do have
;;; the need to log into the remote host as a different user (including root).
;;; I found the following to be very useful
;;;
(defadvice ange-ftp-dired-run-shell-command (before tiad-rsh-cmd dis)
  "Launch rsh -l if needed."
  (setq ange-ftp-remote-shell-file-name
        (format "rsh -l %s" (nth 1 (ange-ftp-ftp-path default-directory)))
	))

;;}}}
;;{{{ built-ins

;;; ........................................................ &built-in ...

;;; ----------------------------------------------------------------------
;;;
(when (tiad-activate-p 'rename-buffer)

(defadvice rename-buffer (around tiad dis)
  "Gives old buffer name for editing."
  (interactive
   (list
    (read-from-minibuffer
     "Rename buffer (to new name): "
     (buffer-name)
     )))
  ad-do-it
  )
)

;;}}}
;;{{{ compile

;;; ......................................................... &compile ...


;;; ----------------------------------------------------------------------
;;; (ad-disable-advice 'compilation-find-file 'before 'tiad)
;;; (ad-activate       'compilation-find-file)
;;;
(defadvice compilation-find-file  (before tiad act)
  "Move to some non dedicated frame."
  (use-non-dedicated-frame))


;;; ----------------------------------------------------------------------
;;; Sep 14 1995 <jari.aalto@poboxes.com> (Finland) Public domain.
;;; See also variable compilation-last-buffer
;;;
;;; - Handling other frames corrected by James (Jim) R.R. Service
;;;   jservice@rd.hydro.on.ca. (the extra t's)
;;;
;;; - This has been reported to be corrected in 19.30
;;;
(when (tiad-activate-p 'compile-internal)

(defadvice compile-internal (after tiad-scroll dis comp)
  "Forces compile buffer to scroll."
  (let* ((ob (current-buffer))
	 (obw  (get-buffer-window ob t))
	 win
	 )
    (save-excursion
      (if (or (null (setq win (get-buffer-window ad-return-value t)))
	      (null obw))
	  nil
	(select-window win)
	(goto-char (point-max))
	(select-window obw)
	))
    ))
)

;;; ----------------------------------------------------------------------
;;; "tap" -- listen secretly :-)
;;;
(defadvice compile-internal (around tiad-tap-buffer dis comp)
  "Save compile buffer name to 'tiad-:compile-internal-buffer'.
See `tiad-:compile-internal-hook'."
  (prog1
      ad-do-it
    (setq tiad-:compile-internal-buffer ad-return-value)))


;;; ----------------------------------------------------------------------
;;;
(defadvice compile-internal (after tiad-run-hook last act comp)
  "Run hook 'tiad-:compile-internal-hook'.
You can put lazy-lockfontifying here."
  (run-hooks 'tiad-:compile-internal-hook))



;;; ----------------------------------------------------------------------
;;;
(defun tiad-compile-save-buffers ()
  "Check what buffers for current compilation target should be saved."
  (interactive)
  (let* ((case-fold-search	nil)	;case sensitive
	 (re-file               tiad-:compile-save-re)
	 )

    ;; Save only interesting buffers, don't care about others.
    (ti::b-buffer-list
     re-file nil nil
     '(and (buffer-modified-p)
	   (y-or-n-p (format "Buffer %s modified. Save it? "
			     (buffer-name)))
	   (save-buffer)))
    ))


;;; ----------------------------------------------------------------------
;;; - More smarter buffer saving.
;;;
(defadvice compile (around tiad dis)
  "Replace original function. More smarter buffer saving.
See function `tiad-compile-save-buffers'."
  (interactive
   (if compilation-read-command
       (list (read-from-minibuffer "Compile command: "
                                 compile-command nil nil
                                 '(compile-history . 1)))
     (list compile-command)))
  (setq compile-command command)

  (if (null compilation-ask-about-save)
      (save-some-buffers (not compilation-ask-about-save) nil)
    (tiad-compile-save-buffers))
  (compile-internal compile-command "No more errors")
  )



;;; ----------------------------------------------------------------------
;;; run compile with the default command line
;;;
(defadvice recompile (around tiad dis)
  "Replace original function.
More smarter buffer saving, seefunction tiad-compile-save-buffers."
  (interactive)
  (if (null compilation-ask-about-save)
      (save-some-buffers (not compilation-ask-about-save) nil)
    (tiad-compile-save-buffers))
  (compile-internal compile-command "No more errors"))


;;}}}
;;{{{ completion

;;; ...................................................... &completion ...

;;; ----------------------------------------------------------------------
;;;
(defadvice PC-complete  (around tiad dis)
  "In file name prompt, use case sensitive completion.
Set `completion-ignore-case' locally to nil."
  (let* ((completion-ignore-case  completion-ignore-case)
	 word
	 )
    (setq word (or (save-excursion (ti::b-read-space-word)) ""))

    (if (string-match "^[/~]" word)
	(setq completion-ignore-case nil))
    ad-do-it
    ))


;;}}}

;;{{{ debugger


;;; -------------------------------------------------------- &debugger ---
;;;
(defadvice debugger-eval-expression (around tiad dis)
  "Chnage interactive so that it offer word from buffer."
  (interactive
    (list
     (read-from-minibuffer
      "(tiad) Eval: "
      (or (ti::b-read-space-word) "")
      read-expression-map t
      'read-expression-history
      )))
  ad-do-it
  )


;;}}}
;;{{{ dired

;;; ........................................................... &dired ...

;;; ----------------------------------------------------------------------
;;; - I love to have highlight in emacs, but if it makes thigs slow
;;;   in emacs, then it's not so important. dired is too slow alredy.
;;; - I think that lazy-lock would be more appropriate here.
;;;
(defadvice dired-insert-set-properties (around tiad dis)
  "Replace function.
Inhibits setting mouse-face properties so thatdired executes faster."
  nil)


;;; ----------------------------------------------------------------------
;;; May 6 1996, <pbreton@i-kinetics.com>
;;;
(defadvice dired-man (before tiad dis)
 "Make sure man variables are initialized"
  (require 'man)
  (Man-init-defvars))


;;}}}

;;{{{ ediff

;;; ----------------------------------------------------------------------
;;; 25 Oct 1996, kevinr@ihs.com (Kevin Rodgers), gnu.emacs.help
;;; This affects `ediff-directories3' and `ediff-merge-directories' as well.")
;;;

(defvar tiad-:ediff-directories-different-files t
  "*See `ediff-intersect-directories'.
If non-nil, `ediff-directories' only includes files that differ
from one another (plus directories) in the session.")

(defadvice ediff-intersect-directories (before tiad-different-files act)
  "Return files with the same name but different contents.
If `tiad-:ediff-directories-different-files' is non-nil and COMPARISON-FUNC
is nil, only return files with the same name but different contents
\(plus directories with the same name\)."
  ;; Warning: The new COMPARISON-FUNC depends on the dynamically scoped
  ;; binding of `auxdir1', `auxdir2', and `auxdir3' established by
  ;; `ediff-intersect-directories'.
  ;;
  (if (and tiad-:ediff-directories-different-files
	   (null (ad-get-arg 6)))	; comparison-func
      (ad-set-arg 6 (lambda (file-1 file-2)
		      (and (string= file-1 file-2)
			   (let ((directory-1 auxdir1)
				 (directory-2 (if (and dir3 common)
						  auxdir3
						auxdir2)))
			     (/= (call-process "cmp" nil nil nil
					       "-s"
					       (expand-file-name file-1
								 directory-1)
					       (expand-file-name file-2
								 directory-2))
				 0)))))))

;;}}}
;;{{{ env

;;; ............................................................. &env ...

;;; ----------------------------------------------------------------------
;;;
;;; 28 Jun 1995, gnu.emacs.sources, kevinr@ihs.com (Kevin Rodgers)
;;; provide completion
(defun tiad-read-envvar (prompt &optional require-match)
  "Read an environment variable name from the minibuffer.
Prompt with PROMPT and complete from `process-environment'.
If optional arg REQUIRE-MATCH is non-nil, only defined variable
names are allowed."
  (completing-read
   prompt
   (mapcar (function
            (lambda (var=value)
              (list (substring var=value 0
                               (string-match "=" var=value)))))
           process-environment)
   nil
   require-match))

;;; ----------------------------------------------------------------------
;;; 28 Jun 1995, gnu.emacs.sources, kevinr@ihs.com (Kevin Rodgers)
;;;
;;; Comment by jari: Hanged for me sometimes, don't know why..
;;; Currently owned by "my" and disabled. Enable this manyally in load-hook
;;; if you want to try it.
;;;
(defadvice getenv (around my dis)
  "Make command more comfortable. Completion."
  (interactive (list (tiad-read-envvar "Get environment variable: " t)))
  ad-do-it
  (if (and (interactive-p) ad-return-value)
      (message "%s" ad-return-value)
    ad-return-value
    ))


;;; ----------------------------------------------------------------------
;;; 28 Jun 1995, gnu.emacs.sources, kevinr@ihs.com (Kevin Rodgers)
;;; May 29 1996, Added editing previous content
;;;
(defadvice setenv (around tiad dis)
  "Add interactive completion."
  (interactive
   (if current-prefix-arg
       (list (tiad-read-envvar "Clear environment variable: " t) nil t)
     (let ((var (tiad-read-envvar "Set environment variable: ")))
       (list var
	     (read-from-minibuffer
	      (format "Set %s to value: " var)
	      (or (getenv var) "")
	      ))
       )))
  ad-do-it
  (if (and (interactive-p) value)
      (message "%s" value)
    value
    ))

;;}}}

;;{{{ grep

;;; ------------------------------------------------------------ &grep ---
;;;
(defadvice grep  (around tiad  dis)
  "Complete filenames with TAB.
Read word from the current pointand put it into grep prompt."
  (interactive
   (ti::f-complete-filename-minibuffer
     (list
      (read-from-minibuffer
       "(tiad) Run grep: "
       (concat grep-command  (or (ti::b-read-space-word) ""))
       map nil 'grep-history
       ))))
  ad-do-it
  )

;;}}}

;;{{{ files

;;; ........................................................... &files ...
;;; - You really should get the tinypatch.el too. With that and these
;;;   adviced functions you can enjoy smallish VAX support again!
;;; - It allows loading VAX files in dired, "f", and saving the
;;;   VAX file buffer with C-x C-s
;;;



;;; ----------------------------------------------------------------------
;;; ANGE things...
;;; - Ange ftp gets "listing" when it tries to guess if the file
;;;   exists or if it's new file. The listing is produced with the call
;;;   insert-file-contents
;;;
;;; find-file-noselect (filename &optional nowarn)
;;;  ...
;;;  ange-ftp-insert-file-contents
;;;    ..file-exists-p
;;;
(defadvice after-find-file (around tiad-file dis)
  "Suppress call if no buffer-file-name. This may happen with ange-ftp."
  (if buffer-file-name
      ad-do-it
      ))



;;; ----------------------------------------------------------------------
;;;
(defadvice find-file (around tiad dis)
  "Check if there is compressed file and load it instead.

If there is both files, uncomressed and compressed; then load the
one that is newest and issue warning.

    `find-file' FILE.txt
    FILE.txt
    FILE.txt.gz	    -- this is never, load it.

    `find-file' FILE.txt.gz  -- do nothing special
    `find-file' FILE.txt.Z   -- do nothing special

You have to set `tiad-:file-compress-support' to non-nil to enable
this feature."
  (if tiad-:file-compress-support
      (ad-set-arg 0 (tiad-convert-filename (ad-get-arg 0))))
  ad-do-it
  )

;;; ----------------------------------------------------------------------
;;; 19.30 doesn't offer the filename, so enable this in all emacs versions
;;;
(defadvice find-alternate-file  (around tiad dis)
  "Interactive change: offer buffer filename as default.
Reference:
  `tiad-:find-alternate-file-flag'"
  (interactive
   (list
    (read-file-name"find alternate file: "
     (file-name-directory (or (buffer-file-name)
			      default-directory))
     nil t
     (if (buffer-file-name)
	 (file-name-nondirectory (buffer-file-name))
       (if tiad-:find-alternate-file-flag
	   (buffer-name) ""))
     )))
  ad-do-it
  )

;;; ----------------------------------------------------------------------
;;;
(defadvice recover-file  (around tiad dis)
  "Offer current buffer's filename in prompt."
  (interactive
   (list
    (read-file-name
     "(tiad) Recocer file: "
     (file-name-directory (or (buffer-file-name)
			      default-directory))
     nil t
     (if (buffer-file-name)
	 (file-name-nondirectory (buffer-file-name))
       (if tiad-:find-alternate-file-flag
	   (buffer-name) "")))
    ))
  ad-do-it
  )



;;; ----------------------------------------------------------------------
;;; Aug 29 1995, idea by kevinr@ihs.com (Kevin Rodgers)
;;; Feb 16 1996, reprogrammed by <jaalto@tre.tele.nokia.fi>
;;;
(defadvice write-file (around tiad-file dis)
  "File handling additions.

Interactive change:

    Changes the interactive prompt so, that full `buffer-file-name' is given
    for editing.

Confirm overwrite:

    When called interactively, require confirmation if FILENAME already exists.
    If FILENAME matches `tiad-:write-file-no-confirm', no confirmation
    is asked.

Vax support:

    Also modifies the file name so that you can write VAX files by removing
    the versin extension. [you need tinypatch.el for VAX files]

    LOGIN.COM;98 --> written as LOGIN.COM

    You cannot normally write with the version extension, the VAX will
    increase the number when using the \"raw\" name only

Compressed file support, .gz and.Z:

    If `write-file' is called to write FILE.txt and there only exists
    FILE.txt.gz, then write to that file instead. This allows you
    to compress files and not needing to make any changes to eg. lisp
    packages that save files (eg. in GNUS 5 you can now have
    .newsrc.eld.gz which takes much less space)

    If called with FILE.txt and there exists both files FILE.txt and
    FILE.txt.gz, then warning about possible conflict is showm and
    the original file argument is used (FILE.txt).

    You have to enable this feature by setting
    `tiad-:file-compress-support' to non-nil."
  (interactive
   ;; Change    "Fwrite to file: "
   (list
    (read-file-name
     "write to file: "
     (or (buffer-file-name) "~/")
     )))


  (let* ((fn           (ad-get-arg 0))
	 ;;  Tmp buffers do not have filename
	 (buffer-file  (or fn (buffer-file-name) ""))
	 (pass         t)
	 )

    ;;  Check if this is VAX file, remove the version extension
    ;;  for the buffer if this is
    ;;
    (if (and  (stringp buffer-file)
	      (o-string-match "\\(.*\\);[0-9]+$" buffer-file))
	(setq fn (read-match 1 buffer-file)))

    (if tiad-:file-compress-support
	(ad-set-arg 0 (tiad-convert-filename fn 'cautious))
      (ad-set-arg 0 fn))

    (if (stringp fn)
	(setq pass (string-match-case tiad-:write-file-no-confirm fn)))

    (if (or (not (interactive-p))	;only when user call it, do checks
	    (not (file-exists-p fn))
	    pass
	    (y-or-n-p (format "%s already exists; overwrite it? " fn)))
	ad-do-it
      (message "Aborted"))
    ))

;;; ----------------------------------------------------------------------
;;;
(defadvice write-region (around tiad-file dis)
  "See `write-file' which explains the advice behavior."
  (interactive "r\nFwrite region: ")
  (let* ((fn           (ad-get-arg 2))
         (buffer-file  fn)
	 (pass         t)
	 )

    ;;  Check if this is VAX file, remove the version extension
    ;;  for the buffer if this is
    ;;
    (if (and (stringp buffer-file)
	     (o-string-match "\\(.*\\);[0-9]+$" buffer-file))
	(setq fn (read-match 1 buffer-file)))

    (if tiad-:file-compress-support
	(ad-set-arg 2 (tiad-convert-filename fn 'cautious))
      (ad-set-arg 2 fn))

    (if (stringp fn)
	(setq pass (string-match-case tiad-:write-file-no-confirm fn)))

    (if (or (not (interactive-p))
	    (not (file-exists-p fn))
	    pass
	    (y-or-n-p (format "%s already exists; overwrite it? " fn)))
	ad-do-it
      (message "Aborted")))
  )

;;}}}
;;{{{ fill

;;; ............................................................ &fill ...


;;; ----------------------------------------------------------------------
;;; In new cc-mode there variable c-hanging-comment-ender-p
;;; which does exactly same than this advice.
;;;
;;; We install this advice for older emacs only.
;;;
;;;
(when (tiad-activate-p 'fill-paragraph)

(defadvice fill-paragraph (after tiad dis)
  "Touch C comment filling, otherwise do nothing.
If the fill was done to C comment. It usually levaes it like this,
while this advice corrects it a bit and moves the last asterisk to
the next line.

/* comment ...         /* comment ...
 * ends here. */        * ends here.
                        */

This function does not affect C comments that occupy only one line."
  (let* (col
	 line
	 )
    (when (and (save-excursion
		 (beginning-of-line)
		 (and
		  ;;  If this is continuing line "*", then search back
		  ;;  otw we're at "/*" already
		  ;;
		  (if (looking-at "^[ \t]*[*]")
		      (re-search-backward "^[ \t]*/[*]" nil t)
		    (looking-at "^[ \t]*/[*]"))
		  (re-search-forward  "^[ \t]*/[*]" nil t)
		  (setq col (current-column)  line (LINE))
		  ))
		 (re-search-forward "[*]/" nil t)

		 ;;  - The "/*" and "*/" must be at different lines,
		 ;;    because only then we want to adjust the last "*/"
		 ;;  - Skip one line comments.
		 (not (eq (LINE) line))
		 )
      (delete-backward-char 2) (insert "\n")
      (move-to-column (1- col) 'force)
      (insert "*/")
      )
    ))
)

;;}}}

;;{{{ gud

;;; ............................................................. &gud ...

;;; ----------------------------------------------------------------------
;;; gud.el
;;;
(defadvice gud-display-line (after tiad dis)
  "Highlights current line."
  (let* ((ov tiad-:gud-overlay)
	 (bf (gud-find-file true-file))
	 )
    (save-excursion
      (set-buffer bf)
      (move-overlay ov (BOLP) (EOLP) (current-buffer))
      )))

;;}}}

;;{{{ imenu

;;; ........................................................... &imenu ...

;;; ----------------------------------------------------------------------
;;; May 6 1996, <pbreton@i-kinetics.com>
;;;
(defadvice imenu (before tiad dis)
  "Widen the buffer before activating imenu"
  (widen))

;;}}}
;;{{{ mail

;;; ............................................................ &mail ...


;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;; mailabbrev.el
;;;
(defadvice sendmail-pre-abbrev-expand-hook (around tiad-no-abbrevs-in-body dis)
  "Do not expand any abbrevs in the message body through self-insert-command."
  (if (or (mail-abbrev-in-expansion-header-p)
	  ;; (not (eq last-command 'self-insert-command)) ; can't be used
	  ;; since last-command is the previous, not the current command
	  (not (integerp last-command-char))
	  (eq (char-syntax last-command-char) ?w)) ; this uses that
      ;; the last char in {C-x '} {C-x a '} {C-x a e} is `w' syntax
      ad-do-it
    (setq abbrev-start-location (point) ; this hack stops expand-abbrev
	  abbrev-start-location-buffer (current-buffer))
    ))



;;}}}
;;{{{ map-ynp

;;; ......................................................... &map-ynp ...

;; 19.28
;; - map-ynp.el::map-y-or-n-p  Get's loaded in loadup.el, it pops up
;;   an Dialog Box of questions if the input is event type, I hate
;;   asking yes/no in dialog box... It much quicker to hit SPACE/DEL
;; - Hmm actually it looks back what the command was by looking at
;;   last-nonmenu-event variable, so I should reset it instead.
;; - *argh* I was wrong, is is the y-or-n-p (built-in) command that pops up
;;   the dialog, anyway the advice works for it too: built-in doesn't
;;   matter
;;
;; The way to do this in XEmacs is:
;;
;;    (setq use-dialog-box nil)
;;
(cond
 ((and (ti::xe-window-system) (emacs-p))

  (defadvice map-y-or-n-p (before tiad dis)
    "Reset any mouse event to key event, so that you don't get dialogs."
    (if (listp last-nonmenu-event)
	;; replace with some harmless value
	(setq last-nonmenu-event ?\n)
      ))

  (defadvice y-or-n-p (before tiad dis)
    "Reset any mouse event to key event, so that you don't get dialogs."
    (if (listp last-nonmenu-event)
	;; replace with some harmless value
	(setq last-nonmenu-event ?\n)
      ))

  ))

;;}}}
;;{{{ mouse

;;; ........................................................... &mouse ...

;;; ----------------------------------------------------------------------
;;;  Too many accidents happened...
;;;
(defadvice mouse-delete-other-windows  (around tiad dis)
  "Confirms window delete."
  (if (y-or-n-p "Really delete _all_ windows ")
      ad-do-it
    (message "")
    ))


;;; ----------------------------------------------------------------------
;;;
(defadvice mouse-delete-window  (around tiad dis)
  "Confirms window delete."
  (if (y-or-n-p "Really delete _this_ window ")
      ad-do-it
    (message "")
    ))

;;}}}
;;{{{ replace.el

(defadvice occur  (before tiad act)
  "Iinteractive change: ask if user want the occur to start from point-min.
also Possibly unfold/un-outline the code."


  (when (and (interactive-p)
	     (not (eq (point) (point-min)))
	     (y-or-n-p "tiad: Start occur from point-min? ")
	     )
    (if (and (or (and (featurep 'folding)
		      (symbol-value 'folding-mode))
		 (and (featurep 'outline)
		      (symbol-value 'outline-mode))
		 )
	     (save-excursion
	       (PMIN)
	       (re-search-forward "\r" nil t))
	     (y-or-n-p "tiad: Open buffer's selective display too? ")
	     )
	(ti::b-xtra-open-outline)
	)
    ))

;;}}}

;;{{{ simple.el

;;; .......................................................... &simple ...



;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;; simple.el
;;;
(defadvice exchange-point-and-mark (around tiad-pop-if-prefix dis)
  "If given prefix, call `set-mark-command' to pop previous mark positions."
  (if current-prefix-arg
      (call-interactively 'set-mark-command))
  ad-do-it
  )

;;; ----------------------------------------------------------------------
;;; May 6 1996, <pbreton@i-kinetics.com>
;;;
(defadvice goto-line (around tiad dis)
  "Widen the buffer before and after `goto-line' command."
  (widen)

  ad-do-it
  ;;  We do this because, the folding.el sets narrowing in effect,
  ;;  when the goto-line has finished.
  (widen)
  )


;;; ----------------------------------------------------------------------
;;; 17 Nov 1995, pot@cnuce.cnr.it (Francesco Potorti`), gnu.emacs.sources
;;; Redefine indent-for-comment to kill the comment with negative
;;; prefix
;;;
(defadvice indent-for-comment (around tiad dis)
  "Kill the comment with negative prefix."
  (if (eq current-prefix-arg '-)
      (kill-comment nil)
    ad-do-it
    ))


;;; ----------------------------------------------------------------------
;;; 6 Dec 1995, pot@cnuce.cnr.it (Francesco Potorti`), gnu.emacs.sources
;;; Redefine insert-buffer to insert a visible buffer, if there's one.
;;;
(defadvice insert-buffer (before tiad dis)
  "Use a more reasonable default, the other window's content."
    (interactive
     (list
      (progn
	(barf-if-buffer-read-only)
	(read-buffer "Insert buffer: "
		     (if (eq (selected-window)
			     (next-window (selected-window)))
			 (other-buffer (current-buffer))
		       (window-buffer (next-window (selected-window))))
		     t)))))


;;; ----------------------------------------------------------------------
;;; 28 Nov 1995, schrod@iti.informatik.th-darmstadt.de (Joachim Schrod)
;;; avoid deactivation of region when buffer end or beginning is reached
;;;
(defadvice line-move (around tiad dis)
  "Avoid deactivation of region. in `beginning-of-buffer' or `end-of-buffer'."
  (condition-case ()
      ad-do-it
    ((beginning-of-buffer end-of-buffer)
     (if (bobp)
	 (message "Beginning of buffer.")
       (message "End of buffer.")
       ))))



;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;;
(defadvice set-mark-command (around tiad-global-if-negative dis)
  "If the argument is negative, call pop-global-mark."
  (if (< (prefix-numeric-value current-prefix-arg) 0)
      (pop-global-mark)
    ad-do-it
    ))



;;; ----------------------------------------------------------------------
;;;
(defadvice what-cursor-position (around tiad dis)
  "Displays line nbr info too."
  ad-do-it
  ;;  we have to use 'princ' because there is percentage mark
  ;;  in returned string and that would run 'message' beserk,
  ;;  since it thinks it's formatting code
  (princ (concat ad-return-value (int-to-string (safe-widen (LINE)))))
  )



;;; ----------------------------------------------------------------------
;;; Code by [jari]
;;; - Jerome Santini <santini@chambord.univ-orleans.fr> suggested adding
;;;   find-file.
;;;
(defadvice switch-to-buffer (around tiad dis)
  "When called interactively: Confirm switch to non-existing buffer.

References:
  `tiad-:switch-to-buffer-find-file'	,if non-nil, suggest `find-file' for
				         non-existing buffers"
  (interactive "Bbuffer name: ")
  (let ((buffer-name (ad-get-arg 0))
	)
    (if (or (not (interactive-p))	;user didn't call us
	    (get-buffer buffer-name))	;it exists
	ad-do-it
      (cond
       ((y-or-n-p (format "`%s' does not exist, create? " buffer-name))
	ad-do-it)			;create new buffer

       (tiad-:switch-to-buffer-find-file ;is this enabled ?
	(find-file (read-file-name "(tiad) Find-file: "
				   nil
				   nil
				   nil
				   buffer-name)))
       ))

    (message "")			;clear the echo area
    )
  )

;;; ----------------------------------------------------------------------
;;;
(defadvice switch-to-buffer-other-frame  (around tiad dis)
  "Replace function. Don't ever create new frame; reuse some existing frame."
  (let ((free-frames (ti::w-frame-list nil 'exclude-current))
	stat
	)
    (if (null free-frames)
	(pop-to-buffer buffer)
      (cond
       ((setq stat (ti::w-get-buffer-window-other-frame buffer))
	;;  buffer is displayed already in some OTHER frame; go to it.
	(raise-frame (car stat))
	(select-frame (car stat))
	(select-window (cdr stat))
	)
       (t
	;;  Go to some free frame and pop up there
	(raise-frame  (car free-frames))
	(select-frame (car free-frames))
	(switch-to-buffer buffer)
	)))))

;;}}}
;;{{{ subr.el

;;; ----------------------------------------------------------------------
;;; May 6 1996, <pbreton@i-kinetics.com>
;;;
(defadvice save-buffers-kill-emacs (around tiad dis)
  "Redefine `save-buffers-kill-emacs' to prevent accidental logouts"
  (cond
   ((and (interactive-p)
	 (y-or-n-p "tiad: Really quit emacs? "))
    (message "")
    ad-do-it
    )
   ((not (interactive-p))
    ad-do-it
    ))
  )


;;; ----------------------------------------------------------------------
;;; - This puts cursor to generated list. Propably what we
;;;   want 99% of the time.
;;;
(defadvice list-buffers  (after tiad dis)
  "Select the list buffer after displaying."
  (if (interactive-p)
      (select-window (get-buffer-window "*Buffer List*"))))

;;}}}
;;{{{ time

;;; ............................................................ &time ...

;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;; this is for reporter.el by Barry A. Warsaw in the xemacs distribution
;;;
(defadvice display-time-process-this-message (around tiad-no-junk-mail dis)
  "Suppress message in modeline.
If display-time-announce-junk-mail-too is nil, suppress the [Junk mail]
message on the modeline."
  ((let ((modeline display-time-mail-modeline)
	 )
     ad-do-it
     (if (and ad-return-value		; junk-p
	      (not display-time-announce-junk-mail-too))
	 ;; restore non-junk modeline
	 (setq display-time-mail-modeline modeline))
     ad-return-value
     ))
  )


;;}}}
;;{{{ tq.el -- 19.30

;;; .............................................................. &tq ...


;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;;
(defadvice tq-process-buffer (around tiad-be-quiet dis protect)
  "It was overly loud when the webster server times out.
Turns 'pop-to-buffer to 'ignore and 'error to 'message during its execution."
  ;;  Actually I use another webster now, but the old one had
  ;;  thesaurus and stemming
  (let ((pop (symbol-function 'pop-to-buffer))
	(err (symbol-function 'error)))
    (fset 'pop-to-buffer 'ignore)
    (fset 'error 'message)
    ad-do-it
    (fset 'pop-to-buffer pop)
    (fset 'error err)
    ad-return-value)
  )

;;}}}
;;{{{ vc

;;; .............................................................. &vc ...


;;; ----------------------------------------------------------------------
;;;
(defun tiad-rcs-initial-comment (file)
  "Add initial comment leader to RCS FILE."
  (let* (buffer
	 file-type
	 str
	 )
    (when (stringp file)		;if not nil

      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ type of file ^^^
      (job-buffer (get-file-buffer file)
	(setq file-type (or (tib-info nil 'variable)
			    (symbol-name major-mode)))
	(setq str comment-start))

      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ default comment ^^^
      (cond
       ((string-match "lisp" file-type)
	(setq str ";; "))

       ((string-match "c[+]+" file-type)
	(setq str "// "))

       ((stringp str)			;original comment, leave it as is
	nil)

       (t
	(setq str "# ")		;Not set? Suggest shell comment
	))

      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ setting comment ^^^
      (unless (var-nil-p		;only if given something
	       (setq str
		     (read-from-minibuffer
		      "Set RCS comment leader to:" str)))
	(setq str (format "rcs -c\"%s\" %s" str file)) ;Shell command

	(message "tinyad: setting rcs comment...")
	(if (shell-exec-nok-p (shell-execute str "*vc*" ))
	    (pop-to-buffer "*vc*"))

        ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ fixing emacs buffer ^^^
	;;  - Now, the rcs -u only modified the delta file in RCS tree,
	;;    we must take the version out of the tree, so that the new
	;;    comment setting takes place: Do "co" and reread the file
	;;    into emacs.
	;;
	(message "tinyad: refreshing the file comment...")

	(shell-execute (format "co %s" file))

	(job-buffer (get-file-buffer file)
	  (let* (find-file-hooks	;prevent VC this time
		 buffer-read-only
		 enable-local-eval
		 )
	    (find-alternate-file file)
	    (pop-to-buffer (current-buffer))
	    (setq buffer (current-buffer))
	    ))

	(job-buffer "*VC-log*"
	  ;;  Fix this variable, because we reread the file
	  ;;  see vc-finish-logentry
	  (setq vc-parent-buffer buffer))


	(message "tinyad: refreshing the file comment ...done")
	))
    ))



;;; ----------------------------------------------------------------------
;;; - I don't want to use AROUND advice, I reserve it to user.
;;; - That's why the combination before -- after.
;;;
(defadvice vc-do-command  (before tiad-vc  dis)
  "Set flag `tiad-:vc-p' if file is version controlled.
Used by TIAD after advice to determine if initial
comment leader needs to be set."
  ;;  - The arg 'file' is nil when vc calls this command with
  ;;    "rcs" nil nil "-V". We are not interested in those cases.
  ;;
  (if (stringp file)
      (setq tiad-:vc-p (vc-registered file))))


;;; ----------------------------------------------------------------------
;;;
(defadvice vc-do-command  (after tiad-vc  dis)
  "Set initial RCS comment leader.
According to flag `tiad-:vc-p', if file was not version controlled,
ie. the CheckIn was done first time, ask from user about the initial
comment leader and set it."
  (if (and (stringp file)
	   (null tiad-:vc-p)		;Initial CheckIn
	   )
      (tiad-rcs-initial-comment file)))


;;; ----------------------------------------------------------------------
;;; vc-hooks.el , vc-mode-line (file &optional label) [28..30]
;;;
;;; - The string displayed is put into vc-mode variable
;;; - My modeline has 10 modes and it's not good to use long
;;;   mode names...
;;; - This function is called by vc-rcs-status, and the original
;;;   functions doesn't always determine the revision number right.
;;;
(defadvice vc-mode-line (around tiad-vc dis)
  "Change the modeline name to more shorter prompt.
Remove 'RCS' or other backend prefix -- this leaves more space
to other mode names. Add word 'b' if current revision is
in the middle of the (b)ranch and not the last one."
  (let* ((now  (if (and file (vc-registered file))
		   (ti::m-rcs-guess-buffer-version file)))
	 ver
	 )

    (if now			;under version control ?
	(setq ver (ti::m-rcs-head-version file)))

    (if (not (and (and (stringp now) (stringp ver))))
	(ad-set-arg 1 "")
      (cond
       ((string=  now ver)		;it's the highest out
	(ad-set-arg 1 ""))
       (t
	(ad-set-arg 1 "b")
	)))
    ad-do-it
    ))



;;; ----------------------------------------------------------------------
;;;
(defadvice vc-print-log (around tiad-vc dis)
  "Position cursor to current revision"
  (let* (ver
	 )
    (setq ver (strmatget  "[.0-9]+" 0 (or vc-mode "")))

    ad-do-it

    (when ver
      ;;  the version must end directly,
      ;;  "1.4" must not match "1.4.1.1"
      ;;
      ;;  Watch out for this statement too, thats why we start
      ;;  searching from the end of buffer.
      ;;  revision 3.4.1.2	locked by: jaalto;
      ;;
      (PMAX)
      (re-search-backward (concat "revision +" ver "[^.]") nil t)
      )))







;;; ----------------------------------------------------------------------
;;; May 6 1996, <pbreton@i-kinetics.com>, some improvements by [jari]
;;;
;;; - Normally each dir have an RCS dir.
;;; - But sometimes user want to keep all RCS files in one RCS dir,
;;;   so he just creates symlinks to that main RCS dir.
;;;
;;;	      /dir/RCS	    main RCS dir
;;;   dir1/RCS ----| |      Symlink 1
;;;   dir2/RCS ------|	    Symlink 2
;;;
;;;
(defadvice vc-register (before tiad-create-rcs-dir dis)
  "RCS directory must exist. Ask to create one if it does not exist."
  (if (not (and (file-exists-p "RCS")
		(file-directory-p "RCS")))
      (if (y-or-n-p
	   (concat
	    "Y = Create new RCS dir, "
	    "N = create symlink to main depository? "
	    ))
	  (make-directory "RCS")

	(if (not (file-exists-p tiad-:vc-main-rcs-dir))
	    (error
	     (format
	      "No main RCS dir found: %s"
	      tiad-:vc-main-rcs-dir)))

	(if (shell-exec-nok-p
	     (shell-execute
	      (format "ln -s %s RCS" tiad-:vc-main-rcs-dir)
	      (use-tmp-buffer tiad-:tmp-buffer 'clear)
	      ))
	    (pop-to-buffer tiad-:tmp-buffer) ; Show shell error
	  )))
  )


;;; ----------------------------------------------------------------------
;;;
(defadvice vc-rcs-status (around tiad-vc dis)
  "Fix this function so that it always set the correct revision."

  ad-do-it

  (let* ((now  (if (and file (vc-registered file))
		   (ti::m-rcs-guess-buffer-version file)))
	 ver
	 )
    (if (and (stringp now)
	     ;;  VC doesn't agree with me, hmpf ? I may be wrong too,
	     ;;  but you can always turn off this advice
	     ;;  and trust VC :-)
	     (not (string-match now ad-return-value))

	     ;;  Last cmd, used by kill-match
	     (string-match "[0-9]+[.][0-9.]+" ad-return-value)
	     )
	(setq ad-return-value (kill-match 0 now ad-return-value)))
    ))



;;}}}

;;{{{ non-emacs distribution files.

;;; ....................................................... &non-emacs ...


;;; ----------------------------------------------------------------------
;;; 29 Apr 1996, Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;; hyberbole package
;;;
(defadvice hkey-help-show (around tiad-shrink-window act)
  "Shrink auxiliary windows to buffer size.
For help-mode,switch `view-mode' off."
  ;;
  ;; hkey-help-show is part of Bob Wiener's Hyperbole. In pure emacs
  ;; a hook is more appropriate: with-output-to-temp-buffer asks the
  ;; function in the variable temp-buffer-show-function (if non-nil)
  ;; to take care of the showing. That function also must call
  ;; temp-buffer-show-hook. Take your pick.
  ;;
  (if (and (not current-window)       ; second arg
           (get-buffer-window buffer))
      (delete-window (get-buffer-window buffer))) ; force recreation
  ad-do-it
  (if (and (not current-window)       ; second arg
           (not (one-window-p t)))    ; not counting the minibuffer
      (shrink-window-if-larger-than-buffer (get-buffer-window buffer)))
  (if (and (eq major-mode 'help-mode)
           (boundp view-mode) view-mode)
      (view-exit)))



;;}}}
;;{{{ extra "my" advices -- disabled

;;; .............................................................. &my ...

;;; ----------------------------------------------------------------------
;;; From: ilya@math.ohio-state.edu (Ilya Zakharevich)
;;; Newsgroups: gnu.emacs.help
;;; Subject: Re: Mapping two commands to a fctn key
;;;
;;; I use the following piece of code (it is 2 days old, and I suspect it
;;; still has some bugs): (After you apply it, the above command should
;;; work, as well as a lot of others.)
;;;
(defun my-define-key-subst-args (name)
  "Used in advice for `define-key' and friends.
NAME should be the symbol name of the \"action\"
argument of the adviced function."
  (let ((def (eval name))
	doc list
	)

    ;; XEmacs 19.14 Bytecomp, shut up "bound but not referenced"
    (if doc (setq doc doc))

    (if (or list
	    (and (listp def) (symbolp (car def))
		 (not (memq (car def) '(lambda keymap))))) ; Extended syntax
	(progn
	  (if (and list (stringp def))
	      (setq doc (list def))	; Process docstring
	    (setq list (cons def list))) ; Now list contains all the actions
	  (eval (list
		 'setq name '(append '(lambda (n raw))
				    doc '((interactive "p\nP")) list)))))))


;;; ----------------------------------------------------------------------
;;; From: ilya@math.ohio-state.edu (Ilya Zakharevich)
;;;
(defadvice define-key
  (before my-key-extra-args (keymap key def &rest list) dis)
  "Allow DEF to be a list to be evaled when key is pressed.
Free variables `n' and `raw' contain numerical and raw repeat counts. Also
supports syntax
\(define-key KEYMAP KEY DOC DEF1 DEF2 ...)
and
\(define-key KEYMAP KEY DEF1 DEF2 ...).
All the DEFi are evaluated, DOC can contain a documentation string."
  (my-define-key-subst-args 'def))



;;; ----------------------------------------------------------------------
;;; From: ilya@math.ohio-state.edu (Ilya Zakharevich)
;;;
(defadvice global-set-key
  (before my-key-extra-args (keys my-function &rest list) dis)
  "Allow DEF to be a list to be evaled when key is pressed.
Free variables `n' and `raw' contain numerical and raw repeat counts. Also
supports syntax
\(global-set-key KEYS DOC DEF1 DEF2 ...)
and
\(global-set-key KEYS DEF1 DEF2 ...).
All the DEFi are evaluated, DOC can contain a documentation string."
  (my-define-key-subst-args 'function))



;;; ----------------------------------------------------------------------
;;; From: ilya@math.ohio-state.edu (Ilya Zakharevich)
;;;
(defadvice local-set-key
  (before my-key-extra-args (keys function &rest list) dis)
  "Allow DEF to be a list to be evaled when key is pressed.
Freevariables `n' and `raw' contain numerical and raw repeat counts. Also
supports syntax
\(local-set-key KEYS DOC DEF1 DEF2 ...)
and
\(local-set-key KEYS DEF1 DEF2 ...).
All the DEFi are evaluated, DOC can contain a documentation string."
  (my-define-key-subst-args 'function))

;;}}}
;;{{{ Examples

;;; ........................................................ &examples ...
;;; Rip code with with tinylib.el/ti::m-pkg-rip-magic


;;; To fontify automatically, whenever compilation buffer runs:

;;* _
;;* (add-hook 'tiad-:compile-internal-hook 'my-compile-font-lock)
;;* _
;;* (defun my-compile-font-lock  ()
;;*   "Compile buffer fontifying immediately."
;;*   (interactive)
;;*   (let* ((buffer tiad-:compile-internal-buffer)
;;*          )
;;*     ;; According to buffer you could set up different font
;;*     ;; keyword parameters, say for
;;*     ;; *compilation*
;;*     ;; *grep*
;;*     ;; *igrep*
;;*     ;; ...
;;* _
;;*     ;;  My setup automatically turn on the lazy-lock too, see
;;*     ;;  font-lock-mode-hook
;;*     (job-buffer
;;* 	buffer
;;*       (font-lock-mode 1))
;;*     ))

;;}}}



(provide   'tinyad)
(run-hooks 'tiad-load-hook)

;;; tinyad.el ends here
