;; @(#) tinyappend.el -- A simple text gathering to buffer utility.

;; @(#) $Id: tinyappend.el,v 1.23 1997/05/14 07:58:25 jaalto NotReleased $
;; @(#) $Keywords: utilities, text, collect, append $
;; $KnownCompatibility: 19.28, XEmacs 19.14 $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1994,95,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      March 20 1994
;;
;; To get information on this program use ident(1) or do M-x tia-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyappend|Jari Aalto|jari.aalto@poboxes.com|
;; Gather text cumulatively to *append* buffer. Store line, region, yank ..|
;; 14-May-1997|$Revision: 1.23 $|~/misc/tinyappend.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;* 	(require 'tinyappend)
;;
;;  Autoload, prefer this one, your emacs starts quicker.
;;
;;* _
;;* 	(autoload 'tia-beg			"tinyappend"	t t)
;;* 	(autoload 'tia-end			"tinyappend"	t t)
;;
;;  If you do not want the default key bindings, add this before the
;;  require command
;;
;;* _
;;* 	(setq tia-:load-hook nil)
;;*
;;  If you have any questions, suggestions, use this function
;;
;;      M-x tia-submit-bug-report


;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...

;; RESERVERD prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tia-" in front of
;;   every function & variable. It stands for '(ti)ny (a)ppend

;;; Commentary:

;;  Preface, march 94
;;
;;	There is nothing fancy, I just wrote this for my own purpose so
;;	that I could gather text from buffers with easy. Later on I could
;;	then peek on that buffer, arrange text etc.
;;
;;	C-x a is handy when appending data to buffer, but it's annoying
;;	that you have to give "buffer name" all the time This one adds to
;;	buffer "*append*" automatically, creating one if it doesn't exist.
;;
;;	I'd strongly recommend you to keep transient-mark-mode (Emacs) on
;;	all the time, so that you can see if you're adding a selected
;;	region into the *append* buffer. If the region is not active, these
;;	functions normally add the current line to the *append* buffer.
;;
;;
;;  Default bindings
;;
;;	My keyboard (HP-UX / X window) just happens to access
;;	these keys easily, so I chose them. Change the tia-:load-hook
;;	if you want to use another keybindings.
;;
;;	    C-c =   Append to the end
;;	    C-c -	Append to the beginning
;;	    C-c _	underscore, Kill (empty) *append* buffer
;;

;;}}}
;;{{{ History

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May 14	1996	[jari]		19.28	v1.23		NotReleased
;; - XEmacs20 compatibility changes (char handling). Internal changes.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Jan 15	1996	[jari]		19.28	v1.17-1.18	NotReleased
;; - Added lib 'm' require. Text
;;
;; Sep 25	1996	[jari]		19.28	v1.16		NotReleased
;; - Rearranged loading. Now uses autoloads.
;;
;; Sep 2	1996	[jari]		19.28	v1.14-1.15	NotReleased
;; - Bug corr: didn't initialize the buffer-file-name for tia-:buffer.
;;
;; Aug 30	1996	[jari]		19.28	v1.13		NotReleased
;; - Rewrote key function to behave more smartly. Now tells if appended
;;   line/region. Corrected bug where the line addition didn't include
;;   newline
;; - Added saving/reloading buffer from file. This makes sure you don't
;;   accidentally lose data. Suggestion made by Stephen Eglen
;;   <stephene@rsunh.crn.cogs.susx.ac.uk>
;;
;; Aug 20	1996	[jari]		19.28	v1.12		NotReleased
;; - XEmacs compatibility. Byte compile errors removed.
;;
;; Jun 28	1996	[jari]		19.28	v1.11		NotReleased
;; - XEmacs compatibility. Used ti::xe-mark-active-p.
;;
;; Jun 7	1996	[jari]		19.28	v1.9-1.10	NotReleased
;; - Thaks to Martin Steppler <steppler@comnets.rwth-aachen.de>
;;   tia-kill				:! corrected
;;   installation			:! changed
;;
;; Feb 17	1996	[jari]		19.28	v1.8		NotReleased
;; - Rewrote the tia-beg and tia-end. They didn't work at all for regions.
;; - Rewrote the whole key binding setup
;;
;; Feb 1	1996	[jari]		19.28	v1.6-1.7	NotReleased
;; - Ahh, this was my first package to ohio nad I remember how
;;   much I needed this small utility. And i still use it :-)
;; - Corrected few bugs, that prevented adding to the end of buffer.
;; - Modified tia-append-to-buffer to take parameter C-u intead of
;;   number.
;; - Checked code with lisp-mnt.el's lm-verify.
;;
;; Sep 7	1995	[jari]		19.28	v1.6		Release_4
;; - It's been long time I used this, but today I just had to get some
;;   lines easily from buffer. Made some cosmetic changes. Added load-hook
;;   and folding to code. Functions now copy current line if area is not
;;   given.
;;
;; Feb 27	1995	[jari]		19.28	v1.4-1.5	Release_3
;; - Corrected LCD entry, now supports Autoload.
;;
;; Nov	26	1994	[jaalto]	19.28	v1.2-1.3	Release_2
;; - File body rewritten. Naming conventions adjusted.
;;   No functional changes
;; - Right GPL licence changed, documentation polished.
;; - Clearer installation
;;
;; March 20	1994 	[jaalo]		18.57	v1.1		Release_1
;; - First release

;;}}}

;;{{{ setup: require

;;; Code:

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))

;;}}}
;;{{{ setup: variables

;;; ......................................................... &v-hooks ...

(defvar tia-:load-hook '(tia-install-default-key-bindings)
  "*Run when file has been loaded.")

;;; ....................................................... &v-private ...

;;; ........................................................ &v-public ...
;;; User configurable

(defvar tia-:buffer  "~/.append"
  "*Buffer where to save text. If this variable has star at the beginnig
of name, like *append*, it is considered that the buffer
doesn't need saving to any file.

Otherwise if file with the same name exists when tinyappend.el is
beeing loaded, the buffer will hold the contents of the file
_only_ if buffer is empty initially.
")

;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyappend.el"
 "tia"
 tia-:version-id
 "$Id: tinyappend.el,v 1.23 1997/05/14 07:58:25 jaalto NotReleased $"
 '(tili-:version-id
   tia-:load-hook
   tia-:buffer
   )))

;;}}}

;;; ########################################################### &funcs ###

;;{{{ code: misc

;;; ----------------------------------------------------------------------
;;;
(defun tia-install-default-key-bindings ()
  "Install tinyappend.el keys."
  (interactive)
  (global-set-key  "\C-c=" 'tia-end)     ;; HP non-shift key
  (global-set-key  "\C-c-" 'tia-beg)     ;; HP non-shift key
  (global-set-key  "\C-c_" 'tia-kill)
  (global-set-key  "\C-c|" 'tia-yank)
  )


;;; ----------------------------------------------------------------------
;;;
(defmacro tia-line-area-args (msg)
  "Return region of current line: (beg end msg) including newline."
  (`
   (list
    (BOLPP)
    (save-excursion
      (end-of-line)
      (safe-do (forward-char 1))	;get newline, unless EOB
      (point))
    (, msg)
    )))


;;; ----------------------------------------------------------------------
;;;
(defun tia-get-buffer ()
  "Creates tia-:buffer and initalizes its content from file if it exists.

Return:
  buffer pointer
"
  (interactive)
  (save-excursion
    (set-buffer (get-buffer-create tia-:buffer))
    (when (not (char= (aref tia-:buffer 0) ?\* ))
      (if (and (file-exists-p tia-:buffer)	;history file exists ?
	       (empty-buffer-p))
	  (insert-file-contents tia-:buffer))
      ;;  link to file so that Emacs asks to save the buffer
      ;;  when you quit with C-x C-c
      ;;
      (setq buffer-file-name tia-:buffer)
      (rename-buffer tia-:buffer)
      ))
  (get-buffer tia-:buffer)
  )

;;}}}
;;{{{ code: main

;;; ----------------------------------------------------------------------
;;;
(defun tia-append-to-buffer (beg end &optional arg msg verb)
  "Stores region to tia-:buffer, default is to stote the end of buffer.
prefix arguments:
  0   = kill append buffer
  C-u = adds to the beginning
"
  (let ((verb  (or verb (interactive-p)))
	(msg   (or msg  ""))
	)
    (save-excursion
      (if (not (eq 0 arg))
	  (copy-region-as-kill beg end))
      (set-buffer (tia-get-buffer))

      (cond				; According to prefix
       ((eq arg 0)			; yank to the beginning
	(kill-buffer (current-buffer))
	(and verb (message (concat "TIA buffer killed")))
	)

       ((and (not (null arg))		; yank to the beginning
	     (listp arg))
	(goto-char (point-min)) (yank)
	(if verb (message (format "*appended %s BEG*" msg)))
	)

       (t
	(goto-char (point-max)) (yank)
	(if verb (message  (format "*appended %s*" msg)))
	))
      )))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tia-end (&optional beg end msg)
  "Stores region or current line to the END of tia-:buffer buffer."
  (interactive
   (if (ti::xe-mark-active-p)
       (list (region-beginning) (region-end) "region")
     (tia-line-area-args "line")))

  (tia-append-to-buffer beg end nil msg 'verb)
  )

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tia-beg (&optional beg end msg)
  "Stores region or current line to the BEGINNING of tia-:buffer buffer."
  (interactive
   (if (ti::xe-mark-active-p)
       (list (region-beginning) (region-end) "region")
     (tia-line-area-args "line")))

  (tia-append-to-buffer beg end '(4) msg 'verb)
  )

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tia-kill ()
  "Kills tia-:buffer  buffer."
  (interactive)
  (if (get-buffer tia-:buffer)
      (kill-buffer tia-:buffer)))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tia-yank (&optional kill)
  "Yanks the tia-:buffer to the current position. Optionally also kills the
storage buffer."
  (interactive "P")
  (if (null (get-buffer tia-:buffer))
      (message (concat "Can't yank, there is no buffer: " tia-:buffer))
    (insert-buffer tia-:buffer)
    (if kill (tia-kill))
    ))

;;}}}

(if (not (get-file-buffer tia-:buffer))
    (tia-get-buffer))

(provide   'tinyappend)
(run-hooks 'tia-:load-hook)

;;; tinyappend.el ends here
