;; @(#) tinycompile.el -- Compile buffer additions. Minor mode.

;; @(#) $Id: tinycompile.el,v 1.9 1997/06/04 06:41:13 jaalto NotReleased $
;; @(#) $Keywords: tools $
;; $KnownCompatibility: 19.28+ XEmacs19.14+ $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is part of GNU Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Sep 19 1996
;;
;; To get information on this program use ident(1) or do M-x txx-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinycompile|Jari Aalto|jari.aalto@poboxes.com|
;; Compile additions: shorten file names, guess compile command for buffer.|
;; 04-Jun-1997|$Revision: 1.9 $|~/misc/tinycompile.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;  Put this file on your Emacs-Lisp load path, add following into your
;;  ~/.emacs startup file. Rip code with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;*     (require 'tinycompile)
;;
;;  or use this autoload; your ~/.emacs loads quicker
;;
;;* _
;;*     (autoload 'tico-mode            "tinycompile" t t)
;;*     (autoload 'turn-on-tico-mode    "tinycompile" t t)
;;*     (add-hook 'compilation-mode-hook 'turn-on-tico-mode 'append)
;;

;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tico-" in front of
;;   every function & variable. It stands for '(ti)ny
;; - variable names contain letters 'tico-:', excecpt version/hook/mode vars.

;;  Preface, Mar 1997
;;
;;	When I was doing grepping over multiple files with igrep.el
;;	the results that were inserted into buffer were too long:
;;	There were 2-6 directory paths which occupied 40 characters and the
;;	actual grep hits were continued with \ character to the right.
;;
;;	That was awfull to read. I couldn't get clear look at the grep
;;	results. I decided that there must be a way to clarify the
;;	results somehow, so I started doing this package.
;;
;;  Overview of features
;;
;;	o   Shortening directory paths
;;	o   Killing non-interesting files from the buffer
;;	o   Hiding commented lines

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	29	1997	[jari]		19.28	v1.7-1.10	NotReleased
;; - Checkdoc 1.42 clear, added defcustom support.
;; - Thanks to Rick Flower <flower@ms31.sp.trw.com> errorneous defvar
;;   corrected (supposed to be defcustom)
;;
;; May	14	1997	[jari]		19.28	v1.5-1.6	NotReleased
;; - Internal changes. now uses minor mode wizard.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Mar	21	1997	[jari]		19.28	v1.2-1.4	NotReleased
;; - Haven't looked at this pacakge for very long time, because I have
;;   been doing other projects. Seems that some functions have been working
;;   very reliably.
;; - Layout of pacakge changed. Many funcktions renamed/programmed.
;;
;; Sep	19	1996	[jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ setup: misc

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyCompile tico-: tools
    "Compile buffers additions.
  Overview of features

	o   Shortening directory paths
	o   Killing non-interesting files from the buffer
	o   Hiding commented lines"
    )


;;; ............................................................ &mode ...
;;;###autoload (autoload 'tico-mode "tinycompile" t t)

(eval-and-compile
(ti::m-fmacro-minor-mode-wizard
 "tico-" " Tco" "'" "Tco" 'TinyCompile "tico-:" 		;1-6

 "Additional commands to Compile buffer. You can kill lines or
shorten the file names and hide comments.

Mode description:
\\{tico-mode}

"
 "TinyCompile"
 (progn
   (if (and tico-mode verb
	    (not (string-match "compil" (symbol-name major-mode))))
       (message-quick "Are you sure this is compile buffer?"))

   )


 "Compile buffer extras."
 (list
  tico-:mode-easymenu-name
  ["Kill matching file lines at point" tico-kill-all-file-lines t]
  ["Shorten directory names"    tico-shorten-lines		    t]
  ["Show(toggle) or hide comments"	tico-show-hide-toggle	    t]
  ["Hide by regexp"		    tico-hide-by-regexp		    t]
  ["Jumpt to file on line"	    tico-parse-line		    t]
  )

 (progn
   (if (eq 1 (length p))
       (define-key map (concat p p)	 'self-insert-command))

    (define-key map (concat p "\177")	'tico-kill-all-file-lines)
    (define-key map (concat p "\b")	'tico-kill-all-file-lines)
    (define-key map (concat p "k")	'tico-kill-all-file-lines)

    (define-key map (concat p "s")	'tico-shorten-lines)
    (define-key map (concat p " ")	'tico-show-hide-toggle)
    (define-key map (concat p "h")	'tico-hide-by-regexp)

    (define-key map (concat p "`")	'tico-parse-line)

    ;;  Overwrite compilation-minor-mode definitions
    ;;
    (define-key map "\C-c\C-c"		'tico-parse-line)
    (define-key map "\C-m"		'tico-parse-line)
    (define-key map [(button2)]		'tico-parse-line)
    ))
)


;;; ......................................................... &v-hooks ...

(defcustom tico-:load-hook nil
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyCompile
  )

;;}}}
;;{{{ setup: public

;;; ........................................................ &v-public ...
;;; User configurable


(defcustom tico-:table-hide
  '(("^.*\\.el:"			;lisp
     "^.*:[ \t]*[;\"'].*"
     )
    ("^.*\\.\\([cC][cC]?\\|[hH][hH]?\\):" ;C/C++
     ":[ \t]*/[/*].*"

     ))
  "*List of FILENAME and HIDE regexps.
If filename in the beginning of line matches elt1 then
show/hide all lines matching elt2.

Format:
 '((FILENAME-REGEXP HIDE-REGEXP)
   (F H)
   ...
   )"
  :type '(repeat
	  (string :tag "File Regexp")
	  (string :tag "Hide Regexp")
	  )
  :group 'TinyCompile
  )

;;}}}
;;{{{ setup: version

;;; ....................................................... &v-version ...

(ti::m-fmacro-version-bug-report
 "tinycompile.el"
 "tico"
 tico-:version-id
 "$Id: tinycompile.el,v 1.9 1997/06/04 06:41:13 jaalto NotReleased $"

 '(tico-:load-hook
   tico-:version-id
   ))

;;}}}

;;{{{ code: macros

;;; ########################################################## &macros ###

;;; ----------------------------------------------------------------------
;;;
(defsubst tico-get-files  (&optional max-point)
  "Return all filenames in compile buffer, optionally until MAX-POINT."
  (beginning-of-line)
  (tico-get-error-lines max-point 'car))

;;}}}

;;; ########################################################### &Funcs ###


;;; ----------------------------------------------------------------------
;;;
(defun tico-shorten-lines ()
  "Shorten the filenames in compile buffer.

Line format must be
  FILE:LINE: results"
  (interactive)
  (let* (dir
	 cd
	 path
	 prev
	 file
         )
    (buffer-enable-undo)
    (save-excursion
      (PMIN)
      (setq cd (bufmatget "^cd +\\(.*\\)" 1))
      (while (re-search-forward "^\\([/.][^:]+\\):" nil t)
	(setq path (read-match 1))

	;; ./pie-mail/hypb.el --> {cd}/pie-mail/hypb.el
	(if (char= (aref path 0) ?.)
	    (setq path (concat cd (substring path 2))))

	(when path
	  (file-components path path file)
	  (kill-match 1 file)

	  (when
	      (or (null prev)
		  (null dir)
		  (string= dir prev))
	    (setq dir path))

	   (unless
	       (string= dir prev)
	     (setq prev dir   dir path)
	     (beginning-of-line)
	     (insert "\ncd " dir "\n\n")
	     )
	   )
	(end-of-line)
	))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tico-parse-line  (&optional verb)
  "Find dir information from the previous 'cd' command. VERB.
Look current line first and if it hashas no directory part,
search backward.

If fuction is called interactively or VERB is set, the found file
is loaded to emacs and cursor put to line. This works like
`compile-goto-error'.

Line format must be:

  FILE:LINE: results

Return:

  (file . line)		,information
  nil			,not valid line"
  (interactive)
  (let* ((path "")
	 line
	 file
	 ret
         )
    (VERB)
    (save-excursion
      (beginning-of-line)
      (when
	  (looking-at "^\\([^:]+\\):\\([0-9]+\\):")
	(setq file (read-match 1)
	      line (string-to-int (read-match 2)))
	(unless
	    (string-match "/" file)
	  (re-search-backward "^cd +\\(.*\\)") ;signal error if not found
	  (setq path (read-match 1)))
	(setq ret (cons (concat path file) line))
	))

    (if verb
	(if (null ret)
	    (message "tico: Can't read file/line information.")
	  (switch-to-buffer-other-window
	   (find-file-noselect (car ret)))
	  (goto-line (cdr ret))
	  ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tico-get-error-lines  (&optional max-point list-func)
  "Gets error lines in compile buffer from current point forward.
Input:
  MAX-POINT    ,max search point, defaults to `point-max'
  LIST-FUNC    ,if given apply it to extract data member.
		Eg 'car, gives you only list of filenames

Return:
 '((\"filename\" . NBR) (F . N) ..)
 or whatever format LIST-FUNC says."
  (let* ((max-point   (or max-point (point-max)))
	 table
	 elt
	 )
    (save-excursion
      (while (and (re-search-forward "^\\([^:]+\\):[0-9]+:" nil t)
		  (< (point) max-point))

	(setq elt (tico-parse-line))
	(if list-func
	    (setq elt (funcall list-func elt)))

	(if (null (member elt table))
	    (push elt table))
	)
      (nreverse table)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun tico-kill-all-file-lines ()
  "Kill all lines associated with the file on the current line."
  (interactive)
  (let* ((elt (tico-parse-line))
	 file
	 re
         )
    (if (null elt)
	(message "Can't find file name in this line.")
      (save-excursion
	(beginning-of-line)
	(setq file (car elt))
	(setq
	 re
	 (format "^%s:\\|^%s:"
		 (file-name-nondirectory file)
		 file
		 ))
	(buffer-enable-undo)
	(PMIN)
	(delete-matching-lines re)
	))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tico-hide-by-regexp  (regexp)
  "Hide lines matching REGEXP."
  (interactive "sHide strings matching: ")
  (tico-show-hide-toggle regexp))


;;; ----------------------------------------------------------------------
;;;
(defun tico-show-hide-toggle  (&optional regexp)
  "Hide or show comment lines matching REGEXP.
We can't kill the lines from buffer because Compile expexts the
lines to be there.

References:
 `tico-:table-hide'"
  (interactive)
  (let* ((list tico-:table-hide)
	 search
	 elt
	 show
         )
    (save-excursion

      (unless regexp			;Find right value
	(setq show (y-or-n-p "Y = show, N = hide "))
	(list-loop list elt
	  (setq search (car elt))
	  (if (re-search-check search)
	      (setq list  nil  regexp (nth 1 elt)))))

      (PMIN)
      (cond
       (show
	(set-text-properties (point-min) (point-max) nil)
	(redraw-display)		;Won't update well otherwise
	)
       (t
	(if (null regexp)
	    (message "No matching regexp in tico-:table-hide")
	  (tima-re-search
	   regexp nil nil nil
	   (if show 'null '(owner tico  invisible t)))
	  )))
      )))



(add-hook 'compilation-mode-hook	'turn-on-tico-mode 'append)
(add-hook 'tico-:mode-define-keys-hook	'tico-mode-define-keys)

(provide   'tinycompile)
(run-hooks 'tico-:load-hook)

;;; tinycompile.el ends here
