;; @(#) tinyff.el -- Regexp find file utility

;; @(#) $Id: tinyff.el,v 1.14 1997/01/15 11:39:46 jaalto NotReleased $
;; @(#) $Keywords: tools, file $
;;
;; $KnownCompatibility: 18.57 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of GNU emacs

;;{{{ Id

;; Copyright (C) 1994,95,96,97 Jari Aalto.
;; Author:       Jari Aalto <jari.aalto@ntc.nokia.com>
;; Maintainer:   Jari Aalto <jari.aalto@ntc.nokia.com>
;; Created:      Nov 3rd 1994
;;
;; To get information on this program use ident(1) or do M-x tiff-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyff|Jari Aalto|jari.aalto@ntc.nokia.com|
;; Regexp find file utility|
;; 15-Jan-1997|$Revision: 1.14 $|~/misc/tinyff.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file, the autoload choice is preferable.
;;
;;     ;;  regular load
;;     (require 'tinyff)
;;     (tiff-install-keys "g")     ; if you want default keys for 18,19
;;
;;     ;;  Autoload
;;     ;;  this will install keys too when used for the first time
;;     (autoload 'tiff-find-file "tinyff" "" t)
;;     (setq tiff-:load-hook '(lambda () (tiff-install-keys "g")))
;;

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; ========================================
;; - To avoid collisions to other modules I use "tiff-" in front of
;;   every function & variable. It stands for '(ti)ny (f)ind (f)ile.
;; - variable names contain letters 'tiff-:', excecpt version/mode vars.

;;  Preface, nov 1994
;;
;;	Inspired by post that I read in gnu.emacs.help in 24 Oct 1994:
;;	"Subject: Re: Q: Wild card to specify a file to be read? (another try)"
;;	The poster  Kai Grossjohann <grossjoh@ls6.informatik.uni-dortmund.de>
;;	told that "There is regexp-find-file.el...." , but I was unable
;;	to locate such packet. Neither did anyone else seemed to know
;;	where it could be ftp'd.
;;
;;	At the time of reading the article I really wasn't hungry for such
;;	.el, but after couple of days when I started C++ project at work
;;	I really missed such beast --> eventually this .el was born.
;;
;;  Overview of features
;;
;;	o   Finds files by user supplied emacs regexp. Two funcs used.
;;	    1) gets files and lets user edit regexp results
;;	    2) when satisfied, use another function to load files in buffer.
;;	    This case can be by-passed, if wanted.
;;	o  Keeps log about failed files & reasons why not loaded.
;;	o  Thrashes temp buffers automatically if wanted.
;;
;; Other find-file ways -- are they better ?
;;
;;	Yes, you propably know that there is one way to load bunch
;;	of files. It goes like this:
;;
;;	    (require 'complete)
;;	    find-file:  *.c
;;
;;	And you get nicely all *.c files.
;;
;;	The completion package gives as a side effect filename completion
;;	too, but I considered quite dangerous: what if you had 20 files in
;;	the directory and not wanting to load all of them...
;;
;;	This package's advantages compared to complete are:
;;
;;	o   you can use _full_ emacs regexp to find files
;;	o   you can edit the files before loading to make sure
;;	    you got the right ones.
;;	o   you can grep from _long listing [dates?] or from regular
;;	    short listing.
;;
;;  Remeber this
;;
;;	You're dealing with emacs regexps, so the following parameter
;;	is invalid:
;;
;;	    (tiff-find-file "*el")
;;
;;	You have mixed the "shell" way o doing things. Instead you have to use
;;	this:
;;
;;	    (tiff-find-file "el$")
;;

;;}}}
;;{{{ History

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = jari.aalto@ntc.nokia.com(work), ssjaaa@uta.fi(University)
;;
;; Jan	15	1997	[jari]		19.28	v1.14	NotReleased
;; - Cleared compilation errors. *THIS* is not ready module.
;;
;; Feb	10	1996	[jari]		19.28	v1.12	NotReleased
;; - Started developing direct find-file feature, so that there is
;;   no intermediate buffer shown to user. **THIS** is not ready module.
;; - Used lisp-mnt's lm-verify to check layout. Rewrote feedback funcs.
;;
;; Feb	27	1995	[jari]		19.28	v1.11	Release_5
;; - Corrected LCD entry. Now supports Autoload.
;;
;; Jan  11	1995	[jari]		19.28	v1.10	Release_4
;; - If user deleted some files from the list, they were still
;;   loaded! --> The copy-region-as-kill was bad choice for copying
;;   the files to tmp buffer. Now uses copy-to-buffer.
;; - If regexp deleted all files, or it didn't find any, user got
;;   empty buffer. Now it just prints message.
;; - Corrected window config restore bug. Changed nicer installation.
;;   Now informs how many files were loaded
;;
;; Dec  14	1994    [jari]		18.57	 v1.9	Release_3
;; - Corr. 2 typing errors, the find-file-this-window refused to load.
;;
;; Dec  13	1994	[jaalto]	18.57	 v1.8	Release_2
;; - Corrected several bugs. Now tries to keep windows as they were.
;; - find-file Prefix arg is now user-variable. Tested in 19.28
;; - Keybindings.
;;
;; Nov	16	1994 	[jari]		18.57	 v1.5	Release_1
;; Nov	3 	1994 	[jari]		18.57	 v1.1
;; - First implementation


;;}}}


;;; Code:
;;; ......................................................... &require ...

(require 'sort)
(require 'tinylibm)


(eval-when-compile
  (defvar byte-compile-dynamic nil "19.29 var") ;silence byteCompiler
  (make-local-variable 'byte-compile-dynamic)
  (setq byte-compile-dynamic t)         ;19.29+
  )

;;{{{ setup: variables
;;; .......................................................... &v-bind ...


(defun tiff-set-keys (key-func)
  "Installs keys."
    (if (string< emacs-version "19")
	(progn
	  (funcall key-func "\C-c\~" 'tiff-find-file )
	  (funcall key-func "\C-c\`" 'tiff-find-file-this-buffer ))

      ;; I suppose we are in 18 then, with extra ESC modifier
      (funcall key-func "\C-x\M-f" 'tiff-find-file )
      (funcall key-func "\C-c\M-f" 'tiff-find-file-this-buffer )
      )
    nil					;if used in hook, must return NIL
    )

(defun tiff-install-keys (env)
  "Installs 'local or 'global keybindings to ENV l,g,b "
  (interactive "slocal global both [lgb]: ")
  (cond
   ((string-equal env "l")
    (tiff-set-keys 'local-set-key))

   ((string-equal env "g")
    (tiff-set-keys 'global-set-key))

   ((string-equal env "b")
    (tiff-set-keys 'local-set-key)
    (tiff-set-keys 'global-set-key)
    )))


;;; ......................................................... &v-hooks ...


(defvar tiff-:load-hook nil
  "*Hook run when file has been loaded.")


;;; ..... user configurable ................................... &conf ...

(defvar tiff-:ignore-case nil
  "*When using RE match , should we ignore case? Identical to
case-fold-search. t means: yes, ignore case.")

(defvar tiff-:ls-cmd "ls"
  "*Way to to get filename information. Should return only files, where
directories are marked with trailing slash. See  tiff-:ls-cmd-o")

;; "o"  means options
(defvar tiff-:ls-cmd-o "-a1F"
  "*How to get files: every file should be on its own row.
Directories must have trailing slash.")

(defvar tiff-:ls-long-cmd-o "-lF"
  "*How to get files in long form: every file should be on its own row.
Directories must have trailing slash.")


(defvar tiff-:ignore-files-re
  "^\\(\.\.\\|\.\\)$\\|\\(\.[o~#]\\)$\\|/$"
  "*Files that are discarded. The default setting ignores:
. ..    directories.
/       general directory
.o      object files
~#      emacs backup and restore files.
")

(defvar tiff-:trash-buffer nil
  "*If set to non-nil always kills temporary buffers, so that they don't
float around your emacs. ")


(defvar tiff-default-parg nil
  "Default prefix argument to use with tiff-find-file.
nil Has no effect on behaviout, t forces always to load directly whithout
editing the files.")


(defvar tiff-:max-files-no-ask 5
  "*For safety, it's good to limit the numver of files loaded into emacs
without asking. One day you just call *.cc and find yourself in
big trouble..

This variable takes effect when ARG parameter is non-nil when calling
function tiff-find-file-1. See also tiff-find-file, which sets this
parameter to t by default.
")


;;}}}
;;{{{ setup: -- private

;;; ..... private variables ................................ &private ...

(defvar tiff-:file-buffer "*tiff*"
  "*The name of the buffer used when loading files.
Should be named as temporary with * marks surrounding.")

(defvar tiff-:file-error-buffer "*tiff-err*"
  "*Files that caused problems and thus were not loaded are stored here.")

(defconst tiff-:kill-ls-F-chars "[*@|]"
  "The ls -F procuces [* exe] [@ link] [| fifo], which are not part of
file name. We remove these.")


;;}}}
;;{{{ setup: -- version

;;; ..... version notice ................................... &version ...

(defconst tiff-version-id
  "$Id: tinyff.el,v 1.14 1997/01/15 11:39:46 jaalto NotReleased $"
  "Latest modification time and version number.")



;;; ----------------------------------------------------------------------
(defun tiff-version ()
  "version information."
  (interactive)
  (require 'tinylib)
  (ti::m-pkg-version-info "tinyff.el"))

;;; ----------------------------------------------------------------------
;;;
(defun tiff-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (require 'tinylib)
  (ti::m-pkg-submit-feedback "tinyff.el"))


;;}}}

;;{{{ code: misc

;;; ############################################################# &misc ##


;;; ----------------------------------------------------------------------
;;;
(defun tiff-trash-buffer (bp &optional flag)
  "Kills buffers used in this module. FLAG forces killing."
  (let* ((b1 (get-buffer tiff-:file-buffer))
	 (b2 (get-buffer tiff-:file-error-buffer))
	 )
    (if flag
	(progn
	  (kill-buffer b1) (kill-buffer b2))
      (if (get-buffer bp) (kill-buffer bp)))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tiff-subs-last-word (beg end)
  "Substitutes region with last words of every line. Words are supposed to
be separated by spaces and tabs."
  (interactive "r" )
  (let* (str
	 pb pe
	 (move t))
    (save-excursion
      (goto-char beg)
      (while (and move
		  (< (point) end))
	(beginning-of-line) 	(setq pb (point))
	(end-of-line)           (setq pe (point))
	;;  find word, first ignore trailing spaces
	(skip-chars-backward " \t" pb)
	(if (null (re-search-backward "[ \t]" pb t)) nil
	  ;;  Was is word really ?
	  (if (not (looking-at " [^- \t/]")) nil ;skip, not a word
	    (setq str (buffer-substring (1+ (point)) pe))
	    (beginning-of-line) (kill-line)
	    (insert str)))
	;; forward word indicates if we're at the end of buffer.
	(setq move (eq 0 (forward-line 1)))
	))
    ))

;;}}}
;;{{{ this buffer

;;; ########################################################### &file  ###
;;; ----------------------------------------------------------------------
;;;
(defun tiff-find-file-this-buffer (&optional buffer)
  "Loads all files listed in separate lines in this buffer.
The files must on their own lines and stick to left margin. Trailing
spaces are ignored. If there are multiple words on the line, the
last one is considered as filename (eg. ls -l listing).

If file was not loaded, the buffer error buffer  will contain list of files
that were not loaded.

Tip: narrow-to-region is quite usefull sometimes!

References:
  tiff-:file-error-buffer
"
  (interactive)
  (let* (
	 (bn		tiff-:file-error-buffer)
	 (trash		tiff-:trash-buffer)
	 (buffer-work	(use-tmp-buffer tiff-:file-buffer 'clear))

	 (file-re	"^[^-~\n\t ]")	;skip these: they are not valid files
	 (file-end	"[ \t\n]")
	 (not-loaded	0)

	 bp				;buffer pointer
	 fn p err
	 count				;how many files to load
	 )

    (if buffer (set-buffer buffer))

    ;; We do all editing unvisible to user. Use tmp buffer
    ;; NOTE:  We can't use copy-region-as-kill -- yank pairs, because
    ;;        it would 'yank' back all text user just killed
    ;;        to form appropriate files. Somehow the user's kill is
    ;;        inserted into buffer.

    (copy-to-buffer buffer-work (point-min) (point-max));move files there
    (set-buffer buffer-work)


    (PMIN)				;Now there is only filenames
    (setq count (count-lines (point-min) (point-max)))



      (while (re-search-forward file-re nil t)
	(beginning-of-line)
	(setq p (point))
	;;   the RE moves automatically to next line.
	(setq fn nil)
	(if (re-search-forward file-end nil t)
	    (setq fn (buffer-substring p (- (point) 1))))

	(setq err nil)
	(if (file-readable-p fn)
	    (if (null (find-file-noselect fn))
		(setq err " find-file error while loading."))
	  (setq err " Not readable.\n"))

	(if (null err)
	    nil				;loaded ok
	  (setq bp (get-buffer-create bn))
	  (save-excursion
	    (set-buffer bp)
	    (if (eq 0 not-loaded) (erase-buffer));add first item
	    (insert (concat fn " Not readable. -- " err  "\n")))
	  (setq not-loaded (1+ not-loaded)))
	)


      (kill-buffer (current-buffer))	;work buffer

      (if (= not-loaded 0)
	  (message (concat "tiff: All files loaded ok [" count "]" ))
	(if trash (tiff-trash-buffer t t));kill both
	(message (concat "tiff: Couldn't load " not-loaded " files.")))

      ))


;;}}}
;;{{{ engine

;;; ----------------------------------------------------------------------
;;;
(defun tiff-find-file (file)
  "This is top level function for tiff-find-file-1. Mostly for bypassing
several extra arguments. The RE is contructed from the ending filename.


"
  (interactive "sFind file: ")
  (let* (filename
	 dir
	 )
    (file-components file dir filename 'strict)

    (tiff-find-file-1 dir filename "n" t)
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tiff-find-file-1 (dir re &optional cmd-type arg)
  "Loads files according to regexp. Lets user to choose/edit files to
load, when match list is constructed.

Input:
  dir		,directory for files
  re            ,lisp regexp to pick files
  cmd-type      ,[Yes]     =  use ls -l otw no long listing
  arg           ,[non-nil] = files are loaded with no-questions-asked.

References:
   tiff-:ls-cmd-o
   tiff-:ls-long-cmd-o
   tiff-:max-files-no-ask
"
  (interactive "Ddirectory: \nsFind file RE: \nsls -l y/[n]: \nP")
  (let* (
	 ;; if not given, use default then
	 (arg		(or arg tiff-default-parg))

	 (bn		tiff-:file-buffer)
	 (bp		(get-buffer-create bn))
	 (plain-files-cmd tiff-:ls-cmd)
	 (cmd-o		tiff-:ls-cmd-o)
	 (long-o	tiff-:ls-long-cmd-o)

	 (case-copy	case-fold-search)
	 (re-ign	tiff-:ignore-files-re)
	 (Fchars	tiff-:kill-ls-F-chars)
	 (no-ask-max	tiff-:max-files-no-ask)
	 (ls-err-re	"not found")

	 count
	 err
	 )

    (save-window-excursion
      (save-excursion
	(set-buffer bp)
	(erase-buffer)
	(setq dir
	      (file-name-as-directory	; make sure it has trailing "/"
	       (expand-file-name dir)))	; sh doesn't get '~'

	;;   Did user say yes to ls -l?
	(if (and cmd-type
		 (string-match "^[ \t]*[yY]" cmd-type))
	    (setq cmd-o long-o))

	(call-process plain-files-cmd  nil bp nil  cmd-o dir)

	(PMIN)
	;;  Get rid of those extra -F flags on files
	(save-excursion (replace-regexp (concat Fchars "$") ""))
	(sort-lines nil (point-min) (point-max))

	;;  add directory name in front of every file
	(PMIN)
	(while (not (eobp))
	  (beginning-of-line)
	  (insert dir)
	  (forward-line) (end-of-line))

	;;   Now pick the files user wanted
	;;   - Check if ls said :"No files found"
	(if (re-search-forward ls-err-re nil t)
	    (progn
	      (setq err t)
	      (message "No files found."))
	  (PMIN)
	  (setq case-fold-search tiff-:ignore-case)
	  (delete-matching-lines re-ign)
	  (delete-non-matching-lines re)
	  (setq count (count-lines (point-min) (point-max)))
	  ;;  Now we have files that user wants to load
	  )) ;; .............................................save-excursion

      (setq case-fold-search case-copy)	;restore value
      (if (> count 0)
	  nil
	(message (concat "Regexp didn't match."))
	(setq err t))

      (if err
	  nil				;skip, error occurred, handled already
       (cond
	((and (<= count no-ask-max) arg);no confirmation
	  (tiff-find-file-this-buffer bp))

	(t
	 (switch-to-buffer-other-window bp)
	 (message
	  (substitute-command-keys
	   (concat "Edit file list, use "
		   "\\[tiff-find-file-this-buffer] to load, "
		   "\\[kill-file] to abort."
		   ))))))

      )))

;;}}}


(provide 'tinyff)
(run-hooks 'tiff-:load-hook)

;;; tinyff.el ends here
;;; ............................................................. &eof ...

