; @(#) tinyhotlist.el -- Hotlist of buffers in X, easy add, easy remove

;; @(#)$Id: tinyhotlist.el,v 1.20 1997/05/14 08:04:43 jaalto NotReleased $
;; @(#) $Keywords: X, popup, hotlist $
;;
;; $KnownCompatibility: 19.28 , XEmacs 19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 15 1995
;;
;; To get information on this program use ident(1) or do M-x tiho-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyhotlist|Jari Aalto|jari.aalto@poboxes.com|
;; Hotlist of buffers in X, easy add, easy remove|
;; 14-May-1997|$Revision: 1.20 $|~/misc/tinyhotlist.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;;; Intallation:
;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Rip code with tinylib.el/ti::m-pkg-rip-magic
;;
;;*     (add-hook 'tiho-load-hook 'tiho-load-save-from-file)
;;*     (require 'tinyhotlist)
;;
;; or use autoload, prefered because your emacs starts up faster
;;
;;* _
;;*     (autoload 'tiho-control              "tinyhotlist" t t)
;;*     (autoload 'tiho-load-save-from-file  "tinyhotlist" t t)
;;
;; Suggested keybinding
;;
;;* _
;;*     ;;  for windowed system
;;*     (global-set-key [C-S-mouse-3] 'tiho-control)
;;* _
;;*     ;;  for non-windowed, close to C-x b , switch-to-buffer
;;*     (global-set-key "\C-cb" 'tiho-control-kbd)
;;
;; Before you can use hotlist, read the documentation of function
;; 'tiho-control'. Example setup is at the end of file.
;;
;; If you have any questions, use this function
;;
;;      M-x tiho-submit-bug-report


;;}}}
;;{{{ Briefly



;;; .................................................... &t-commentary ...
;;; Commentary:

;;}}}
;;{{{ Documentation

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tiho-" in front of
;;   every function & variable. It stands for '(ti)ny (ho)tlist'
;; - variable names contain letters 'tiho-:', except hook/mode vars.

;;  Preface, may 1995
;;
;;	I'm big fan of 'msb.el', but when it comes to files I really need
;;	some  companion with it. When I'm having an emacs session, it's
;;	very common that when I have 20 C++ files in emacs, I start
;;	reading news while the compile it going on its own buffer
;;	[normally I have 15min break], then I see some interesting
;;	article and I want to try some lisp code .. let's load couple of
;;	.emacs.XXXX configurable files. Then I realize that there is mail
;;	coming, because dragbar-time.el tells me that, I switch to rmail
;;	and start reading the latest messages... Within that 15 minute my
;;	emacs if full of buffers and When I use MSB to navigate through
;;	them I get frustrated: "Where was that buffer again, do I need to
;;	go level 3 before I can get that file visible.."
;;
;;	The navigation is especially problem if I'm only working with
;;	handful of source files _actively_, while I still have 30+ files
;;	_loaded_ "just in case", or "for reference lookup".
;;
;;	What I really need, is a simple hotlist for my most recent files,
;;	where I can put/remove items very easily, and which I can
;;	immediately call. No more deep level stepping with msb.el.
;;
;;	Don't get me wrong, I'd never give msb away, it's superb in class
;;	of it's own, but I also need the hotlist, because the 'most
;;	recent files' page in msb changes dynamically whenever I change
;;	buffers, but my hotlist stays the same, unless I remove a item.
;;
;;  Description
;;
;;	Provides X popup where you can add/remove current buffer
;;	with ease, kinda 'most recent work file list'
;;
;;  How to use the hotlist
;;
;;	When you load this el, it declares hotlist cache to store the
;;	items. Obviously the list will be empty at first, but after you
;;	have added entry to it, you'll get the hotlist visible. How
;;	entries are added or removed from hotlist, is explained in function:
;;
;;	    tiho-control
;;
;;	If you can spend some keybindings more and you use lot of
;;	add/remove commands, you can use something like the following too:
;;
;;	    (global-set-key [f1] 'tiho-add)
;;	    (global-set-key [f2] 'tiho-remove)
;;
;;	Non-windwed users can use the popups via completion menu, while
;;	it's not as nice as X-popup. Call this function from keyboard.
;;
;;	    tiho-control-kbd
;;
;;  Cache house keeping -- silently in background
;;
;;	Every time you _select_ item from hotlist, the cache is checked
;;	for possibly non-existing files and all found ones are silently
;;	removed from the list. If you happen to kill lot's of buffers,
;;	there obviously is no need to go and kill those buffers one by one
;;	from hotlist.
;;
;;	If you don't select any buffer eg. by closing the menu by
;;	clicking outside of it, the house keeping isn't performed.
;;
;; Saving/restoring the hotlist
;;
;;	You can save the hotlist to a file with command
;;
;;	    C-u M-x tiho-load-save-from-file RET
;;
;;	And when this package gets loades in your ~/.emacs, the hotlist
;;	is also restored by the load hook:
;;
;;	    (add-hook 'tiho-load-hook 'tiho-load-save-from-file)


;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change log:
;;
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	14	1997	[jari]		19.28	v1.14-1.20	NotReleased
;; - Rearranged loading. Now uses autoloads.
;; - Added lib m require due to macros.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Aug	30	1996	[jari]		19.28	v1.13	        NotReleased
;; - Finally this package is XEmacs compatible. Uses new lib funcs.
;;
;; Aug	12	1996	[jari]		19.28	v1.10-v1.12	NotReleased
;; - Added automatic holist saver/loader
;;   tiho-load-save-from-file		:+
;;
;; Apr	25	1996	[jari]		19.28	v1.9		NotReleased
;; - Changed version variables not to have ":"
;; - Rewrote the example without using the tinylib sort function,
;;   since it no longer exists.
;;
;; Feb	8	1996	[jari]		19.28	v1.8		NotReleased
;; - Added example setup. put all extra funcs to tinylib.
;; - Added non-x user support too. Now uses completion to read buffer.
;;   todo: should use up/down keys to select hotlist buffer
;;
;; Jun	5	1995	[jari]		19.28	v1.7		NotReleased
;; - Added tiho-:default-list, which is default list that cache always
;;   holds.
;; + tiho-:default-func , to initalize default list
;; + tiho-get-buf, to get matching buffers
;; + tiho-re-add , to add many buffers at once
;; ! tiho-kill, now initalizes after kill if requested.
;;
;; May	29	1995	[jari]		19.28	v1.6		Release_2
;; - Added fast function tiho-add, tiho-remove and renamed lowlevel
;;   ones as tiho-remove-1, tiho-add-1
;; - Added documentation about house keeping.
;;
;; May	19	1995	[jari]		19.28	v1.5		NotReleased
;; - Added package id variables. Added ###autolad directive. Tested
;;   byte compilation. Code arranged into right folds, bookmark
;;   sections added.
;;
;; May	19	1995	[jari]		19.28	v1.4		Release_1b
;; - Only corrected some spelling errors in text, LCD etc. Sent to ohio
;;
;; May	19	1995	[jari]		19.28	v1.3		Release_1a
;; - Added sorting of list. Couple of user options more. Removed
;;   library dependency, since only 1 func used. Have used this
;;   extensively now, and seems working ok. Posted to g.e.help
;;
;; May	16	1995	[jari]		19.28	v1.2		NotReleased
;; - Some minor message bugs, when adding/removing. Corrected.
;; - One serious bug: no selection was not handled. Corrected.
;;
;; May	16	1995	[jari]		19.28	v1.1		NotReleased
;; - Finished the code. Seems working quite well, I expect bugs
;;   still...
;;
;; May	15	1995	[jari]		19.28	v1.0		Created


;;}}}

;;; Code:

;;{{{ setup: require

(require 'tinylibm)

(eval-when-compile
  (defvar byte-compile-dynamic nil "19.29 var") ;silence byteCompiler
  (make-local-variable 'byte-compile-dynamic)
  (setq byte-compile-dynamic t)         ;19.29+
  )



;;}}}
;;{{{ setup: private
;;; .......................................................... &v-bind ...

(defvar tiho-:history-keymap  nil
  "Keymap for history.")


(if tiho-:history-keymap		;unused currently
    nil
  (setq tiho-:history-keymap (make-sparse-keymap))
  (define-key  tiho-:history-keymap	[up]	'ignore)
  (define-key   tiho-:history-keymap	[down]  'ignore)
  )

;;; ......................................................... &v-hooks ...

(defvar tiho-load-hook nil
  "*Hook run when file is loaded.")

;;; ....................................................... &v-private ...


(defvar tiho-:cache nil
  "Hotlist cache.")

(defconst tiho-:package 'tiho
  "Package name.")

(defvar tiho-:msg-id (symbol-name tiho-:package)
  "Identifier for package's messages.")

(defvar tiho-:history  nil
  "History for completion.")

(defvar tiho-:autosave-file  "~/.emacs.var.hotlist"
  "*Default hotlist configuration save/restore file.")




;;}}}
;;{{{ setup: user config

;;; ........................................................ &v-public ...

(defvar tiho-:list-max 40
  "*Maximum members in hotlist.")

(defvar tiho-:title "     hotlist     "
  "*Title of menu")

;;  handy if you want to call from non-mouse, eg. pressing key.
;;  --> set event parameter to nil when calling func  tiho-control
;;
(defvar tiho-:x-coord 170
  "*Default menu coordinate.")

(defvar tiho-:y-coord 170
  "*Default menu coordinate.")

(defvar tiho-:msg-sleep 1
  "*Message display time. Set this at least to 1, since there may be
successive messages and moving mouse will clear message buffer otw.")

(defvar tiho-:default-list nil
  "*When the cache is emptied, it will always start with these buffers.
Variable is a list if buffer names '\(buffer buffer..\)

Don't worry if the buffer does not exist, because house keeping is automatic.
")

(defvar tiho-:default-func nil
  "*Function that is called when killing hotlist. It should
fill the tiho-:default-list with default walues.")


(defvar tiho-:use-x-popup t
  "*If you prefere not to use popup-like dialog box for hotlist items,
then set ths variable to nil. This variable is valid only if you're
running in X-windowed system.")


;;}}}

;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyhotlist.el"
 "tiho"
 tiho-:version-id
 "$Id: tinyhotlist.el,v 1.20 1997/05/14 08:04:43 jaalto NotReleased $"
 '(tiho-:version-id
   )))


;;}}}

;;; ########################################################### &Funcs ###



;;{{{ misc

;;; ----------------------------------------------------------------------
;;;
;;;
(defun tiho-msg (error &rest args)
  "Prints packages messages. If error is t, indicate error condition."
  (let* ((pkg (if  tiho-:msg-id
                  (concat tiho-:msg-id ": ")
                ""))
         )
    (setq args (mapconcat 'concat args ""))
    (message  (concat pkg error args))
    (sleep-for tiho-:msg-sleep)
    ))


;;}}}

;;{{{ menu handle
;;; ........................................................ &handling ...


;;; ----------------------------------------------------------------------
;;;
(defun tiho-add-1 (buffer)
  "Adds buffer to hotlist. Arg must be STRING.

Returns:
   t or nil if added.
"
  (let* ((ptr tiho-:cache)
	 el
	 exist
	 )
    ;;  We have to check if it exists already...
    (list-loop ptr el
      (if (equal el buffer)
	  (setq ptr nil exist t)) 	;Stop scanning
      )

    ;;  Add only if not there.
    (if exist
	nil
      (setq tiho-:cache (append tiho-:cache (list buffer)))
      ;;  Keep it in sorted order.
      (setq tiho-:cache (sort tiho-:cache 'string-lessp))
      t)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tiho-add-list (list)
  "Adds list of buffers to hotlist."
  (let* (elt
	 )
    (list-loop list elt
      (tiho-add-1 elt))))



;;; ----------------------------------------------------------------------
;;;
(defun tiho-re-add (re &optional  temp-buf)
  "Adds all buffers matchig RE to hotlist. If optional TEMP-BUF prefix arg
is non-nil the mach is made against temporary buffers also. Normally they
are not counted.
"
  (interactive "sAdd buffers matching: \nP")
  (let* (list)
    (setq list (ti::b-buffer-list re temp-buf))
    (tiho-add-list list)
    ))




;;; ----------------------------------------------------------------------
;;;
(defun tiho-remove-1 (item )
  "Removes buffer from hotlist. Arg is buffer regexp.

Returns:
   t or nil if removed.
"
  (let* ((ptr tiho-:cache)
	 list el
	 ret
	 )
    (list-loop ptr el
      (if (not (string-match item el))
	  (setq list (append list (list el)))
	;;  Stop scanning
	(setq tiho-:cache (append list (cdr ptr)))
	(setq ptr nil  ret t)
	))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tiho-kill (&optional default)
  "Kills hotlist. If optional DEFAULT is non-nil the hotlist is initalized
with default values.

References:
   tiho-:default-list.
"
  (interactive)
  (let* ((func  tiho-:default-func)
	 )

    (if (fboundp func)			;create default list
	(funcall func))
    (setq tiho-:cache nil)

    (if (and default (listp tiho-:default-list))
	(setq tiho-:cache tiho-:default-list))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tiho-cache-update ()
  "Removes all killed buffers from cache. "
  (let* ((ptr tiho-:cache)
	 buffer
	 list
	 )
    (list-loop ptr buffer
      (if (get-buffer buffer)
	  (setq list (append list (list buffer)))
	))
    (setq tiho-:cache list)
    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tiho-load-save-from-file (&optional mode)
  "Save or restore hotlist configuration from file.
Input:
  mode:   nil     ,load from file
          non-nil ,save to file

References:
  tiho-:autosave-file
"
  (interactive "P")
  (let* ((file  tiho-:autosave-file)
	 buffer
	 elt
	 tmp-file
	 list
         )

    (cond
     ((and (null mode)
	   (file-exists-p file))	;forget if there is no file
      (load-file file)
      (list-loop tiho-:cache elt
	(when (file-exists-p elt)
	  (find-file-noselect elt)
	  (push (file-name-nondirectory elt) list)
	  ))
      (setq tiho-:cache list)		;reset it to normal values
      )

     (t
      ;;  We don't need save-excursion here, because we kill the buffer
      ;;
      (set-buffer (setq buffer (use-tmp-buffer "*tiho-config*" 'clear)))

      (insert
       (concat
        ";;\n"
	";; File: Emacs hotlist history file\n"
        ";; Desc: Load this with M-x tiho-load-save-from-file\n"
        "\n"
        ))

      (lisp-mode)
      (insert "(defconst tiho-:cache\n'(\n"  )

      (list-loop tiho-:cache elt
	(if (and (get-buffer elt)
		 (setq tmp-file (buffer-file-name (get-buffer elt)))
		 )
	  (insert "\"" tmp-file "\"\n")
	  ))

      (insert "\n")


      (goto-char (point-max))
      (insert
       (concat
        "))\n\n"
        ";;; " (file-name-nondirectory file) " ends here"
        ))

      (indent-region (point-min) (point-max) nil)

      ;; Save it
      (write-region (point-min) (point-max) file)
      (kill-buffer buffer)
      ))
    ))


;;}}}
;;{{{ X menu

;;; ........................................................... &xMenu ...


;;; ----------------------------------------------------------------------
;;;
(defun tiho-complete (list)
  "Show LIST in completion menu.
Return:
 buffer or nil"
(let* ((menu  (list-to-assoc-menu list))
       (def   (car-safe tiho-:history))
       ret
       )
  (setq ret (completing-read "hot item: " menu nil t def 'tiho-:history))

;;;  (setq ret (read-from-minibuffer
;;;	     "hot item: "          def
;;;	     tiho-:history-keymap  nil
;;;	     tiho-:history))


  (if (var-nil-p ret)			;really selected ?
      nil
    ret)
  ))


;;; ----------------------------------------------------------------------
;;;
(defun tiho-show-menu (event &optional title)
  "Pops the menu and selects the buffer.
If EVENT is nil, uses default coordinates to display the menu.

Return:
  buffer name or nil.
"
  (interactive "e")
  (let* ((list   tiho-:cache)
	 (title  (or title tiho-:title))
	 (x      (cond
		  ((and tiho-:use-x-popup	;; permits use of popup
		        (ti::xe-window-system))
		   t)
		  (t  nil)))			;; no X available...
	 )

;;    (setq x t)

    (if x
	(ti::xe-popup list event nil title)
      (tiho-complete list))

    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tiho-control-kbd (&optional arg)
  "Same as tiho-control, but you can call this from keyboard."
  (interactive "P")
  (tiho-control (ti::xe-make-fake-event tiho-:x-coord tiho-:y-coord) arg)
  )


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tiho-control (event &optional arg)
  "Control center of hotlist.

Optional ARG can be:
  nil           ,show the hotlist
  0             ,kill all members from hotlist.
  9             ,kill all, but initalize with defaults.
  - nbr         ,[negative nbr], remove item from hotlist
  + nbr         ,[positive nbr], add current active buffer to hotlist
  C-u           ,[or listp], remove active buffer from hotlist
"
  (interactive "e\nP")
  (let* ((buffer (buffer-name))
	 (menu   tiho-:cache)
	 ret
	 )
    (cond
     ;; ...................................................... display ...
     ((null arg)
      (if (null menu)
	  (tiho-msg nil  "Empty hotlist.")
	(setq ret (tiho-show-menu event))
	(if (null ret)
	    nil				;no selection
	  (if (get-buffer ret)
	      (switch-to-buffer ret)
	    (tiho-msg nil "Not exist [" ret "]. Removed from hotlist.")
	    (tiho-cache-update))))
	)
     ;; ................................................... remove/add ...
     ((integerp arg)
      (cond
       ((= 0 arg)
	(tiho-kill)
	(tiho-msg nil "Hotlist killed.")
	)

       ;;  I hear you: "Why number 9??" --   Because it's next to number 0
       ((= 9 arg)
	(tiho-kill t)
	(tiho-msg nil "Hotlist killed/initalized.")
	)

       ((> arg 0)
	(if (tiho-add-1 buffer)
	    (tiho-msg nil "Added [" buffer "] to hotlist.")
	  (message "Already in hotlist.")))

       ((< arg 0)
	(if (null menu)
	    (message "Empty hotlist.")
	   (setq ret (tiho-show-menu event "--Remove item--"))
	   (if (null ret) nil		;user cancelled
	     (tiho-remove-1 ret)
	     (tiho-msg nil "Removed. [" ret "]")
	     ))
	)))
     ;; ............................................... remove current ...
     ((listp arg)
      (if (tiho-remove-1 buffer)
	  (tiho-msg nil "removed [" buffer "] from hotlist.")
	(tiho-msg nil
		  "Not in hotlist.")))

     )
    ))



;;; ----------------------------------------------------------------------
;;;
;;;
;;;###autoload
(defun tiho-add ()
  "Adds current buffer to hotlist."
  (interactive)
  (tiho-control nil 1))

;;; ----------------------------------------------------------------------
;;;
;;;
;;;###autoload
(defun tiho-remove ()
  "Remove current buffer from hotlist."
  (interactive)
  (tiho-control nil -1))

;;}}}
;;{{{ Example

;;; ......................................................... &example ...
;;; - This gives you ides how to use package efectively.
;;; - Code can be easily extracted with function tinylib.el/ti::m-pkg-rip-magic
;;; - The ti::  functions are from the lib

;;*   (autoload 'tiho-control		"tinyhotlist"	t t)
;;*   (setq tiho-load-hook 'my-tiho-init)
;;* _
;;*   (defun my-tiho-init ()
;;*     "Sets default statye for hotlist"
;;*     (setq tiho-:msg-id nil)
;;*     (setq tiho-:default-func		'my-tiho-defaults)
;;*     (global-set-key [C-S-mouse-3]	'tiho-control)
;;*     (my-tiho-defaults)			;fill it too
;;*     )
;;* _
;;* _
;;* _
;;*   (defun my-tiho-defaults ()
;;*     "Default hotlist"
;;*     (setq
;;*      tiho-:default-list
;;*      (sort
;;*       (ti::b-buffer-list
;;*        (concat
;;* 	"RMAIL$\\|scratc"
;;* _
;;* 	;; emacs project files
;;* 	"\\|emacr?s\\|funcs.ja.el\\|tiny"
;;* _
;;* 	;; btree.cc  project
;;* 	"\\|b.*[ch]"
;;* _
;;* 	;;  WMP C++ project
;;* 	"\\|wmpmea.*cc"
;;* _
;;* 	;; GNUS
;;* 	"\\|article\\|newsgroup\\|Summary"
;;* 	)
;;*        ;;  include tmp buffers too
;;*        t)
;;* _
;;*        'string<
;;*        ))
;;*     nil					;for possible hook
;;*     )
;;* _
;;*   ;; ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ window-system ^ ^
;;*   )

;;}}}

(provide   'tinyhotlist)
(run-hooks 'tiho-load-hook)

;;; tinyhotlist.el ends here
