;; @(#) tinylibm.el -- library of usefull s(M)all (M)acros or functions

;; @(#) $Id: tinylibm.el,v 1.123 1997/07/04 12:47:57 jaalto NotReleased $
;; @(#) $Keywords: extensions library macros $
;;
;; $KnownCompatibility: (18.57) , 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 6 1995
;;
;; To get information on this program use ident(1) or do M-x tilibm-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinylibm|Jari Aalto|jari.aalto@poboxes.com|
;; General elisp library of small macros and functions.|
;; 04-Jul-1997|$Revision: 1.123 $|~/misc/tinylibm.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;     	(require 'tinylibm)
;;
;; WARNING
;;
;; macro names are not unique in this file, so they may override
;; some existing functions. REPORT IMMEDIATELY IF THIS HAPPENS so
;; that I can correct this unfortunate clash.


;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;;
;;	None. functions/macros may have any names. Be warned!
;;	Normally I have included safe guard to functions that
;;	may exist in emacs, in those cases you get Emac's function
;;	and not implementations from this library.

;; Preface, 1995
;;
;;	This is lisp function library, package itself does nothing.
;;	It contains small functions that you can't live without if you make
;;	your own progrmas
;;
;; Code policy
;;
;;	If I have found some usefull macro from someone's .el or from
;;	news, the proper source and name of author + time is preserved. All
;;	the credit goes to those marvellous programmers!
;;
;;	Please get the 'tinylib.el' so that you can see the code guidelines.
;;
;; Description of package, 'm' macro library
;;
;;	This library contains *small* pieces of code that can easily
;;	solve minor tasks. If the code is small enough, it is usually
;;	in macro form, but don't count on that, it may be regular function
;;	as well.
;;
;;	You must not autoload this package; but always include
;;
;;	    (require 'tinylibm)
;;
;;	in your own pacakges if uou use function. The good thing is
;;	that you don't need any other require commands, because all
;;	my other llibraries are included with that statement when their
;;	functions are called. This is possible because this pacakge
;;	defines all autoloads to those other libraries.
;;
;;	Repeat: you don't have to put this in your pacakges:
;;
;;	    (require 'tinylibm)
;;	    (require 'tinylib)   ;; leave this out
;;	    (require 'tinyliby)  ;; not needed either.


;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jul	4	1997	[jari]		19.28	1.118-123	NotReleased
;; - Updated 'mt' autoloads. Corrected d! function
;;   characterp				:+
;;   vec-arr-init			:now inits to power of two (128)
;;   fexec				:! no mode ByteComp whinings
;;
;; Jun	18	1997	[jari]		19.28	1.115-117	NotReleased
;;    function-adviced-p		:! error
;;    defalias-p			:! now stops is symbol is not function
;;    file-pure-find			:! buffer file name not set
;;
;; Jun	9	1997	[jari]		19.28	1.113-114	NotReleased
;;   save-buffer-modified		:+
;;   int-to-bin-string			:+
;;   bin-string-to-int			:+
;;   int-to-hex-string			:+
;;   int-to-oct				:+
;;   radix				:+
;;   roct, rbin, rhex			:+
;;   int-to-net				:+
;;   rmac, ctime			:+
;;   kill-buffer-safe			:! now sets buffer modified flag.
;;
;; Jun	5	1997	[jari]		19.28	1.108-1.112	NotReleased
;; - Dan Rich <drich@cisco.com> reported that new customize wasn't
;;   loaded in XEmacs right. Corrected the customize loading, but
;;   it needed many eval-and-compile FORMS to be inserted into
;;   this library: I had to know the function-args-p during compilation.
;; - Checkdoc 1.42 clear. (x-overlay error, should have quotes around it)
;; - Now prints "info" that the old custom lib is fine too. no more
;;   load custom multiple times. (featurep) tests.
;;
;; Jun	3	1997	[jari]		19.28	1.107		NotReleased
;; - Defcustom didn't load in XEmacs20 right. Added xemacs-pp test there.
;;
;; May	31	1997	[jari]		19.28	1.105-1.106	NotReleased
;; - the defcustom loading and byte compile messages rewritten.
;; - Lot of unused code removed (aroun 10-20 functions)
;;   find-buffer-visiting		:! error; dind't work
;;
;; May	30	1997	[jari]		19.28	1.103-1.04	NotReleased
;; - The defcustom loading is now complete different than in previous
;;   version. New defcustom works only in 19.34 and 19.15 and up.
;;   make-local-hook			:+
;;   defgroup-tiny			:+
;;   ti::m-defgroup-mail		:+ here, because belongs to defgroup
;;   find-buffer-visiting		:! new implementation
;;
;; May	27	1997	[jari]		19.28	1.99-1.102	NotReleased
;; - Checkdoc 1.29 documentation string errors fixed
;;   function-args-p			:! failed in xemacs.
;;   defcustom				:! displays messages now
;;
;; May	18	1997	[jari]		19.28	1.97-1.98	NotReleased
;; - Added some XE lib autoloads. I should propably rearrange xe
;;   lib better some time and move all autoloads here. Removed
;;   nreverse calls (list-find broke)
;; - Added eval-when for easy-menu, because macros aren't handled by
;;   'macro autoload calls.
;;   use-events				:! changed
;;   use-dynamic-compilation		:! is now macro
;;   use-timer				:! load timer package (XE and E)
;;
;; May	14	1997	[jari]		19.28	1.95-1.96	NotReleased
;; - Changed all non standard boolean tests to use when and unless
;;   line-wrap-p			:+
;;   ECOL				:-
;;   list-c-add-cdr			:! push call error
;;
;; May	2	1997	[jari]		19.28	1.94		NotReleased
;; - removed list-* add and prepend functions, because they're in cl.el
;;   explicit-shell-file-name		+ XEmacs Emulation.
;;   splice				+ nsplice
;;   define-buffer-local-keymap		+
;;   defcustom				+
;;   loop-for				! rewritten, now uses cl macros
;;
;; Mar	23	1997	[jari]		19.28	1.91-1.93	NotReleased
;; - Updated new autolaods for main library (new functions)
;;
;; Mar	21	1997	[jari]		19.28	1.86-1.90	NotReleased
;; - changed char-equal commands to char=, because char= is case sensitive.
;; - Deleted all mouse overlay functions because they were not used.
;;   char=				:+
;;   char-in-list-case			:! correction, now case sensitive
;;   char-assq				:! now case sensitive
;;
;; Mar	17	1997	[jari]		19.28	1.85		NotReleased
;; - more XEmacs20 changes and character handling corrections.
;;   char-in-list-case			:+
;;   char-assq				:+
;;   tinyck-maybe-activate		:+
;;   read-char-safe-until		:! rewritten
;;
;; Mar	12	1997	[jari]		19.28	1.82-1.84	NotReleased
;; - Rewrote read-char-safe-until so that it takes care of XEmacs 20
;;   specific character comparision. Emacs treats characters as ints,
;;   but in XEmacs 20 they are whole different type (MULE support)
;; - Rewrote read-char-safe-until because it was too slow in Emacs.
;;   (You had to wait betwween every typed character.)
;;
;; Mar	6	1997	[jari]		19.28	1.74-1.81	NotReleased
;; - Had to move code around due to compile problems. Now the undefined
;;   functions are defined first and then the rest of the pacakge
;;   is compiled.
;; - Corrected read-char-safe to use char-int because read-char changed
;;   in XEmacs20. Thanks to Dan Rich <drich@cisco.com>
;; - Removed Emas v18 support.
;;   char-int				:+ not exist in XEmacs19.14
;;   xemacs-pp				:+ test XEmacs20 or XEmacs19.15
;;   occur-do-macro			:+
;;   clear-buffer			:+
;;   find-buffer-visiting		,+ XEmacs compatibility (defalias)
;;   define-key-if-free			:! bug
;;
;; Feb	28	1997	[jari]		19.28	1.74-1.76	NotReleased
;; - Lib 'y' autoloads added. (missing)
;;   overlay-supported-p		:! not working in XEmacs
;;   re-search-check			:! changed; now starts from p-min
;;
;; Feb	27	1997	[jari]		19.28	1.73-1.75	NotReleased
;; - Removed overlay autoloads [lib o] because they can't be used in XEmacs
;; - Updated libmt and tinymatch autoloads.
;; - Moved code around the library because it didn't compile in XEmacs.
;;   "not defined before used" syndrome
;;   keep-lower-order			:+
;;   face-supported-p			:+
;;   use-dynamic-compilation		:+
;;
;; Feb	25	1997	[jari]		19.28	1.72		NotReleased
;;   no-action-in-progress		:+
;;   push-definition			:+
;;   pop-definition			:+
;;   define-key-if-free			:+
;;   run-hooks-until			:! bug
;;
;; Feb	16	1997	[jari]		19.28	1.69-1.71	NotReleased
;;   window-pmax-line-p			:! corrected
;;   window-pmin-line-p			:! corrected
;;   selective-display-line-p		:+
;;   copy-key-definition		:+
;;
;; Jan	22	1997	[jari]		19.28	1.66-1.68	NotReleased
;; - Updated 'mt' autoloads.
;;   file-mode-protect			:+
;;   window-pmax-line-bol		:+
;;   count-char-in-region		:+
;;   LINE				:! uses now count-char-in-region
;;
;; Dec	17	1996	[jari]		19.28	v1.64-1.65	NotReleased
;;   file-mode-make-read-only-all	:+
;;   VERB				:+
;;   current-frame			:+
;;   return-work-frame			:+
;;   use-prefix-key			:+
;;   file-delete-safe			:! corrected, now does chmod
;;
;; Dec	3	1996	[jari]		19.28	v1.62-1.63	NotReleased
;; - updated autoloads for main lib.
;;   vec-arr-prop			:! new parameter
;;
;; Nov	23	1996	[jari]		19.28	v1.60-1.61	NotReleased
;; - Corrected some autoload statements. i-macros corrections
;;   ti::p-http-request			:+ new autoload
;;   nconcc				:+ fastest list add method
;;   emacs-v18				:! bug corrected
;;   run-hooks-until			:! now accepts list of hooks
;;
;; Nov	10	1996	[jari]		19.28	v1.59		NotReleased
;; - New autoloads.
;;   i-macro-region-ask			:+
;;   i-macro-region-body		:+
;;   face-p				:!
;;
;; Nov	9	1996	[jari]		19.28	v1.58		NotReleased
;; - Generated autoloads for all libs. Lib 'q' is NOT USED any more.
;;   The functions have been moved to main library.
;;
;; Nov	8	1996	[jari]		19.28	v1.57		NotReleased
;;   safe-narrow			:+
;;   buffer-modified-pp			:+
;;   re-search-check			:!
;;   file-delete-safe			:! XEmacs support
;;   safe-do				:! XEmacs support
;;
;; [Version log deleted]
;;
;; May	6	1995	[jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ code: Emacs compatibility, aliases, byteCompiler


;;; ################################################### &compatibility ###
(eval-and-compile

(if (not (fboundp 'eval-after-load))	;XEmacs 19.14 doesn't have this
    (defun eval-after-load (arg1 form)
      (eval form)))

(if (not (fboundp 'window-edges))	;XEmacs doesn't have this
    (autoload 'window-edges "tinylibxe"))

(if (not (fboundp 'region-exists-p))	;Emacs doesn't have this
    (defalias 'region-active-p	'ti::xe-mark-active-p))

(if (not (fboundp 'number-to-string))
    (defalias 'number-to-string 'int-to-string))

(if (not (fboundp 'set-text-properties))
    (defalias 'set-text-properties 'ignore))

(if (not (fboundp 'string-to-number))
    (defalias 'string-to-number 'string-to-int))


(if (not (fboundp 'match-string))	;only in 19.30+
    (defalias 'match-string	'read-match))


(if (not (fboundp 'make-local-hook))	;Only in 19.30+ I guess
    (defun make-local-hook (hook)
      "tinylibm.el make HOOK local"
      (add-hook-local hook nil)))


(if (not (fboundp 'find-buffer-visiting)) ;not in XEmacs 19.14
    (defun find-buffer-visiting (file)
      "tinylibm.el. FILE."
      ;;   file-truename  dies if there is no directory part in the name
      ;;   Check it first
      (or (and (string-match "^/" file)
	       (get-file-buffer (file-truename file)))
	  (get-file-buffer file)
	  )))



(if (not (fboundp 'backward-line))
    (defun backward-line (&optional arg)
      (forward-line (if (integerp arg)
			(- 0 arg)
		      -1))))

(if (not (fboundp 'abs))
    (defun abs (x)
      "absolute values."
      (if (< x 0) (- x) x)))




;;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. XEmacs20 char . .

(if (not (fboundp 'char-int))		;Not in XEmacs19.14
    (defalias 'char-int 'identity))

(if (not (fboundp 'char-int-p))		;Not in Emacs (in XEmacs20 MULE)
    (defun char-int-p (ch)
      (and (integerp ch)
	   (> ch -1)			;valid range 0-255
	   (< ch 255)
	   )))

(if (not (fboundp 'int-to-char))	;Not in Emacs (exist in XEmacs 20)
    (defalias 'int-to-char 'identity))	;In Emacs chars are ints

(if (not (fboundp 'char-to-int))	;Not in Emacs (exist in XEmacs 20)
    (defalias 'char-to-int 'identity))

(if (not (fboundp 'char-equal))
    (defalias 'char-equal 'equal))

(if (not (fboundp 'characterp))
    (defalias 'characterp 'integerp))	;exists in  XEmacs

(if (not (fboundp 'char=))
    (defun char= (ch1 ch2 &optional ignored-arg) ;exists in  XEmacs 20.1
      (let* (case-fold-search)		;case sensitive
	(char-equal ch1 ch2))))


(if (not (fboundp 'char-in-list-case))
    ;; In XEmacs20, you can't use following
    ;; (memq ch '(?a ?b ?c ?d ?e ?f)), because 'eq' test against
    ;; characters is wrong.
    ;;
    ;; Neither is this format recommended.
    ;; (memq (char-int ch) (mapcar 'char-int '(?a ?b ?c ?d ?e ?f)))
    ;;
    ;; Vladimir Alexiev <vladimir@cs.ualberta.ca> also suggests
    ;; cl's (member* ch '(?a ?b) :test 'char=)
    ;;
    (defun char-in-list-case (ch list)
      "If CH can be found in LIST, return a pointer to it.
       The Match is case sensitive. And nil CH is ignored."
      (let* (case-fold-search
	     ret
	     )
	(while (and list ch)
	  (if (char= ch (car list))
	      (setq ret list  list nil)	;Stop there
	    (setq list (cdr list))))
	ret
	)))


(if (not (fboundp 'char-assq))
    (defun char-assq (ch alist)
      "If CH can be found in ALIST, return entry. If CH is nil, do nothing."
      (let (case-fold-search
	    ret
	    )
	(while (and ch alist)
	  (setq ret (car alist))
	  (if (char= ch (car ret))
	      (setq alist nil)
	    (setq alist (cdr alist)  ret nil)
	    ))
	ret
	)))



;;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..  19.30 match . .

;; 19.30 changed the matching operations
;; o = original or old
;;
(cond
 ((boundp 'posix-string-match)
  (defalias 'o-string-match	'posix-string-match)
  (defalias 'o-looking-at	'posix-looking-at)
  (defalias 'o-search-forward	'posix-search-forward)
  (defalias 'o-search-backward	'posix-search-backward)
  )
 (t
  (defalias 'o-string-match	'string-match)
  (defalias 'o-looking-at	'looking-at)
  (defalias 'o-search-forward	'search-forward)
  (defalias 'o-search-backward	'search-backward)
  ))



;; Emacs and XEmacs differ here. Convert Emacs func --> XEmacs name
;;
(if (and (fboundp 'shell-command-to-string)
	 (not (fboundp 'exec-to-string)))
    (defalias 'exec-to-string 'shell-command-to-string))

(if (not (boundp 'explicit-shell-file-name))
    (defvar explicit-shell-file-name nil)) ;XEmacs 19.14 doesn't have this.

;; Eval-and-compile ending paren
)
;;}}}
;;{{{ setup: require

;;; ......................................................... &require ...
;;; These macros are defined before used

(require 'backquote)
(require 'cl)
;;  (require 'advice)			;Needed by many packages.


(eval-and-compile
  ;;  We have to put these functions inside here, because
  ;;  we later use construct
  ;;
  ;;  (eval-and-compile
  ;;    (when (xemacs-p)
  ;;      (autoload ....
  ;;
  ;;  And that doesn't succeed, because Compiler doesn't yet know
  ;;  that xemacs-p exists inside eval-and-compile. By putting the
  ;;  definitions here; the compiler is gurranteed to see them.
  ;;

  (defsubst xemacs-p ()
    "Test if we're in XEmacs. Returns main revision as integer (eg. 19)."
    (and (string-match "^\\([0-9]+\\)\\..*XEmacs" emacs-version)
	 (match-end 1)
	 (string-to-int
	  (substring emacs-version 0 (match-end 1) )
	  )))

  (defsubst xemacs-pp  ()
    "Check if XEmacs20+ or XEmacs 19.15+ is used. This was a big leap
and many changes were done."
    (let ((ver (xemacs-p))
	  )
      (and ver				;This is Emacs if ver is not set
	   (or (eq ver 20)
	       (and (eq ver 19)
		    (> emacs-minor-version 14)
		    )))))

  (defsubst emacs-p ()
    "Test if we're in Emacs."
    (not (boundp 'xemacs-logo)))


  ;;  forward declaration
  ;;
  (if (not (fboundp 'count-char-in-region))
      (autoload 'count-char-in-region "tinylibm"))

  (autoload 'occur	"replace" t t)

  ;; Use defvar to silence byteCompiler
  ;;
  (defvar byte-compile-dynamic nil "19.29 var")
  (make-local-variable 'byte-compile-dynamic)
  (setq byte-compile-dynamic t)         ;19.29+
  )

;;}}}
;;{{{ setup: variables

;;; ......................................................... &v-hooks ...

(defvar ti:m-load-hook nil
  "*Run when file has been loaded.")

;;; ....................................................... &v-private ...

(defconst ti:m-debug-buffer  "*d!!*"
  "*Debug buffer where to write. Make a wrapper to use function d!!
In your programs, like:

  (defvar my-package-:debug nil
    \"Debug. On/offf.\")

  (defvar my-package-:debug-buffer \"*my-package*\"
    \"Debug record buffer.\")


  (defmacro my-package-debug (&rest args)
    \"Record debug info.\"
    (`
     (let* ( ;; write data to package private buffer.
	    (ti:m-debug-buffer my-package-:debug-buffer)
	    )
       (if my-package-:debug
	   (d!! (,@ args))))
   ))

  ;; this is how you use the debug capability in functions.
  ;; You must enable debug with (setq my-package-:debug t)
  ;;
  (defun my-package-some-function ()
     ;; ... code
     (my-package-debug \"here\" var1 win1ptr buffer \"\\n\" )
     ;; ... code
     )")


;;}}}

;;; --++-- --++-- --++-- --++-- --++-- --++-- --++-- --+ compile start --
(eval-and-compile

;;{{{ code: Autoload 'main' lib

(autoload 'ti::p-uude-cmd                       "tinylib" nil nil)
(autoload 'ti::p-uu-cmd                         "tinylib" nil nil)
(autoload 'tinylib-version                      "tinylib" nil t)
(autoload 'tinylib-submit-feedback              "tinylib" nil t)
(autoload 'ti::s-sqz                            "tinylib" nil nil)
(autoload 'ti::s-verify-ends                    "tinylib" nil nil)
(autoload 'ti::s-del-re                         "tinylib" nil nil)
(autoload 'ti::c-format-string                  "tinylib" nil nil)
(autoload 'ti::c-url-to-ange-ftp                "tinylib" nil nil)
(autoload 'ti::c-upcase-words-to-variable-names "tinylib" nil t)
(autoload 'ti::c-time-list                      "tinylib" nil t)
(autoload 'ti::c-ch2escape                      "tinylib" nil nil)
(autoload 'ti::c-str2re                         "tinylib" nil nil)
(autoload 'ti::c-int-to-access-mode-string      "tinylib" nil nil)
(autoload 'ti::m-rcs-delta-get-file             "tinylib" nil nil)
(autoload 'ti::m-rcs-delta-lock-status          "tinylib" nil nil)
(autoload 'ti::m-rcs-str-find                   "tinylib" nil nil)
(autoload 'ti::s-rcs-look-id                    "tinylib" nil nil)
(autoload 'ti::m-rcs-sort-same-level-list       "tinylib" nil nil)
(autoload 'ti::m-rcs-files-in-dir               "tinylib" nil nil)
(autoload 'ti::m-rcs-guess-buffer-version       "tinylib" nil nil)
(autoload 'ti::m-rcs-buffer-version             "tinylib" nil nil)
(autoload 'ti::m-rcs-previous-version           "tinylib" nil nil)
(autoload 'ti::m-rcs-get-all-branches           "tinylib" nil nil)
(autoload 'ti::d-standard-date                  "tinylib" nil t)
(autoload 'ti::d-s-month-n                      "tinylib" nil t)
(autoload 'ti::d-time-diff-days                 "tinylib" nil nil)
(autoload 'ti::s-repeat                         "tinylib" nil nil)
(autoload 'ti::s-syntax-info                    "tinylib" nil t)
(autoload 'ti::s-tabify                         "tinylib" nil nil)
(autoload 'ti:s-read-match-subs                 "tinylib" nil nil)
(autoload 'ti:s-read-match-list                 "tinylib" nil nil)
(autoload 'ti::s-case-replace                   "tinylib" nil nil)
(autoload 'ti::s-index                          "tinylib" nil nil)
(autoload 'ti::s-index-substring                "tinylib" nil nil)
(autoload 'ti::s-1-space                        "tinylib" nil nil)
(autoload 'ti::s-get-blocks                     "tinylib" nil nil)
(autoload 'ti::s-listify                        "tinylib" nil nil)
(autoload 'ti::b-find-ange-buffer               "tinylib" nil nil)
(autoload 'ti::b-find-ange-to-dired-buffer      "tinylib" nil nil)
(autoload 'ti::b-uu-area                        "tinylib" nil nil)
(autoload 'ti::b-uu-line-p                      "tinylib" nil t)
(autoload 'ti::b-area-bounds                    "tinylib" nil nil)
(autoload 'ti::b-read-if-solid                  "tinylib" nil nil)
(autoload 'ti::b-get-wspc                       "tinylib" nil nil)
(autoload 'ti::b-read-line                      "tinylib" nil nil)
(autoload 'ti::b-grep-lines                     "tinylib" nil nil)
(autoload 'ti::b-looking-back-at                "tinylib" nil nil)
(autoload 'ti::b-read-char                      "tinylib" nil nil)
(autoload 'ti::b-read-word                      "tinylib" nil nil)
(autoload 'ti::b-read-space-word                "tinylib" nil nil)
(autoload 'ti::b-read-syntax-word               "tinylib" nil nil)
(autoload 'ti::b-read-nth-word                  "tinylib" nil nil)
(autoload 'ti::b-zap-to-regexp                  "tinylib" nil t)
(autoload 'ti::b-leave-nth-word                 "tinylib" nil t)
(autoload 'ti::b-kill-line                      "tinylib" nil t)
(autoload 'ti::b-u2d-crlf                       "tinylib" nil t)
(autoload 'ti::b-arrow-control                  "tinylib" nil nil)
(autoload 'ti::b-insert-line-numbers            "tinylib" nil t)
(autoload 'ti::b-randomize-lines                "tinylib" nil t)
(autoload 'ti::b-make-dup-line                  "tinylib" nil t)
(autoload 'ti::b-inc-string-nbr                 "tinylib" nil t)
(autoload 'ti::b-copy-line-and-inc-numbers      "tinylib" nil t)
(autoload 'ti::b-copy-word                      "tinylib" nil t)
(autoload 'ti::b-add-newlines-to-region         "tinylib" nil t)
(autoload 'ti::b-cnv-empty-lines                "tinylib" nil t)
(autoload 'ti::b-del-dup-lines                  "tinylib" nil t)
(autoload 'ti::b-replace-in-region-arguments    "tinylib" nil nil)
(autoload 'ti::b-replace-in-rectangle           "tinylib" nil t)
(autoload 'ti::b-delete-until-non-empty-line    "tinylib" nil t)
(autoload 'ti::b-trim-blanks                    "tinylib" nil t)
(autoload 'ti::b-replace-regexp                 "tinylib" nil nil)
(autoload 'ti::b-diff-type-p                    "tinylib" nil nil)
(autoload 'ti::b-xtra-open-outline              "tinylib" nil t)
(autoload 'ti::b-get-re                         "tinylib" nil t)
(autoload 'ti::b-buffer-list                    "tinylib" nil nil)
(autoload 'ti::b-buffer-list-files              "tinylib" nil nil)
(autoload 'ti::w-frame-list                     "tinylib" nil nil)
(autoload 'ti::w-list                           "tinylib" nil nil)
(autoload 'ti::w-get-buffer-window-other-frame  "tinylib" nil t)
(autoload 'ti::w-find-bottom                    "tinylib" nil nil)
(autoload 'ti::w-match-buffers                  "tinylib" nil nil)
(autoload 'ti::k-function-bind-info             "tinylib" nil nil)
(autoload 'ti::k-reinstall-minor-mode           "tinylib" nil nil)
(autoload 'ti::k-add-minor-mode                 "tinylib" nil nil)
(autoload 'ti::k-map-bind-control               "tinylib" nil nil)
(autoload 'ti::k-xlat-table                     "tinylib" nil nil)
(autoload 'ti::k-put-abc-map                    "tinylib" nil nil)
(autoload 'ti::k-put-map                        "tinylib" nil nil)
(autoload 'ti::k-mapkeys                        "tinylib" nil nil)
(autoload 'ti::t-wipe-properties                "tinylib" nil t)
(autoload 'ti::t-try-set-colors                 "tinylib" nil nil)
(autoload 'ti::m-word-move                      "tinylib" nil t)
(autoload 'ti::m-find-duplicate-same-word       "tinylib" nil t)
(autoload 'ti::m-count-words                    "tinylib" nil t)
(autoload 'ti::m-count-chars-in-delimited-area  "tinylib" nil t)
(autoload 'ti::m-move-paragraph-to-column       "tinylib" nil t)
(autoload 'ti::m-surround-with-char             "tinylib" nil t)
(autoload 'ti::m-add-spaces-80                  "tinylib" nil t)
(autoload 'ti::m-long-line                      "tinylib" nil nil)
(autoload 'ti::m-scramble-reg                   "tinylib" nil t)
(autoload 'ti::m-add-str-reg                    "tinylib" nil t)
(autoload 'ti::m-sort-regexp-fields             "tinylib" nil nil)
(autoload 'ti::m-passwd-grep-user-alist         "tinylib" nil nil)
(autoload 'ti::m-passwd-build-alist             "tinylib" nil nil)
(autoload 'ti::m-passwd-read-entry              "tinylib" nil nil)
(autoload 'ti::m-mail-abbrevs-get-alist         "tinylib" nil nil)
(autoload 'ti::m-mail-abbrevs-email-list        "tinylib" nil t)
(autoload 'ti::m-make-2D-array                  "tinylib" nil nil)
(autoload 'make-ND-array                        "tinylib" nil nil)
(autoload 'ti::m-momentary-string-flash         "tinylib" nil nil)
(autoload 'ti::m-find-function-name             "tinylib" nil nil)
(autoload 'ti::m-reindent                       "tinylib" nil t)
(autoload 'ti::f-ange-completed-message         "tinylib" nil nil)
(autoload 'ti::f-ange-status                    "tinylib" nil nil)
(autoload 'ti::f-ange-download-file             "tinylib" nil nil)
(autoload 'ti::f-ange-file-handle               "tinylib" nil nil)
(autoload 'ti::f-chmod-w-toggle                 "tinylib" nil nil)
(autoload 'ti::f-find-shadows                   "tinylib" nil t)
(autoload 'ti::f-dir-files                      "tinylib" nil nil)
(autoload 'ti::f-files-only                     "tinylib" nil nil)
(autoload 'ti::f-newer-exist                    "tinylib" nil nil)
(autoload 'ti::f-get-ext                        "tinylib" nil nil)
(autoload 'ti::f-get-load-path                  "tinylib" nil t)
(autoload 'ti::f-user-home                      "tinylib" nil nil)
(autoload 'ti::f-file-list                      "tinylib" nil nil)
(autoload 'ti::f-complete-file-name             "tinylib" nil nil)
(autoload 'ti::f-complete-file-name-word        "tinylib" nil t)
(autoload 'ti::f-complete-filename-minibuffer   "tinylib" nil t 'macro)
(autoload 'ti::f-read-file-list                 "tinylib" nil nil)
(autoload 'ti::p-finger-error                   "tinylib" nil nil)
(autoload 'ti::p-finger                         "tinylib" nil t)
(autoload 'ti::p-http-request                   "tinylib" nil t)
(autoload 'ti::p-uu-save-buffer                 "tinylib" nil t)
(autoload 'ti::p-uu-make                        "tinylib" nil t)
(autoload 'ti::p-zip                            "tinylib" nil nil)
(autoload 'ti::p-zip-dir-files                  "tinylib" nil nil)
(autoload 'ti::p-zip-view-command               "tinylib" nil t)
(autoload 'ti::p-tar-zip-view-maybe-command     "tinylib" nil nil)
(autoload 'ti::p-tar-view-command               "tinylib" nil t)
(autoload 'ti::p-tar-read-listing-forward       "tinylib" nil nil)
(autoload 'ti::p-comint-run-command             "tinylib" nil t)
(autoload 'ti::q-read-input-invisible           "tinylib" nil nil)
(autoload 'ti::q-read-input-as-password         "tinylib" nil nil)
(autoload 'ti::m-selective-display-copy-to      "tinylib" nil t)
(autoload 'ti::m-selective-display-print        "tinylib" nil t)
(autoload 'ti::m-ad-control                     "tinylib" nil nil)
(autoload 'ti::m-pkg-submit-feedback            "tinylib" nil t)
(autoload 'ti::m-pkg-submit-bug-report          "tinylib" nil t)
(autoload 'ti::m-pkg-version-info               "tinylib" nil t)
(autoload 'ti::m-pkg-get-header                 "tinylib" nil nil)
(autoload 'ti::m-pkg-ins-example                "tinylib" nil t)
(autoload 'ti::m-pkg-rip                        "tinylib" nil t)
(autoload 'ti::m-pkg-rip-magic                  "tinylib" nil t)
(autoload 'ti::m-pkg-make-mode-magic            "tinylib" nil t)
(autoload 'ti::m-pkg-make-mode                  "tinylib" nil t)
(autoload 'ti::m-pkg-make-var                   "tinylib" nil nil)
(autoload 'ti::m-pkg-make                       "tinylib" nil nil)
(autoload 'ti::m-pkg-autoload-create-on-file    "tinylib" nil t)
(autoload 'ti::m-pkg-install-pgp-tar            "tinylib" nil t)
(autoload 'ti::xe-overlay-some                  "tinylib" nil nil)
(autoload 'ti::xe-overlay-properties            "tinylib" nil nil)
(autoload 'ti::xe-overlays-at                   "tinylib" nil nil)
(autoload 'ti::xe-overlay-put                   "tinylib" nil nil)
(autoload 'ti::xe-overlay-move                  "tinylib" nil nil)
(autoload 'ti::xe-activate-region               "tinylib" nil nil)
(autoload 'ti::xe-read-password                 "tinylib" nil nil)
(autoload 'ti::xe-mouse-call-original           "tinylib" nil t)
(autoload 'ti::xe-popup                         "tinylib" nil t)
(autoload 'ti::xe-display-depth                 "tinylib" nil nil)
(autoload 'ti::xe-read-event                    "tinylib" nil nil)
(autoload 'ti::xe-executing-macro               "tinylib" nil nil)
(autoload 'ti::xe-make-x-popup-event            "tinylib" nil nil)
(autoload 'ti::xe-make-fake-event               "tinylib" nil nil)
(autoload 'ti::xe-modeline-update               "tinylib" nil nil)
(autoload 'ti::xe-set-frame-name                "tinylib" nil nil)
(autoload 'ti::xe-set-frame-parameter           "tinylib" nil t)
(autoload 'ti::xe-get-frame-parameter           "tinylib" nil nil)
(autoload 'ti::xe-frame-window-config           "tinylib" nil nil)
(autoload 'ti::xe-window-system                 "tinylib" nil nil)
(autoload 'ti::xe-timer-list-control            "tinylib" nil nil)
(autoload 'ti::xe-timer-elt                     "tinylib" nil nil)
(autoload 'ti::xe-timer-process-status          "tinylib" nil nil)
(autoload 'ti::xe-timer-key-delete              "tinylib" nil nil)
(autoload 'ti::xe-mark-active-p                 "tinylib" nil nil)
(autoload 'ti::xe-mail-abbrev-table             "tinylib" nil nil)
(autoload 'ti::xe-expand-mail-aliases           "tinylib" nil t)
(autoload 'ti::xe-set-mode-line-format          "tinylib" nil nil)
(autoload 'ti::m-vmacro-minor-mode              "tinylib" nil nil 'macro)
(autoload 'ti::m-vmacro-minor-mode-1            "tinylib" nil nil)
(autoload 'ti::m-fmacro-minor-mode              "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-minor-mode-1            "tinylib" nil t)
(autoload 'ti::m-fmacro-minor-mode-on           "tinylib" nil t)
(autoload 'ti::m-fmacro-minor-mode-off          "tinylib" nil t)
(autoload 'ti::m-fmacro-minor-mode-install      "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-minor-mode-install-1    "tinylib" nil t)
(autoload 'ti::m-fmacro-define-keys             "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-define-keys-1           "tinylib" nil nil)
(autoload 'ti::m-fmacro-version-bug-report      "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-version-bug-report-1    "tinylib" nil t)
(autoload 'ti::m-fmacro-debug                   "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-debug-1                 "tinylib" nil t)
(autoload 'ti::m-fmacro-install-pgp-tar         "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-install-pgp-tar-1       "tinylib" nil t)
(autoload 'ti::m-fmacro-minor-mode-wizard       "tinylib" nil nil 'macro)
(autoload 'ti::m-fmacro-minor-mode-wizard-1     "tinylib" nil nil)

;; ti::m-fmacro-minor-mode-wizard-1 pfx					;1   mode-Name				;2   mode-Name-prefix-key			;3   easymenu-Name			;4   custom-group				;5   variable-style			;6   doc-str				;7   mode-desc				;8   minor-mode-body			;9   easymenu-doc				;10   easymenu-body			;11   define-key-body			;12
;; ti::m-fmacro-minor-mode-wizard pfx					;1   mode-Name				;   mode-Name-prefix-key			;   easymenu-Name			;   custom-group				;   variable-style			;6   doc-str				;7   mode-desc				;   minor-mode-body			;   easymenu-doc				;10   easymenu-body			;   define-key-body			;12
;; ti::m-fmacro-install-pgp-tar-1 func-ins-sym elisp-file &optional log-buffer
;; ti::m-fmacro-install-pgp-tar func-ins-sym elisp-file &optional log-buffer
;; ti::m-fmacro-debug-1 debug-function debug-toggle-function debug-variable debug-buffer
;; ti::m-fmacro-debug debug-function debug-toggle-function debug-variable debug-buffer
;; ti::m-fmacro-version-bug-report-1 filename   prefix version-variable version-value   bug-var-list   &optional bug-body
;; ti::m-fmacro-version-bug-report filename   prefix   version-variable version-value   bug-var-list   &optional bug-body
;; ti::m-fmacro-define-keys-1 func-def-sym   keymap-sym   prefix-key-sym   easymenu-sym   easymenu-Name-sym   easymenu-doc-str   easy-menu-forms   body
;; ti::m-fmacro-define-keys func-def-sym   keymap-sym   prefix-key-sym   easymenu-sym   easymenu-Name-sym   easymenu-doc-str   easy-menu-forms   eval-body
;; ti::m-fmacro-minor-mode-install-1 func-ins-sym   mode-sym   map-sym   mode-name-sym   hook-sym   &rest body
;; ti::m-fmacro-minor-mode-install func-ins-sym   mode-sym   map-sym   mode-name-sym   hook-sym   &rest body
;; ti::m-fmacro-minor-mode-off          mode-func-sym
;; ti::m-fmacro-minor-mode-on           mode-func-sym
;; ti::m-fmacro-minor-mode-1 func-min-sym    doc-str				;1    install-func			;2    mode-var				;3    mode-Name				;4    prefix-var				;5    menu-var				;6    no-mode-msg				;7    mode-desc				;8    hook				;9    &rest body				;10
;; ti::m-fmacro-minor-mode    func-min-sym   doc-str   install-func				;3   mode-var   mode-Name				;5   prefix-var   menu-var				;7   no-mode-msg   mode-desc				;9   hook   &optional body
;; ti::m-vmacro-minor-mode-1 pfx   mode-Name   mode-Name-prefix-key   easymenu-Name   custom-group   &optional prefix-style
;; ti::m-vmacro-minor-mode pfx   mode-Name   mode-Name-prefix-key   easymenu-Name   custom-group   &optional style
;; ti::xe-set-mode-line-format          fmt
;; ti::xe-expand-mail-aliases           beg end &optional alias-alist
;; ti::xe-mail-abbrev-table
;; ti::xe-mark-active-p
;; ti::xe-timer-key-delete              key
;; ti::xe-timer-process-status
;; ti::xe-timer-elt                     function
;; ti::xe-timer-list-control            &optional mode
;; ti::xe-window-system
;; ti::xe-frame-window-config
;; ti::xe-get-frame-parameter           parameter &optional frame
;; ti::xe-set-frame-parameter prop-or-list value &optional frame
;; ti::xe-set-frame-name                string &optional frame get
;; ti::xe-modeline-update
;; ti::xe-make-fake-event               x y &optional mouse-sym
;; ti::xe-make-x-popup-event            x y
;; ti::xe-executing-macro
;; ti::xe-read-event
;; ti::xe-display-depth
;; ti::xe-popup    string-list &optional event mode title
;; ti::xe-mouse-call-original minor-mode-symbol &optional event
;; ti::xe-read-password                 &optional prompt
;; ti::xe-activate-region               &optional off
;; ti::xe-overlay-move ov-sym beg end &optional make-local face
;; ti::xe-overlay-put                   ov-sym prop val
;; ti::xe-overlays-at                   point
;; ti::xe-overlay-properties            overlay
;; ti::xe-overlay-some
;; ti::m-pkg-install-pgp-tar            dir &optional log-buffer source
;; ti::m-pkg-autoload-create-on-file    file &optional buffer no-show
;; ti::m-pkg-make                       beg end str ch
;; ti::m-pkg-make-var
;; ti::m-pkg-make-mode                  beg end
;; ti::m-pkg-make-mode-magic            beg end
;; ti::m-pkg-rip-magic                  beg end
;; ti::m-pkg-rip                        re ch &optional beg end
;; ti::m-pkg-ins-example                lib &optional re
;; ti::m-pkg-get-header                 lib header-list
;; ti::m-pkg-version-info               lib &optional arg
;; ti::m-pkg-submit-bug-report          lib id var-list
;; ti::m-pkg-submit-feedback            lib
;; ti::m-ad-control single-or-list regexp &optional disable verb msg
;; ti::m-selective-display-print        beg end
;; ti::m-selective-display-copy-to      beg end buffer &optional verb
;; ti::q-read-input-as-password         &optional prompt max echo-char
;; ti::q-read-input-invisible
;; ti::p-comint-run-command buffer-name clear form wait-re string-arg-list
;; ti::p-tar-read-listing-forward
;; ti::p-tar-view-command               file &optional buffer nice verb
;; ti::p-tar-zip-view-maybe-command     file
;; ti::p-zip-view-command file &optional buffer nice zip-cmd verb
;; ti::p-zip-dir-files zip-file dir re &optional zip-cmd
;; ti::p-zip       zip-file files &optional zip-cmd clear
;; ti::p-uu-make   file &optional zip-ext insert buffer verb
;; ti::p-uu-save-buffer                 file &optional do-it verb
;; ti::p-http-request command &optional port timeout buffer verb
;; ti::p-finger    email &optional port timeout buffer verb
;; ti::p-finger-error                   &optional buffer
;; ti::f-read-file-list                 &optional message
;; ti::f-complete-filename-minibuffer   &rest body
;; ti::f-complete-file-name-word        &optional no-msg
;; ti::f-complete-file-name             file-name &optional dir flist
;; ti::f-file-list                      dirs re
;; ti::f-user-home
;; ti::f-get-load-path fn paths &optional all-paths verb
;; ti::f-get-ext                        file &optional mode
;; ti::f-newer-exist                    f1 f2
;; ti::f-files-only                     list &optional eval-form
;; ti::f-dir-files dir re &optional absolute form not-re-form
;; ti::f-find-shadows                   &optional path
;; ti::f-chmod-w-toggle                 file
;; ti::f-ange-file-handle mode user host dir lcd file-list &optional not-bg msg-func
;; ti::f-ange-download-file ange-ref download-dir &optional not-bg
;; ti::f-ange-status                    ange-ref
;; ti::f-ange-completed-message         &rest args
;; ti::m-reindent                       beg end
;; ti::m-find-function-name             &optional point
;; ti::m-momentary-string-flash         STRING &optional wait-for-key
;; make-ND-array                        dimensions &optional value
;; ti::m-make-2D-array                  n m &optional value
;; ti::m-mail-abbrevs-email-list        &optional abbrev-alist
;; ti::m-mail-abbrevs-get-alist         &optional expand-until
;; ti::m-passwd-read-entry              &optional user cmd
;; ti::m-passwd-build-alist             cmd
;; ti::m-passwd-grep-user-alist         re &optional not-re passwd-alist
;; ti::m-sort-regexp-fields list level re &optional numeric reverse case
;; ti::m-add-str-reg                    beg end str &optional re-look
;; ti::m-scramble-reg                   beg end &optional char
;; ti::m-long-line                      beg end len
;; ti::m-add-spaces-80                  beg end &optional column
;; ti::m-surround-with-char             char
;; ti::m-move-paragraph-to-column       beg end col
;; ti::m-count-chars-in-delimited-area  arg &optional verb
;; ti::m-count-words                    beg end
;; ti::m-find-duplicate-same-word       &optional back
;; ti::m-word-move                      set &optional back
;; ti::t-try-set-colors                 list face &optional attribute
;; ti::t-wipe-properties                beg end
;; ti::k-mapkeys                        map-key-fun args
;; ti::k-put-map                        map &optional func
;; ti::k-put-abc-map                    map &optional func
;; ti::k-xlat-table                     &optional arg
;; ti::k-map-bind-control               map-symbol get-set prop key
;; ti::k-add-minor-mode mode-func-sym mode-name-sym  mode-map &optional remove
;; ti::k-reinstall-minor-mode           mode-name-symbol
;; ti::k-function-bind-info             function-sym &optional map
;; ti::w-match-buffers                  buffer-name-list
;; ti::w-find-bottom                    win-list
;; ti::w-get-buffer-window-other-frame  buffer
;; ti::w-list                           &optional buffers
;; ti::w-frame-list &optional all exclude-current win
;; ti::b-buffer-list-files              &optional re str
;; ti::b-buffer-list test &optional temp-buf exclude action-form
;; ti::b-get-re                         re &optional level
;; ti::b-xtra-open-outline
;; ti::b-diff-type-p
;; ti::b-replace-regexp re level str &optional back beg end
;; ti::b-trim-blanks                    beg end
;; ti::b-delete-until-non-empty-line    &optional backward point
;; ti::b-replace-in-rectangle           match replace corner1 corner2
;; ti::b-replace-in-region-arguments    &optional iact
;; ti::b-del-dup-lines beg end &optional len white-lines
;; ti::b-cnv-empty-lines                beg end &optional nbr strict
;; ti::b-add-newlines-to-region         beg end &optional arg
;; ti::b-copy-word                      n
;; ti::b-copy-line-and-inc-numbers      &optional increment
;; ti::b-inc-string-nbr re inc-val increment &optional level
;; ti::b-make-dup-line                  &optional count
;; ti::b-randomize-lines                beg end
;; ti::b-insert-line-numbers beg end &optional line grow format
;; ti::b-arrow-control                  buffer &optional mode str pos
;; ti::b-u2d-crlf                       &optional arg force
;; ti::b-kill-line                      &optional delete count
;; ti::b-leave-nth-word                 beg end &optional nbr strict
;; ti::b-zap-to-regexp                  arg regexp
;; ti::b-read-nth-word &optional count mode back charset
;; ti::b-read-syntax-word               syntax &optional mode
;; ti::b-read-space-word
;; ti::b-read-word                      &optional charset strict
;; ti::b-read-char                      &optional direction distance
;; ti::b-looking-back-at                re
;; ti::b-grep-lines                     re &optional beg end inc-prop
;; ti::b-read-line                      &optional len skip
;; ti::b-get-wspc                       &optional pp
;; ti::b-read-if-solid
;; ti::b-area-bounds                    beg end
;; ti::b-uu-line-p                      &optional string
;; ti::b-uu-area
;; ti::b-find-ange-to-dired-buffer
;; ti::b-find-ange-buffer               user host
;; ti::s-listify                        string &optional sep
;; ti::s-get-blocks                     str sep &optional inc
;; ti::s-1-space                        str
;; ti::s-index-substring str char &optional include right seek-end
;; ti::s-index                          str char &optional reverse
;; ti::s-case-replace model str &optional symmetry rest-case
;; ti:s-read-match-list match-list level-list string &optional terminate
;; ti:s-read-match-subs level-list &optional string terminate
;; ti::s-tabify                         string &optional mode
;; ti::s-syntax-info                    char &optional verb
;; ti::s-repeat                         count char-or-string
;; ti::d-time-diff-days                 std1 std2
;; ti::d-s-month-n                      arg &optional mode
;; ti::d-standard-date                  &optional short time
;; ti::m-rcs-get-all-branches           rev rev-list
;; ti::m-rcs-previous-version           version v-list
;; ti::m-rcs-buffer-version             &optional buffer
;; ti::m-rcs-guess-buffer-version       file &optional user
;; ti::m-rcs-files-in-dir               &optional dir re
;; ti::m-rcs-sort-same-level-list       list
;; ti::s-rcs-look-id                    str
;; ti::m-rcs-str-find                   str &optional mode
;; ti::m-rcs-delta-lock-status          &optional user-name
;; ti::m-rcs-delta-get-file             file &optional buffer
;; ti::c-int-to-access-mode-string      mode
;; ti::c-str2re                         str
;; ti::c-ch2escape                      item
;; ti::c-time-list                      &optional zero-form time-string
;; ti::c-upcase-words-to-variable-names beg end &optional case-fold
;; ti::c-url-to-ange-ftp                str
;; ti::c-format-string                  str
;; ti::s-sqz                            string
;; tinylib-submit-feedback
;; tinylib-version                      &optional arg
;; ti::p-uu-cmd                         file
;; ti::p-uude-cmd                       file



;;}}}
;;{{{ code: Autoload 'mt' lib

(autoload 'tinylibmt-version                    "tinylibmt" nil t)
(autoload 'tinylibmt-submit-feedback            "tinylibmt" nil t)
(autoload 'ti::mt-body-empty-p                  "tinylibmt" nil nil)
(autoload 'ti:mt-body-clear                     "tinylibmt" nil nil)
(autoload 'ti::mt-message-length                "tinylibmt" nil nil)
(autoload 'ti::mt-get-2re                       "tinylibmt" nil nil)
(autoload 'ti::mt-required-headers              "tinylibmt" nil nil)
(autoload 'ti::mt-mail-p                        "tinylibmt" nil nil)
(autoload 'ti::mt-header-area-size              "tinylibmt" nil nil)
(autoload 'ti::mt-hmax                          "tinylibmt" nil nil)
(autoload 'ti::mt-text-start                    "tinylibmt" nil nil)
(autoload 'ti::mt-narrow                        "tinylibmt" nil nil)
(autoload 'ti::mt-mail-buffer-name              "tinylibmt" nil nil)
(autoload 'ti::mt-rename-buffer                 "tinylibmt" nil t)
(autoload 'ti::m-mail-simple-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-vm-macro                      "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-mh-macro                      "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-gnus-macro                    "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-rmail-macro                   "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-rmail-do-message-macro        "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-rmail-copy-message            "tinylibmt" nil t)
(autoload 'ti::mt-pgp-v3xx-p                    "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-p                         "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signed-conventional-p     "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signed-conventional-multi-p "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signed-xpgp-p             "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signed-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-public-key-p              "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-remail-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-comment-file-p            "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-encrypted-p               "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-normal-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-headers-p                 "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-re                        "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-block-area-kill-forward   "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-block-area                "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-re-search                 "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-exe-version-number        "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-data-type                 "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-trim-buffer               "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-chop-region               "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-header-kill-in-body       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-data-char-to-int          "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-data-string-to-bin-string "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-data-bin-string-to-int-list "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-data-study-ctb-byte       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-study-1-key-id     "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-pgp-stream-study-1-time       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-study-enc          "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-study-signed       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-study-pring        "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-study              "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-forward-xpgp       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-forward            "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-forward-and-study  "tinylibmt" nil t)
(autoload 'ti::mt-pgp-stream-forward-info       "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-stream-data-elt           "tinylibmt" nil nil)
(autoload 'ti::mt-pgpk-id-lines-in-region       "tinylibmt" nil nil)
(autoload 'ti::mt-pgpk-public-get-region        "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signature-remove          "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signature-normal-do-region "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-pgp-signature-normal-info     "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-sig-header-info-v2xx      "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signature-header-info-v3xx "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-signature-header-info     "tinylibmt" nil nil)
(autoload 'ti::mt-mime-parse-header             "tinylibmt" nil nil)
(autoload 'ti::mt-pgp-pkey-read                 "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-close                    "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-anonymize-headers        "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-reply-type               "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-block                    "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-reply-block              "tinylibmt" nil nil)
(autoload 'ti::mt-pgpr-parse-levien-list        "tinylibmt" nil nil)
(autoload 'ti::mt-email-from-string             "tinylibmt" nil nil)
(autoload 'ti::mt-email-find-region             "tinylibmt" nil nil)
(autoload 'ti::mt-t-parse-name                  "tinylibmt" nil nil)
(autoload 'ti::mt-parse-name                    "tinylibmt" nil nil)
(autoload 'ti::mt-parse-email                   "tinylibmt" nil nil)
(autoload 'ti::mt-parse-received                "tinylibmt" nil nil)
(autoload 'ti::mt-parse-date                    "tinylibmt" nil nil)
(autoload 'ti::mt-signature-insert-break        "tinylibmt" nil nil)
(autoload 'ti::mt-yank                          "tinylibmt" nil nil)
(autoload 'ti::mt-trim-buffer                   "tinylibmt" nil nil)
(autoload 'ti::mt-field-start                   "tinylibmt" nil nil)
(autoload 'ti::mt-next-field-start              "tinylibmt" nil nil)
(autoload 'ti::mt-kill-field-in-body            "tinylibmt" nil nil)
(autoload 'ti::mt-kill-field                    "tinylibmt" nil nil)
(autoload 'ti::mt-get-field-1                   "tinylibmt" nil nil)
(autoload 'ti::mt-get-field                     "tinylibmt" nil nil)
(autoload 'ti::mt-add-field                     "tinylibmt" nil nil)
(autoload 'ti::mt-add-to-field-string           "tinylibmt" nil nil)
(autoload 'ti::mt-kill-field-elt                "tinylibmt" nil nil)
(autoload 'ti::mt-kill-non-rfc-fields           "tinylibmt" nil nil)
(autoload 'ti::mt-get-all-email-addresses       "tinylibmt" nil nil)
(autoload 'ti::mt-news-buffer-p                 "tinylibmt" nil t)
(autoload 'ti::mt-news-group                    "tinylibmt" nil nil)
(autoload 'ti::mt-set-article-buffer            "tinylibmt" nil nil)
(autoload 'ti::mt-get-article-re                "tinylibmt" nil nil)
(autoload 'ti::mt-news-reply-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-anon-penet-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-anon-penet-to-p               "tinylibmt" nil nil)
(autoload 'ti::mt-nymserver-email-convert       "tinylibmt" nil nil)
(autoload 'ti::mt-mime-maybe-p                  "tinylibmt" nil nil)
(autoload 'ti::mt-mime-p                        "tinylibmt" nil t)
(autoload 'ti::mt-mime-qp-decode                "tinylibmt" nil nil)
(autoload 'ti::mt-qp-mime-prepare               "tinylibmt" nil t)
(autoload 'ti::mt-sendmail-reset-send-hooks     "tinylibmt" nil nil)
(autoload 'ti::mt-sendmail-pure-env-macro       "tinylibmt" nil nil 'macro)
(autoload 'ti::mt-sendmail-macro                "tinylibmt" nil nil 'macro)

;; ti::mt-sendmail-macro                to subject send &rest body
;; ti::mt-sendmail-pure-env-macro       &rest body
;; ti::mt-sendmail-reset-send-hooks
;; ti::mt-qp-mime-prepare
;; ti::mt-mime-qp-decode                from to
;; ti::mt-mime-p
;; ti::mt-mime-maybe-p
;; ti::mt-nymserver-email-convert       email &optional na-mode
;; ti::mt-anon-penet-to-p
;; ti::mt-anon-penet-p                  email
;; ti::mt-news-reply-p
;; ti::mt-get-article-re                re &optional level
;; ti::mt-set-article-buffer
;; ti::mt-news-group
;; ti::mt-news-buffer-p
;; ti::mt-get-all-email-addresses &optional field-list abbrev-alist
;; ti::mt-kill-non-rfc-fields           &optional list
;; ti::mt-kill-field-elt                re &optional field
;; ti::mt-add-to-field-string           field string &optional sep
;; ti::mt-add-field field text &optional look-field mode replace
;; ti::mt-get-field                     field &optional any mode
;; ti::mt-get-field-1                   field
;; ti::mt-kill-field                    field-re &optional replace-str
;; ti::mt-kill-field-in-body            list
;; ti::mt-next-field-start              &optional move back max
;; ti::mt-field-start                   field-re &optional move max
;; ti::mt-trim-buffer
;; ti::mt-yank                          &optional prefix
;; ti::mt-signature-insert-break        &optional point
;; ti::mt-parse-date                    str
;; ti::mt-parse-received                &optional not-matching
;; ti::mt-parse-email                   line
;; ti::mt-parse-name                    line
;; ti::mt-t-parse-name
;; ti::mt-email-find-region             &optional beg end no-dupes
;; ti::mt-email-from-string             string
;; ti::mt-pgpr-parse-levien-list        &optional buffer control-list
;; ti::mt-pgpr-reply-block              pgp-email
;; ti::mt-pgpr-block mode &optional type email key latent
;; ti::mt-pgpr-reply-type               property-string
;; ti::mt-pgpr-anonymize-headers mode &optional no-ins arg1 arg2 hash
;; ti::mt-pgpr-close
;; ti::mt-pgp-pkey-read                 &optional raw kill-file
;; ti::mt-mime-parse-header             header-string &optional downcase
;; ti::mt-pgp-signature-header-info
;; ti::mt-pgp-signature-header-info-v3xx
;; ti::mt-pgp-sig-header-info-v2xx
;; ti::mt-pgp-signature-normal-info
;; ti::mt-pgp-signature-normal-do-region &rest body
;; ti::mt-pgp-signature-remove          &optional add no-cnv
;; ti::mt-pgpk-public-get-region        &optional beg end buffer relax
;; ti::mt-pgpk-id-lines-in-region       beg end
;; ti::mt-pgp-stream-data-elt           data elt
;; ti::mt-pgp-stream-forward-info       &optional search any
;; ti::mt-pgp-stream-forward-and-study  &optional search any
;; ti::mt-pgp-stream-forward            &optional any
;; ti::mt-pgp-stream-forward-xpgp
;; ti::mt-pgp-stream-study              ctb stream
;; ti::mt-pgp-stream-study-pring        length stream
;; ti::mt-pgp-stream-study-signed       length stream
;; ti::mt-pgp-stream-study-enc          length stream
;; ti::mt-pgp-stream-study-1-time       stream
;; ti::mt-pgp-stream-study-1-key-id     stream result
;; ti::mt-pgp-data-study-ctb-byte       int
;; ti::mt-pgp-data-bin-string-to-int-list string
;; ti::mt-pgp-data-string-to-bin-string string
;; ti::mt-pgp-data-char-to-int          char
;; ti::mt-pgp-header-kill-in-body
;; ti::mt-pgp-chop-region               beg end
;; ti::mt-pgp-trim-buffer
;; ti::mt-pgp-data-type
;; ti::mt-pgp-exe-version-number        &optional exe-file-location
;; ti::mt-pgp-re-search &optional mode move end no-anchor
;; ti::mt-pgp-block-area                mode &optional inside max
;; ti::mt-pgp-block-area-kill-forward   mode &optional move
;; ti::mt-pgp-re                        str
;; ti::mt-pgp-headers-p
;; ti::mt-pgp-normal-p                  &optional point
;; ti::mt-pgp-encrypted-p               &optional check-pgp-dash-line
;; ti::mt-pgp-comment-file-p            &optional point
;; ti::mt-pgp-remail-p
;; ti::mt-pgp-public-key-p              &optional point
;; ti::mt-pgp-signed-p
;; ti::mt-pgp-signed-xpgp-p
;; ti::mt-pgp-signed-conventional-multi-p
;; ti::mt-pgp-signed-conventional-p
;; ti::mt-pgp-p
;; ti::mt-pgp-v3xx-p
;; ti::mt-rmail-copy-message            &optional nbr separate
;; ti::mt-rmail-do-message-macro        nbr mode &rest body
;; ti::mt-rmail-macro                   &rest body
;; ti::mt-gnus-macro                    &rest body
;; ti::mt-mh-macro                      &rest body
;; ti::mt-vm-macro                      &rest body
;; ti::m-mail-simple-p
;; ti::mt-rename-buffer                 &optional arg
;; ti::mt-mail-buffer-name
;; ti::mt-narrow                        &optional text
;; ti::mt-text-start                    &optional move
;; ti::mt-hmax                          &optional move noerr
;; ti::mt-header-area-size
;; ti::mt-mail-p
;; ti::mt-required-headers
;; ti::mt-get-2re                       re str
;; ti::mt-message-length
;; ti:mt-body-clear
;; ti::mt-body-empty-p
;; tinylibmt-submit-feedback
;; tinylibmt-version                    &optional arg


;;}}}
;;{{{ code: Autoload 'y' lib

(autoload 'tinyliby-version                     "tinyliby" nil t)
(autoload 'tinyliby-submit-feedback             "tinyliby" nil t)
(autoload 'ti::y-load-cleanup                   "tinyliby" nil nil)
(autoload 'ti::y-lh-where                       "tinyliby" nil nil)
(autoload 'ti::y-lh-get                         "tinyliby" nil nil)
(autoload 'ti::y-feature-kill                   "tinyliby" nil nil)
(autoload 'ti::y-unload-symbols                 "tinyliby" nil nil)
(autoload 'ti::y-unload                         "tinyliby" nil nil)
(autoload 'ti::y-unload-feature                 "tinyliby" nil t)
(autoload 'ti::y-feature-list-unload            "tinyliby" nil nil)
(autoload 'ti::y-remove-from-hooks              "tinyliby" nil nil)
(autoload 'ti::y-get-symbols                    "tinyliby" nil nil)
(autoload 'ti::y-get-file-documentation         "tinyliby" nil t)
(autoload 'ti::y-describe-symbols-i-args        "tinyliby" nil nil)
(autoload 'ti::y-describe-symbols               "tinyliby" nil t)
(autoload 'ti::y-symbol-summary                 "tinyliby" nil t)

;; ti::y-symbol-summary                 re &optional verb
;; ti::y-describe-symbols pattern &optional mode exclude-re bind-info out-buffer
;; ti::y-describe-symbols-i-args        &optional arg
;; ti::y-get-file-documentation         file &optional verb
;; ti::y-get-symbols                    re &optional test-form
;; ti::y-remove-from-hooks              symlist re
;; ti::y-feature-list-unload            list
;; ti::y-unload-feature                 sym &optional verb
;; ti::y-unload                         mode list
;; ti::y-unload-symbols                 list
;; ti::y-feature-kill                   sym
;; ti::y-lh-get                         sym
;; ti::y-lh-where                       sym
;; ti::y-load-cleanup                   ELT
;; tinyliby-submit-feedback
;; tinyliby-version                     &optional arg



;;}}}
;;{{{ code: Autoload 'o' lib

(autoload 'tinylibo-version                     "tinylibo" nil t)
(autoload 'tinylibo-feedback                    "tinylibo" nil t)
(autoload 'ti::o-make-match                     "tinylibo" nil nil)
(autoload 'ti::o-buffer-substring               "tinylibo" nil nil)
(autoload 'ti::o-mouse-on-p                     "tinylibo" nil nil)
(autoload 'ti::o-get-mouse                      "tinylibo" nil nil)
(autoload 'ti::o-get-prop                       "tinylibo" nil nil)
(autoload 'ti::o-get-prop-val                   "tinylibo" nil nil)
(autoload 'ti::o-re-search                      "tinylibo" nil nil)
(autoload 'ti::o-re-search-move                 "tinylibo" nil nil)
(autoload 'ti::o-get-within-area                "tinylibo" nil nil)
(autoload 'ti::o-remove-area                    "tinylibo" nil t)

;; ti::o-remove-area                    &optional beg end propl propl-t
;; ti::o-get-within-area                propl &optional propl-t beg end
;; ti::o-re-search-move                 ov re &optional level back max
;; ti::o-re-search re level list &optional max back reuse reuse-t
;; ti::o-get-prop-val                   ovl prop-list
;; ti::o-get-prop                       ovl prop-list
;; ti::o-get-mouse
;; ti::o-mouse-on-p                     ov
;; ti::o-buffer-substring               ov &optional no-properties
;; ti::o-make-match                     level plist
;; tinylibo-feedback
;; tinylibo-version                     &optional arg
;; ti::o



;;}}}
;;{{{ code: Autoload 'xe' lib

;;; ........................................................... &libxe ...

;;; Autoload only some funcions

(when (xemacs-p)
  (autoload 'run-at-time "tinylibxe")
  ;;  We can emulate these
  (autoload 'window-edges			"tinylibxe")
  (autoload 'event-start                        "tinylibxe")
  (autoload 'posn-point                         "tinylibxe")
  (autoload 'posn-window			"tinylibxe")
  )

;;}}}

;;{{{ code: Autoload tinymatch.el -- coloring support

(autoload 'tima-search-face-reset               "tinymatch" nil nil 'macro)
(autoload 'tima-search-face-set                 "tinymatch" nil nil 'macro)
(autoload 'tima-face                            "tinymatch" nil nil 'macro)
(autoload 'tima-save-data                       "tinymatch" nil nil)
(autoload 'tima-undo                            "tinymatch" nil t)
(autoload 'tima-clear-buffer-properties         "tinymatch" nil t)
(autoload 'tima-clear-region-properties         "tinymatch" nil t)
(autoload 'tima-get-mouse-property              "tinymatch" nil nil)
(autoload 'tima-match-level                     "tinymatch" nil nil)
(autoload 'tima-re-search                       "tinymatch" nil t)
(autoload 'tima-property-search-and-modify      "tinymatch" nil nil)
(autoload 'tima-read-regexp                     "tinymatch" nil nil)
(autoload 'tima-looking-at                      "tinymatch" nil t)
(autoload 'tima-buffer                          "tinymatch" nil t)
(autoload 'tima-re-search-forward               "tinymatch" nil t)
(autoload 'tima-re-search-backward              "tinymatch" nil t)
(autoload 'tima-mouse-mark-region               "tinymatch" nil t)
(autoload 'tima-mouse-unmark-region             "tinymatch" nil t)
(autoload 'tima-unmark-region                   "tinymatch" nil t)
(autoload 'tima-mark-region                     "tinymatch" nil t)

;; tima-mark-region                     beg end &optional remove face
;; tima-unmark-region                   beg end
;; tima-mouse-unmark-region             beg end event
;; tima-mouse-mark-region               beg end event
;; tima-re-search-backward re &optional level face mode save-undo
;; tima-re-search-forward re &optional level face mode save-undo
;; tima-buffer     re &optional level face mode save-undo
;; tima-looking-at re &optional level face-or-plist
;; tima-read-regexp
;; tima-property-search-and-modify match-plist set-plist &optional beg end
;; tima-re-search  re &optional direction level maxp face mode save-undo
;; tima-match-level &optional level face-or-propl beg end
;; tima-get-mouse-property
;; tima-clear-region-properties         beg end &optional propl
;; tima-clear-buffer-properties         &optional  propl
;; tima-undo
;; tima-save-data                       re level func mode beg
;; tima-face                            face
;; tima-search-face-set                 face
;; tima-search-face-reset





;;}}}
;;{{{ code: Autoload other 'tiny tools'

(autoload 'tinyck-advice-control	    "tinyck")
(autoload 'tib-info			    "tinybufid")
(autoload 'tib-cnv-txt2comment		    "tinybufid")
(autoload 'timu-menu			    "tinymenu")

;;}}}
;;{{{ code: Autoload easymenu.el (19.28)

;;  These are from XEmacs 19.14, they should suffice
;;
(autoload 'easy-menu-define                     "easymenu" nil nil 'macro)
(autoload 'easy-menu-do-define                  "easymenu" nil nil)
(autoload 'easy-menu-add                        "easymenu" nil nil)
(autoload 'easy-menu-remove                     "easymenu" nil nil)

;; easy-menu-remove                     menu
;; easy-menu-add                        menu &optional map
;; easy-menu-do-define                  symbol maps doc menu
;; easy-menu-define                     symbol maps doc menu

;;; ..  ..  ..  ..  ..  ..  ..  ..  ..  ..  ..  ..  ..  .. Emacs 19.30  ..
;; (autoload 'easy-menu-define                     "easymenu" nil nil 'macro)
;; (autoload 'easy-menu-do-define                  "easymenu" nil t)
;; (autoload 'easy-menu-create-keymaps             "easymenu" nil nil)
;; (autoload 'easy-menu-change                     "easymenu" nil nil)
;; (autoload 'easy-menu-remove                     "easymenu" nil nil)
;; (autoload 'easy-menu-add                        "easymenu" nil nil)
;;
;; ;; easy-menu-add                        menu &optional map
;; easy-menu-remove                     menu
;; easy-menu-change                     path name items
;; easy-menu-create-keymaps             menu-name menu-items
;; easy-menu-do-define                  symbol maps doc menu
;; easy-menu-define                     symbol maps doc menu


;;}}}


)
;;; --++-- --++-- --++-- --++-- --++-- --++-- --++-- --++- compile end --

;;{{{ setup: version

;;; ......................................................... &version ...

(defconst tinylibm-version
  (substring "$Revision: 1.123 $" 11 16)
  "Latest version number.")

(defconst tinylibm-version-id
  "$Id: tinylibm.el,v 1.123 1997/07/04 12:47:57 jaalto NotReleased $"
  "Latest modification time and version number.")


;;; ----------------------------------------------------------------------
;;;
(defun tinylibm-version (&optional arg)
  "Show version information. ARG will instruct to print message to echo area."
  (interactive "P")
  (ti::m-pkg-version-info "tinylibm.el" arg))

;;; ----------------------------------------------------------------------
;;;
(defun tinylibm-submit-bug-report ()
  "Submit bug report."
  (interactive)
  (ti::m-pkg-submit-bug-report
   "tinylibm.el"
   tinylibm-version-id
   '(tinylibm-version-id
     )))


;;}}}
;;{{{ code: function test

;;; ...................................................... &func-tests ...
;;; We define these here becasue they are used lated in this library
;;; "define before using"

(eval-and-compile

;;; ----------------------------------------------------------------------
;;;
(defmacro func-car-test (symbol test-val &optional test-func)
  "Test car of the SYMBOL against TEST-VAL with TEST-FUNC.
Function must be symbol, not a lambda form.

Return:
  symbol      ,yes, test succeeded
  nil         ,test failed"
  (`(if (and (not (sequencep (, symbol))) ;; list ?
	     (symbolp (, symbol))	  ;; chokes if not sequencep
	     (fboundp (, symbol))

	     ;;  Eg. symbol-function 'car  doesn't return list.
	     ;;
	     (listp (symbol-function (, symbol)))
	     (eq (, test-val)
		 (funcall (or (, test-func) 'car)
		  (symbol-function (, symbol)))))
	(, symbol)
      nil
      )))

;;; ----------------------------------------------------------------------
;;;
(defun defalias-p (symbol)
  "If function SYMBOL is alias, return it's truename. Otw Return nil."
  (let* (sym
	 prev
	 ret
	 )
    (if (or (sequencep symbol)		;lambda form ?
	    (not (symbolp symbol))
	    (not (fboundp symbol)))
	nil
      (setq sym (symbol-function symbol))
      (if (not (symbolp sym))
	  nil
	(while (and (symbolp sym)	;was alias, go into nesting levels
		    (fboundp sym))	;must be function or user made mistake
	  (setq prev sym)
	  (setq sym (symbol-function sym))
	  )
	(setq ret prev)
	))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun defmacro-p (symbol)
  "Test if function SYMBOL is in fact macro, created with defmacro.

Return:
  symbol     ,this can be truename of the function if it was aliased
  nil"
  (func-car-test symbol 'macro))


;;; ----------------------------------------------------------------------
;;;
(defun lambda-p (symbol)
  "Test if function SYMBOL was created with defsubst or is in lambda form.

Return:
  symbol     ,this can be truename of the function if it was aliased
  nil"
  (func-car-test symbol 'lambda))



;;; ----------------------------------------------------------------------
;;;
(defun autoload-p (symbol)
  "Test if function SYMBOL is in its autoload form.
Works with aliased symbols too.

Return:
  symbol     ,this can be truename of the function if it was aliased
  nil"
  ;;  Get the REAL name if it is alias or use the func's SYMBOL name
  (let* ((func (or (defalias-p symbol) symbol))
	 )
    (func-car-test func 'autoload)))


;;; ----------------------------------------------------------------------
;;;
(defsubst function-adviced-p  (symbol)
  "Test if function SYMBOL has any advice attached. ."
  (when (boundp 'ad-advised-functions)
    (member (symbol-name symbol) (symbol-value 'ad-advised-functions))))


;;; --++-- --++-- --++-- --++-- --++-- --++-- --++--  eval-and-compile --
)

;;}}}
;;{{{ code: [f]boundp macros, small FORMS

;;; ########################################################## &boundp ###
;;; - To see what the'll become use for example:
;;;   (macroexpand '(dec x))



;;; ----------------------------------------------------------------------
;;;
(put 'nafboundp 'lisp-indent-function 2)
(defmacro nafboundp (func re &rest body)
  "Execute body if certain condition is true.

a) If not fboundp FUNC is true and b) if the function is not in (a)utoload
   state and
b) RE match autoload file.

In short. Do BODY only if the autoload refer to file
matching RE. This is usefull, if you define your own function that does
not exist in current emacs, but may exist in newer releases. Suppose
following situation.

 (if (xemacs-p)
     (autoload 'run-at-time \"tinylibxe\"))   ;; Say it's in 'xe' lib

in file tinylibxe.el:

 (nafboundp 'run-at-time \"tinylibxe\"
  ;; XEmacs does not have it, but it may be fboundp due to autoload.
  ;; But, because it refers to us, we define the function,
  ;; if the autoload refers somewhere else, then this form doesn't
  ;; take in effect. Somebody else has done the definition.
  ;;
  ...

  )"
  (` (cond
      ((or (and (fboundp (, func))
		(autoload-p (, func))
		(string-match
		 (, re )
		 (nth 1 (symbol-function (, func))))
		)
	   (not (fboundp (, func)))
	   )

       (,@ body)
       ))))


;;; ----------------------------------------------------------------------
;;; - thousends of lines are coded without this handy macro,
;;;   to me this is a life saver...
;;;
(put 'job-buffer 'lisp-indent-function 1)
(defmacro job-buffer (buffer &rest body)
  "Go to BUFFER temporarily and execute BODY."
  (`
   (save-excursion
     (set-buffer (, buffer))
     (,@ body)
     )))


;;; ----------------------------------------------------------------------
;;; - And you need this every time you deal with lists.
;;;
;;; Using CL is little longer and maybe slower
;;;
;;; (loop for elt in list do
;;;   ...
;;;   )
;;;
(put 'list-loop 'lisp-indent-function 2)
(defmacro list-loop (list var &rest body)
  "Loop over LIST, (car LIST) is put to VAR, and do BODY.
List is advanced with cdr. You can stop the loop by assigning nil to LIST."
  (`
   (while (, list)
     (setq (, var) (car (, list)))
     (,@ body)
     (setq (, list) (cdr (, list)))
     )
   ))


;;; ----------------------------------------------------------------------
;;; - I hate to type all the forms in code, completing this command
;;;   is much nicer.
;;;
(put 'safe-narrow 'lisp-indent-function 2)
(defmacro safe-narrow (beg end &rest body)
  "Narrow temprarily to BEG END and do BODY.
This FORM preserves restriction and excursion with one command."
  (`
   (save-excursion
     (save-restriction
       (narrow-to-region (, beg) (, end))
       (,@ body)
       ))))


;;; ----------------------------------------------------------------------
;;; - Using this is much more confortable that the long form..
;;;   if you need to deal with the errors, you can go the long way...
;;;
(put 'safe-do 'lisp-indent-hook 0)
(defmacro safe-do (&rest body)
  "Shortcut to `condition-case'. Ignore all errors while executing BODY."
  (`
   (let* (debug-on-signal
	  )
     ;;  In XEmacs, if this variable is t, it will stop condition-case
     ;;  altogether. The 'if' statement is just an no-op for
     ;;  XEmacs bytecompiler which stops whining about defined, but
     ;;  non used variable
     ;;
     (if debug-on-signal (setq debug-on-signal nil))

     (condition-case nil
	 (progn
	   (,@ body))
       (error nil)))))


;;; ----------------------------------------------------------------------
;;; Note that nconc works only if the initial
;;; list is non-empty, that's why we have to initialize it in the
;;; first time with if.
;;;
(defmacro nconcc (list x)
  "(LIST X) like nconc, but can also add to empty list."
  (`
   (setq (, list) (nconc (, list) (list (, x))))
   ))

;;}}}
;;{{{ small ones

;;; ########################################################### &funcs ###

;;; ----------------------------------------------------------------------
;;; buffer may be narrowed.
;;;
(defsubst EQPP ()
  "Check if `point-min' is equal to `point-max'.
Used for checking possibleempty buffer."
  (eq (point-min) (point-max)))


;;; ----------------------------------------------------------------------
;;; This returns true nature of point-min point-max in buffer
;;; EQ = equal
;;; B  = buffer marker
;;; PP = points
;;;
(defsubst EQBPP ()
  "Check if buffer's min point is equal to max point.
Uses `point-min-marker' and `point-max-marker'. Used for checking
possible empty buffer."
  (eq (point-min-marker) (point-max-marker)))



;;; ----------------------------------------------------------------------
;;;   Great add to comint processess.
;;;
(defsubst PROC-MARK (&optional buffer)
  "Return process mark for current buffer or optional BUFFER.
If there is no process at buffer, return nil"
  (let* ((proc (get-buffer-process
		(or buffer (current-buffer))))
	 )
    (if proc
	(process-mark proc))
    ))

;;; ----------------------------------------------------------------------
;;; - These are needed all the time, makes code readable
;;; - Note: Seems to be included in 'cl', macros 'decf'
;;;   I may remove these later, so don't use them!
;;;
(defmacro inc (var)
  "Increments VAR by 1."
  (` (setq (, var) (1+ (, var)))))

(defmacro inci (var i)
  "Increments VAR  by I."
  (` (setq (, var) (+ (, var) (, i)))))

(defmacro dec (var)
  "Decrement VAR by 1."
  (` (setq (, var) (1- (, var)))))

(defmacro deci (var i)
  "Decrement VAR by I."
  (` (setq (, var) (- (, var) (, i)))))


;;; ----------------------------------------------------------------------
;;;
(defmacro VERB ()
  "Setq variable 'verb'.
If interactive flag is set or if 'verb' variable is set.
This is usually the verbosity flag that allows printing messages.

Note:

  You have to define variable 'verb' prior calling this macro,
  preferably in function argument definition list.

Example:

  (defun my-func (arg1 arg2 &optional verb)
    (interactive
      ...do something, ask parameters
      )
    (VERB)     ;; set verbose if needed
    (if verb (message 1))
    ..code
    (if verb (message 2))
    )"
  (`
   (setq verb (or verb (interactive-p)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst PMIN ()
  "Goes to `point-min'."
  (goto-char (point-min)))

;;; ----------------------------------------------------------------------
;;;
(defsubst PMAX ()
  "Goes to `point-max'."
  (goto-char (point-max)))


;;; ----------------------------------------------------------------------
;;; NOTE: this macro seems to be included in 'cl Emacs distribution
;;; package. The function name is 'pop'.
;;;
;;;
;;;
(defmacro NEXTP (list)
  "Advance LIST pointer with cdr. For iterating, please use `list-loop'."

  (` (setq (, list) (cdr (, list)))))


;;; ----------------------------------------------------------------------
;;;
(nafboundp 'int-to-float "tinylibm"
  (defmacro int-to-float (nbr)
    "Convert integer to float."
    (` (read (concat (int-to-string (, nbr)) ".0"))
       )))

;;; ----------------------------------------------------------------------
;;; see also:  (dotimes (i n)
;;;
(put 'loop-for 'lisp-indent-function 2)
(defmacro loop-for (beg end &rest body)
  "Loop from BEG to END and do BODY. Variable 'COUNTER' refers to iterator."
  (`
   (loop for COUNTER from (, beg) to (, end) do (progn (,@ body))   )))


;;; ----------------------------------------------------------------------
;;; - This is real time saver in my .emacs, because I have same
;;;   ~/.emacs in different sites. Now I don't have to care about
;;;   the fact that if some function is missing or not, eg.
;;;   in lisp mode hooks I can write
;;;
;;;   (add-hook 'lisp-mode-hook 'my-lisp-mode-hook)
;;;   (defun my-lisp-mode-hook ()
;;;     "My lisp settings"
;;;     (fexec 'expand-mode 1)
;;;     ..
;;;     )
;;;
;;;   I have very limited quota in the University, so the expand.el
;;;   is not installed in my private dir there. Whereas it _is_
;;;   installed in my work's machine --> the above call takes effect.
;;;
;;;   This is lot more nicer way to add calls, than the lenghty
;;;   if..fboundp way..
;;;
;;; - I strongly advice you to copy this macro and add it it as a
;;;    first line in  your ~/.emacs
;;;
;;;
(defmacro fexec (func-sym &rest args)
  "Call function symbol FUNC-SYM with ARGS if function exists.
This is like doing:

	(if (fboundp func)
	    (funcall func args))

But this macro saves you lot of trouble if you write that multiple times.
This macro also shuts up unnecessary compiler whinings fro non existing
function call `func' in the example above.

Normally you would use this for calling extra function that may add
some value: but calling them is not essential (like coloring, or
turning on some handy minor mode)"
  (`
   ;;   We use let to hide the function name from byte compiler:
   ;;   it would otherwise claim that function does ;not esist,
   ;;   while we really wouldn't even use it due to fboundp test!
   ;;
   (let* ((func (, func-sym)))
     (when (fboundp (, func-sym))
       (apply func (,@ args) nil)))))



;;; ----------------------------------------------------------------------
;;;
(defmacro sfuncall (func-sym &rest args)
  "Call FUNC-SYM with ARGS.
Like funcall, but uses `symbol-function' to get the function definition.
This shuts up some whinings from byte compiler.

The full story:

  Byte Compiler isn't very smart when it comes to knowing if
  symbols exist or not. If you have following statement in your function,
  it still complaints that the function \"is not known\"

  (if (fboundp 'some-non-existing-func)
      (some-non-existing-func arg1 arg2 ...))

  instead use:

  (if (fboundp 'some-non-existing-func)
      (sfuncall 'some-non-existing-func arg1 arg2 ...)

  to get rid of the unnecessary warning.

Warning:

  You _cannot_ use sfuncall if the function is in autoload state, because
  `symbol-function' doesn't return a function to call. Rearrange
  code so that you do (require 'package) or (autoload-p func) test before
  using sfuncall."
  (`
   (apply (symbol-function (, func-sym)) (,@ args) nil)
   ))


;;; ----------------------------------------------------------------------
;;; Emacs distribution, sun-fns.el -- Jeff Peck
;;;
(nafboundp 'logtest "tinylibm"
 (defun logtest (x y)
   "True if any bits set in X are also set in Y.
Just like the Common Lisp function of the same name."
   (not (zerop (logand x y)))))



;;; ----------------------------------------------------------------------
;;;
(defun bin-string-to-int  (8bit-string)
  "Convert 8BIT-STRING  string to integer."
  (let* ((list  '(128 64 32 16 8 4 2 1))
	 (i   0)
	 (int 0)
         )
    (while (< i 8)
      (if (not (string= "0" (substring 8bit-string i (1+ i))))
	  (setq int (+ int (nth i list) )))
      (inc i)
      )
    int
    ))


;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun int-to-bin-string (n &optional length)
  "Convert integer N to bit string (LENGTH, default 8)."
  (let* ((i    0)
	 (len  (or length 8))
	 (s    (make-string len ?0))
	 )
    (while (< i len)
      (if (not (zerop (logand n (ash 1 i))))
          (aset s (- len (1+ i)) ?1))
      (setq i (1+ i))
      )
    s
    ))


;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun int-to-hex-string (n &optional separator pad)
  "Convert integer N to hex string. SEPARATOR between hunks is \"\".
PAD says to padd (bit hex string with leading zeroes."
  (or separator
      (setq separator ""))
  (mapconcat
   (function (lambda (x)
	       (setq x (format "%x" (logand x 255)))
	       (if (= 1 (length x))
		   (if pad (concat "0" x) "")
		 x)))
   (list (ash n -24) (ash n -16) (ash n -8) n)
   separator))



;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun int-to-oct (n &optional separator)
  "Convert integer N into Octal. SEPARATOR between hunks is \"\"."
  (or separator
      (setq separator ""))
  (mapconcat
   (function (lambda (x)
	       (setq x (format "%o" (logand x 511)))
	       (if (= 1 (length x)) (concat "00" x)
		 (if (= 2 (length x)) (concat "0" x) x))))
   (list (ash n -27) (ash n -18) (ash n -9) n)
   separator))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun radix (str base)
  "Convert STR to BASE."
  (let ((chars "0123456789abcdefghijklmnopqrstuvwxyz")
        (case-fold-search t)
        (n 0)
        i)
    (mapcar
     (function
      (lambda (c)
	(setq i (string-match (make-string 1 c) chars))
	(if (>= (or i 65536) base)
	    (error "%c illegal in base %d" c base))
	(setq n (+ (* n base) i)))
      (append str nil)
      ))
    n))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun rbin (str)
  "Convert STR into binary."
  (radix str 2))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun roct (str)
  "Convert STR into octal."
  (radix str 8))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun rhex (str)
  "Convert STR into hex."
  (if (string-match "\\`0x" str) (setq str (substring str 2)))
  (radix str 16))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun int-to-net (float)
  "Decode packed FLOAT 32 bit IP addresses."
  (format "%d.%d.%d.%d"
          (truncate (% float 256))
          (truncate (% (/ float 256.0) 256))
          (truncate (% (/ float (* 256.0 256.0)) 256))
          (truncate (% (/ float (* 256.0 256.0 256.0)) 256))
          ))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun rmac (string)
  "Decode STRING x-mac-creator and x-mac-type numbers."
  (if (numberp string)
      (setq string (format "%X" string)))
  (let ((i 0)
        (r ""))
    (while (< i (length string))
      (setq r (concat r
               (make-string 1
                (rhex (concat (make-string 1 (aref string i))
                              (make-string 1 (aref string (1+ i)))))))
            i (+ i 2)))
    r))

;;; ----------------------------------------------------------------------
;;; 08 Jun 1997 Jamie Zawinski <jwz@netscape.com> comp.emacs
;;;
(defun ctime (time)
  "Print a time_t TIME."
  (if (and (stringp time) (string-match "\\`[0-9]+\\'" time))
      (setq time (string-to-number (concat time ".0"))))
  (let* ((top (floor (/ time (ash 1 16))))
	 ;; (bot (floor (mod time (1- (ash 1 16)))))
         (bot (floor (- time (* (ash 1 16) (float top)))))
         )
    (current-time-string (cons top bot))))


;;; ----------------------------------------------------------------------
;;; - Can't remember where this originates, not mine...
;;;
(defsubst rand0 (n)
  "Random number in [0 .. N]."
  (cond
   ((<= n 0) 0)
   (t (abs (% (random) n)))))


;;; ----------------------------------------------------------------------
;;; - Can't remember where this originates, not mine...
;;;
(defsubst rand1 (n)
  "Random number [1 .. N]."
  (1+ (rand0 n)))


;;; ----------------------------------------------------------------------
;;; - Can't remember where this originates, not mine...
;;;
(defun randij (i j)
  "Random number [I .. J]."
  (cond
   ((< i j) (+ i (rand0 (1+ (- j i)))))
   ((= i j) i)
   ((> i j) (+ j (rand0 (1+ (- i j)))))
   (t (error "randij wierdness %s %s" (2str i) (2str j)))))


;;; ----------------------------------------------------------------------
;;; - Can't remember where this originates, not mine...
;;;
(defun 2str (x)
  "Return a string with some reasonable print-representation of X.
If X is an integer, it is interpreted as an integer rather than
a character: (2str 65) ==> \"65\" not \"A\"."
  (cond
   ((stringp x) x)
   ((symbolp x) (symbol-name x))
   ((numberp x) (int-to-string x))
   (t (prin1-to-string x))))



;;; ----------------------------------------------------------------------
;;; This is very usefull for printing any variables.
;;; (mesasge (concat "DEBUG:" (print1cat nil buffer frame overlay..)))
;;;
(defun prin1cat (separator &rest args)
  "Cats elements separated by single space or with SEPARATOR.
The ARGS can be anything."
  (let* (ret
	 (sep  (concat "%s" (or separator " ")))
	 )
    (if (null sep) (setq sep sep))	;XEmacs 19.14 ByteComp silencer

    (mapcar
     (function
      (lambda (x)
	(setq ret
	      (concat
	       ret

	       (cond
		((integerp x)
		 (format
		  (concat "%d" (or separator " "))
		  x))

		((stringp x)
		 (format
		  (concat "%s" (or separator " "))
		  x))

		((symbolp x)
		 (format
		  (concat "'%s" (or separator " "))
		  x))

		((and (not (null x))
		      (listp x))
		 (prin1-to-string
		  (eval   ;; -expression
		   (quote x)))
		 )
		(t
		 (format
		  (concat "%s" (or separator " "))
		  x)
		 ))))))
     args)
    ret
    ))



;;; ----------------------------------------------------------------------
;;; - The world's oldest way to debug program by inserting breakpoints...
;;; - This has to be defmacro, because of the &rest arguments
;;;   are passed "as is" to next function.
;;;
(defmacro d! (&rest args)
  "Debug. Show any ARGS information and wait for keypress.
Use inside lisp code to debug variables."
  (`
   (save-excursion
     (save-match-data
       (read-from-minibuffer (prin1cat "|" (,@ args)))))))


;;; ----------------------------------------------------------------------
;;; - This logs to buffer, when you can't display values, eg. in loop
;;;   or while you're in minibuffer and reading input.
;;; - see tinydiff.el how to use this productively.
;;;
(defmacro d!! (&rest args)
  "Stream debug. Record any information in ARGS to debug buffer.
Use inside lisp code to debug any variables.

References:
  `ti:m-debug-buffer'"
  (`
   (let* ((myBufferTo  (use-tmp-buffer ti:m-debug-buffer))
	  )
     (append-to-buffer-string
      myBufferTo
      (prin1cat "|" (,@ args))))))


;;; ----------------------------------------------------------------------
;;;
(defsubst str-left (str count)
  "Use STR and read COUNT chars from left.
If the COUNT exeeds string length or is zero, whole string is returned.

Mostly usefull, when you want to be sure string doesn't exceed certain
length, like displaying 10 last chars from the left only.

This function never calls error."
  (if (> count 0)
      (substring str 0 (min (length str) count))
    str
    ))


;;; ----------------------------------------------------------------------
;;;  - You can do this with negative argument to substring, but if you exceed
;;;    the string len, substring will barf and quit with error.
;;;  - This one will never call 'error'.
;;;
(defsubst str-right (str count)
  "Use STR and read COUNT chars from right.
If the COUNT exeeds string length or is zero, whole string is returned.

Mostly usefull, when you want to be sure string doesn't exceed certain
length, like displaying 10 last chars from pathname at max.

This function never calls error."
  (let* ((pos (- (length str)  count))
	 )
    (if (> pos 0)
	(substring str (- 0 count))
      str
      )))


;;; ----------------------------------------------------------------------
;;; - This old version is equivalent to the new one. The NEW one
;;;   was needed because emacs 19.30+ didn't allow integer in
;;;   'concat function any more.
;;; - This is interesting macro, but ... Hmm, I think it is
;;;   too slow to be used regularly. Use with care in places where
;;;   time is not critical.
;;;
;;old (defmacro strcat (var-sym &rest body)
;;old"Shorthand to (setq VAR-SYM (concat VAR-SYM ...))"
;;old   (` (setq (, var-sym) (concat (or (, var-sym) "") (,@ body)))))

(defmacro strcat (var &rest body)
  "Like strcat. Put results to VAR using BODY forms.
Integers and variables passed in BODY to VAR
Example call:  (strcat var \"hello \" \"there \" 1234 \" \" 33)"
  (` (setq (, var)
	   (concat
	    (or (, var) "")
	    (mapconcat
	     (function
	      (lambda (x)
		(cond
		 ((stringp x) x)
		 ((integerp x) (int-to-string x))
		 (t   (eval x))          ;; it's variable
		 )))
	     (quote (, body))
	     ""
	     )))))

;;; ----------------------------------------------------------------------
;;;
(defsubst month-list ()
  "Return LIST: month names in short format."
  (list "Jan" "Feb" "Mar" "Apr" "May" "Jun"
	"Jul" "Aug" "Sep" "Oct" "Nov" "Dec"))

;;; ----------------------------------------------------------------------
;;;
(defun month-list-re (&optional cat-str)
  "Return month regexp separated by ' \\\\|' or CAT-STR.
There is intentional space, since short month name is supposed to
follow something else."
  (let* ((ret
	 (mapconcat    'concat    (month-list) (or cat-str " \\|")))
	 )
    ;;  The last item must be handled separately
    (if (null cat-str)
	(concat ret " "))
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst month-mm-alist ()		;mm = month first
  "Short month names in alist form: ((\"Jan\" 1) ..)."
  '( ("Jan" . 1) ("Feb" . 2) ("Mar" . 3)
     ("Apr" . 4) ("May" . 5) ("Jun" . 6)
     ("Jul" . 7) ("Aug" . 8) ("Sep" . 9)
     ("Oct" . 10) ("Nov" 11) ("Dec" . 12)))

;;; ----------------------------------------------------------------------
;;;
(defsubst month-nn-alist ()		;nn = nbr first
  "Short month names in alist form: ((1 \"Jan\") ..)."
  '( (1 . "Jan") (2 . "Feb") (3 . "Mar")
     (4 . "Apr") (5 . "May") (6 . "Jun")
     (7 . "Jul") (8 . "Aug") (9 . "Sep")
     (10 . "Oct") (11 . "Nov") (12 . "Dec")))


;;; ----------------------------------------------------------------------
;;;
(defsubst date-eu-list ()
  "Return list: European date list."
  (list "Mon" "Tue" "Wed" "Thu" "Fri" "Sat" "Sun"))


;;; ----------------------------------------------------------------------
;;;
(defsubst date-us-list ()
  "Return list: US date list."
  (list "Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat"))


;;; ----------------------------------------------------------------------
;;;
(defun date-list-re (&optional cat-str)
  "Return date regexp.
There is intentional SPACE, since short month name is supposed
to follow something else.

Input:
  CAT-STR      ,defaults to \" \\\\|\""
  (let* ((ret
	 (mapconcat    'concat    (date-eu-list) (or cat-str " \\|"))))

    ;;  The last item must be handled separately
    (if (null cat-str)
	(concat ret " "))
    ))

;;; ----------------------------------------------------------------------
;;;
(defmacro message-flash (time sleep &rest body)
  "Flash message briefly for TIME [default 0.2] in seconds and wipe it away.

SLEEP instructs to use `sleep-for' instead of `sit-for'. It's recommended to
use SLEEP if you want to be sure the message is seen. Possible values:
  'sleep        ,use `sleep-for' function
   any          ,use `sit-for' function

The BODY can be any expression yielding a string."
  (`
   (progn
     (message (,@ body))
     (if (eq 'sleep (, sleep))
	 (sleep-for (or (, time) 1))
       (sit-for (or (, time) 0.2)))
     (message "")
     )))

;;; ----------------------------------------------------------------------
;;;
(defmacro message-quick (&rest body)
  "Shortcut to `message-flash'. Time interval is set to 1 second.
Uses `sleep-for' to make sure message is noticed.

Input:
  BODY    ,any form returning a string."
  (` (message-flash 1 'sleep (,@ body))))

;;; ----------------------------------------------------------------------
;;;
(defsubst read-char-safe (&optional prompt)
  "Wait for character until given and ignore all other events with PROMPT.
The `read-char' command chokes if mouse is moved while reading input.
This function returns 'ignore if the `read-char' couldn't read answer.
Otherwise it returns normal character.

Note:
  The cursor is not in the echo area when character is read. This
  may be confusing to user if you read multiple characters.

References:
  `read-char-safe-until'

Return:
  ch        ,character integer code, character type
  'ignore   ,if read failed due to non-char event."
  (condition-case nil
      (progn
	(message (or prompt ""))	;prevent echoing keycodes...
	(discard-input)			;this is a must before we read

	;; char-int
	;;   Emacs: this is no-op
	;;   XEmacs19.14: char-int doesn't exist.
	;;   XEmacs20:  read-char has changed, it does not return
	;;          int, but a character type, and we need conversion
	;;
	(read-char)
	)
    (error
     'ignore)))



;;; ----------------------------------------------------------------------
;;; Note: see function read-char-exclusive in never emacs versions, 19.29+
;;;       It is nt almost like this function... so we don't use it.
;;;
(defun read-char-safe-until (&optional prompt list)
  "Read character until given. Discards any events that are not characters.

Input:
  PROMPT    text displayed when asking for character
  LIST	    list of characters. The prompting won't stop until some of
	    them is selected.

Return:
  character  ,character type"
  (let* (ch
	 )
    (cond
    ((null list)
     (while (symbolp (setq ch (read-char-safe prompt)))))

    (list
     ;; We don'r have to do character conversion, because they are
     ;; treated as ints
     ;;
     (while (or (symbolp (setq ch (read-char-safe prompt)))
		(null ch)
		(not (char-in-list-case ch list))))
     ))

    (message "")
    ch
    ))



;;; ----------------------------------------------------------------------
;;;
;;; This will also remove the properties:
;;;
;;;   (format "%s" string)
;;;
(defsubst remove-properties (string)
  "Remove properties from STRING. Modifies STRING permanently.
Return:
   string	,with no properties."
  (progn
    (set-text-properties 0 (length string) nil string)
     string
     ))




;;; ----------------------------------------------------------------------
;;; - this is from fsf-translate-keys.el
;;;   01 Feb 1996, rainer@physik3.gwdg.de (Rainer Blome)
;;;
(defmacro applycar (function-form list-form)
  "Like mapcar, but does (apply FUNCTION-FORM (car LIST-FORM)).
Instead of (funcall FUNCTION (car LIST)). This is very useful for
invoking some function with many different sets of arguments.

Examples:

    (applycar 'global-set-key
      '(
	([f12]	repeat-complex-command)	; Again 	L2
     	([f14]	undo)			; Undo		L4
     	([f16]	copy-region-as-kill)	; Copy		L6
     	([f18]	yank)			; Paste		L8
     	([f20]	kill-region)		; Cut		L10
	))
  -->  (nil nil nil nil nil) ;; global - set - key returns 'nil

    (applycar (lambda (a b) (list b a)) ;; swaps arguments
      '((1 2)(3 4))
      )
  -->  ((2 1) (4 3))"
  (let ((spec-name (gensym)))
    (` (mapcar (lambda ((, spec-name))
		 (apply (, function-form) (, spec-name)) )
	 (, list-form) )
       )))


;;; ----------------------------------------------------------------------
;;; - Hmmm, too bad sleep-for accepts only seconds and sometimes
;;;   I just want to hang around for fraction.. like flashing messages
;;; - sit-for won't do, because it stops sit'n if any event is generated.
;;;
(defsubst sleep-for-count (count)
  "Mimics `sleep-for'.
Give COUNT times to loop. Good fraction of secondsecond loop time is 3000."
  (let* ((i 0))
    (while (and (> count 0)		;no negative numbers!
		(<= i count))
      (setq i (1+ i)))))



;;; ----------------------------------------------------------------------
;;;
(defsubst kill-control-chars ()
  "Kill all control characters from the whole buffer."
  (interactive)
  ;;  Hmm, strictly Emacs lisp says not to use replace-regexp
  ;;  in lisp programs. If this bothers you, use ti::b-replace-regexp
  ;;  instead.
  (save-excursion
    (PMIN)
    ;; Excludes tab,ff,cr,lf.
    (replace-regexp "[\000-\010\016-\037]+" "")
    ))



;;; ----------------------------------------------------------------------
;;;
(defsubst add-command-line-arg (arg &optional func)
  "Add ARG into `command-switch-alist' if it's not already there.
This inhibits argument to be treated as filename.

Optional FUNC is called when arg is found. Default FUNC used is 'ignore."
  ;;  make sure it's not there already
  ;;
  (or (assoc arg command-switch-alist)
      (setq command-switch-alist
	    (cons (cons arg (or func 'ignore))
		  command-switch-alist)
	    )))


;;; ----------------------------------------------------------------------
;;; - This should be standard function
;;; - Std distrib cplus-md.el
;;;
(nafboundp 'count-char-in-string"tinylibm"
  (defun count-char-in-string (c s)
    "Count CHARACTER in STRING."
    (let ((count 0)
	  (pos   0)
	  )
      (while (< pos (length s))
	(if (char= (aref s pos) c)
	    (inc count))
	(inc pos))
      count
      )))


;;; ----------------------------------------------------------------------
;;;
(nafboundp 'count-char-in-region "tinylibm"
  (defun count-char-in-region  (beg end char)
    "count characters. Eg to have real line count in buffer that
is running folding.el or outline, you should not call
count-lines function , but (count-char-in-region ?\\n)"
    (interactive "r\ncChar: ")
    (let* ((i 0)
	   )

      (setq end (max beg end)
	    char (char-to-string char))
      (save-excursion
	(goto-char (min beg end))
	(while (search-forward char end  t)
	  (inc i)))
      (if (interactive-p)
	  (message "%d hits in region." i))
      i
      )))

;;}}}
;;{{{ tests; small ones

;;; ......................................................... &testing ...

;;; ----------------------------------------------------------------------
;;;
(defsubst buffer-modified-pp  (&optional buffer)
  "Same as `buffer-modified-pp' but acceps arg BUFFER."
  (if (null buffer)
      (buffer-modified-p)
    (job-buffer buffer (buffer-modified-p))))

;;; ----------------------------------------------------------------------
;;;
(nafboundp 'first-line-p "tinylibm"
  (defsubst first-line-p  ()
    "Check if cursor is at first line"
    (save-excursion
      (beginning-of-line)
      (bobp))))


;;; ----------------------------------------------------------------------
;;;
(nafboundp 'last-line-p "tinylibm"
  (defsubst last-line-p  ()
    "Check if cursor is at last line"
    (save-excursion
      (end-of-line)
      (eobp))))


;;; ----------------------------------------------------------------------
;;;
(defsubst buffer-narrowed-p ()
  "Check if buffer is narrowed."
  (not (eq 1 (point-min))))

;;; ----------------------------------------------------------------------
;;; - it's better to use this func, so that you don't have to test
;;;   against 'magical' numbers.
;;;
(defsubst shell-exec-nok-p (nbr)
  "Check if last `shell-execute' return NBR was not successful."
  (not (eq 0 nbr)))


;;; ----------------------------------------------------------------------
;;;
(defun empty-buffer-p (&optional buffer)
  "Check if BUFFER is empty.
Buffer is considered empty if a) real `point-min' == `point-max'
b) or it contains only whitespace characters.

Return:
  nil		,buffer contains something
  t		,it's clear
  'empty	,contains only whitespace"
  (save-excursion
    (set-buffer (or buffer (current-buffer)))
    (if (EQBPP)				;fast way to tell...
	t
      (PMIN)
      (if (re-search-forward "[^ \n\t]" nil t)
	  nil
	'empty
	))
      ))

;;; ----------------------------------------------------------------------
;;;
(defun tinyck-maybe-activate  (&optional type mode)
  "Activate keybinding conversion is underlyiing Emacs needs it.
Call `tinyck-advice-control' with parameter mode if key conversion needed.
This ensures that binding work in any emacs (XEmacs and Emacs).
If you only use STRING bindings only use string notation

    (global-set-key \"\\C-c\\C-f\" ...)

then you don't need this function.

TYPE tells how you have written the keybindings. The 'xemacs binding
type is already supported by 19.33+ Emacs releases, but if you want your
packages be backward compatible you want to call this fucntions prior
bind definitions.

		# The Control-a binding is stylistically exploded due to
		# checkdoc.el
		#
    'emacs	Your bindings are like [?\\C - a] and [f10]
    'xemacs	Your bindings are like [(control ?a)] and [(f10)]

MODE

    nil		You pass this argument bfore you start defining keys
    'disable	You pass this, when you have finished.

Example:

    (tinyck-maybe-activate 'emacs)	  ;; turn conversion on in Xemacs
    (define-key [f1] 'xxx-function-call)
    <other key definitions ...>
    (tinyck-maybe-activate 'emacs 'disable) ;; conversion off

Recommendation:

    It is recommended that you write using the 'xemacs style, which
    is also supported in later Emacs releases 19.30+. If you do so,
    then calling this function is no-op in those Emacsen that support
    XEmacs style and you save the call to tinyck.el pacakge.

Return:
    t	    ,conversion activated
    nil"
  (let* ((xe-support (or (xemacs-p)
			 (and (emacs-p)
			      (> emacs-minor-version 33))))
         )
    (cond
     ((memq type '(xe XE xemacs))
      (if xe-support
	  nil				;Do nothing
	(when (emacs-p)			;XEmacs bindings and We're in Emacs
	  (tinyck-advice-control mode)
	  t
	  )))

     ((eq type 'emacs)
      (when (xemacs-p)			;Emacs bindings and we're in XEmacs
	(tinyck-advice-control mode)
	t
	))
     (t
      (error "Unknown type %s" type mode)
      ))
    ))




;;; ----------------------------------------------------------------------
;;; See register.el::insert-register
;;;
(defsubst register-live-p (char)
  "Test if register CHAR contain valid window configuration or mark."
  (let ((val (get-register char))
       )
    (if (or (consp val)		;window config
	    (and (markerp val)	;mark
		 (marker-buffer val)) ;not killed, reverted
	    )
	t
      nil
      )))


;;; ----------------------------------------------------------------------
;;;
;;; (1 2)   (cdr el) --> (2)   ,this is list
;;; (1)     (cdr el) --> nil   ,this too
;;; (1 . 2) (cdr el) --> 2     ,listp returns nil
;;;
(defsubst conspp (elt)
  "Test if ELT is in _really_ in format (X . X)."
  (and (consp elt)			;must be some '(...) form
       (null (listp  (cdr elt)))))


;;; ----------------------------------------------------------------------
;;; - How may time I need this! Can't count... this definitely should
;;;   be standard emacs function.
;;;
(defsubst listpp  (list)
  "Test if the there _really_ is elements in the LIST.
A nil is not accepted as a true list."
  (and (not (null list))
       (listp list)))


;;; ----------------------------------------------------------------------
;;;
(defsubst dos-file-p ()
  "Check if there is CR/LF ie.
\015[^M] + \n code at the end ofcurrent line."
  (save-excursion
    (end-of-line)
    (char= (preceding-char) ?\015)))


;;; ----------------------------------------------------------------------
;;; The 19.28 featurep is buggy, since unload-feature command does not
;;; remove entry from "features" list
;;;
(defsubst feature-exist-p (sym)
  "Check if the feature _really_ exist by loooking `load-history'.
Return:
   LIST		,`load-history' list for feature SYM
   nil          ,no feature information"
  (and (featurep sym)
       (assoc (symbol-name sym) load-history)))

;;; ----------------------------------------------------------------------
;;; - This allows you to load features easily in your .emacs
;;;   If they exist, then fine, if don't then you don't have them,
;;; - This is usually usegfull if you have same ~/.emacs in multiple
;;;   sites, but in one site you can only keep half of
;;;   the lisp packages.
;;;
(defsubst feature-load (feature-sym)
  "Try to load FEATURE-SYM if it does not exist. Signal no error.
Return non-nil if feature is provided.

Example:
  (feature-load 'mldrag)"
   (or (featurep feature-sym)
       (load (symbol-name feature-sym) 'noerr 'nomsg)))





;;; ----------------------------------------------------------------------
;;;  nil as a buffer ptr returns current buffer.., that's why AND test
;;;
(defsubst killed-buffer-p (buffer)
  "Check if BUFFER exist. Buffer must be buffer-pointer."
  (progn
    (if (stringp buffer)
	(null (get-buffer buffer))
      (not (and buffer
		(buffer-name buffer))))))


;;; ----------------------------------------------------------------------
;;;
(defsubst space-p (char)
  "Return t if character CHAR is space or tab."
  (or (char= char ?\t)
      (char= char ?\ )))


;;; ----------------------------------------------------------------------
;;; 11 Oct 1996, Kim-Minh Kaplan <kimminh.kaplan@eunet.fr>
;;;
(defun face-p (face-symbol)
  "XEmacs ad Emacs compatibility, Check if the FACE-SYMBOL exists."
  (cond
    ((fboundp 'find-face)
     (sfuncall 'find-face face-symbol))
    ((fboundp 'face-list)
     (memq face-symbol (sfuncall 'face-list)))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun face-supported-p  ()
  "Check if colors can be used (that thay can be displayed)."
  (let (ver
	)
    (cond
     ((emacs-p)
      (ti::xe-window-system))		;Emacs supports only in X

     ((setq ver (xemacs-p))
      (or (memq ver '(20))		;XEmacs 20 does tty
	  (and (eq ver 19)		;> 19.15 does too
	       (> emacs-minor-version 14)))
      ))))


;;; ----------------------------------------------------------------------
;;;
(defun overlay-supported-p  ()
  "Check if overlays are supported."
  (let* (
         )
    (or (emacs-p)
;;; The overlay.el library isn't working right yet: XEmacs19.15, XEmacs20
;;;	(and (xemacs-p)
;;;	     (fboundp 'make-overlay)	;Check basic set
;;;	     (fboundp 'move-overlay)	;See XEmacs 19.15/overlay.el
;;;	     (fboundp 'overlay-put))
	     )
    ))

;;}}}
;;{{{ misc, matching

;;; ......................................................... &m-match ...
;;; - The functions must be here, because defsubsts must be defined
;;;   before used
;;; - I chose to keep these names as possible, they consist of 3 char groups:
;;;   BUF(fer)  MAT(ching)


(eval-and-compile
;;; ----------------------------------------------------------------------
;;;
;;;
(defsubst read-match (level &optional string)
  "Read match from buffer atsubmatch LEVEL. Optionally from STRING.
Return nil, if match at LEVEL doesn't exist.

You have to call `o-looking-at' etc. before using this function.

Please use
   bufmatget, strmatget instead of calling this if you can

Call:
  (level &optional string)"
  (if (match-end level)
      (if (stringp string)
	  (substring
	   string
	   (match-beginning level) (match-end level))
	(buffer-substring
	 (match-beginning level) (match-end level)))
    ))


;;; ----------------------------------------------------------------------
;;; The replace-match doesn't allow to do strings, this does  :-)
;;;
(defun kill-match (level &optional replace string)
  "Kill match from buffer at submatch LEVEL or replace with REPLACE.
Point sits after the replaced or killed area.

Optionally you can give STRING. If level didn't match, do nothing.

Call:
  (level &optional replace string)

Return:
  t	,action taken
  nil	,if match at LEVEL doesn't exist.
  str   ,if string was given  ,use (setq str (kill-match 1 replace str))"
  (if (null string)
      (cond
       ((match-end level)
	(delete-region (match-beginning level) (match-end level))

	;;  I think emacs has bug, because cursor does not sit at
	;;  match-beginning if I delete that region, instead it is off +1
	;;  --> force it to right place
	(and replace
	     (goto-char (match-beginning level))
	     (insert replace))
	))
    ;; Handle string case
    (cond
     ((match-end level)
      (concat
       (substring string 0 (match-beginning level))
       (if (stringp replace) replace "")
       (substring string (match-end level)))))
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst strmatget (re level str)
  "Return RE match at LEVEL from STR. Nil if no match at level.
Call:
  strmatget (re level str)"
  (if (o-string-match re str)
      (read-match level str)))


;;; ----------------------------------------------------------------------
;;;
(defsubst bufmatget (re level)
  "Return string matching RE from _buffer_ at LEVEL. Use `looking-at'.
Nil if no match at level.

Call:
  bufmatget (re level)"
  (if (looking-at re)
      (read-match level)))

;; eval-and
)
;;}}}
;;{{{ tests cont'd


;;; ----------------------------------------------------------------------
;;;
(defsubst selective-display-line-p ()
  "Check if this line is collapsed with selective display.
Note: `selective-display' variable is usually t and the line contains \\r."
  (save-excursion
    (beginning-of-line)
    (looking-at ".*\r")
    ))

;;; ----------------------------------------------------------------------
;;;
(eval-and-compile
(defun function-args-p (symbol)
  "Return function SYMBOL argument list string or nil.
Works for byte compiled functions too.

Notes:
  if function is alias, the real function behing it is examined.
  if function is in autoload state, \"(autoload-args)\" is returned."
  (let* ((args-re "[^(]+\\([^)]+)\\)")
	 str
	 ret
	 )
    (if (autoload-p symbol)
	;;  We can't know the args. And we don't want to find out,
	;;  since it would load the package unnnecessarily
	;;
	(setq ret "(autoload-args)")

      (setq str (prin1-to-string
		 (symbol-function
		  (or (defalias-p symbol) symbol))))

      (cond
       ((emacs-p)
	(if (or (string-match "^(lambda +nil" str)
		(string-match "^#\\[nil" str)
		(and
		 (setq ret (strmatget args-re 1 str))
		 ;;  Empty arg list
		 (string= ret "")))
	    (setq ret nil))
	)
       (t
	;;  XEmacs has different Byte compilation format
	;;  #<compiled-function (from "custom.elc") nil "...(7)
	;;
	(cond
	 ((string-match
	   (concat "compiled-function +\(from.*\) +" args-re) str)
	  (setq ret (read-match 2)))
	 ((string-match "^(lambda +nil" str)) ;bypass
	 ((setq ret (strmatget args-re 1 str)))
	 ))
       ))
    ret
    ))
)

;;; ----------------------------------------------------------------------
;;;
(defsubst bool-p (var)
  "Test if VAR is nil or t."
  (or (eq var nil) (eq var t)))



;;; ----------------------------------------------------------------------
;;;
(defmacro print-p (ch)
  "Determines if character CH can be printed normally.
CH can be anything and this function won't choke. The \\t \\r \\n and \\f
codes are cosidered printable.

Return:
  t
  nil"
  (` (if (and (not (null (, ch)))	;it must not be nil
	      (or (char-in-list-case (, ch) '(?\t ?\n ?\r ?\f))
		  (and
		   (> (char-int (, ch)) 31)
		   (< (char-int (, ch)) 127))))
	 t nil)
     ))


;;; ----------------------------------------------------------------------
;;;
(defun char-case-p (char)
  "Check if character is uppercase or lowercase.

Return:
  t	,uppercase
  nil	,lowercase
  nbr	,if character isn't in set [A-Za-z] it returns CHAR."
  (cond
   ((and (>= char  97) (<= char 122))
    nil)
   ((and (>= char  65) (<= char 90))
    t)
   (t
    char)))


;;; ----------------------------------------------------------------------
;;;
(defsubst var-nil-p (var)
  "Test if VAR is empty.
Variable with only whitespaces [ \\f\\n\\t]* is considered empty too.

Example:
  (if (var-nil-p (setq answer (read-from-minibuffer \"give dime? \")))
      (message \"No fruit juice for you then.\"))"
  (or (eq nil var)
      (and (stringp var)
	   (or (string= var "")
	       (not (o-string-match "[^ \t\n\f]" var)))
	   )))


;;; ----------------------------------------------------------------------
;;;
(defsubst window-pmin-visible-p ()
  "Check if the `point-min' is visible in current window."
  (= (window-start) (point-min)))


;;; ----------------------------------------------------------------------
;;;
(defmacro window-pmax-visible-p ()
  "Check if the `point-max' is visible in current window."
  (= (window-end) (point-max)))


;;; ----------------------------------------------------------------------
;;;  Window pmin == the area of buffer that user sees, top line
;;;
(defun window-pmax-line-p ()
  "Check if cursor is on the same line as window's `point-max'."
  (let (point
	)
    (save-excursion
      (beginning-of-line)
      (setq point (point))

      (goto-char (window-end))

      ;;  a) if the last line DOES NOT exceed the window len; then the
      ;;     (window-end) is in next unvisible line. --> backward char
      ;;     brings it to previous line
      ;;  b) if the last line exceed the window len; then the
      ;;     (window-end) puts cursor at the last line. --> backward-char
      ;;     is no-op.
      ;;
      (backward-char 1)
      (beginning-of-line)

      (eq (point) point)
      )))


;;; ----------------------------------------------------------------------
;;;
(defsubst window-pmin-line-p ()
  "Check if cursor is on the same line as window's `point-min'."
  (save-excursion
    (beginning-of-line)
    ;;  The 1- is due to fact that there is NEWLINE, where C-e command
    ;;  does not ever go.
    (= (point) (window-start))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun window-pmax-line-bol (&optional eol-point)
  "Return window's last line's beginnning of point or EOL-POINT."
  (save-excursion
    ;; This is past of visible window, that why we go up one line
    ;;
    (goto-char (window-end))
    (backward-char 1)
    (if eol-point
	(end-of-line)
      (beginning-of-line))
    (point)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun window-middle-line ()
  "Computes middle line NBR in current window."
  (let* (
	 (win-min	(count-lines (point-min) (window-start)))
	 (win-max	(count-lines (point-min) (window-end)))
	 (middle	(+ win-min (/ (1+ (- win-max win-min)) 2)))
	 )
    middle
    ))



;;; ----------------------------------------------------------------------
;;; By Noah Friedman <friedman@prep.ai.mit.edu>
;;; in his eldoc.el lisp programming aid (post command hook)
;;;
(defun no-action-in-progress-p (mode)
  "Return t if there is no action currently in progress.
This means that following cases indicate that action is in progress
and it should be interfered.

o   if cursor it at minibuffer
o   keyboard macro is executing

Input MODE

 'timer
 This says that the function that calls us is currently run
 by an timer functin (19.34+)

 'post-command
 Same as above; but this time calling command is running in post hook.


This function is usually called from background processes that are
run by timers or post-command*hook functions and they want to print
something in the echo area. Use this"
  (and
   (not executing-kbd-macro)

   ;; Having this mode operate in an active minibuffer/echo area causes
   ;; interference with what's going on there.
   ;;
   (not cursor-in-echo-area)
   (not (eq (selected-window) (minibuffer-window)))
   (cond
    ((eq mode 'timer)
     ;;  If this-command is non-nil while running via an idle
     ;;  timer, we're still in the middle of executing a command,
     ;;  e.g. a query-replace where it would be annoying to
     ;;  overwrite the echo area.
     (and (not this-command)
	  (symbolp last-command)
	  ))

    ((eq mode 'post-command)
     ;;  If this-command is non-nil while running via an idle
     ;;  timer, we're still in the middle of executing a command,
     ;;  e.g. a query-replace where it would be annoying to
     ;;  overwrite the echo area.
     (and (symbolp this-command)
	  (sit-for 0.50))
     ))))


;;}}}
;;{{{ line

;;; ............................................................ &line ...
;;;
;;; Q: Why BOLP and BOLPP ??
;;; -  Depending on your needs, you may only need to know eol *OR* bol,
;;;    then you'd use EOLP, BOLP
;;; -  If you need to know both, it's more efficent to write it this way,
;;;    where save-excursion is used only once.
;;;
;;;    (save-excursion
;;;      (setq eolp (EOLPP))
;;;      (setq bolp (BOLPP)))
;;;

(defsubst BOLP ()
  "Return beginning of line. Preserves point."
  (save-excursion
    (beginning-of-line) (point)))


(defsubst BOLPP ()
  "Return end of line. Moves point."
  (beginning-of-line) (point))


(defsubst EOLP ()
  "Return end of line. Preserves point."
  (save-excursion
    (end-of-line) (point)))


(defsubst EOLPP ()
  "Return beginning of line. Moves point."
  (end-of-line) (point))


;;; ----------------------------------------------------------------------
;;;   Should return the same as goto-line, does it always ?
;;;
(defun LINE (&optional pmin)
  "Return current line nuber from the beginning of buffer.
If PMIN is non-nil the `point-min' is used for starting point, this
is usefull eg. for narrowed case. Normally returns true line number.

This function counts the number of \\n chartacters, so it will
return right count even in folding/outline buffers where selective
display is used. Using command `count-lines' would return false value.

Lines are counted from 1..x"
  ;;  - There is difference in using BOLP or point in certain
  ;;    situations, so always use line beginning as reference
  ;;  - The count-lines returns 0 for 1st line --> 1+
  (1+ (count-char-in-region
	(if pmin
	    (point-min)
	  (point-min-marker))
	(BOLP)
	?\n
	)))


;;; ----------------------------------------------------------------------
;;;
;;;  For more advanced line readings, see ti::b-read-line
;;;
(defsubst read-full-line ()
  "Retun whole line."
  (save-excursion
    (buffer-substring (EOLPP) (BOLPP))))



;;; ----------------------------------------------------------------------
;;;
(defsubst line-length (&optional point)
  "Length of current line. Optionally from POINT."
   (save-excursion
     (if point (goto-char point))
     (end-of-line)
     (current-column)))


(defsubst line-wrap-p  ()
  "Check if line wraps. ie. line is longer that current window."
  (> (line-length) (nth 2 (window-edges))))


;;; ----------------------------------------------------------------------
;;;
(defun re-search-check (re &optional level start-form read)
  "Check whole buffer if there is RE.

Input:
  RE		,regexp to search
  LEVEL		,which sublevel in regexp to match, default is 0
  START-FORM	,form yielding starting point of search. Default is `point-min'
  READ          ,read the match instead of returning point

Return:
  start point of match at level.
  string
  nil)"
  (save-excursion
    (if start-form
	(goto-char (eval start-form))
      (PMIN))

    (when (re-search-forward re nil t)
      (if read
	  (read-match (or level 0))
	(match-beginning (or level 0))
	))))





;;}}}


;;{{{ Special lists, assoc

;;; ................................................... &lists-special ...

;;; ----------------------------------------------------------------------
;;; Many times you want to have data structure with some KEY
;;; lias-c = cons list handling
;;;
(defmacro list-c-add-cdr (func key list add)
  "Add to the ASSOC list new ELT.
List must be in format, K = key, E = element.
  ( (K . (E E) (K . (E E)) .. )

Input:

  FUNC      'assq or 'assoc or any other to get inner list
  KEY       ,key
  LIST      ,list
  ADD       ,element to add

Example:

  (setq list '( (1 . (a b)) (2 . (c d))))
  (list-c-add-cdr list 'assq 1 'x)

  -->
  '( (1 . (a b x)) (2 . (c d))))"
  (`
   (let* (EL-T
	  LIS-T
	  )
   (when (setq EL-T (funcall (, func) (, key) (, list)))
     (setq LIS-T (cdr EL-T))
     (push (, add) LIS-T)
     (setcdr EL-T LIS-T)
     ))
   ))

;;}}}
;;{{{ list

;;; ............................................................ &list ...

;;; ----------------------------------------------------------------------
;;; 18 Apr 1997, Erik Naggum <erik@naggum.no>; gnu.emacs.help
;;;
(defun splice (target replacement beg end)
  "Mimic perl 'splice'. TARGET is source list. REPLACEMENT is destination list.
Position to slice is range BEG END.
  (setq x '(a b c))
  (splice x '(d e) 1 2) => (a d e c)
  x => (a b c)"
  (append (subseq target 0 beg)
          replacement
          (nthcdr end target)))

;;; ----------------------------------------------------------------------
;;; 18 Apr 1997, Erik Naggum <erik@naggum.no>; gnu.emacs.help
;;;
(defun nsplice (target replacement beg end)
  "Mimic perl 'splice'. TARGET is source list. REPLACEMENT is destination list.
Position to slice is range BEG END.
  (setq x '(a b c))
  (nsplice x '(d e) 1 2) => (a d e c)
  x => (a d e c)"
  (setf (nthcdr beg target) (append replacement (nthcdr end target)))
  target)

;;; ----------------------------------------------------------------------
;;; - This is handy for Function lists that can have parameters..
;;;   a little smarter way than doing run-hooks ...
;;;
(defsubst list-eval-until  (list)
  "Evaluate each item in LIST until some of them return non nil.

Return:
  nil
  non-nil	,result of evaled form"
  (let* (ret
	 )
    (while (and list
		(null (setq ret (eval (car list)))))
      (setq list (cdr list)))
    ret
    ))



;;; ----------------------------------------------------------------------
;;; - Very usefull when you can accept single or list argument and treat it
;;;   as list in mapcar
;;;
(defsubst list-make (single-or-list)
  "Converts SINGLE-OR-LIST into list.
If argument is already a list this macro is no-op."
  (if (listp single-or-list)
      single-or-list
    (list single-or-list)))



;;; ----------------------------------------------------------------------
;;; 31 Oct 1995, jsc@kontiki.sun.com (John Cooper), gnu.emacs.help
;;; - unfortunately recursion is quite slow, but this is
;;;  exceptional example!
;;;
;;;*(defun list-flatten (l)
;;;*  (cond ((consp l) (append (flatten (car l)) (flatten (cdr l))))
;;;*	((null l) l)
;;;*	(t (list l))))
;;;
;;; miracle@informatik.uni-oldenburg.de (Jan Schormann)
;;; 1 Nov 1995, gnu.emacs.help replied with non recursive solution
;;; - slightly layout modified by jari
;;;
;;;
(defun list-flatten (l)
  "Flatten list L."
  (let (result stack)
    (while (or stack l)
      (if l
	  (if (consp l)
	      (setq stack  (cons (cdr l) stack)     l (car l))
	    (setq   result (cons l result)          l nil)
	    )
	(setq l (car stack)    stack (cdr stack))
	))
    (reverse result)
    ))



;;; ----------------------------------------------------------------------
;;; #todo : should this use prin1-to-string, before extarcting elements,
;;;         any toughts ?
;;;
(defun list-join (list &optional join-str)
  "Joins string LIST with JOIN-STR, whic defualts to space."
  (let* (ret
	 (ch  (or join-str " "))
	 )
    (while list
      (setq ret (concat (or ret "") (car list)))
      (setq list (cdr list))
      (if list				;only if still elements
	  (setq ret (concat ret ch)))
      )
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun list-to-assoc-menu (list)
  "Converts string or number items in LIST into assoc menu.
Items are numbered starting from 0.

'(1 2 \"a\" \"b\")  --> '((\"1\" . 1) (\"2\" . 2) (\"a\" . 3) (\"b\" . 4))

This is usefull, if you call x popup menu or completion. For example:

(completing-read \"complete number: \"
		 (list-to-assoc-menu '(111 222 333 444)))"
  (let* ((i 0)
	 ret
	 str
	 )
    (list-loop list str
      (if (integerp str)
	  (setq str (int-to-string str)))

      (setq ret (append ret (list (cons str i))))
      (inc i)
      )
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun list-remove-successive  (list function)
  "Remove succesive same elements from LIST.

Input:

  LIST		,list
  FUNCTION	,accept Arg1 and Arg2 in list, should return non-nil
		 if elements are the same. Arg1 and Arg2 are taken
		 as 'car' in the list.

Example:

  (list-remove-succesive '(1 1 2 2 3) 'eq)
  --> '(1 2 3)
  (list-remove-succesive '(\"1\" \"1\" \"2\" \"2\" \"3\") 'string=)
  --> '(\"1\" \"2\" \"3\")"
  (let* (new-list
	 elt prev
	 )
    (list-loop list elt
      (if (funcall function prev elt)
	  nil				;skip duplicates
	(setq prev elt)		;prev value
	(push elt new-list))
      )

    (reverse new-list)
    ))

;;}}}
;;{{{ list

;;; ............................................................ &list ...



;;; ----------------------------------------------------------------------
;;; This is very usefull when contruction interactive calls
;;; (interactive
;;;   (list-merge-elements
;;;    (region-beginning)
;;;    (region-end)
;;;    (funcall get-3-arg-list)     ;; this returns '(arg1 arg2 arg3)
;;;    ))
;;;
;;; -->  (1 100 arg1 arg2 arg3)
;;;
(defun list-merge-elements (&rest args)
  "Merge single elements, ARGS, and on n dimensional lists to one list.
Example:
  (list-merge-elements 1 2 'some '(type here))
  -->
  '(1 2 some type here)"
  (let* (elt
	 x
	 ret
	 )
    (list-loop args elt
      (cond
       ((listpp elt)
	(list-loop elt x (push x ret)))
       (t
	(push elt ret)
	)))
    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;; - Ever struggled with peeking the lists..?
;;; - I have, and printing the contents of auto-mode-alist into
;;;   the buffer is very easy with this.
;;; - Should be default emacs function.
;;;
(defun list-print (list)
  "Insert content of LIST into current point."
  (interactive "XLisp symbol, list name: ")
  (mapcar
   (function
    (lambda (x)
      (insert (2str x) "\n")))
   list))


;;; ----------------------------------------------------------------------
;;; 1990, Sebastian Kremer, Institute for Theoretical Physics, West Germany
;;; BITNET: ab027@dk0rrzk0.bitnet
;;;
(defsubst list-to-string (list &optional separator)
  "Convert LIST into string. Optional SEPARATOR defaults to \" \".

Input:
  '(\"str\" \"str\" ...)

Return
  str"
  (mapconcat
   (function identity)			;returns "as is"
   list
   (or separator " ")
   ))



;;; ----------------------------------------------------------------------
;;; These two are almost direct copy from shadowfile.el, 19.28 distrib.
;;; Why not making them widely available ?
;;;
;;; Boris Goldowsky <boris@cs.rochester.edu>, Great code Boris !!
;;;
(defun list-union (a b &optional test)
    "Add members of list A to list B.
If TEST is non nil add only if they are not equal to items already in B.

There result order of elements is as follows:

(list-union '(1 2 3) '(1 4 4))
--> 3 2 1 1 4 4

To preserve order, you must call this function like this:
(list-union (reverse '(1 2 3)) '(1 4 4))
--> 1 2 3 1 4 4"
  (if (null a)
      b
    (if (and test (member (car a) b))
        (list-union (cdr a) b test)
      (list-union (cdr a) (cons (car a) b) test))))


;;; ----------------------------------------------------------------------
;;;
(defsubst list-previous-elt (list arg &optional test-form)
  "Get previous element from LIST.
See `list-position' for arguments ARG TEST-FORM."
  (let* ((pos (list-position list arg test-form))
	 )
    (if (and pos
	     (not (eq 0 pos)))		;can't go back
	(nth (1- pos) list))
    ))


;;; ----------------------------------------------------------------------
;;; This enables you to access previous and next element easily.
;;;
(defun list-position  (list arg &optional test-form)
  "Return position 0..x in list.

Input:
  LIST		,list
  ARG		,this position in list is sought
  TEST-FORM	,defaults to 'equal, you can use ARG and LIST in the
		 test form. Example:  '(string= (car list) arg)

Return:
  nil  ,no ARG in list"
  (let* ((i 0)
	 ret
  	 )
    (while list
      (if (if test-form
	      (eval test-form)
	    (equal (car list) arg))
	  (setq ret i  list  nil)
	(inc i)
	(setq list (cdr list))
	))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun list-find (table arg &optional test-form all-matches)
  "Loops through TABLE until element matching ARG is found.

Input:

  TEST-FORM	defaults to (o-string-match (nth 0 (car table)) arg)
		and the supposed list is assumed to be:
		'( (\"REGEXP\"  ANY_DATA)  ..)
  ALL-MATCHES	flag, if non-nil return list of matches.

You can refer to these items in the test-form

  table         ,current table
  arg           ,argument
  element       ,current item beeing compared, also the actual element
		 stored to list if match return t. Defaults to (car table)

Examples:

   (defconst my-list '((\"1\" \"a\") (\"2\" \"b\")))

   ;;  This is like using 'assoc'
   ;;
   (list-find my-list \"1\")
   --> (\"1\" \"a\")

   ;;  Do match against member 2
   ;;
   (list-find my-list \"b\" '(o-string-match (nth 1 (car table)) arg))
   --> (\"2\" \"b\")

   ;;  This is little tricky, we search all '.fi' sites, and then
   ;;  remove all whitespaces around the items.
   ;;
   (defconst my-list2 '(\"   foo@a.fi \" \"Bar <man@b.fi>   \" \"gee@c.uk  \"))
   (list-find my-list2  \"[.]fi\"
              '(and
		 (o-string-match arg (car table))
		 (setq element (ti::s-sqz (car table)))
		 )
	'all-matches)

   --> (\"foo@a.fi\" \"Bar <man@b.fi>\")

Return:

  nil
  element      ,single element
  list	       ,list is returned if all-items is non-nil"
  (let* (element
	 ret
	 )
    (list-loop table element
      (when (if test-form
		(eval test-form)
	      (string-match (nth 0 (car table)) arg))
	(if all-matches		;how to put results ?
	    (nconcc ret element)
	  (setq ret element   table nil)))
      )
    ret
    ))

;;}}}
;;{{{ misc, window, frame, events, popup

;;; ........................................................ &m-events ...


;;; ----------------------------------------------------------------------
;;;
(nafboundp 'current-frame "tinylib"
  (defsubst current-frame  ()
    "Return current fram pointer."
    (window-frame (get-buffer-window  (current-buffer)))))


;;; ----------------------------------------------------------------------
;;;
(defsubst return-work-frame  (&optional win)
  "Return some non-dedicated frame. The current frame which is checked is WIN."
  (if (window-dedicated-p (selected-window))
      (car (ti::w-frame-list nil nil win))
    ;;  current frame
    (window-frame (get-buffer-window  (current-buffer)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst use-non-dedicated-frame  ()
  "Maybe move to some non dedicated frame(window).
Eg. you can'tperform `find-file', `switch-to-buffer' in the dedicated frame."
  (if (window-dedicated-p (selected-window))
      (raise-frame (select-frame (car (ti::w-frame-list))))))


;;; ----------------------------------------------------------------------
;;;
(defsubst string-match-case (re str &optional case-fold start)
  "Do local case sensitive match.
See `string-match' for parameters RE STR.
The CASE-FOLD has same value as `case-fold-search'. nil means sensitive.
START is parameter in `string-match'."
  (let ((case-fold-search case-fold))
     (string-match re str start)))



;;; ----------------------------------------------------------------------
;;;
(defmacro use-dynamic-compilation  ()
  "Turn on dynamic cmpilation in this pacakge.
You add this staement to the beginning of your file:

   (eval-when-compile (use-dynamic-compilation))"
  (`
   (progn
     (defvar byte-compile-dynamic nil "19.29 var") ;silence byteCompiler
     (make-local-variable 'byte-compile-dynamic)
     (setq byte-compile-dynamic t)         ;19.29+
     )))


;;; ----------------------------------------------------------------------
;;;
(defmacro use-timer ()
  "Emacs and XEmacs compatible. Load correct timer package.
Recommended usage:

  (eval-and-compile (use-timer))"
  (`
   (progn
     (if (emacs-p)
	 (require 'timer)		;Emacs
       (require 'itimer)		;XEmacs
       ))))

;;}}}
;;{{{ misc

;;; ............................................................ &misc ...




;;; ----------------------------------------------------------------------
;;; 17 Feb 1997, Erik Naggum <erik@naggum.no> comp.emacs,
;;; Slightly adjusted by [jari]
;;;
(defun push-definition (symbol &optional func-flag)
  "Push current definition of SYMBOL to stack.
If FUNC-FLAG is non-nil, then push function definition.

Stack is at kept in property 'definition-stack"
  (if func-flag
      (push (symbol-function symbol) (get symbol 'definition-stack))
    (push (symbol-value symbol) (get symbol 'definition-stack))
    ))


;;; ----------------------------------------------------------------------
;;; 17 Feb 1997, Erik Naggum <erik@naggum.no> comp.emacs,
;;; Slightly adjusted by [jari]
;;;
(defun pop-definition (symbol &optional func-flag)
  "Retrieve previous definition of SYMBOL from stack.
If FUNC-FLAG is non-nil, then pop function definition.

Stack is at kept in property 'definition-stack"
  (if func-flag
      (setf (symbol-function symbol) (pop (get symbol 'definition-stack)))
    (setf (symbol-value symbol) (pop (get symbol 'definition-stack)))
    ))



;;; ----------------------------------------------------------------------
;;;
(defsubst use-prefix-key  (map key)
  "Define to MAP a prefix KEY. If KEY is not keymap, allocate the key.
Return KEY's original binding."
  (if (not (keymapp (lookup-key map key)))
      ;;  Make it available
      (prog1
	  (lookup-key map key)
	(define-key map key nil))))


;;; ----------------------------------------------------------------------
;;; 19 Apr 1997, kyle_jones@wonderworks.com (Kyle Jones), comp.emacs
;;;
(defmacro define-buffer-local-keymap ()
  "Copy current local keymap and execute `use-local-map'.
After that your commands with `local-set-key' are buffer local."
  (use-local-map
   (copy-keymap (or (current-local-map) (make-sparse-keymap)))))

;;; ----------------------------------------------------------------------
;;;
(defmacro define-key-if-free (map key object &optional callback)
  "Put key to map if key is not assigned already.
then key can be assigned if
o   slot is nil
o   slot has function 'ignore
o   slot has already object

Any other case generates error: the slot is already occupied.

You normally call this function from package that want's to define
eg. function keys permanently and if there is already user definition
you can stop right there and print message.

Input:

  MAP	   ,map where to define the key eg. `global-map'
  KEY	   ,key eg. [f10]
  OBJECT   ;assin object to key.
  CALLBACK ,on error call function CALLBACK with argument KEY and the
            result of `lookup-key'.

Example:

  (define-key-if-free global-map  [f10]
    'xxx-func 'xxx-define-key-error)

  (defun xxx-define-key-error (key def)
    (error
     (format \"package xxx: key %s is already occupied with %s\"
	     \"Please use manual customization.\"
	     key def)))"
  (`
   (let ((def (lookup-key (, map) (, key )))
	 )
     ;; Lookup key returns NBR if the sequence of keys exceed
     ;; the last keymap prefix
     ;; C-cck  --> C-cc  is undefined, so there is no C-c c map yet
     ;;
     ;;

     (if (or (eq (, object) def)
	     (memq def '(nil ignore))
	     (integerp def))
	 (define-key (, map) (, key ) (, object))
       (if (, callback)
	   (funcall (, callback) (, key ) def)
	 (error
	  (format "Already occupied, key: %s slot content: %s "
		  (, key)
		  (prin1-to-string def)))))
     )))


;;; ----------------------------------------------------------------------
;;;
(defmacro copy-key-definition (map to-key from-key)
  "Put to MAP a TO-KEY that is bound to FROM-KEY.
You can use this function eg. in minor modes, where when minor
mode is turned on, it moves some key definitions to somewhere
else. For example if minor mode wants to take over PgUp and PgDown
keys, but preserve their original menaing under some other key,
it could copy the function calls to sme other key like
control-PgUp and control-PgDown.

Example:

    ;;  move PgUp/Down under Control key. Preserve their original
    ;;  function that may not be simple scroll-down!
    ;;
    (copy-key-function map [C-prior] [prior])
    (copy-key-function [C-next] [prior])

    ;; Now occupy  minor map definition
    (define-key [prior] 'minor-mode-function)"
  (`
   (define-key (, map) (, to-key)
     (or (and (current-local-map)
	      (lookup-key (current-local-map) (, from-key)))
	 (lookup-key global-map          (, from-key)))
     )))


;;; ----------------------------------------------------------------------
;;;
(defun make-vector-keybinding  (&rest args)
  "Make vector keybinding. ARGS can be 1-char-strings, chars or symbols.
This function is usefull if user can configure the first character
as a prefix key. Usually you need this function only to contruct the
[x-event] keybinding, where you need symbols like mouse - 1, mouse - 2.

Note:

    This function is solely for Emacs only.

Example:

    (setq xx-mode-key-prefix \"\C-z\")

    (defun xx-mode-default-keybindings ()
        (setq mouse-key
              (make-vector-key
                '(xx-mode-key-prefix ?i 'mouse-1)))
        (define-key xx-mode-map mouse-key 'xx-func)
	)"
  (let* ((i  0)
	 vec
	 elt
         )
    ;;	mouse binding "Prefix + i mouse-1"
    ;;

    (setq vec (make-vector (length args) nil))
    (list-loop args elt
      (cond
       ((stringp elt)
	(setq elt (aref elt 0)))	;pick only first char

       ((integerp elt) nil)		;do nothing

       ((symbolp elt)
	(setq elt (intern (symbol-name elt))))

       (t
	(error "Invalid arg %s" elt)
	))

      (aset vec i elt)
      (inc i)
      )
    vec
    ))



;;; ----------------------------------------------------------------------
;;;
(defsubst beginning-of-defun-point (&optional end)
  "Search function beginning or END. Point is preserved. No errors.
Return:
 point
 nil	,not found"
  (save-excursion
    (safe-do
     (if end
	 (end-of-defun)
       (beginning-of-defun))
     (point)
     )))


;;; ----------------------------------------------------------------------
;;;
(defsubst digit-len (arg)
  "Return number of digits in ARG which must be either number or string.
If ARG is string, the length of string is returned."
  (let ((val arg))
    (if (integerp arg)
	(setq val (int-to-string arg)))
    (length val)))


;;; ----------------------------------------------------------------------
;;; - add-hook should accept many parameters...
;;;
(defun add-hooks (hook-or-list single-or-list &optional remove append)
  "Run `add-hook' to insert every element in HOOK-OR-LIST to SINGLE-OR-LIST.

Remember:

  `Add-hook' creates a hook variable if it doesn't exist.

Input:

  HOOK-OR-LIST  ,hook symbol, or list of hook symbols
  LIST		,single function or list of functions
  REMOVE	,flag; run `remove-hook' instead.
  APPEND        ,parameter to `add-hook'

Example:

  ;;  Add function to all 3 hooks
  (add-hooks '(mode1-hook mode2-hook) 'turn-on-xxx-mode)"
  (let* ((list	(list-make single-or-list))
	 (hlist (list-make hook-or-list))
	 hook
	 )
    (list-loop hlist hook
      (mapcar
       (function
	(lambda (x)
	  (if remove
	      (remove-hook hook x)
	    (add-hook hook x append))
	  ))
       list))
    ))


;;; ----------------------------------------------------------------------
;;; - Finally Exist in new emacs
;;; - This is in defmacro form, because I didn't know how to
;;;   write the following
;;;
;;;      (apply (car LisT) (,@ args) nil)
;;;
(defmacro run-hooks-until  (hook-sym &optional rest-hooks &rest args)
  "Run all functions in HOOK-SYM. Stop when first one return non-nil.

Input:

  HOOK-SYM		,hook symbol, or list of functions.
  REST-HOOKS		,return value option t/nil
  ARGS			,arguments to functions. if NIL, functions
			 are called without arguments.

Example:

  (run-hooks-until 'my-hook       nil arg1 arg2 arg3)

Return:

  a.	Hook's non-nil return value
  b.	If REST-HOOKS is non-nil, return rest of the hooks that
	were not run."
  (`
   (let* ((LisT
	   ;;  'hook ==> eval ==> '(func1 func2)
	   (cond
	    ((listpp (, hook-sym))  (, hook-sym))
	    (t
	     (list-make
	      (cond
	       ((and (symbolp (, hook-sym)) ;Single function
		     (fboundp (, hook-sym)))
		(, hook-sym))
	       ((symbolp (, hook-sym))	;Variable
		(eval (, hook-sym)))
	       (t
		;;  Common mistake, prevent it
		(error "Need hook symbol")
		)))
	     )))
	  ReT
	  )
     (while (and LisT
		 (null
		  (setq ReT
			(if (eq nil (quote (, args)))
			    (funcall (car LisT))
			  (apply (car LisT) (,@ args) nil)
			  )))
		 )
       (setq LisT (cdr LisT))
       )

     (if (null ReT)		;no-op, Silence XEmacs 19.14 ByteComp.
	 (setq ReT nil))

     (if (, rest-hooks)  LisT ReT)
     )))


;;; ----------------------------------------------------------------------
;;;
;;; - I need locals so many times it make sme cry, eg. post-command-hook
;;; - And why doesn't the add-hook accepts list by default ??
;;;
;;; - This aapplies to 19.29.1 and newer
;;;       (add-hook HOOK FUNCTION &optional APPEND LOCAL)
;;;       Do not use `make-local-variable' to make a hook
;;;       variable buffer-local.  Use `make-local-hook'
;;;       instead.
;;;
(defun add-hook-local (var-sym maybe-list)
  "Like `add-hook', but make VAR-SYM hook local to buffer.
MAYBE-LIST argument can be a single symbol or list of symbols.

Eg. For making your private post command:

   (add-hook-local 'post-command-hook 'func1)
   (add-hook-local 'post-command-hook '(func1 func2))"
  ;; the variable may be local already, but we do not do
  ;; any checkings
  (make-local-variable var-sym)

  ;; Copy this because add-hook modifies the list structure.
  (set var-sym (copy-sequence (eval var-sym)))


  (if (and (not (null maybe-list))
	   (listp maybe-list))
      (mapcar
       (function
	(lambda (x)
	  (add-hook var-sym x)))
       maybe-list)
    (add-hook var-sym maybe-list)
    )
  nil				;just some neutral ret value
  )



;;; ----------------------------------------------------------------------
;;;
(defsubst shell-execute (command &optional buffer)
  "Executesshell COMMAND and optionally output to BUFFER.

References:
  `shell-file-name'	variable
  `shell-exec-nok-p'	function

Return:
  0	,error
  nbr	,ok"
   (call-process
    shell-file-name
    nil
    buffer
    nil
    "-c"
    command
    ))


;;; ----------------------------------------------------------------------
;;; XEmacs : replace-in-string
;;;
(defun subst-char (str char to-char)
  "Replace in STR every CHAR with TO-CHAR."
  (let ((len   (length str))
	(ret   (copy-sequence str))	;because 'aset' is destructive
	)
    (while (> len 0)
      (if (char= (aref str (1- len)) char)
	  (aset ret (1- len) to-char))
      (dec len))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun subst-char-with-string (string &optional char to-string)
  "In STRING, converts CHAR with TO-STRING.
Default is to convert all tabs in STRING with 8 spaces."
  (let* ((to-string	(or to-string	"        "))
	 (char		(or char	?\t))
	 (len           (length string))
	 (i             0)
	 elt
	 ret
	 )

    (while (< i len)
      (setq elt (char-to-string (aref string i)))
      (if (char= char (aref string i))
	  (setq elt to-string))
      (setq ret (concat ret elt))
      (inc i))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; Thu, 15 Feb 96, "Christopher J. Madsen" <ac608@yfn.ysu.edu>
;;;
(defun prefix-arg-to-text (arg)
  "Return a string describing the current prefix argument ARG."
  (cond
   ((null     arg)    "")
   ((eq '-    arg)    "C-u - ")
   ((integerp arg)    (format "C-u %d " current-prefix-arg))
   (t
    (apply 'concat
	   (make-list (round (log (car arg) 4)) "C-u ")))))


;;; ----------------------------------------------------------------------
;;;
(defmacro keep-lower-order (var1-sym var2-sym)
  "Keep VAR1-SYM < VAR2-SYM."
  (` (let ((MiN (min (, var1-sym) (, var2-sym)))
	   (MaX (max (, var1-sym) (, var2-sym)))
	   )
       (setq (, var1-sym) MiN)
       (setq (, var2-sym) MaX)
   )))

;;; ----------------------------------------------------------------------
;;;
(defmacro var-toggle (var &optional arg)
  "Toggle VAR according to ARG like mode would do.
Usefull for for functions that use arg 0/-1 = off, 1 = on, nil = toggle.
Minor modes behave this way.

VAR is set to following values when ARG is:
  arg 0/-1  VAR -> nil
  arg nbr   VAR -> t
  arg nil   VAR -> not(var)     toggles variable"
  (` (setq (, var)
	   (cond
	    ((memq (, arg) '(0 -1))
	     nil)
	    ((null (, arg))
	     (not (, var)))
	    (t
	     t)))
     ))

;;}}}

;;{{{ buffers

;;; .......................................................... &buffer ...



;;; ----------------------------------------------------------------------
;;; - Why doesn't Emacs have this by default, the erase-buffer
;;;   doesn't take arguments ... :-(
;;;
(defun clear-buffer (&optional buffers)
  "Clear BUFFERS. Buffer existense is not checked."
  (let (elt
	)
    (setq buffers (or (list-make buffers)
		      (list (current-buffer))))
    (save-excursion
      (list-loop buffers elt
        (set-buffer elt)
        (erase-buffer)
        ))
    ))


;;; ----------------------------------------------------------------------
;;; - The buffer is *not* cleared by default, only put to consistent state
;;;
(defun use-tmp-buffer (&optional buffer clear)
  "Create and reset temporary BUFFER.
Remove read-only. buffer name is \"*tmp*\" by default.
Put buffer to `fundamental-mode' and remove any narrowing.
if CLEAR is non-nil, delete old buffer content.

Return:
  buffer"
  (let* ((buffer  (get-buffer-create (or buffer "*tmp*")))
	 )
    (save-excursion
      (set-buffer buffer)
      (fundamental-mode)		;No fancy modes here
      (setq buffer-read-only nil)

      ;; - This call has been commented for now, because it prints
      ;;   unecessary message every time it's beeing called.
      ;; - Besides the modified flag is not much used for "star",tmp, buffers
      ;;
      ;; (set-buffer-modified-p nil)

      ;; - We don't check the possible narrowing. We just want to be sure.
      ;;
      (widen)
      (if clear
	  (erase-buffer))
      )
    buffer
    ))



;;; ----------------------------------------------------------------------
;;;
(defun get-buffer-substring (&optional buffer beg end)
  "Like `buffer-substring', but accept BUFFER parameter.
The BEG and END defaults to `point-min' and `point-max'.

with no parameters, acts like `buffer-string' function."
  (let* (ret
	 )
    (cond
     (buffer
      (save-excursion
	(set-buffer buffer)
	(if (and beg end)
	    (setq ret (buffer-substring beg end))
	  (setq ret (buffer-string)))
	))
     (t
      (if (and beg end)
	  (setq ret (buffer-substring beg end))
	(setq ret (buffer-string)))
      ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst append-to-buffer-string (buffer string &optional beg-flag)
  "Append to BUFFER a STRING. If BEG-FLAG is non-nil, prepend to buffer."
  (save-excursion
    (set-buffer buffer)
     (if beg-flag
	 (PMIN)
       (PMAX))
     (insert string)))

;;; ----------------------------------------------------------------------
;;;
(defun set-buffer-safe (buffer)
  "Execute `set-buffer' if BUFFER exists. Does not signal any error.
Return
  BUFFER-PTR    ,if `set-buffer' executed
  nil           ,buffer does not exist"
  (let* (ret)
    (cond
     ((stringp buffer)
      (if (get-buffer  buffer)
	  (setq ret (set-buffer  buffer))))
     ((bufferp buffer)
      ;; it's a buffer-pointer type
      (if (killed-buffer-p buffer)
	  nil
	(setq ret (set-buffer buffer))))
     )
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst kill-buffer-safe (buffer)
  "Do `kill-buffer' only if BUFFER exists. Does not signal any error.
Return
  t             ,killed
  nil           ,no such buffer"
  (save-excursion
    (when (set-buffer-safe buffer)
      (set-buffer-modified-p nil)	;No confirmation when we kill it
      (kill-buffer buffer))))

;;}}}
;;{{{ hash table

;;; ...................................................... &hash table ...
;;; These are normally calld hash tables, or Emacs says they are obarrays.
;;; whatever...
;;;
;;; The idea is to store uniq ITEMS into vectors, like filenames.
;;; Then each filename can have properties, like rcs version number,
;;; locker, date of creation etc.
;;;
;;;

;;; ----------------------------------------------------------------------
;;;
(defmacro vec-arr-init (table &optional size init-val)"Clears vector TABLE. Default SIZE is 128 buckets. INIT-VAL defaults to nil."
  (` (setq (, table) (make-vector (or (, size) 128) (or (, init-val) nil)))
     ))

;;; ----------------------------------------------------------------------
;;;
(defmacro vec-arr-get (table item &optional allocate)
  "Read vector TABLE and return ITEM. ALLOCATE if ITEM does not exist."
  (` (if (, allocate)
         (intern (, item) (, table))
       (intern-soft (, item) (, table)))
     ))

;;; ----------------------------------------------------------------------
;;;
(defun vec-arr-prop (table item prop &optional put-value force-set)
  "In vector TABLE and ITEM, get or put property PROP.

Input:
  TABLE		hash table
  ITEM		If ITEM is not allocated bucket, signal error.
  PROP		property symbol
  PUT-VALUE	value to put. If this is non-nil value is stored.
  FORCE-SET	flag, if non-nil then put anything that was in  put-value
		Eg. value nil can be stored this way."
  (let* (sym)
    (if (null (setq sym (vec-arr-get table item)))
        (error "No bucket found for item. [item not in table] %s" item)
      (if (or put-value force-set)
          (put sym prop put-value)
        (get sym prop)))
    ))

;;}}}

;;{{{ file

;;; ############################################################ &file ###

;;; ----------------------------------------------------------------------
;;; The lisp primitive call isn't very descriptive. This short
;;; macro looks better in code.
;;;
(defsubst file-changed-on-disk-p (&optional buffer)
  "Check if BUFFER's file has recently changed on disk."
  (not (verify-visited-file-modtime
	(or (current-buffer) buffer))))


;;; ----------------------------------------------------------------------
;;;
(defsubst file-mode-make-read-only (mode)
  "Make MODE bit user read-only."
  (logand mode 383))

;;; ----------------------------------------------------------------------
;;;
(defsubst file-mode-make-read-only-all (mode)
  "Make MODE bit read-only to all."
  (logand mode 292))			;444oct

;;; ----------------------------------------------------------------------
;;;
(defsubst file-mode-make-writable (mode)
  "Raise MODE bit's write flag."
  (logior mode 128))

;;; ----------------------------------------------------------------------
;;;
(defsubst file-mode-make-executable (mode)
  "Raise MODE bit's executable flag."
  (logior mode 64))			;oct 100


;;; ----------------------------------------------------------------------
;;;
(defsubst file-mode-protect (file &optional mode)
  "Set FILE modes to -rw------- or if MODE is non-nil to -r--------."
  (interactive)
  (cond
   (mode    (set-file-modes file 256)) ;; 400oct
   (t       (set-file-modes file 384)) ;; 600oct
   ))

;;; ----------------------------------------------------------------------
;;;
(defsubst file-toggle-rw (mode)
  "Toggle MODE bit's user write flag."
  (if (= 0 (logand mode 128))		;-r-------- , 400 oct, 256 dec
	 (file-mode-make-writable mode)	;R --> W  200
       (file-mode-make-read-only mode)  ;W --> R, 577
       ))


;;; ----------------------------------------------------------------------
;;;
(defsubst file-owned-p (file)
  "Test if current `user-uid' [uid] owns the FILE."
  (eq (user-uid) (nth 2 (file-attributes file))))


;;; ----------------------------------------------------------------------
;;; - If you own the file, you can turn on the write flag..
;;;
(defsubst file-modify-p (file)
  "Test if we can modify FILE. It must be file, not dir, owned by us."
  (and (file-exists-p file)
       (file-owned-p file)))


;;; ----------------------------------------------------------------------
;;; - I do this so often that a macro is handy
;;;
(defsubst file-loadable-p (file)
  "Check if FILE is loadable. Non-string args are accepted too.
The FILE is not expanded."
  (and (stringp file)
       (file-readable-p file)))


;;; ----------------------------------------------------------------------
;;;
(defsubst file-read-only-p (file)
  "Check if FILE is read only.
Only checks if there is no +w flags,other flags are not checked.

Eg. you may have permissions ---x------ which this function
reports as read-only, bcause there is no +w flags on."
  (let (modes
	)
    (if (not (file-exists-p file))
	(error "No file '%s'" file)
      (if (null (setq modes (file-modes file)))
	  (error "File modes failed?")
	;;  222oct is 146dec "--w--w--w" if any of these write flags
	;;  is on, then this returns true.
	(if (eq 0 (setq modes (logand modes 146)))
	    t
	  nil
	  )))))

;;; ----------------------------------------------------------------------
;;;
(defun file-name-run-real-handler (caller-sym operation args)
  "You can call this function only from `file-name-handler-alist' handler.
See Info page Node: Magic File Names.

Input:
  CALLER-SYM	,the caller's function symbol
  OPERATION	,handler operation, see info page.
  ARGS		,in &rest form"
  (let ((inhibit-file-name-handlers
	 ;;  Prevent infinite loop, don't call my-handler again.
         (cons caller-sym
               (and (eq inhibit-file-name-operation operation)
                    inhibit-file-name-handlers)))
        (inhibit-file-name-operation operation)
	)
    (apply operation args)))


;;; ----------------------------------------------------------------------
;;;
;;; I need both all the time, so this macro is handy
;;;
(defmacro file-components (str dir-var file-var &optional dir-flag)
  "Shortcut for extracting directory part and file part at once.

Input:
  STR	    absolute filename
  DIR-VAR   variable symbol
  FILE-VAR  variable symbol
  DIR-FLAG  If non-nil, then empty directory portion is returned as \"./\""
  (`
   (let* ((dIrI (, str))
	  )
     (setq (, dir-var)   (or (file-name-directory dIrI)
			     (and (, dir-flag) "./"))
	   (, file-var)  (file-name-nondirectory dIrI)
	   ))))

;;; ----------------------------------------------------------------------
;;; #obsolete 19.30 insert-file-contents-literally
;;;
;;; - The problem with "loading into emacs" is that all kinds of hooks
;;;   are run, eg. folding and outline might get activated when the file is
;;;   loaded. Eg. if we do eval, it can't see the functions if they are
;;;   behind selective display.
;;;
;;; - The function name starts with word "file" because all file related
;;;   commands are prefixed with that word in this package. You may want to
;;;   use this alias for interactive use:
;;;
;;;   #defalias (defalias 'find-file-pure 'file-pure-find)
;;;
(defun file-pure-find (file &optional buffer show-it)
  "Like `find-file' but disables everything which might affect loading.
No hooks are run, no other special setups.

If there existed same file, the buffer name will reflect the file name
with letters \"<2>\" or so.

Input:
  FILE		file to load
  BUFFER	optional buffer where to insert the file
  SHOW-IT       displays buffer. This is on when called interactively.

Return:
  buffer pointer"
  (interactive "fFind file: ")
  (let* (
	 ;;   This mode does not run any hooks.
	 (default-major-mode 'fundamental-mode)

	 ;;   This makes sure we truly load the file.
	 ;;   If there were that file in emacs, emacs won't load it.
	 (fn  (file-name-nondirectory file))
	 (bp  (or buffer (generate-new-buffer fn)))

	 ;;   Prohibit emacs from doing anything fancy while
	 ;;   we load the file
	 (enable-local-eval  nil)
	 (find-file-hooks    nil)        ;; jka doen't use this

	 (verb (or show-it (interactive-p)))
	 )
     (save-excursion
       (set-buffer bp)
       (insert-file-contents file)
       (if verb (switch-to-buffer bp))
       (set-buffer-modified-p   nil)
       (setq buffer-file-name (expand-file-name file))
       bp
       )))


;;; ----------------------------------------------------------------------
;;;
(defun file-eval (file)
  "Like `load-file', but read FILE and eval it in temporary buffer.

The advantage over `load-file' is that physical loading also uncompresses
the file if there is proper elisp package to handle it, thus your elisp
can be in any file *form* that packages allow for loading."
  (let* ((name   "*file-eval*")
	 (buffer (use-tmp-buffer name 'clear))
	 )
    (setq buffer (file-pure-find file name))
    (save-excursion
      (set-buffer buffer)
      (if (and (xemacs-p)		;XEmacs compatibility
	       (fboundp 'eval-buffer))
	  (sfuncall 'eval-buffer)
	(sfuncall 'eval-current-buffer))

      (kill-buffer buffer)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun dir-writable-p (file-or-dir)
  "Check if FILE-OR-DIR is writable.

Call:
  (file-or-dir)"
  (let* ((dir		(file-name-directory (expand-file-name file-or-dir)))
	 (file		"#9#_%")
	 (fn		(concat dir file))
	 )
    (if (not (stringp file))
	(error "invalid arg"))

    (file-writable-p fn)
    ))

;;; ----------------------------------------------------------------------
;;; - When removing temporary files; I don't care if they succeed or not
;;;
(defun file-delete-safe (files)
  "Deletes file or list of FILES. Read only files are chmod'd to writable.
All errors are ignored."
  (let* ((list (list-make files))
	 file
	 mods
	 )
    (list-loop list file
      (safe-do
	(when (file-exists-p file)
	  (setq mods (file-mode-make-writable (file-modes file)))
	  (set-file-modes file mods)
	  (delete-file (car list))
	  )))
    ))

;;; ----------------------------------------------------------------------
;;; - The buffer is *not* cleared, only put to consistent state
;;;
(defun use-tmp-file (file)
  "Prepare temporary file for use. Deletes old file with the same name.
sure you have write permission to the file.

Aborts with error if can't prepare the conditions to use FILE."
  (if (file-exists-p file)
      (condition-case nil
	  (delete-file file)
	(error
	 (error "Cannot delete old file"))))

  ;;  See if the we have permissions to dir to write this new file ?
  ;;
  (if (not (file-writable-p file))
      (error "Can't write to file"))
  )

;;}}}
;;{{{ mouse

;;; ........................................................... &mouse ...


;;; ----------------------------------------------------------------------
;;;
(defsubst get-mouse-point (&optional event)
  "Return mouse's working point. Optional EVENT is a mouse click."
  (if (or mouse-yank-at-point
	  (null event))
      (point)
    (if (xemacs-p)
	(point)
      (sfuncall 'posn-point (sfuncall 'event-start event)))))


;;}}}

;;{{{ special: i-macros for interactive

;;; ......................................................... &imacros ...
;;; You put these macros inside 'interactive'
;;;
;;; (defun test (beg end)
;;;   (interactive (tipgp-i-region-ask-macro))
;;;   ;;  code continues
;;;   )
;;;
;;;

;;; ----------------------------------------------------------------------
;;;
(defsubst i-macro-region-ask (&optional prompt)
  "Macro, usually called from 'interactive' command.
Ask to include whole buffer with PROMPT if region is not selected. If there is
no region given, signal error.

Return:
   '(beg end)"
  (cond
   ((ti::xe-mark-active-p)
    (list (region-beginning) (region-end)))

   ((y-or-n-p
     (or
      prompt
      "Hmmm.. no region selected. Use whole buffer? "))
    (list (point-min) (point-max)))

   (t
    (error "No region.")
    )))


;;; ----------------------------------------------------------------------
;;;
(put 'i-macro-region-body 'lisp-indent-function 0)
(defmacro i-macro-region-body  (&rest body)
  "Macro, usually called from 'interactive' command.
Return selected region and execute BODY. Signal error if
region is not selected/

Return:
  '(beg end BODY-return-value)"
  (`
   (if (null (ti::xe-mark-active-p))
       (error "No region selected.")
     (list
      (region-beginning)
      (region-end)
      (,@ body)
      ))))


;;}}}
;;{{{ FORMS: special

;;; ........................................................... &forms ...

;;; ----------------------------------------------------------------------
;;;
(put 'save-buffer-modified 'lisp-indent-function 0)
(defmacro save-buffer-modified (&rest body)
  "This FORM savext modified state during execution of body.
Suppose buffer is _not_ modified when you do something in the BODY,
eg. set face properties. Unfortunately changing face also signifies
to emacs that buffer has been modified. But the result is that when
BODY finishes; the original buffer modified state is restored."
  (` (let* ((B-modiF (buffer-modified-p))
	    )
       (prog1 (,@ body)
	 (if B-modiF (set-buffer-modified-p t)))
       )))

;;; ----------------------------------------------------------------------
;;; - Instead of writing defvar, you can use 19.35's defcustom
;;;   to let user to run M-x customize RET xxx RET. Example:
;;;
;;; (defcustom xxx-value 6
;;;   "*Integer defining interval between..."
;;;    :type 'integer
;;;    :group 'strokes)
;;;
;;; Originally by By Bill Perry; Email address unknown,
;;; idea modified by [jari]
;;;
(eval-and-compile
  (let* ((list  load-path)
	 dir
	 try
	 path
	 )
  (cond
   ;; ..................................................... no custom ...
   ;;  The reason why newest custom.el does not work in prior releases it the
   ;;  new bacquote macro syntax it uses. It needs new emacs lisp parser to
   ;;  read the macros.
   ;;
   ((or (and (emacs-p)
	     (< emacs-minor-version  34))
	(and (xemacs-p)
	     (or (< emacs-minor-version  14))
	     (not (xemacs-pp))		;19.15, 20.x and up
	     )
	)
    (defmacro defgroup (&rest args) nil)

    (defmacro defcustom (var value doc &rest args)
      (` (defvar (, var) (, value) (, doc))))
    )
   ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. custom maybe . .
   (t
    ;; Explanation: When I say (require 'custom) in -batch byte
    ;; compile; and the load-path HAS my private ~/elisp at from,
    ;; it still loads old custom.elc from XEmacs 19.14 distribution.
    ;;
    ;; Why? Don't know. That's why I want to load it manually here.
    ;;
    ;;
    (while (and (null path)		;Where it is?
		(setq dir (car list)))
      (setq try
	    (if (string-match "/$" dir)
		(concat dir "custom.el")
	      (concat dir "/custom.el")))

      (when (file-exists-p try)
	(setq path (file-name-directory try)))
      (setq list (cdr list))
      )

    ;; ............................................... load new custom ...
    (condition-case ()
	(progn
	  ;;  The order of these loads is very important
	  ;;
	  (if (not (featurep 'widget))
	      (load (concat path "widget.el")))

	  (if (not (featurep 'x-overlay))
	      (load (concat path "x-overlay.el")))

	  (if (not (featurep 'custom))
	      (load (concat path "custom.el")))

	  (if (not (featurep 'cus-face))
	      (load (concat path "cus-face.el")))

	  (if (not (featurep 'wid-edit))
	      (load (concat path "wid-edit.el")))

	  (if (not (featurep 'cus-edit))
	      (load (concat path "cus-edit.el"))) ;load function 'customize'


	  )
      (error nil))

    (cond
     ((and (featurep 'custom) (fboundp 'custom-declare-variable))
      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . case 1 ..
      ;;  19.14 has very old custom.el, that shouldn't be used any more
      ;;
      (unless (function-args-p 'customize)
	(message "\n\
tinylibm.el: ** [Info, The compilation is still going fine...]
	     ** Hm, loading custom didn't go quite right. Reasons:
	     ** a. You have too old custom.el library, because I can't
	     **    see `customize' function to take ONE argument.
	     **    Be sure to have new cus-edit.el too.
	     ** b. Your load-path is set so that the old custom.el
	     **    was laoded.
	     ** 27 May 1997 http://www.dina.kvl.dk/~abraham/custom/"))
      nil
      )
     ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . case 2 ..
     (t
      (message "\n\
tinylibm.el: ** Old custom libs; http://www.dina.kvl.dk/~abraham/custom/
	     ** [Info, The compilation is still going fine...]")
      ;; We have the old custom-library, hack around it!
      ;;
      (defmacro defgroup (&rest args) nil)

      (defmacro defcustom (var value doc &rest args)
	(` (defvar (, var) (, value) (, doc))))
      ))

    ))))


;;; ----------------------------------------------------------------------
;;; so that I can keep the URL link in one place.
;;;
(put 'defgroup-tiny 'lisp-indent-function 3)
(defmacro defgroup-tiny (symbol prefix group &optional doc)
  "Define defcustom.el group for tiny* files.

Input:

  SYMBOL    The package's defgroup name
  PREFIX    Package's variable prefix
  GROUP	    The upper level custom group where SYMBOL belong
	    (eg. extenstions).
  DOC	    Group documentation string."
  (`
   (defgroup (, symbol) nil
     (, doc)

       ;; You can also use (url-link "mailto:jari.aalto@poboxes.com)
       ;;
       :link '(url-link :tag "Update site"
			"ftp://cs.uta.fi/pub/ssjaaa/")
       :prefix (symbol-name (quote (, prefix)))
       :group  (quote (, group))

       ;;  Now define custom contact function when you click link
       :link '(link
	       :tag "Contact maintainer"
	       :func-args (list
			   (symbol-name (quote (, prefix)))
			   (, symbol)
			   )
	     :action    ti::m-defgroup-mail
	     )
       )))

;;; ----------------------------------------------------------------------
;;; This would actually belong to main library, but I want to to keep it
;;; close to defcustom code.
;;;
;;; The following autload tells that function exists (used in function)
;;;
(eval-when-compile
  ;;  For some reason Emacs 19.30 doesn't see :func-args
  ;;  as class parameter if compiled without custom? Hm. Any ideas,
  ;;  how to tell that it is not a variable?
  ;;
  (when (and (not (fboundp 'widget-get))
	     (and (emacs-p)
		  (eq emacs-minor-version  30))
	     )
    (message "\n\
tinylibm.el: ** ignore following byte compiler message if you see it
             ** 'reference to free variable :func-args'")
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-defgroup-mail  (widget &rest ignore)
  "Called from defcustom/defgroup with WIDGET and IGNORE rest args.
Send mail to tiny* package maintainer. Read keyword :func-args
which should hold elements

  '(list PACKAGE-PREFIX PACKAGE-NAME)  ;; nth 0 \"list\" is ignored.

The PACKAGE-PREFIX is in format \"xxx-:\" where a contact function
name `xxx-submit-bug-report' is derived."

  ;; Due to sfuncall, functions must not be in autoload state.
  ;;
  (require 'wid-edit)
  (let* ((args (sfuncall 'widget-get widget :func-args))
	 (arg1 (eval (nth 1 args)))
	 (arg2 (nth 2 args))

	 ;;  from variable pfx "tipgp-:" --> to function prefix "tipgp-"
	 (pfx   (substring arg1 0 (1- (length arg1))))
	 (func  (concat pfx "submit-bug-report"))
	 sym
         )
    (if (setq sym (intern-soft func))
	(call-interactively sym)
      (message "Can't find contact function %s. Load %s.el first."
	       func arg2)
      nil
      )))


;;; ----------------------------------------------------------------------
;;;
(put 'occur-do-macro 'lisp-indent-function 2)
(defmacro occur-do-macro (re &optional hook &rest body)
  "Run occur with RE starting from `point-min' and call HOOK after BODY.

Execute BODY after occur statement in occur buffer.
Run HOOK in occur buffer last; this arg can also be nil if there is no hook."
  (`
   (progn
     (save-excursion			;save user's active point
       (PMIN)
       (occur (, re))
       )
     (pop-to-buffer "*Occur*")
     (,@ body)
     (PMIN)
     (if (, hook)
	 (run-hooks (quote (, hook))))
     )))



;;; ----------------------------------------------------------------------
;;; gnu.emacs.help, William G. Dubuque <wgd@martigny.ai.mit.edu>
;;; #note This already exists in XEmacs, but we have to in ZE 19.15
;;; but we must define it here because it's MACRO and can't
;;; be wrapped inside (nfoundp, since the ByteCompiler doesn't see it in
;;; next
;;; function
;;;
;;;
(defmacro with-output-to-string (&rest body) ;XEmacs has this
  "Please use `exec-to-string'. Execute BODY and return content as string."
  (`
   (save-excursion
     (set-buffer (get-buffer-create " *string-output*"))
     (setq buffer-read-only nil)
     (buffer-disable-undo (current-buffer))
     (erase-buffer)
     (let ((standard-output (current-buffer)))
       (,@ body))
     (buffer-string)
     )))



;;; ----------------------------------------------------------------------
;;; Example: (ti::m-exec-to-string "uname -n")
;;;
(nafboundp 'exec-to-string  "tinylibm"	;XEmacs has this
  (defun exec-to-string (command)
    "Returns shell COMMAND's ouput as string. Tinylibm."
    (with-output-to-string
     (call-process shell-file-name nil t nil "-c" command))))





;;; ----------------------------------------------------------------------
;;; #todo: should use help-mode ?
;;;
(put 'momentary-do-output 'lisp-indent-function 3)
(defmacro momentary-do-output (buffer &optional echo-msg win1 &rest body)
  "Momentarily execute body in buffer.
You normally use this to display messages to user.
Buffer is buried after this form finishes.

The output is accomplished using `with-output-to-temp-buffer', so
you have to use 'princ' to write output.

Input:

  BUFFER	,string
  ECHO-MSG      ,displayed at echo area. If nil, default message is used.
  WIN1		,flag, if non-nil, occupie full window
  BODY		,rest of the lisp code.

Example:

    (momentary-do-output
	\"*notes*\" \"howdy! Press some key\" nil
      (princ \"This is the message\"))"
  (`
   (save-excursion
     (save-window-excursion
       (with-output-to-temp-buffer (, buffer)
	 (,@ body))
       (select-window  (get-buffer-window (, buffer)))
       (if (, win1)
	   (delete-other-windows (get-buffer-window (, buffer))))
       (read-char-safe-until
	(or (, echo-msg) "Press something to delete window."))

       (bury-buffer (, buffer))
       ))))


;;; ----------------------------------------------------------------------
;;; - Sometimes you just want to switch buffer temporarily and
;;;   set point to somewhere else, like scroll the buffer, but
;;;   with save-excursion it's impossible.
;;;
(put 'save-excursion-window 'lisp-indent-hook 0)
(defmacro save-excursion-window (&rest body)
  "Like `save-excursion` BODY, but return to buffer where it took off.
No other values are preserved. Also the `select-window'
is executed if the original buffer had `window-live-p'. (ie. it was visible)

Use this if you want to eg. scroll some buffer."
  (`
   (let* ((oRig-Buf (current-buffer))
	  (oRig-Win (get-buffer-window oRig-Buf))
	  )
     (prog1
	 (progn
	   (,@ body))
       (set-buffer oRig-Buf)		;restore buffer.

       (cond
	((and (windowp oRig-Win)		;no window visible
	      (window-live-p oRig-Win))
	 (select-window oRig-Win)))	;and the visible window
       ))))

;;; ----------------------------------------------------------------------
;;;
(put 'window-loop 'lisp-indent-function 1)
(defmacro window-loop (win-list &rest body)
  "(win-list &rest body) Loop WIN-LIST and do BODY only if window is active.
The variable 'win' hold current window pointer.

Notes:
  WIN-LIST must be _variable_, not anything else.
  WIN-LIST is looped and will be NIL when he loop terminates."
  (`
   (let* (win
	  )
     (list-loop (, win-list) win
       (cond
	((window-live-p win)
	 (,@ body)
	 ))
       )
     )))


;;; ----------------------------------------------------------------------
;;;
(put 'save-with-marker 'lisp-indent-hook 0)
(defmacro save-with-marker (&rest body)
  "Save the line position by using the marker and execute BODY.
Marker is assigned to current position. Caution: If you delete text where the
marker is, there is no way to set the previous point. In this case the
results are undefined.

Call
 (&rest body)

Notes:
  Make sure you don't insert to immediate marker position, because
  markers moves along with the text!"
  (`
   (let* ((MarK (point-marker))
	  )
     (prog1
	 (progn (,@ body))
       (when (marker-position MarK)
	 (goto-char (marker-position MarK)))
      ))
   ))

;;; ----------------------------------------------------------------------
;;;
(put 'save-line-col 'lisp-indent-function 2)
(defmacro save-line-col (fail-form col-form &rest body)
  "Save line and column position.
When you kill/add lines from buffer, you cannot normally save the current
point with `save-excursion', since the point no longer is the
same spot or it may be have been deleted.

This macro saves the position by remembering line and column position.

Call
  (fail-form col-form &rest body)

Error conditions:
  If the line position cannot be preserved, Then FAIL-FORM is evaled: it can
  put the cursor at desired place.

  If column position cannot be preserved COL-FORM is evaled.

Example:
  ;;  1st and 2nd forms act like no-ops after erase buffer command
  (save-line-col nil nil (erase-buffer))

  ;;  1st: Put cursor at the beg. of buffer when failure.
  ;;  2nd: If col is missed, put cursor at beg. of line
  ;;  3rd: The form executed is all the rest of the lines
  ;;
  (save-line-col
   (goto-char (point-min))
   (beginning-of-line)
   (flush-lines \"*\\.txt\")
   )

Return:
  Last value returned by BODY"
  (` (let* ((SLC-sLc-col  (current-column)) ;prevent variable suicide
	    (SLC-sLc-line (LINE))
	    )
       (prog1
	   (progn (,@ body))
	 (goto-line SLC-sLc-line)
	 (move-to-column  SLC-sLc-col)
	 (cond
	  ((not (eq (LINE) SLC-sLc-line))
	   (, fail-form))
	  ((not (eq (current-column) SLC-sLc-col))
	   (, col-form))
	  ))
       )))


;;; ----------------------------------------------------------------------
;; - Example: Use narrow-to-region interactively [select reg first]
;;   in some buffer and then try to run this func with M-x tt.
;;
;;* (defun tt ()
;;*   (interactive)
;;*   (safe-widen
;;*    (PMIN) (y-or-n-p "ok?")
;;*    ))
;;
;;
(put 'safe-widen 'lisp-indent-hook 0)
(defmacro safe-widen (&rest body)
  "(&rest body) Widen buffer end execute BODY.
Preserves possible narrowing when done.

The BODY is not protected against errors or surrounded by `save-excursion'
Return:
  last value of BODY"
  (` (let ((BeG		(point-min-marker))
	   (EnD		(point-max-marker))
	   (EnD-max	(point-max))
	   EnD-wmax
	   ReT
	   )
       (unwind-protect
	   (progn
	     (widen)
	     (setq EnD-wmax (point-max))
	     (setq ReT (progn (,@ body)))
	     )
	 (save-excursion
	   (set-buffer (marker-buffer BeG))
	   ;; what about after widen ? Were we in narrow mode ?
	   (if (not (= EnD-wmax EnD-max))
	       (narrow-to-region BeG EnD))


	   (if (null ReT)		;no-op, Silence XEmacs 19.14 ByteComp.
	       (setq ReT nil))

	   ReT
	   )
	 ))))

;;}}}
;;{{{ misc

;;; ............................................................ &misc ...
;;; Some function that must be defined at the end of file so that
;;; the macros or defsubs get defined first.

(nafboundp 'buffer-substring-no-properties "tinylibm"
  (defun buffer-substring-no-properties (beg end)
    "Implemented in tinylibm.el"
    (remove-properties (buffer-substring beg end))))

;;}}}


(provide   'tinylibm)
(run-hooks 'ti:m-load-hook)

;;; tinylibm.el ends here
