;; @(#) tinyreplace.el -- handy query-replace, area, case preserve, words

;; @(#) $Id: tinyreplace.el,v 1.59 1997/05/31 12:44:10 jaalto NotReleased $
;; @(#) $Keywords: tools $
;;
;; $KnownCompatibility: 19.28 XEmacs 19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 1 1995
;;
;; To get information on this program use ident(1) or do M-x tire-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyreplace|Jari Aalto|jari.aalto@poboxes.com|
;; Query-replace, hilit, block area support, latex, words, case pserve |
;; 31-May-1997|$Revision: 1.59 $|~/misc/tinyreplace.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; - Put this file on your Emacs-Lisp load path, add following into your
;;   ~/.emacs startup file
;; - Code can be easily extracted with tinylib.el / ti::m-pkg-rip-magic
;;
;;* _
;;*     (require 'tinyreplace)
;;
;; Or you can use autoload (preferred) , your emacs starts up faster
;;
;;* _
;;* 	(autoload 'tire-replace-forward		"tinyreplace" t t)
;;* 	(autoload 'tire-replace-region		"tinyreplace" t t)
;;*	(autoload 'tire-replace-over-files	"tinyreplace" t t)
;;*     (autoload 'tire-define-keys-compile-map "tinyreplace" t t)
;;*     (autoload 'tire-replace-over-files-compile-buffer "tinyreplace" t t)
;;*     (add-hook 'compilation-mode-hook 'tire-define-keys-compile-map)
;;
;;
;; You can also use the functions from a echo area menu, like this
;;
;;* _
;;*     (global-unset-key "\C-z")      ;; make key available
;;* 	(global-set-key "\C-z%"        ;; or C-z5 for non-shifted key
;;* 	  '(lambda () "Replace menu" (interactive) (timu-menu 'tire-:menu)))
;;
;; Also make sure you have colors on, otherwise the replaced region
;; may not be visible.
;;
;;_
;;*	(set-face-background 'highlight "blue")
;;
;; If you have any questions, use function
;;
;;	M-x tire-submit-bug-report	,send bug report


;;}}}
;;{{{ Documentation

;; .................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tire-" in front of
;;   every function & variable. It stands for '(ti)ny (re)place
;; - variable names contain letters 'tire-:', excecpt version/hook/mode vars.

;;  Preface, 1995
;;
;;	I saw post in gnu.emacs.help where brianp@ssec.wisc.edu (Brian Paul)
;;	asked for help to replace his C variables: "Suppose I want to
;;	replace all occurances of the variable i in my C program with j."
;;	The normal emacs function query-replace wasn't suitable for this
;;	task because it offered too many false hits. (Guess how many i
;;	characters are used in non-variable context!).
;;
;;	Well later I rembered that one could have used \bi\b to search
;;	words. But the nature of "word" is very different here as it
;;	would have been with \b; it relies on syntax table which you
;;	seldom want to change, whilst the "word definition " here can be
;;	changed on the fly :-/
;;
;;	Things are not that simple always, in fact, the first
;;	implementation of this package had to do with the latex math
;;	equation replace, so that program would automatically skip over
;;	normal text and perform replace within the blocks only. I think
;;	the engine offers lots of other usages too.
;;
;;	I decided to pull out the v1.0 and make it complete package.
;;
;;  Overview of features
;;
;;	o   Companion to emacs's query-replace. Simple interface.
;;	o   Text beeing replaced is highlighted AND terminals that cannot see
;;	    the highlight will see "=>" string marking the line beeing
;;	    processed
;;	o   Preserve case while replacing "FoO" --> "BaR" (symmetry)
;;	o   Toggle case sensitivity and symmetry during the replace.
;;	o   Word match mode on/off during replace: 'matchTHIS or THIS '
;;	o   "Narrow to function", go to "beg of file" when you start
;;	    replacing. You're put back to position where you were when
;;	    you quit.
;;	o   Can replace over many files. (Reads compile buffer output)
;;	    ChecksOut files from RCS when needed (if they are not locked)
;;      o   You can define `tire-exlude-line-regexp' that skips any line
;;          matching looking-at regexp at the beginning of line.
;;
;;  How to use
;;
;;	If you know lisp, you can go and take straight advantage of the
;;	engine function:
;;
;;	    tire-replace-region-1
;;
;;	Normally functions work within area defined by you, but
;;	there is 'applications' section which offers several ready to run
;;	functions for various needs:
;;
;;	    tire-replace-region	    ;like query replace but in selected area
;;	    tire-replace-forward    ;start from current point.
;;
;;	    tire-latex-blk-replace  ;replacer text surrounded by latex BLOCKS
;;	    tire-latex-math-replace ;replace text within latex
;;				    ;math equations only.
;;
;;  What commands do I have while replacing interactively?
;;
;;	There is some handy commands that normal emacs replace lacks
;;
;;	o   toggle case sensitivity during replace
;;	o   toggle symmetry during replace
;;	o   Go to start of buffer _now_ (return back when you exit replace)
;;	o   search backward
;;	o   limited undo
;;	o   Narrow to current function, so that you can replace local variables
;;	o   Flash function name where you're (only for some programming
;;	    languages.)
;;
;;	See function  tire-replace-region-1 for command explanation.
;;	To abort the search, you just press Ctrl-g or 'Q' and  you'll be
;;	returned to the starting point of search.
;;
;;  Command line prompt explanation
;;
;;	The command line prompt will look like this
;;
;;	    Replace 'xx' with 'yy' (a,bvuBFNU? [+CSX] !ynqQ)
;;
;;	Where the flag settings active are displayed between brackets.
;;	+ means that you have used (N)arrow command, C indicates
;;	case sensitivity, S tells that symmetry is activated and
;;	X means that line exclude is oboeyed. For full explanation
;;	of the commands, please press
;;
;;	   ?
;;
;;	Which will print the command summary and explanation momentarily.
;;
;;  Special commands in command line
;;
;;	When you edit the Seach string or destination string, there is
;;	some keys that you can use.
;;
;;	    C-l	    Yank the text under point to current prompt
;;	    C-o	    Yank previous SRING1
;;
;;	The Yank command is C-l not C-y, because if you edit and
;;	kill inside the promt line, you can use regular C-y to yank
;;	text back. The C-l command read space separated text from the
;;	buffer and you most like have to edit the imported text a little.
;;
;;	The C-o command yank first string(the one that is searched)
;;	to the prompt. This is also handy if you used C-l to yank
;;	then you eidited a lot the yanked text and want to move it
;;	to the next prompt. This way you don't have to do the
;;	editing again, but only modify the previous string.
;;
;;	    <	Move buffer's yank point backward
;;	    >	Move buffer's yank point forward
;;
;;	When you select region and you're ready to give replace
;;	arguments; the point in the buffer propably isn't in a suitable
;;	place for C-l yank command. You can move the buffer's yank
;;	point with these command. The text to the left shown to
;;	you briefly tells where the point currenly is.
;;
;;	This feature propably is at its best in acompile buffer where
;;	you have grep results and you draw region around the the files
;;	where you want the replace to happen. Move a little with [<>]
;;	and you will be soon in a line that has the grep word, then yank
;;	it to the replace prompt.
;;
;;
;;
;;
;;  Note about the arrow pointer
;;
;;	Users that do not have highlight capability to see which portion
;;	of text will be replaced will appreciate the arrow at the beginning
;;	of line to show where the text is located.
;;
;;	The option "a" that refreshes the arrow marker is *forced* to ask
;;	a minibuffer question in order to change the state of arrow (hide
;;	or show). I found no other way to do this and I think it's a bug
;;	in 19.28 emacs, because the state is not immediately shown in
;;	buffer.
;;
;;	Drop me a mail if you know how to do the updating without using
;;	the extra read-from-minifuffer "Refresh arrow.." question.
;;
;;  Note about commands
;;
;;	The commands are hard wired in this module and you cannot add
;;	new ones as you can in replace.el which is minor-mode based.
;;	This package is meant to be companion to replace.el, and for
;;	that reason the interface has been designed to be as simple
;;	as possible without any additional modes.
;;
;;	If you need some new command, please mail me direcly and I'll
;;	see what I can do. There is no plans to ever convert this module
;;	to minor mode.


;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	28	1997	[jari]		19.28	v1.58-1.59	NotReleased
;; - defcustom support. Checkdoc 1.29 clear. Libary func changes.
;;
;; May	16	1997	[jari]		19.28	v1.56-1.57	NotReleased
;; - Added new "C" command to compile buffer relace. if you want to replace
;;   over many files without prompting confirmation each time; this is
;;   real handy.
;; - Changed all non standard boolean tests to use when and unless.
;; - corrected tire-:menu.
;;
;; Apr	18	1997	[jari]		19.28	v1.55		NotReleased
;; - Internal changes due to library functions.
;;
;; Mar	22	1997	[jari]		19.28	v1.54		NotReleased
;; - Added new "w" command to match words only. Rewrote overlay code,
;;   now highlighting works in XEmacs too.
;;
;; Mar	18	1997	[jari]		19.28	v1.46-1.53	NotReleased
;; - XE20 compatibility: character handlig has been changed.
;; - Fixed CS and N flag showing in command prompt. Fixed <> command
;;   in compile buffer. Updated instalklation instructions.
;;
;; Mar	5	1997	[jari]		19.28	v1.45		NotReleased
;; - tire-replace-over-files-compile-buffer had serious error: It narrowed
;;   to the beg end area and this prevented seeing the 'cd' directory
;;   ie. contructing the absolute file name. Now it doesn't narrow
;;   any more.
;; - Programmed more commands to the compile buffer replace, now you
;;   have a chance to save & kill the buffer after it has been used
;;   for replace calls.
;;
;; Feb	28	1997	[jari]		19.28	v1.44		NotReleased
;; - overlay library loading in XEmacs cancelled, it doesn't work right.
;;
;; Feb	27	1997	[jari]		19.28	v1.43		NotReleased
;; - Now the highlighting is not used in XEmacs that doesn't support overlays.
;;
;; Feb	23	1997	[jari]		19.28	v1.41.-1.42	NotReleased
;; - Wrote more documentation (command line prompt handling)
;; - Added new user command C-o to yank STRING1.
;; - now the 'replace XX with YY ' won't overflow any more. The
;;   XX and YY are shortened when needed.
;;
;; Feb	22	1997	[jari]		19.28	v1.40		NotReleased
;; - Now asks to go the beginning of buffer in the start of replace if
;;   there is no matches forward.
;; - Compilation relace now open folded/outlined buffers.
;;
;; Jan	27	1997	[jari]		19.28	v1.39		NotReleased
;; - I saw a post where D.G . Christensen said that IBM mainframes had
;;   editor SPF which had capability to 'x-ouot' lines from replacing
;;   and he was wondering if emacs could do the same.
;; - Now that feature is added to this package. See tire-:exlude-line.
;;
;; Jan	15	1997	[jari]		19.28	v1.35-1.38	NotReleased
;; - Added <> commands to find right word for yanking. Now compile
;;   buffer replace function handles dedicated compile frame.
;;
;; Now	3	1996	[jari]		19.28	v1.32-1.34	NotReleased
;; - Rearranged loading. Now uses autoloads. Autoloads are now in 'm' lib
;;
;; Aug	21	1996	[jari]		19.28	v1.31		NotReleased
;; - Corrected all XEmacs 19.14 ByteComp errors. Now this file is
;;   XEmacs compatible.
;;
;; Aug	16	1996	[jari]		19.28	v1.30		NotReleased
;; - Added require tinylibxe, if this file is run in XEmacs. Corrected
;;   slight error in relace-1 function. Now tests arguments before
;;   doing delete-region.
;;
;; Aug	12	1996	[jari]		19.28	v1.29		NotReleased
;; - transient-mark-mode doesn't exist in XEmacs. Remark by
;;   stephene@cogs.susx.ac.uk
;; - Now checks if this mode is present and ignores mode if needed.
;;   tire-:transient-mark-mode		:+
;;   tire-transient-mark-mode		:+
;;   tire-replace-region-1		:! corrected
;;
;; Jun	28	1996	[jari]		19.28	v1.28		NotReleased
;; - Removed some require commands: 'faces 'backquote. XEmacs does not have
;;   feature 'faces. Tip by <samuels@noise.bsd.uchicago.edu>
;;
;; Jun	25	1996	[jari]		19.28	v1.27		NotReleased
;; - Now this package can Use CheckOut to get files from RCS if
;;   you do tire-replace-over-files.
;; - New user command 'U' which is real handy for unfolding the
;;   buffer.
;;   tire-:pre-replace-hook		:+
;;   tire-:tmp-buffer			:+
;;   tire-:err-buffer			:+
;;   tire-:user-function		:+
;;   tire-default-user-function		:+
;;   tire-replace-ask			:!
;;
;; Jun  17	1996    [jari]          19.28   v1.26		NotReleased
;; - No more uses EVENTS, because we can read chars with read-char-safe.
;;
;; Jun  12	1996    [jari]          19.28   v1.25		NotReleased
;; - Added function to do replacing over multiple files.
;;   tire-replace-over-files			:+
;;   tire-replace-over-files-compile-buffer	:+
;;
;; Jun  12	1996    [jari]          19.28   v1.24		NotReleased
;; - Corrected the main routine. now uses the marker to detect the
;;   last position. Cat'd lines should now be handled ok and the bug
;;   in 1.23 should no more exist.
;;
;; Jun  7	1996    [jari]          19.28   v1.23           NotReleased
;; - Bug fixes. The commands "v" + "u" (the undo backward) and "b"
;;   to find original word backward (if you had answered actually
;;   NO for replace).
;; - Now works better.
;; - Known bugs: Replacement does not work right if all lines are
;;   concatenated together without newlines. I should use markers,
;;   not points to detect BEG END are. Will be corrected in next release.
;;
;; May  28	1996    [jari]          19.28   v1.22           NotReleased
;; - Serious erro in main loop corrected. If function was called
;;   by a program it didn't work at all since all the string-from
;;   string-to variables were set in the 'ask' interactive part.
;; - Now works again from programs
;;   tire-replace-1			:!
;;
;; May  24	1996    [jari]          19.28   v1.21           NotReleased
;; - Corrected bug, when trying to replace "%l" with "%li"  in C++ code.
;;   The problem is that "%" character in message. Requires tinylib 1.46
;;
;; May  8	1996    [jari]          19.28   v1.20           NotReleased
;; - Added smarter input engine. Now you can yank the word to the
;;   "Replace" "With" commands and clear the line easily.
;; - Requies new lib 'M' 1.30. No more compilation warnings.
;;   tire-:args-keymap-hook		:+
;;   tire-:args-history			:+
;;   tire-:args-keymap			:+
;;   tire-:replace-buffer		:+
;;   tire-:read-args-function		:+
;;   tire-interactive-region-args	:+
;;   tire-key-clear-input		:+
;;   tire-key-yank-word			:+
;;   tire-args-keymap-create		:+
;;   tire-read-args			:+
;;
;; Apr  19      1996    [jari]          19.28   v1.19           NotReleased
;; - Reverted to previous version 1.19, because I found better way
;;   to cope with XEmacs. There is now new lib tinylibxe.el
;;
;; Apr	18	1996	[jari]		19.28	v1.18		NotReleased
;; - Used general 'O' lib function for overlay handling to make XEmacs
;;   portable code. Still there is overlay-arrow-position, which I don't
;;   know if it exists in XEmacs.
;;
;; Feb	15	1996	[jari]		19.28	v1.17		NotReleased
;; - Added bug reporting functions. Corrected bytecompiler errors.
;;
;; Feb	15	1996	[jari]		19.28	v1.16		NotReleased
;; - Small error corrrecten in "N" command.
;;
;; Feb	15	1996	[jari]		19.28	v1.15		NotReleased
;; - Removed some private variables. Now the event reading is hard-coded.
;;   Also uses read-char-safe-until instead of read-char
;; - The Narrow to function was too loose. Now it's more stricter,
;;   If both bounds are found _only_ then use narrow.
;;   tire-:events-p		:- removed
;;   tire-replace-ask		:! better event reading
;;   tire-replace-1		:! better narrow check
;;
;; Feb	15	1996	[jari]		19.28	v1.14		NotReleased
;; - I got an exellent idea. When replacing something in the C++
;;   buffer I got easily lost..what the function name was I was replacing?
;;   Same happened me to lisp too.
;; - I also wanted the replace happen in function boundary only. It
;;   had been my wish for long time.
;;   "F"			:+ show function name
;;   "N"			:+ narrow to function
;;
;; Feb	14	1996	[jari]		19.28	v1.13		NotReleased
;; - Added new command "B" to go to the logical beginning of file.
;;   Eg. if you start at the middle of buffer, you can hit "B" immediately
;;   to start the replace from beg of buffer.
;;   tire-replace-ask		:! command added
;;   tire-replace-1		:! narrow removed, disallowed PMIN call
;;
;; Feb	10	1996	[jari]		19.28	v1.12		NotReleased
;; - lisp-mnt's lm-verify checked. Feedback rewrote. ByteComp ok.
;;
;; Jan	22	1996	[jari]		19.28	v1.11		NotReleased
;; - Removed the library functions from this file so that the
;;   maintaining is more easy.
;; - New commands added to main function v,b,u. Now user can undo
;;   changes and go backward.
;;   tire-move-overlay		:+ new func
;;   tire-replace-1		:+ new func
;;   tire-replace-ask		:! new commands
;;   tire-replace-region-1	:! new commands
;;
;; Jan	18	1996	[jari]		19.28	v1.10		NotReleased
;; - some functions needed the regexp-quote command, so that words
;;   were found correcly
;;   tire-replace-region	:!
;;   tire-replace-forward	:!
;;
;; Nov	12	1995	[jari]		19.28	v1.9		NotReleased
;; - Added 'Byte compile' section to clarify possible messages.
;;   tire-arrow-control		:! small changes
;;   tire-replace-ask		:! some errors corrected
;;   tire-replace-region-1	:! arrow control corrected
;;
;; Oct	22	1995	[jari]		19.28	v1.8		NotReleased
;; - I wasn't satisfied with the (y/n) choices while replacing the text,
;;   since I wanted to press "!" like in query replace. So why not code
;;   it :-/
;; - In fact the whole user input were reprogrammed, now it gives quite
;;   flexible control over what is happening: changing the case sensitivity
;;   and changin the symmetry while replacing.
;; - I also added the "arrow" for terminals that cannot "see" the replaced
;;   text marked with color.
;;   tire-:events-p		:+ can I use events for reading input ?
;;   tire-:arrow		:+ the arrow
;;   tire-:arrow-initial-state	:+ start showing it or not
;;   BOLP,EOLP ..		:+ from my lib
;;   ti::b-arrow-control	:+ from my lib
;;   ...use-events		:+ and more .. from my lib
;;   tire-arrow-control		:+
;;   tire-replace-ask		:+ User interaction
;;   tire-replace-region-1	:+ reprogrammed
;;
;; Oct	18	1995	[jari]		19.28	v1.7		Release_2
;; - Added section 'What commands do I have...'
;;   var-toggle			:+ lib func
;;   tire-symmetry-toggle	:+ to ease symmetry choices
;;
;; Oct	7	1995	[jari]		19.28	v1.6		NotReleased
;; - Added "rest-case", what to do the the rest of chars it the model
;;   does not tell?
;;   tire-:symmetry-rest	:+ new
;;   ti::s-case-replace		:! to handle rest-case
;;
;; Oct	7	1995	[jari]		19.28	v1.5		NotReleased
;; - I saw a post where one person wanted to preserve the case while
;;   replacing, so that when replacing "foo" with "bar" the "FOOman" would
;;   become "BARman".
;; - Well, I got interested and wrote some more code to accoplish this. I
;;   think this is a wished feature by many of us.
;; - Added 'autoload' note to the installation. Sent as beta to sources group.
;;
;;   tire-:face			:+ user config face
;;   tire-:symmetry		:+ preserve case ?
;;   char-case-p		:+ lib func
;;   ti::s-case-replace		:+ lib func
;;   tire-replace-region-1	:! rewrote basic structure, now narrows
;;
;; Jun	27	1995	[jari]		19.28	v1.4		NotReleased
;; - Corrected code a little, added unwind-protect to ensure the hilit
;;   is removed if user uses c-g to interrupt the replace process.
;; - Removed regexp-quote command, so that tire-replace-forward
;;   could be used with full REs
;;
;; May	20	1995	[jari]		19.28	v1.3		Release_1
;; - Nothing new seems to appear here, so it's time archive this.
;;   Added little more docs.
;;
;; May	20	1995	[jari]		19.28	v1.2		NotReleased.
;; - Added 'how to use' section. Renamed some funcs.
;;
;; May	20	1995	[jari]		19.28	v1.1		NotReleased.
;; - Added variable replace function  tire-var-replace
;;
;; May	1	1995	[jari]		19.28	v1.0		Created
;; - The original engine was posted under name hilit-replace.el, at that
;;   time is wasn't packaged at all, but it was almost operational.

;;}}}

;;; Code:

;;; ......................................................... &require ...

(require 'tinylibm)


(eval-and-compile
  (autoload 'vc-registered "vc")
  )

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyReplace tire-: tools
  "Overview of features

	o   Companion to emacs's query-replace. Simple interface.
	o   Text beeing replaced is highlighted AND terminals that cannot see
	    the highlight will see '=>' string marking the line beeing
	    processed
	o   Preserve case while replacing FoO --> BaR (symmetry)
	o   Toggle case sensitivity and symmetry during the replace.
	o   Word match mode on/off during replace: matchTHIS or THIS
	o   'Narrow to function', go to 'beg of file' when you start
	    replacing. You're put back to position where you were when
	    you quit.
	o   Can replace over many files. (Reads compile buffer output)
	    ChecksOut files from RCS when needed (if they are not locked)
        o   You can define `tire-exlude-line-regexp' that skips any line
            matching looking-at regexp at the beginning of line."
	)

;;{{{ variables

;;; ......................................................... &v-hooks ...

(defcustom tire-load-hook nil
  "*Hook run when file has been loaded."
  :type 'hook :group 'TinyReplace
  )

(defcustom tire-:args-keymap-hook  nil
  "*Hook which can define additional keybindings to tire-:args-keymap."
  :type 'hook :group 'TinyReplace
  )

(defcustom tire-:pre-replace-hook  nil
  "*Hook to run just before replacing starts in a buffer."
  :type 'hook :group 'TinyReplace
  )

;;; ....................................................... &v-private ...

(defvar tire-:replace-region-overlay nil
  "Overlay used to show the replaced region.")

(defvar tire-:transient-mark-mode nil
  "Transient mark mode state (Emacs).")


(defvar tire-:arrow-state nil
  "Arrow display state.")

(defvar tire-:narrow-state nil
  "Narrowed to function state.")


(defvar tire-:args-history  nil
  "History of replace strings.")

(defvar tire-:args-keymap nil
  "Keymap for reading arguments.")

(defvar tire-:replace-buffer  nil
  "Buffer where to replace.")

(defvar tire-:tmp-buffer  "*tire-temp*"
  "Temp buffer.")

(defvar tire-:err-buffer  "*tire-error*"
  "Error message buffer.")

(defvar tire-:read-point  nil
  "This variable is used in interactive word reading.
It tells where the current point is.")


(defvar tire:o-exclude  nil
  "Private. Temprary variable to keep state when calling another function")

(defvar tire-:string1  nil
  "Private. The asked string1. Set in `tire-read-args'")


(defvar tire-:word-match-mode nil
  "Not a user variable. Hold value t if user switch to exact word matching.
Property 're will have the original regexp.")

;;; ........................................................ &v-public ...
;;; User configurable

(defcustom tire-:exlude-line  nil
  "*When search stops to found position, this variable is consulted.

It can be:

  nil	    do nothing special. Proceed replace.
  regexp    line which matches `looking-at' REGEXP at the bol is skipped.
  function  if it returns t, then the line is skipped and search continues.
	    Function takes no arguments and it can move point, since
	    it is run uder save-excursion. Point is at replace point when
	    the function is called.


Example:

  (setq tire-:exlude-line 'my-tire-exclude)
  (defun my-tire-exclude ()
    ;;  Exclude comment lines
    (cond
     ((eq major-mode 'c++-mode)
      (beginning-of-line)
      (looking-at \"^[ \t]*//\")
      )))"
  :type '(string :tag "Regexp") :group 'TinyReplace
  )


(defcustom tire-:arrow "=>"
  "*Line marker where the replace takes effect.
Especially usefull, when term cannot display colors to show the
replacement place."
  :type '(string :tag "Arrow string") :group 'TinyReplace
  )

(defcustom tire-:arrow-initial-state 'show
  "*When replacing interactively, this is the default arrow state.
If your terminal supports highlighting you may want to set this to 'hide.

The only valid values are 'show and 'hide."
  :type '(choice
	  (const show)
	  (const hide)
	  )
  :group 'TinyReplace
  )



(defcustom tire-:face 'highlight
  "*The match area overlay face."
  :type 'face :group 'TinyReplace
  )

(defcustom tire-:word-boundary "[^a-zA-Z0-9_]"
  "*This is complement set of characters forming a word.
For example if you want to replace 'i' with 'j' , you don't want to match

     \"Ignore ThIs match, but replace i with j\"
                                     ^^^
The complement makes sure the word is full word."
  :type '(string :tag "Word complement charset") :group 'TinyReplace
  )

(defcustom tire-:symmetry nil
  "*Non-nil perform replacement using same symmetry.
When replacing text, it may be desireable to have the same symmetry,
the case of the characters, to be preserved while replace takes effect.
Suppose you have text

	FOO Foo fooUx foo

And you want to preserve the symmetry when doing \"foo\" --> \"bar\".
This is what you get:

	BAR Bar barUx bar

If the symmetry is nil, then the normal replace would have given:

	bar bar barUx bar"
  :type 'boolean :group 'TinyReplace
  )

(defcustom tire-:symmetry-rest nil
  "*See ti::s-case-replace's  parameter REST-CASE."
  :type 'boolean :group 'TinyReplace
  )

;; Not in defcustom; advanced feature and expert knows what to to with this.
;;
(defvar tire-:read-args-function  'tire-read-args
  "*Function to ask two arguments ARG1 and ARG2 for replace.
Input
  String
Output
  '(\"arg1\" \"arg2\")

Function must terminate with error if it cannot return list of
two strings.")


(defcustom tire-:user-function 'ti::b-xtra-open-outline
  "*User function fun from command prmpt key 'U'."
  :type 'function :group 'TinyReplace
  )

(defcustom tire-:rcs-co-command  "co -l %s"
  "*Command to take out files from RCS version conrol.
This is used if you replace over many files."
  :type '(string :tag "Shell command") :group 'TinyReplace
  )


;;; ............................................................ &menu ...
;;;  You propably want to copy this to your ~/.emacs and define your
;;;  own key combinations. See tinymenu.el how to use the menu variable
;;;
(defvar tire-:menu
   '("\
replace: (5)fwd (r)egion (c)ompile buffer files (f)iles replace (lL)atex"
     (
      (?5  . ( (call-interactively 'tire-replace-forward)))
      (?%  . ( (call-interactively 'tire-replace-forward)))
      (?r  . ( (call-interactively 'tire-replace-region)))
      (?c  . ( (call-interactively 'tire-replace-over-files-compile-buffer)))
      (?f  . ( (call-interactively 'tire-replace-over-files)))
      (?l  . ( (call-interactively 'tire-latex-blk-replace)))
      (?L  . ( (call-interactively 'tire-latex-math-replace)))
      ))
  "Help for the menuing commands and what the keys do:

5  calls function `tire-replace-forward'
%  calls function `tire-replace-forward'
r  calls function `tire-replace-region'
c  calls function `tire-replace-over-files-compile-buffer'
f  calls function `tire-replace-over-files'

Special commands
l  calls function `tire-latex-blk-replace'
L  calls function `tire-latex-math-replace'")

;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyreplace.el"
 "tire"
 tire-:version-id
 "$Id: tinyreplace.el,v 1.59 1997/05/31 12:44:10 jaalto NotReleased $"
   '(tire-:version-id
     tire-:arrow-state
     tire-:narrow-state
     tire-:arrow-initial-state
     tire-:face
     tire-:word-boundary
     tire-:symmetry
     tire-:symmetry-rest
     )))



;;}}}

;;; ########################################################### &Funcs ###

;;{{{ misc

;;; ----------------------------------------------------------------------
;;;
(defun tire-define-keys-compile-map  ()
  "Define keybindings."
  (interactive)
  (let (sym
	map
	)
    (when (and  (setq sym 'compilation-mode-map)
		(boundp sym)
		(setq map (symbol-value sym))
		(keymapp map))
      (define-key map "%"
	'tire-replace-over-files-compile-buffer))

    (when (and  (setq sym 'compilation-minor-mode-map)
		(boundp sym)
		(setq map (symbol-value sym))
		(keymapp map))
      (define-key map "%"
	'tire-replace-over-files-compile-buffer))


    ))

;;; ----------------------------------------------------------------------
;;;
(defmacro tire-interactive-region-args  (string)
  "Construct interactive tag for functions that need region.
STRING is argument to tire-:read-args-function.

Return:
 '(BEG END ARG1-STRING ARG2-STRING)"
  (`
   (if buffer-read-only
       (barf-if-buffer-read-only)
     (list-merge-elements
      (region-beginning)
      (region-end)
      (funcall tire-:read-args-function (, string))
      ))
   ))


;;; ----------------------------------------------------------------------
;;;
(defun tire-make-word-regexp  (string)
  "See tire-:word-boundary. Make regexp from STRING."
  (concat tire-:word-boundary
	  "\\(" (regexp-quote string) "\\)"
	  tire-:word-boundary
	  ))


;;; ----------------------------------------------------------------------
;;;
(defun tire-read-args (&optional prompt)
  "Read two arguments with PROMPT. Return '(ARG1 ARG2)"
  (let* ((opoint   (point))
	 arg1
	 arg2
	 )
    (setq tire-:replace-buffer	(current-buffer)
	  tire-:read-point	(point)
	  tire-:string1		nil
	  )

    (tire-args-keymap-create)

    (setq arg1
	  (read-from-minibuffer
	   (concat (or prompt "") " Search: ") nil
	   tire-:args-keymap nil tire-:args-history))
    (setq tire-:string1 arg1)		;Now available

    (setq arg2
	  (read-from-minibuffer
	   "Replace with: " nil
	   tire-:args-keymap nil tire-:args-history))
    (goto-char opoint)			;restore
    (list arg1 arg2)
    ))



;;; ----------------------------------------------------------------------
;;;
(defmacro tire-symmetry ()
  "Return current symmetry value."
  (` tire-:symmetry))

;;; ----------------------------------------------------------------------
;;; - This is for user friendliness
;;;
(defun tire-symmetry-toggle (&optional arg verb)
  "Toggle variable` tire-:symmetry' with ARG. VERB."
  (interactive "P")
  (VERB)
  (var-toggle tire-:symmetry arg)

  (if verb
      (message (concat "Symmetry is now " (if tire-:symmetry "on" "off"))))
  )



;;; ----------------------------------------------------------------------
;;;
(defun tire-transient-mark-mode  (mode)
  "Record `transient-mark-mode' if it exists. MODE can be 'write or 'read."
  (when (boundp 'transient-mark-mode)	;XEmacs doesn't have this
    (cond
     ((eq mode 'write)
      (setq tire-:transient-mark-mode
	    (let* ((var 'transient-mark-mode)) ;XEmacs 19.14 byteComp silencer
	      (symbol-value var))))

     ((eq mode 'read)
      tire-:transient-mark-mode
      ))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tire-arrow-control (buffer mode &optional str)
  "Handles showing the arrow.

Input:
  BUFFER	,buffer pointer
  MODE		,symbol 'show, 'hide, 'toggle, 'maybe or 'move.
  STR		,Used for restoring the original contents when mode is 'hide

Sets global:
  `tire-:arrow-state'

Returns:
  mode		,current state"

  (cond
   ((eq mode 'toggle)
    (if (or (null overlay-arrow-position) ;doesn't exist
	    (not (equal (marker-buffer overlay-arrow-position)
			(current-buffer)))
	    )
	(setq mode 'show)
      (setq mode 'hide))
    )

   ((eq mode 'maybe)
    (cond
     ((null tire-:arrow-state)
      (setq mode 'show)
      )
     (t
      (setq mode tire-:arrow-state)	;follow the mode which is active
;;;      (d! ">> hide" mode)
      )))
    )


  (cond
   ((or (eq mode 'show)
	(eq mode 'move))
    (ti::b-arrow-control buffer mode tire-:arrow  (point))
    (setq tire-:arrow-state 'show)
    )
   ((eq mode 'hide)
    (ti::b-arrow-control buffer 'hide str)
    (setq tire-:arrow-state 'hide)
    ))

  mode
  )

;;; ----------------------------------------------------------------------
;;;
(defun tire-replace-ask (buffer from-str to-str )
  "Perform asking while in interactive replace mode.

Input:
 BUFFER FROM-STR TO-STR

Note:
  `tire:o-exclude'    ,must be set in the calling function"
  (let* ((o-exclude tire:o-exclude)
	 (loop      t)
	 msg
	 ans
	 )

    (while loop
      (setq from-str (ti::c-format-string from-str)
	    to-str   (ti::c-format-string to-str))

      (setq
       msg
       (format "Replace '%s' with '%s' (a,bvuBFNU? [%s%s%s%s] !ynqQ) "

	       ;; We want the promt to fit nicely
	       ;;
	       (if (> (length from-str) 18)
		   (concat (str-left from-str 16) "..")
		 from-str)
	       (if (> (length from-str) 18)
		   (concat (str-left to-str 16) "..")
		 to-str)

	       (if tire-:narrow-state "N " "")
	       (if (tire-symmetry)  "S" "")
	       (if case-fold-search "" "C")
	       (if tire-:word-match-mode "W" "")
	       (if tire-:exlude-line "X" "")
	       ))

      (setq ans (read-char-safe-until msg))

      ;;  I'm purposively using dummy COND case.
      (cond
       ((char= ?a ans)
	(tire-arrow-control buffer 'toggle)
	(read-from-minibuffer
	 "Arrow refresh ok.. Press return to update view."))

       ((char= ?s ans)
	(tire-symmetry-toggle))

       ((char= ?w ans)
	(var-toggle tire-:word-match-mode))

       ((char= ?c ans)
	(var-toggle case-fold-search))

       ((char= ?\  ans)
	(setq ans ?y))

       ((char= ?x ans)	;exclude toggle
	(if tire-:exlude-line
	    (setq tire-:exlude-line nil)
	  (setq tire-:exlude-line o-exclude)) ;Dynamically bound in call func
	)

       ((char= ?F ans)
	(tire-show-function-name (point)))

       ((char= ?N ans)
	(setq  tire-:narrow-state t)
	(setq ans ?N))

       ((char= ?U ans)
	(funcall tire-:user-function)
	(setq ans ?U))

       ((char-in-list-case ans '(?\177 ?\b ?n))
	(setq ans ?n))

;;;       ((char-in-list-case ans '(?? ?b ?v ?u ?B)))	;As is, do nothing
       )

      (if (char-in-list-case ans '(?! ?? ?y ?n ?q ?Q ?b ?v ?u  ?B ?N))
	  (setq loop nil))

      )					;while loop

;;;    (d! ans (char= ans ?B))

    ans
    ))


;;; ----------------------------------------------------------------------
;;; Press Ctrl-g to abort replace.
;;;
(defun tire-show-function-name (point)
  "Flashes function name briefly from POINT."
  (let* ((name (ti::m-find-function-name point))
	 (txt  (if name name "<not found>"))
	 )
    (message-quick txt)))



;;; ----------------------------------------------------------------------
;;; Press Ctrl-g to abort replace.
;;;
(defun tire-move-overlay (beg end)
  "Move overlay to BEG END."
  (ti::xe-overlay-move 'tire-:replace-region-overlay beg end  nil)
  (ti::xe-overlay-put  'tire-:replace-region-overlay 'face tire-:face)
  )

;;; ----------------------------------------------------------------------
;;;
(defun tire-replace-1 (beg end str)
  "Replaces region BEG END with STR, point sith after replace."
  (when (and (integerp beg) (integerp end))
    (delete-region beg end) (goto-char beg)
    (insert str)
    ))

;;}}}
;;{{{ key

;;; ----------------------------------------------------------------------
;;;
(defun tire-key-clear-input  ()
  "Clear the line"
  (interactive)
  (delete-region (BOLPP) (EOLPP))
  )

;;; ----------------------------------------------------------------------
;;;
(defun tire-key-forward-word (&optional count)
  "Forward word. COUNT is argument to `forward-word', and defaults to 1.

This function is meant to position you to to right word which you can
then insert into the replace prompt with \\[tire-key-yank-word]. When"
  (interactive)
  (let* ((obuffer (current-buffer))
	 )
    (select-window (get-buffer-window tire-:replace-buffer))
    (forward-word (or count 1))
    (setq tire-:read-point (point))
    (message
     "reading point %d:%s..."
     (point)
     (str-left
      (or (buffer-substring (point) (EOLP))
	  "")
      40))
    (select-window (get-buffer-window  obuffer))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tire-key-backward-word ()
  "See tire-key-forward-word."
  (interactive)
  (tire-key-forward-word -1))


;;; ----------------------------------------------------------------------
;;;
(defun tire-key-yank-string1  ()
  "Yank previous string (search string)"
  (interactive)
  (if tire-:string1
      (insert tire-:string1)
    (message "Sorry, there is no STRING1 to yank yet ")
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tire-key-yank-word ()
  "Yank word from buffer. `tire-:replace-buffer' must be set."
  (interactive)
  (let* (word
	 )
    (save-excursion
      (set-buffer tire-:replace-buffer)
      (goto-char  tire-:read-point)

      ;;  This is just because of the space-word-command
      ;;
      ;;     Word-here
      ;;     *             << advance this cursor one char
      ;;
      (if (char-in-list-case (preceding-char) '(?\ ?\t ?\n ))
	  (forward-char 1))
      (setq word (ti::b-read-space-word)))

    (when word
      (setq word (remove-properties word))
      (insert word))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tire-args-keymap-create  ()
  "Create keymap"
  (setq tire-:args-keymap (copy-keymap minibuffer-local-map))
  (define-key tire-:args-keymap "\C-l"  'tire-key-yank-word)
  (define-key tire-:args-keymap "\C-o"  'tire-key-yank-string1)
  (define-key tire-:args-keymap "\\"    'tire-key-clear-input)
  (define-key tire-:args-keymap "<"     'tire-key-backward-word)
  (define-key tire-:args-keymap ">"     'tire-key-forward-word)
  (run-hooks 'tire-:args-keymap-hook)
  )

;;}}}
;;{{{ main

;;; ----------------------------------------------------------------------
;;; - The "v" has been chosen because it's close to "b". I first
;;;   used "B" for backward REPLACEMENT, but it was too much
;;;   trouble to reach extra shift key.
;;;
;;;   Eg. If I want to replace backward (to undo some changes),
;;;   you just press "v" and "u". Much more awkward would have
;;;   been "B" and "u". The shift-modifier is not good in this case.
;;;
;;; - The "undo" feature here is hand coded, because I couldn't find
;;;   any emacs command that would undo last change...one at a time.
;;;
(defun tire-replace-region-1 (beg end re str &optional level ask func)
  "Perform replace.

Input:

  BEG END   region
  RE	    regexp to search
  STR	    replace string
  LEVEL	    subexpression level to match in RE. Defaul is 0, whole match.
  ASK	    interactive
  FUNC	    Call func

Commands while ASK is non-nil:     (simple undo backward is = 'v u' )

 y or SPACE		,replace
 n or BACKSPACE		,skip
 !			,replace rest
 q                      ,quit
 Q			,quit at current point.

 search flags
 s                      ,toggle symmetry when replacing
 c			,toggle case-fold-search. This is local change
			 and the original value of `case-fold-search'
		         is preserved when function exits.
 w                      ,toggle word only search mode.
 a                      ,toggle arrow displaying
 x                      ,toggle `tire-:exlude-line' between nil and
                         original value

 search control fwd/back
 b			,search backward for MATCH
 v                      ,search backward for REPLACEMENT
 u			,undo -- This is very limited undo and works
                         only when replacing ordinary strings, not
                         regexps. The symmetry must not be used at all.
 movement, others
 B			,go to Beginning of search
                         Min or Max point depending of search direction
 F			,flash function name
 N			,narrow the replace to function.
			 You can't cancel this if you use it. The
			 'N' is indicated in the command line,
 U			,Run user function `tire-:user-function'


 func                   ,search function.

References:
  `tire-:exlude-line'

Returns:
  nbr   ,last replace area position. This is not same as the END
         parameter, because replacing text modifies the buffer's points."
  (let* (
	 (case          case-fold-search) ;save the value
	 (arrow-orig    overlay-arrow-string)
	 (arrow-init    tire-:arrow-initial-state)
	 (symm-rest	tire-:symmetry-rest)
	 (tire-:exlude-line tire-:exlude-line) ;Make local copy
	 (o-exclude     tire-:exlude-line) ;original value

	 (level		(or level 0))	;default value
	 (buffer        (current-buffer))
	 (func          (or func 're-search-forward))
	 (MARK          (point-marker))	;record user position
	 PREV-ME
	 MARK-MAXP			;marker for the last point of search
	 minp maxp			;logical point min, point-max

	 c-exclude			;Current exclude value
	 str-to
	 do-ask
	 quit-point
	 read-string
;;;	 undo-string
	 fmin fmax
	 pos
         mb me                          ;match area, beg end
	 replace
         )

    (setq tire-:word-match-mode nil)	;Init this
    (put 'tire-:word-match-mode 're re)
    (put 'tire-:word-match-mode 'word (tire-make-word-regexp re))


    (setq tire:o-exclude o-exclude)

    (tire-transient-mark-mode 'write)
    (when (fboundp 'transient-mark-mode) ;turn this off for now..
      (sfuncall 'transient-mark-mode 0)) ;; XEmacs compatibility

    (setq tire-:arrow-state arrow-init)	;<< set global, see 'maybe
    (tire-arrow-control buffer 'maybe)
    (setq tire-:narrow-state nil)	;reset


    (cond
     ((eq func 're-search-forward)
      (setq minp (min beg end)		;logical min and max
	    maxp (max beg end)))
     (t
      (setq minp (max beg end)
	    maxp (min beg end))
      ))

    (save-excursion
      (goto-char maxp)
      (setq MARK-MAXP (point-marker)))

;;;    (d! beg end minp maxp re str)
    (goto-char minp)

    (save-excursion
      (unwind-protect
	  (catch 'cancel

;;;	    (d! "maxp" func (< (point) maxp) maxp (marker-position MARK-MAXP))

	    (run-hooks 'tire-:pre-replace-hook)

	    ;;  Peek a little before we start
	    ;;
	    (when (and
		   (not (eq (point) (point-min))) ;Already PMIN ?
		   (save-excursion
		     (if (null (funcall func re nil t))
			 (y-or-n-p "\
Hmm, there is no matches forward. Go to beginning of buffer?"))))
	      (PMIN))

	    (while (and (funcall func re nil t)
			(< (point) (marker-position MARK-MAXP)))


	      (setq mb (match-beginning level)
		    me (match-end level))

              ;; .. .. .. .. .. .. .. .. .. .. .. .. .. excluding line ..
	      (setq c-exclude tire-:exlude-line)

	      (save-excursion
		(cond
		 ((null c-exclude) ;; No exclude patterns
		  t)		     ;; flag "ok"

		 ((and (symbolp c-exclude)
		       (fboundp c-exclude))
		  (if (funcall c-exclude)
		      (setq mb nil))
		  )

		 ((stringp c-exclude)
		  (beginning-of-line)
		  (if (looking-at c-exclude)
		      (setq mb nil))	;Force forgetting this point

		  )))

	      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ HANDLE it ^^^
	      (if (null mb)
		  nil			;submatch error
		(tire-move-overlay mb me)
		;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ user asking ^^^

;;;		  (d! ask)
		(setq read-string (buffer-substring mb me))
		(setq str-to str)

		(if (null ask)
		    (setq replace t)	;automatic

		  (tire-arrow-control buffer 'maybe)
;;;		    (d!  tire-:arrow-state )

		  (setq do-ask t)
		  (setq read-string (buffer-substring mb me))
		  (setq str-to str)


		  (while do-ask
		    (setq replace (tire-replace-ask buffer read-string str))
		    (setq do-ask nil)

		    (cond
		     (tire-:word-match-mode
		      (setq re (get 'tire-:word-match-mode 'word))
		      (setq level 1)
		      )
		     (t
		      (setq re (get 'tire-:word-match-mode 're))
		      (setq level 0)
		      ))

                    ;; ... ... ... ... ... ... ... ... ... ... ... ... ...
		    (cond
		     ((char= ?B replace)
		      (if (eq func 're-search-forward)
			  (progn
			    (PMIN)
			    (setq minp  (point)))
			(PMAX)
			(setq maxp (point))
			)
		      (redraw-display)
		      (setq replace nil)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... . help  ..
		     ((char= ?? replace)
		      (momentary-do-output
			  "*Replace command help*" nil 'win1
			(princ (documentation 'tire-replace-region-1))
			)
		      (setq replace nil)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... narrow  ..
		     ((char= ?N replace)
		      (cond
		       ((eq func 're-search-forward)
			;;  The other call isn't executed if first fails
			;;
			(and (setq fmin (beginning-of-defun-point))
			     (setq fmax (beginning-of-defun-point 'end)))
			(cond
			 ((and fmin fmax  (not (eq fmin fmax)))
			  (setq minp fmin maxp fmax)
			  (goto-char maxp)
			  (setq MARK-MAXP (point-marker))
			  (goto-char minp))
			 (t
			  (setq tire-:narrow-state nil)
			  (message-quick "Can't find narrow bounds.")))
			)
		       (t
			(and (setq fmin (beginning-of-defun-point))
			     (setq fmax (beginning-of-defun-point 'end)))
			(cond
			 ((and fmin fmax  (not (eq fmin fmax)))
			  (setq minp fmax maxp fmin)
			  (goto-char maxp)
			  (setq MARK-MAXP (point-marker))
			  (goto-char minp))
			 (t
			  (setq tire-:narrow-state nil)
			  (message-quick "Can't find narrow bounds.")))
			))
		      (setq replace nil)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... ... ... ..
		     ((char= ?b replace)
		      (save-excursion
			(goto-char mb)

			(setq mb nil me nil)
			(when (re-search-backward re minp t)
			  (setq mb (match-beginning level)
				me (match-end level))
			  ))

		      (if (null mb)
			  (progn
			    (message-quick "No more hits.")
			    (setq replace nil)
			    )
			(goto-char me)
;;;			(setq undo-string (buffer-substring mb me))
			(tire-move-overlay mb me))
		      (setq replace nil  do-ask t)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... ... ... ..
		     ((char= ?v replace)
		      (save-excursion
			(if PREV-ME (goto-char PREV-ME))

			(setq mb nil me nil)
;;;			(setq P (point)   R (regexp-quote str)   M minp)
			(when (re-search-backward (regexp-quote str) minp t)
			  (setq mb (match-beginning level)
				me (match-end level))
			  ))

		      (if (null mb)
			  (progn
			    (message-quick "No previous hit.")
			    (setq replace nil)
			    )
			(goto-char me)
;;;			(setq undo-string (buffer-substring mb me))
			(tire-move-overlay mb me))
		      (setq replace nil  do-ask t)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... ... ... ..
		     ((char= ?u replace)
		      (tire-replace-1
		       mb me
		       (ti::s-case-replace str read-string
					   (tire-symmetry) symm-rest))
		      (tire-move-overlay mb me)
		      (setq replace nil  do-ask t)
		      )

                     ;; ... ... ... ... ... ... ... ... ... ... ... ... ..
		     ((char= ?! replace)
		      (setq   ask nil     replace t))
		     ((char= ?n replace)
		      (setq replace nil))
		     ((char= ?q replace)
		      (throw 'cancel t))
		     ((char= ?Q replace)
		      (setq quit-point mb)
		      (throw 'cancel t))
		     (t
		      (setq replace t)))
		    ))

		;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ do it ^^^
		(when replace
		  (setq PREV-ME me)

		  (tire-replace-1
		   mb me
		   (ti::s-case-replace read-string str-to
				       (tire-symmetry) symm-rest))
		  ));; if null mb
	      ));; cancel - while

	;; ............................ condition
	;; - Clean up if Ctrl-g pressed
	;; - We're done, dehilit and restore possible transient mode

	(if (and tire-:replace-region-overlay
		 (overlay-supported-p))
	    (sfuncall 'overlay-put tire-:replace-region-overlay 'face nil))

	(setq pos maxp)			;update return value

	(tire-arrow-control buffer 'hide arrow-orig) ;remove it

	(if (tire-transient-mark-mode 'read) ;if it were on previously
	    (when (fboundp 'transient-mark-mode)
	      (sfuncall 'transient-mark-mode 1))) ;; XEmacs compatibility

	(setq case-fold-search case)	;restore value
	))

    (if quit-point			;only when interactive "Q"
        (goto-char quit-point)		;handy if you made mistake...
      (goto-char (marker-position MARK)))


    (message "")
    (setq MARK nil  MARK-MAXP nil)	;kill markers

    pos
    ))


;;}}}
;;{{{ applications

;;; #################################################### &applications ###



;;; ----------------------------------------------------------------------
;;;
(defun tire-read-compile-buffer-filename ()
  "Read filename on line."
  (let (dir
	file
	)
    (save-excursion
      (beginning-of-line)
      (when (setq file (bufmatget "^\\([^ \t:]+\\):" 1))
	(save-excursion
	  (if (re-search-backward "cd[ \t]+\\([^ \t\n]+\\)" nil t)
	      (setq dir (read-match 1))))

	(if (and dir (not (string-match "^/" file)))
	    (setq file (concat dir file)))
	))
    file
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tire-replace-over-files-compile-buffer
  (beg end str1 str2 &optional func verb)
  "Read all files forward in buffer that is in compile buffer format.
Perform replace over the found files. Checks Out files that are
RCS controlled if necessary.

Line format:

  /DIR/DIR/FILE: matched text

Input:

  See function `tire-replace-1'
  BEG END STR1 STR2 &OPTIONAL FUNC VERB

References:

  `tire-:rcs-co-command'"
  (interactive
   (list-merge-elements
    (tire-interactive-region-args "compile")
    nil
    t
    ))
  ;; ................................................. interactive end ...
  ;;
  (let ((co-form        tire-:rcs-co-command)
	(o-frame        (current-frame))
	(w-frame        (return-work-frame))

	(func           (or func 'tire-replace-forward))
	(err-buffer     (use-tmp-buffer tire-:err-buffer 'clear))
	(read-only      0)
	no-confirm
	buffer
	cache
	ro-cache			;read only file list
	file
	ch
	)
    (VERB)
    (save-excursion
      (keep-lower-order beg end)
      (goto-char end) (end-of-line) (setq end (point))
      (goto-char beg)

      (while (and (not (eobp))
		  (< (point) end)
		  )
	(setq file (tire-read-compile-buffer-filename))

	;;  See that the file is loaded only once.
	;;  /users/jaalto/elisp/test.el:;; @(#) ...
	;;  /users/jaalto/elisp/test.el:;; $Id: ...
	;;
;;;	(d! (point) file (read-full-line) )



	(cond
	 ((and file (not (file-exists-p file)))
	  (read-char-safe-until
	   (format "[press] file not exist: %s" file)))

	 ((and file (not (member file cache)))
	  (raise-frame (select-frame w-frame))
	  (push file cache)		;Now we have dealt with it

	  ;;  If it's under RCS and not locked, ask if we should
	  ;;  CheckOut it.
	  (when (and (vc-registered file)
		     (not (file-writable-p file))
		     (y-or-n-p (format "Co rcs file: %s" file))
		     )
	    (when (shell-exec-nok-p
		   (shell-execute (format co-form file) err-buffer))
	      (pop-to-buffer err-buffer)
	      ))

	  (cond
	   ((not (file-writable-p file))
	    (inc read-only)
	    (push file ro-cache)
	    )
	   (t
	    (save-excursion
	      ;;  Also jumps to buffer if it's already in Emacs
	      ;;
	      (setq buffer (find-file file))

	      ;;  Open outline/folding before doing anything
	      ;;
	      (ti::b-xtra-open-outline)
	      (PMIN)

	      (cond
	       (no-confirm
		;; Automatic replace
		;;
		(message "Processing: %s" file)
		(replace-string str1 str2)
		(job-buffer buffer (save-buffer))
		)
	       (t
		(funcall func str1 str2)
		;;  What to do after replace
		;;
		(if (and
		     verb
		     (null no-confirm)

		     (buffer-modified-p)
		     (char-in-list-case
		      (setq
		       ch
		       (read-char-safe-until
			(format
			 "%s: SPC s)ave, DEL n)ext, q)save-kill C)ontinue."
			 (file-name-nondirectory file)
			 )))
		      '(?s ?S  ?n ?N ?\b  ?\ ?C ))
		     )
		    (cond
		     ((char-in-list-case ch '(?\ ?s ?S))
		      (job-buffer buffer (save-buffer)))

		     ((char-in-list-case ch '(?n ?N ?\b))
		      nil)

		     ((char-in-list-case ch '(?C))
		      (job-buffer buffer (save-buffer))
		      (setq no-confirm t))

		     ((char-in-list-case ch '(?q ?Q))
		      ;;  Why job-buffer? Well I have automatic
		      ;;  select-buffer programmed to my my mouse movement,
		      ;;  so if I point some other frame, that buffer
		      ;;  gets activated.
		      ;;
		      ;;  In here we want to be sure that the right buffer
		      ;;  "the replace buffer" is touched.
		      ;;
		      ;;
		      (job-buffer buffer (save-buffer))
		      (kill-buffer buffer)
		      )
		     ))
		))

	      ))))
	 )
	(select-frame o-frame)
	(forward-line 1)
	))

    (raise-frame (select-frame o-frame))

    (when verb
      (read-char-safe-until
       (format
	"Handled %s files, %s read-only: %s"
	(length cache) read-only
	(mapconcat 'concat ro-cache " ")
	)))
    nil
    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tire-replace-over-files
  (str1 str2 shell-file-list &optional func verb)
  "Replace over many files.
Input:
  STR1			search string
  STR2			replace with this
  SHELL-FILE-LIST	list of files, like '(\"*c\" \"*h\")
  FUNC			function to do the replace, eg `query-replace'.
  VERB			verbose messages.

References:
  See *tire-tmp* for matching files."
  (interactive
   (let* (files
	  list
	  )

     (setq list (funcall tire-:read-args-function))
     (setq files (ti::f-read-file-list))
     (list
      (nth 0 list)
      (nth 1 list)
      files
      )))
  ;; ................................................. interactive end ...
  ;;
  (let ((verb           (or verb (interactive-p)))
	(func           (or func 'tire-replace-forward))
	(tmp		(use-tmp-buffer tire-:tmp-buffer))
	file-string
	cmd
	)
    (setq shell-file-list (mapcar 'expand-file-name shell-file-list))
    (setq file-string (mapconcat 'concat shell-file-list " "))
    (setq cmd (concat "egrep  '" str1  "' " file-string " /dev/null" ))

    (job-buffer tmp
      (shell-execute cmd tmp)
      (if (empty-buffer-p)
	  (message "No files matches '%s'" file-string)
	(tire-replace-over-files-compile-buffer
	 (point-min) (point-max) str1 str2 func verb)))
    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tire-replace-region (beg end str1 str2)
  "In region BEG END, find STR1 and replace with STR2."
  (interactive (tire-interactive-region-args "region"))
  (tire-replace-region-1 beg end (regexp-quote str1) str2 0 t))



;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tire-replace-forward (str1 str2)
  "Find STR1 and replace with STR2 from current point forward.
See source code of function `tire-args-keymap-create' what keybindings
you can use. Normally C - l yanks, and \"\\\" key deletes line."
  (interactive (funcall tire-:read-args-function))
  (tire-replace-region-1 (point) (point-max) (regexp-quote str1) str2 0 t))







;;; ........................................................... &latex ...

;;; ----------------------------------------------------------------------
;;; ** Not gurranteed to work interactively.
;;;
;;;###autoload
(defun tire-latex-blk-replace (str1 str2 blk &optional beg-re end-re)
  "Select latex block areas for replace.

Input:
 STR1 STR2	Find and replace with.
 BLK		Block delimiter to find
 BEG-RE END-RE	Region bound regexps."
  (interactive "sLatex equation Search: \nsReplace with: \nsBlock names: ")
  (let* ((cp	    (point))                   ;current point
	 (block-re  (concat "\\(" blk "\\)"))
	 (beg-re    (or beg-re (concat "begin{" block-re "}")))
	 (end-re    (or end-re (concat "end{" block-re "}")))
	 MARK
         beg end
         area-end
	 move
	 )
    (save-excursion
      (goto-char (point-max))       (setq MARK (point-marker))
      (goto-char cp)                    ;start from current point
      (while (and (if (null (re-search-forward beg-re nil t))
                      nil
                    (setq beg (point)))
                  (save-excursion
                     (if (null (re-search-forward end-re nil t))
                         nil
                       (goto-char (match-beginning 0))
                       (setq end (point))))
                  (< (point) (marker-position MARK)))
        (setq area-end                  ;leave the block end
              (tire-replace-region-1 beg end (regexp-quote str1) str2 0 t)
              move
              (+ 2 area-end))
        (if (< move (point-max))
            (goto-char move)
          (goto-char (point-max)))
        ))
    ))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tire-latex-math-replace (str1 str2)
  "Find STR1 and replace with STR2 inside latex math blocks."
  (interactive "sLatex equation Search: \nsReplace with:")
  (let ((math-blocks "equation")
        )
    ;;   first $ .. $ blocks
    ;;
    (save-excursion
      (goto-char (point-min))
      ;;   first $ .. $ blocks
      (tire-latex-blk-replace str1 str2 nil "[^\\][$]" "[^\\][$]")
      ;;  then the rest
      (tire-latex-blk-replace str1 str2 math-blocks)
      )))

;;}}}

(add-hook   'compilation-mode-hook 'tire-define-keys-compile-map)

(provide    'tinyreplace)
(run-hooks  'tire-load-hook)

;;; tinyreplace.el ends here
