;; @(#) tinyscroll.el -- Enable or Disable autoscroll for any buffer.

;; @(#) $Id: tinyscroll.el,v 1.24 1997/06/05 05:47:46 jaalto NotReleased $
;; @(#) $Keywords: extensions $
;;
;; $KnownCompatibility: 19.28,33  XEmacs 19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 10 1996
;;
;; To get information on this program use ident(1) or do M-x tisc-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyscroll|Jari Aalto|jari.aalto@poboxes.com|
;; Enable/Disable autoscroll for any buffer. Keeps cursor at bottom smartly! |
;; 05-Jun-1997|$Revision: 1.24 $|~/misc/tinyscroll.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Rip code with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;*   (require 'tinyscroll)
;;
;; or use autolaod; your .emacs loads up a bit quicker. In this package
;; however the above method is preferred, since it automatically
;; mark *compilation* buffer for autoscrolling. Using the autoload
;; puts the scroll in effect only when you add entry to scroll list
;; with M-x tisc-control.
;;
;;* _
;;*   (autoload 'tisc-control                  "tinyscroll" t t)
;;*   (autoload 'tisc-list                     "tinyscroll" t t)
;;*   (autoload 'tisc-timer-process-control    "tinyscroll" t t)
;;*   (eval-after-load "compile" '(require 'tinyscroll))
;;
;; To activate/deactivate scrolling for buffer, call
;;
;;	M-x tisc-control
;;
;; To set default buffers to scroll, change this variable
;;
;;      tisc-:list
;;
;; If you have any questions, use these functions
;;
;;      M-x tisc-submit-bug-report      ,send bug report


;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tisc-" in front of
;;   every function & variable. It stands for '(ti)ny auto (sc)roll'
;; - variable names contain letters 'tisc-:', excecpt version/hook/mode vars.

;;  Preface, May 1996
;;
;;	I was in the middle of testing my new package tinyck.el which
;;	didn't quite work as I wanted, I was loading all the lisp
;;	files to see if it breaks.  Well, I watched the *Message*
;;	buffer to fill with statements
;;
;;	    Loading abbrev...
;;	    Loading abbrev...done
;;	    ...
;;	    Loading rmail...
;;	    loading rmail done...
;;	    ...
;;
;;	But suddendly the emacs died. It kicked me off to the shell
;;	and I had no idea what package was the last one that got loaded.
;;
;;	You see, the *Message* buffer keeps growing, but you have to
;;	tap the pgDown key to get to the end, all the time. Instead
;;	I decided to pull out some lisp to do general autoscrolling for
;;	any buffer, so that I can just sit back and watch the buffer
;;	move. No more guessing in *Message* buffer what was the last
;;	message before Emacs sunk :-)
;;
;;  Overview of features
;;
;;	o   Select buffer, and hit auto scroll on/off. You can scroll any
;;	    buffer.
;;	o   If there are multiple windows for the same buffer, scroll only the
;;	    bottom one. --> you can have "permanent" look window, while
;;	    the buffer scrolls in other window.
;;	o   Smart scrolling: if buffer's point-max doesn't move, it ignores
;;	    scrolling. This way you can browse buffer after there is
;;	    no more output to window.
;;
;;  How to use this package
;;
;;	The scroling here is based on timers, where the lowest interval can
;;	be one 1 second. This means that you don't get smooth
;;	and continuous scrolling, but regular update of the buffer,
;;	which may in rare cases seem jerky.
;;
;;	If this bothers you I advice you to look at my tinyad.el
;;	where you can find automatic compilation autoscroll for
;;	buffers that are written by live processes (shells usually)
;;	Tailor similar function to your other process buffers.
;;
;;	To enable/disable autoscroll for current buffer, use these:
;;
;;	    M-x tisc-control		;to activate scroll
;;	    C-u M-x tisc-control	;to deactivate scroll
;;
;;  Lowest window of the same buffer always scrolls
;;
;;	It is an interesting problem, when you have SAME buffer
;;	in multiple windows, to decide which window to scroll.
;;	I didn't want to scroll all windows, since otherwise I
;;	wouldn't have used two/or more windows for the same buffer.
;;
;;	I decided that the lowest window for the buffer
;;	always scrolls. You can't change that. This was a design
;;	decision and I won't support scrolling middle/upper
;;	buffers. Just arrange your windows so that the scrolling one goes
;;	at the bottom.
;;
;; Scroll update
;;
;;	The scroll update happens only if the buffer has a scroll window
;;	visible. It must also reside in current frame, other
;;	frames are not scrolled.
;;
;;	This again was a design decision, since I don't see
;;	a need to have carrousel running in frames. The working
;;	(or active) frame should suffice.
;;
;;	Happy experimenting!



;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun 5	1997	[jari]		19.28	v1.23-1.24     	NotReleased
;; - Rick Flower <flower@ms31.sp.trw.com> reported missing list-add
;;   function. Corrected. Some Defcustom definition errors too.
;; - defcustom support. Checkdoc 1.42 clear.
;;
;; May 14	1997	[jari]		19.28	v1.20-1.22     	NotReleased
;; - Internal changes due to library functions.
;; - Changed all non standard boolean tests to use when and unless
;; - Compilation adjustments.
;;
;; Jan  15	1997	[jari]		19.28	v1.16-1.18     	NotReleased
;; - Rearranged loading. Now uses autoloads. Text changes,
;;   eval-after-load installation instruction added.
;;
;; Sep  5	1996	[jari]		19.28	v1.14-1.15     	NotReleased
;; - Peter Simons <simons@petium.rhein.de> reported that the run-at-time
;;   dosn't accept "now" parameter in Emacs 19.33. Changed it to "1 sec",
;;   which works for all Emacs releases.
;; - Rearranged some library loading setups for XEmacs.
;;
;; Aug	23	1996	[jari]		19.28	v1.13     	NotReleased
;; - Some small XEmacs 19.14 compatibility modifications. XEmacs ByteCompiler
;;   errors cleared.
;;
;; Jun	28	1996	[jari]		19.28	v1.11-1.12     	NotReleased
;; - Removed all XEmacs dependent code, and moved them to mail lib, under
;;   ti::xe, XEmacs compatibility function class.
;;
;; Jun	28	1996	[jari]		19.28	v1.10		NotReleased
;; - XEmacs doesn't have feature 'timer. Should use some other way to
;;   to do the same in XEmacs. There seems to be itimer package.
;; - I added the emulation to lib 'xe', but can't guarrantee it works.
;; - Now uses feature-load command to load timer package
;;   tisc-get-timer-process-status	:+
;;   tisc-timer-delete			:+
;;   tisc-process			:!
;;
;; Jun	2	1996	[jari]		19.28	v1.9		NotReleased
;; - Changed the intelligency of tisc-timer-process-control function
;;   so that it doesn't rely on the entry in the timer lists. It always
;;   deletes old entry and adds new one, instead of merely
;;   checking it. This should make sure there is _always_ live
;;   timer process running.
;;
;; Jun	2	1996	[jari]		19.28	v1.8		NotReleased
;; - Only textual changes.
;;
;; Jun	2	1996	[jari]		19.28	v1.7		NotReleased
;; - Added debug capability
;; - Small error pointed by Peter Simons <simons@petium.rhein.de>.
;;   The scroll process was not activated when package got loaded,
;;   that meant that the default tisc-:list didn't get activated either.
;; - Solution: added tisc-timer-process-control call to
;;   tisc-:load-hook, which starts the scroll control process in emacs.
;;
;; May	29	1996	[jari]		19.28	v1.6		NotReleased
;; - Bug in tisc-list, didn't recognize the new format of LIST.
;;
;; May	29	1996	[jari]		19.28	v1.5		NotReleased
;; - Added initalizing of auto scroll to *compilation*. This activates
;;   immediately after loading this file.
;;
;; May	25	1996	[jari]		19.28	v1.4		NotReleased
;; - Rewrote the autoscroll access list code. Now this package does
;;   smart scrolling: only when buffer's point moves, it scrolls.
;;   tisc-add-1				:+
;;   tisc-remove-1			:+
;;   tisc-point-max-moved-p		:+
;;   tisc-buffers			:+
;;   tisc-active-buffer-p		:!
;;   tisc-:list-add			:!
;;   tisc-control			:!
;;
;; May	10	1996	[jari]		19.28	v1.2-3		NotReleased
;; - Finished docs and added "submit" functions. Added autoloads.
;;
;; May	10	1996	[jari]		19.28	v1.1		Created



;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-and-compile
  (use-timer)
  (autoload 'adelete				"assoc")
  )

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyScroll tisc-: extensions
    "Enable or Disable autoscroll for any buffer.
  Overview of features

	o   Select buffer, and hit auto scroll on/off. You can scroll any
	    buffer.
	o   If there are multiple windows for the same buffer, scroll only the
	    bottom one. --> you can have \"permanent\" look window, while
	    the buffer scrolls in other window.
	o   Smart scrolling: if buffer's point-max doesn't move, it ignores
	    scrolling. This way you can browse buffer after there is
	    no more output to window.
    ")


;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defcustom tisc-:load-hook nil
  "*Hook that is run when package is loaded."
  :type 'boolean :group 'TinyScroll
  )

;;; ........................................................ &v-public ...


(defcustom tisc-:interval  2
  "*Interval in seconds when srolling process activates.
Must be bigger that 0."
  :type 'integer :group 'TinyScroll
  )


;;; Initalize this in tisc-:load-hook if you want to have some
;;; other default buffers at startup.
;;;
(defcustom tisc-:list
  '(
    ("*compilation*" . 1)		;set this to auto scroll
    ("*grep*"        . 1)
    )
  "*List of buffers  that have autoscroll active.
Format: '((buffer-name-string . max-point) (BN . POINT) ..)"
  :type '(repeat
	  (string :tag "buffer")
	  (integer :tag "point")
	  )
  :group 'TinyScroll
  )


(defcustom tisc-:debug nil
  "*Debug. On or off. Use only is you are going to report error."
  :type 'boolean :group 'TinyScroll
  )


;;; ....................................................... &v-private ...

(defvar tisc-:tmp-buffer  "*autoscroll*"
  "Temporary buffer to display the active auto-scroll buffers.")


;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyscroll.el"
 "tisc"
 tisc-:version-id
 "$Id: tinyscroll.el,v 1.24 1997/06/05 05:47:46 jaalto NotReleased $"
 '(tisc-:version-id
   tisc-:list
   tisc-:tmp-buffer
   tisc-:interval
   tisc-:debug
   )))



;;}}}

;;; ########################################################### &Funcs ###

;;{{{ code: misc

;;; ----------------------------------------------------------------------
;;;
(defmacro tisc-debug (&rest args)
  "Generate debug if debug is on with ARGS."
  (`
   (if tisc-:debug
       (d!! (,@ args)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-active-buffer-p (buffer-name)
  "Check is BUFFER-NAME name is in `tisc-:list'."
  (assoc buffer-name tisc-:list))

;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-add-1 (buffer-name position)
  "Add BUFFER-NAME and last POSITION to scroll list."
  (push (cons buffer-name position) tisc-:list ))

;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-remove-1 (buffer-name)
  "Remove BUFFER-NAME from scroll list."
  (setq tisc-:list (adelete 'tisc-:list buffer-name)))


;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-point-max-moved-p (buffer-name max)
  "Find BUFFER-NAME; return t if MAX is not stored `point-max' for BUFFER-NAME.
Also updates new `point-max' if MAX is different.
If buffer does not exist, do nothing and return nil."
  (let* ((elt (tisc-active-buffer-p buffer-name))
	 )
    (tisc-debug "Max-p check" elt buffer-name max "\n")
    (when (and elt (not (eq (cdr elt) max)))
      (setcdr elt max)
      max
      )))

;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-buffers ()
  "Return list of buffer that have autoscroll on."
  (mapcar 'car tisc-:list))


;;; ----------------------------------------------------------------------
;;;
(defsubst tisc-use-tmp-buffer ()
  "Set up temporary buffer and displays it."
  (use-tmp-buffer tisc-:tmp-buffer 'clear)
  (pop-to-buffer  tisc-:tmp-buffer)
  )


;;; ----------------------------------------------------------------------
;;; if easier to trap "t" error condition.
;;;
(defun tisc-:list-add (buffer-name position &optional remove)
  "Check is BUFFER-NAME name is in 'tisc:-list'.

Input:

  BUFFER-NAME	buffer name string
  POSITION	`point-max' in the buffer
  REMOVE	flag, remove buffer from list

Return:
  nil		,ok
  t		,action not done"
  (let* ((exist (tisc-active-buffer-p buffer-name))
	 ret
	 )
    (cond
     ((or (and remove		(null exist))
	  (and (null remove)	exist))
      (setq ret t))
     (remove
      (tisc-remove-1 buffer-name))
     (t
      (tisc-add-1 buffer-name position)
      ))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tisc-window-list  ()
  "Gets windows that have autscroll enabled.
Return:
   window list or nil"
  (let* ((alist (ti::w-match-buffers (tisc-buffers)))
	 win
	 elt
	 win-list
	 )
    (list-loop alist elt
      (cond
       ((eq 2 (length elt))
	(push  (nth 1 elt)  win-list))
       (t
	(setq win (ti::w-find-bottom (cdr elt)))
	;;  Select first window in the bottom list
	;;
	(push (car win)  win-list)
	)))
    win-list
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tisc-process ()
  "Scroll all window buffers in `tisc-:list'.
ActivateThis process activates itself only when the window, which
should be scrolled, is visible"
  (let ((list  (tisc-window-list))
	)
    (when list				;if we bother to do anything?
      (window-loop list
	(save-excursion-window
	  (set-buffer (window-buffer win))
	  (tisc-debug "PROC" (window-buffer win)
		    (buffer-name) (point-max) "\n")

	  ;;  Scrolling in fact means that, the point-max is
	  ;;  always visible
	  (select-window win)
	  (when (tisc-point-max-moved-p (buffer-name) (point-max))
	    (PMAX)
	    ))
	))
    nil
    ))

;;}}}
;;{{{ code: interactive

;;; ..................................................... &interactive ...



;;; ----------------------------------------------------------------------
;;;
;;;###autolaod
(defun tisc-timer-process-control (&optional delete)
  "Keep the autscroll process and timer process alive.
Optionally DELETE autoscroll process."
  (interactive "P")
  (let* ((key	(ti::xe-timer-elt 'tisc-process))
	 (verb  (interactive-p))
	 )

    ;;  Don't know what user had in mind but...fix it
    ;;
    (if (not (integerp tisc-:interval))
	(setq tisc-:interval 2))

    (cond
     ((and delete key)
      (ti::xe-timer-key-delete key)
      (if verb (message "Autoscroll process deleted."))
      )

     (t
      (cond
       ((null key)			;doesn't exist
	;; this will also restart timer
	;; In Emacs 19.28 - 19.30 , you could pass parameter
	;; "now", bute later emacs releases do not accept it.
	;;
	(run-at-time "1 sec" tisc-:interval 'tisc-process))

       (t				;no timer running
        ;;  Delete old and start it again. This is more safer
        ;;  than just checking if entry is in the list.
        ;;
	(ti::xe-timer-key-delete key)
	(run-at-time "1 sec" tisc-:interval 'tisc-process)
	))
      (if verb (message "Autoscroll process alive."))
      ))

    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autolaod
(defun tisc-list  (&optional print)
  "Show list of active sutoscroll buffers.
Buffers are listed inecho-area if they fit there, otherwise in separate buffer.

If optional PRINT flag is non-nil, always generate report to tmp buffer.
If list if empty, do nothing.

Return:
 t	,report genrated to tmp buffer
 nil	,no report"
  (interactive)
  (let* ((str	(list-to-string (mapcar 'car tisc-:list)))
	 (verb  (interactive-p))
	 ret
	 )
    (if (and (string= str "")  verb)
	(message "no entries in autoscroll list.")
      (setq ret t)
      (cond
       ((and (null print)
	     (< (length str) 80))
	(message str))
       (t
	(tisc-use-tmp-buffer)
	(insert (list-to-string tisc-:list "\n"))
	(setq buffer-read-only t)
	(shrink-window-if-larger-than-buffer)
	)))
    ret
    ))

;;}}}
;;{{{ code: main

;;; ----------------------------------------------------------------------
;;; - It's a bit slow to create buffer comletions this way.
;;;   Anybody has a better suggestion to amulate "bBuffer"
;;;   interactive tag? Mail me if you know...
;;;
;;;   But I couldn't get the on/off information to the prompt
;;;   otherwise.
;;;
;;;###autolaod
(defun tisc-control (buffer-or-pointer &optional off verb)
  "Turn on autoscroll on/off for current buffer.
If this command is called from `tisc-:tmp-buffer' then the current
word in the line is read and offered for default buffer name.

Input:

  BUFFER-OR-POINTER	buffer to scroll
  OFF			flag, prefix arg; is non-nil turn scrolling off
  VERB			flag, allow verbose messages."
  (interactive
   (list
    (completing-read
     (format "Scroll [%s] buffer: " (if current-prefix-arg "off" "on"))
     (list-to-assoc-menu (ti::b-buffer-list "."))
     nil nil

     ;; Default buffer ...
     (if (string= (buffer-name) tisc-:tmp-buffer)
	 (read-full-line)
       (buffer-name))
     )
    current-prefix-arg
    ))

  (let* ((bufferp	(if (bufferp buffer-or-pointer)
			    buffer-or-pointer
			  (get-buffer buffer-or-pointer)))

	 (verb		(or verb (interactive-p)))
	 buffern
	 msg
	 )

    ;;  Check non-interactive errors
    (if (or (null bufferp)
	    (killed-buffer-p bufferp))
	(error "Invalid arg, buffer %s" bufferp))


    (setq buffern (buffer-name bufferp))

    (if off
	(if (tisc-:list-add buffern 0 'remove)
	    (setq msg "Already removed."))
      ;;  Keep the process alive all the time
      ;;
      (tisc-timer-process-control)
      (save-excursion
	;;  We have to record the point-max
	(set-buffer buffern)
	(if (tisc-:list-add buffern (point-max))
	    (setq msg "Already in list."))
	))


    (if verb
	(message msg))
    ))

;;}}}

(tisc-timer-process-control) ;; wake it up !
(provide   'tinyscroll)
(run-hooks 'tisc-:load-hook)

;;; tinyscroll.el ends here
