;; @(#) tinysword.el -- search word under cursor: backward, forward

;; @(#) $Id: tinysword.el,v 1.32 1997/06/26 19:27:00 jaalto NotReleased $
;; @(#) $Keywords: extensions $
;; $KnownCompatibility: 19.28,30 , XE19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $packageinstallre: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C)  1994,95,96,97 Jari Aalto
;; Author:        Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:    Jari Aalto <jari.aalto@poboxes.com>
;; Created:       Nov 7th 1994
;;
;; To get information on this program use ident(1) or M-x tisw-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;;; LCD Archive Entry:

;; tinysword|Jari Aalto|jari.aalto@poboxes.com|
;; Emacs 19, XE19.13+, search word(charset) under cursor: backward, forward|
;; 22-Mar-1997|$Revision: 1.32 $|~/misc/tinysword.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;;; Intallation:
;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;* 	(require 'tinysword)
;;* _
;;
;; or use autoload, and your ~/.emacs loads quicker
;;
;;*   (autoload 'tisw-search-word-forward "tinysword" t t)
;;*   (autoload 'tisw-search-word-backward "tinysword" t t)
;;*   (global-set-key [f2]   'tisw-search-word-forward)
;;*   (global-set-key [S-f2] 'tisw-search-word-backward)
;;* _
;;*   ;;  Non window system; there is no S-f2; so we have to
;;*   ;;  occupy f2 and f3
;;*   ;;
;;*   (global-set-key "\e[12~"   'tisw-search-word-forward)
;;*   (global-set-key "\e[13~"   'tisw-search-word-backward)
;;
;;
;; Following keys are automatically bound when you load this file,
;; The F2 is common search key in Ms-DOS application (eg. WP). In non
;; window system the keys f2 and f3 are defined, because S-f2 does
;; not exist. Some mouse bindings are defined too.
;;
;;	F2	    tisw-search-word forward
;;	Shift F2    -- '' --         backward
;;
;; If you don't want to use these bindings, add following statement
;; prior any other command.
;;
;;	    (setq tisw-:load-hook nil)
;;
;; More fancy word control example at the end of this file.

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tisw-" in front of
;;   every function & variable. It stands for '(ti)ny (s)earch (w)ord
;; - variable names contain letters '-:'.

;;  Preface, 1994
;;
;;	In 7 Nov 1994 <aep@world.std.com> (Andrew E Page) posted
;;	interesting code by article name 'Script Example: Search for next
;;	word', which I took a look. The idea of code was excellent, but it
;;	didn't work as expected at all. Gradually the idea was crystallized
;;	into this el, which now works with emacs 18 and 19.
;;
;;	"Why we need search word package, when in emacs 19 I can do C-s to
;;	enter search mode: C-w C-w C-w to grap words immediately after
;;	point and finally C-s to start searching...?"
;;
;;	Well, people tend to forget, that life was out there when 19.xx
;;	wan't in hands of developers. This package was originally made for
;;	18 and still in 19.xx I don't feel like giving it up. The advantage
;;	of this package is the variable
;;
;;	    tisw-:word-boundary-set,
;;
;;	which you can easily change whenever you need (eg. thru
;;	functions). To do the same in emacs, you have to go and modify the
;;	syntax entries involved...then come back again when you're done. I
;;	never do that, I seldom touch the syntax entries. Besides all
;;	mode-xxx go crazy if I would do so. Now you see the advantage?
;;
;;	And of course I feel more comfortable to do just one keypress,
;;	like F2 to search forward instead of cubersome C-s C-w C-w
;;	C-w...Cs :-')
;;
;;  Description
;;
;;	Grabs word under oint and searches fwd/back. The word is
;;	inserted into emacs's search ring, so that you can later
;;	continue with C-s or with C-r call.
;;
;;  Why doesn't it find my C++ function class::InitClass() ??
;;
;;	User pressed the search function over the call:
;;
;;	    InitClass(;
;;	    i =  i +1;
;;
;;	Now he can't find the function definition... Well,
;;	remember that this searches 'true' words, not part of them. A
;;	word is surrounded by at least one whitespace, since it's not a
;;	word if it is cat'd together with something else. Obviously the
;;	function definition didn't contain any spaces...
;;
;;	The problem is, that if is you say ':' belongs to character set in C++,
;;	[because you propably want to grab variables easily.
;;	including the scope operator 'clss::variable' or '::global'] ,
;;	the find funtion expects to find word in boundary
;;
;;	    nonWordWORDnonWord
;;
;;	And as you can see, the if ':' belongs to word, it can't
;;	simultaneously belong to NonWord !
;;
;;	Summa summarum: Revert to emacs C-s for a moment, since the
;;	word is automatically added to the search buffer.
;;
;;  Word accept function note:
;;
;;	There is variable `tisw-:accept-word-function', which has
;;	default function
;;
;;	    tisw-accept-word
;;
;;	The function's purpose is to check if the searched word is
;;	accepted and that search should be terminated. Currently there it
;;	contains some programming logic for C/C++ languages, so that
;;	certain hits are ignored. Consider following case:
;;
;;	    struct *foo;   - 1
;;	    foo->x;	   - 2
;;	    x->foo	   - 3
;;
;;	    int foo, x;    - 4
;;	    foo = x;	   - 5        * start of 'foo' and 'x' search backward
;;
;;	C/C++ mode, searching for 'foo' finds 4,2,1  -- Not 3
;;	C/C++ mode, searching for 'x'   finds 5,4,3  -- Not 2
;;	But in text-mode, you would find all occurrances.
;;
;;	The added logic to C++ ignores the struct's MEMBER matches so that
;;	you really can find the "main" variables. If you don't like
;;	this added feature, you can alwasy go to
;;
;;	    M-x text-mode
;;
;;	For a while, or if want to permanently switch this feature off,
;;	you set the variable `tisw-:accept-word-function' to nil, which
;;	causes all hits to be accepted.
;;
;;	Needless to say, that you can use put your own checking
;;	function in that variable to control the accurrances better.

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun	26	1997	19.28	1.30-1.31	[jari]		NotReleased
;; - If you searched backward and the cursor landed on clumn 0, which
;;   had space, the search stopped. Corrected. now searching backward
;;   stays in the beginning of word.
;;
;; Jun	5	1997	19.28	1.29		[jari]		NotReleased
;; - shenjw@eng.umd.edu reported that the overlay stayed in the buffer
;;   after search. Corrected.
;; - Removed all possible v18.xx code. checkdoc 1.42 clear. defcustom
;;   is now supported.
;;
;; Mar	22	1997	19.28	1.24-1.28	[jari]		NotReleased
;; - Renamed all hooks: Renamed all varaibles to use -: convention.
;; - Now overlay code works in XEmacs again.
;;
;; Feb	29	1997	19.28	1.23		[jari]		NotReleased
;; - Rearranged code so that we dont't load overlay.el any more
;;   in XE. It doesn't work right.
;;
;; Feb	27	1997	19.28   1.22-1.23	[jari]		NotReleased
;; - text changes. libxe removed and overlays are only used if
;;   they are supported. Now default keys in non window are f2 and f3
;;
;; Sep	25	1996	19.28   1.21	[jari]			NotReleased
;; - Rearranged loading. now uses autoloads as much as possible.
;;
;; Sep	5	1996	19.28   1.20	[jari]			NotReleased
;; - done@nexus.sorostm.ro (Dan Nicolaescu) suggested wrapping the search,
;;   now there is tisw-:wrap-flag variable to toggle that feature.
;;
;; Sep	5	1996	19.28   1.19	[jari]			NotReleased
;; - Rewrote some code and fixed bug in the main loop. Now everything
;;   is auto-installed, no example code in the package any more.
;;
;; Sep	4	1996	19.28   1.18	[jari]			NotReleased
;; - Satish Balay <balay@mcs.anl.gov> Suggested that the when searching
;;   word "a" in C/C++, it should skip over "struct.a" and "struct->a"
;;   variables.
;; - Implemented. Now there is variable tisw-:accept-word-function
;; - Made some rearrangements and minor corrections to tisw-search-word.
;;
;; Aug	29	1996	19.28   1.15-17	[jari]			Release_8
;; - I don't think I support 18.xx more, allthough the code is
;;   there. Added XE 19.14 compatibility.
;; - Cleared all XE byteCompiler errors. Installs key F2 automatically.
;;
;; Apr	12	1996	19.28   1.14	[jari]			Release_7
;; - Added highlight when emacs 19.xx is present
;;
;; Feb	12	1996	19.28   1.13	[jari]			Release_6
;; - Still little errors in var names. Modernazed naming of the variables.
;;
;; Feb	10	1996	19.28   1.12	[jari]			NotReleased
;; - lisp-mnt's lm-verify checked.
;;
;; Jun	30	1995	19.28   1.11 	[jari]			NotReleased
;; - Documentation[more docs] and code layout changed to reflect my
;;   recent packages.
;; - Example section added to the end of file. Rewrote the the engine,
;;   so that it has two separate functions, one is general word
;;   grabber and the other is searcher engine.
;; - Provide moved to end , added tisw-:load-hook.
;;
;; May	6	1995	19.28   v1.8-1.10 [jari]		Release_5
;; - This had been working so well that hadn't looked it for a long
;;   time... When I finally wanted to change tisw-:word-boundary-set
;;   variable so that it would include ':' for class names in C++, I
;;   was suprised? Not working any more? It turned out that I used
;;   defvar for composing tisw-word-boundary, so actually what
;;   happened was, that this package never saw the change....
;; - I changed them into macros, much better now :-/
;;
;; Nov 30	 1994	19.28   v1.7    [jari]			Release_4
;; - Added hooks, added configurable character set, corrected *again*
;;   the search-word function...
;; - The packet polished: variables and functions
;;
;; Nov 26	1994	19.27	v.1.3	[jari]			Release_3
;; - Now supports 19.27 isearch C-s C-r
;; - Now REALLY grabs whole words. The prev versions worked incorrectly.
;;
;; Nov 21	1994	18.57	v1.2	[jari]			Release_2
;; - Added isearch support for 18.57. Now you can use C-s C-r after word has
;;   been grabbed. A good feature suggested by
;;   hochberg@LightStream.COM (Yigal Hochberg).
;;
;; Nov 7	1994	18.57	v1.1	[jari]			Release_1
;; - Andrew sent quite interesting piece of code which unfortunately
;;   had some errors and weaknesses --> it didn't work. I took
;;   a hobby to look at it and make new version. I think this is good
;;   piece of code which teaches various important facts for newcomers...
;; - The code above simplified the searching by using only 1 function
;;   for either direction.
;; - Have fun :-)

;;}}}

;;; Code:

;;{{{ setup: require

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinySword tisw-: extensions
    "search word under cursor: backward, forward.")


;;}}}
;;{{{ hooks

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defcustom tisw-:before-hook nil
  "*Hook that is run at the BEG of search function.
You can set this to point to function that alters the value of
`tisw-:word-boundary-set' eg. by looking at the file type."
  :type 'hook :group 'TinySword
  )


(defcustom tisw-:final-hook nil
  "*Hook that is _always_ run at the END of search function.
It doesn't care about word grabbings or search failures."
  :type 'hook :group 'TinySword
  )

(defcustom tisw-:load-hook 'tisw-default-keybindings
  "*Run when package has been loaded."
  :type 'hook :group 'TinySword
  )


;;}}}
;;{{{ variables

;;; ....................................................... &v-private ...

(defvar tisw-:direction nil
  "Tell direction of search. nil = forward.")

(defvar tisw-:search-status nil
  "Status of word search. t = successful.")

(defvar tisw-:overlay nil
  "Overlay used for highlighting.
Created and killed during program execution.")


;;; ........................................................ &v-public ...
;;; User configurable

(defcustom tisw-:word-boundary-set "-A-Za-z0-9_"
  "*Character set to conform a single word.
You might want to set this to something else before doing search."
  :type 'hook :group 'TinySword
  )


(defcustom tisw-:wrap-flag  nil
  "*Non-nil means wrap buffer if there is no more match."
  :type 'boolean :group 'TinySword
  )


(defcustom tisw-:accept-word-function  'tisw-accept-word
  "*Function run after the search for word has been successful.
If this variable contains non-existing function (like nil), the
content of the variable is ignored.

Default function:
  'tisw-accept-word

Passed args to function:
 string     ,word being searched

Return values of function:
  t	    ,accept search
  nil	    ,do not accept search, continue searching next word."
  :type 'function :group 'TinySword
  )


;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyef.el"
 "tief"
 tisw-:version-id
 "$Id: tinysword.el,v 1.32 1997/06/26 19:27:00 jaalto NotReleased $"
 '(tisw-:version-id

   tisw-:before-hook
   tisw-:final-hook
   tisw-:load-hook

   tisw-:direction
   tisw-:search-status
   tisw-:overlay
   tisw-:word-boundary-set
   tisw-:wrap-flag
   )))

;;}}}

;;; ########################################################### &Funcs ###

;;{{{ 19.xx isearch add

;;; ----------------------------------------------------------------------
;;;
(defun tisw-add-to-ring-isearch19 (isearch-string)
  "Add search pattern to ISEARCH-RING in Emacs 19.xx.
This code is directly taken from function `isearch-done' By Daniel LaLiberte."
  (if (> (length isearch-string) 0)
      ;; Update the ring data.
      (if isearch-regexp
          (if (or (null regexp-search-ring)
                  (not (string= isearch-string (car regexp-search-ring))))
              (progn
                (setq regexp-search-ring
                      (cons isearch-string regexp-search-ring))
                (if (> (length regexp-search-ring) regexp-search-ring-max)
                    (setcdr (nthcdr (1- search-ring-max) regexp-search-ring)
                            nil))))
        (if (or (null search-ring)
                (not (string= isearch-string (car search-ring))))
            (progn
              (setq search-ring (cons isearch-string search-ring))
              (if (> (length search-ring) search-ring-max)
                  (setcdr (nthcdr (1- search-ring-max) search-ring) nil))))))
  )

;;}}}
;;{{{ main

;;; ----------------------------------------------------------------------
;;;
(defun tisw-default-keybindings  ()
  "Install default key bindings for searching."
  (interactive)

  (tinyck-maybe-activate 'xemacs)

  (global-set-key [(f2)]   'tisw-search-word-forward)
  (global-set-key [(shift f2)] 'tisw-search-word-backward)

  ;;  Non window system; there is no S-f2; so we have to
  ;;  occupy f2 and f3
  ;;
  (global-set-key "\e[12~"   'tisw-search-word-forward)
  (global-set-key "\e[13~"   'tisw-search-word-backward)

  ;; - The following code searches words fwb/back by clicking mouse
  ;;   this and many more examples are presented in tinymouse.el

  (global-set-key [(control meta button2)]       'tisw-search-word-forward)
  (global-set-key [(control meta shift button2)] 'tisw-search-word-backward)

  (tinyck-maybe-activate 'xemacs 'disable)
  )



;;; ----------------------------------------------------------------------
;;;
(defun tisw-accept-word  (word)
  "Determine if we accept searched WORD."
  (let* ((type      (symbol-name major-mode))
	 (ret	    t)			;default, accept search
	 space-word
         )
    (cond
     ((string-match "^c-\\|^cc-\\|c[+]+" type)
      ;; Check C/C++ dependent variables, where rg. 'a' is
      ;; searched
      ;; a = 1			, begin search with 'a'
      ;; a = a + 1		, accepted hit
      ;; struct->a		, not accepted hit, continue search
      ;;
      (setq space-word (save-excursion (or (ti::b-read-space-word) "")))
      (if (string-match (concat "\\(->\\|[.]\\)" (regexp-quote word))
			space-word)
	  (setq ret nil))		;discard this one.
      ))

;;;    (d! ret space-word word)

    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tisw-grab-word (&optional charset beg end )
  "Gets word under cursor limited by CHARSET string.
Optional BEG and END gives maximum search limits.
Default boundary is line limit."
  (let* (
	 re-word-boundary
	 re-word
	 ;;  We accept ':' and '-' , beasuse they are used in c++ and lisp
	 (charset (or charset "-:A-Za-z0-9_"))
	 (beg (or beg (BOLP)))
	 (end (or end (EOLP)))
	 pb pe p re
	 ret
	 )
    (setq re-word-boundary  (concat  "[^" charset "]"))
    (setq re-word (concat  "[" charset "]"))	;considered single word

    ;; Note:  the first search goes backwards to find the start of the
    ;;        word, which is one character in front of the character
    ;;        found by the search.  Then we go forward to the end of
    ;;        word which is one character behind the character found by the
    ;;        search.

    (save-excursion			;conceive original (point)
      (if (re-search-backward re-word-boundary beg t)
	  (setq pb (1+ (point))))
      (if pb nil			;already found
	(setq p (point))
	(beginning-of-line)
	(if (eq p (point))		;we were at the BEG
	    (setq re re-word)
	  (setq re (concat re-word "+"))) ;skip chars
	(if (re-search-forward re (1+ p) t)	; word at the BEG
	    (setq pb beg)))
      )

;;;    (d! re-word-boundary pb)

    ;;  Then search end point
    (save-excursion
      (if (re-search-forward re-word-boundary end t)
	  (setq pe (1- (point))))
      (if pe nil			;already found
	(if (looking-at (concat re-word "+$"))  ; handle word at the END of ln
	    (setq pe end)))
      )
    (if (and pb pe)
	(setq ret (buffer-substring pb pe)))

    ret					;easier to debug this way
    ))



;;; ----------------------------------------------------------------------
;;; - There is lot of re-search-backward/fwd commands and it is intentional,
;;;   so that the code is totally emacs version independent. Newer emacs
;;;   has nice functions that shrink this code to 10 lines :->
;;; - Never grab word function is already coded in tinylib.el
;;;
(defun tisw-search-word-main (&optional backward set)
  "Gets word under cursor and search next occurrence.
If BACKWARD is non-nil, the search will be headed backward, the SET
corresponds to `tisw-:word-boundary-set'.

Before searching is done the tisw-hooks is thrown. This is useful
is you want someone to dynamically change the search-word's idea of
the chars belonging to word. By setting `tisw-:word-boundary-set' you
can set different sets for text and lisp.  [In lisp the '-' is part of
word while in text it normally isn't].

NOTE:
   You cannot search 1 char words with this due to internal
   behaviour of search method and cursor positioning."
  (interactive "P")
  (let ((wrap   tisw-:wrap-flag)
	(loop   0)
	(accept t)
	charset
	word found
	re-word-boundary  re-word
	prev-point
	no-msg
	mb me
	)
    (or tisw-:overlay
	(setq tisw-:overlay (ti::xe-overlay-some)))
    ;; ................................................... set charset ...

    (setq tisw-:direction backward)	;inform possible hook func

    ;;   Let the user set the word criteria
    (if tisw-:before-hook
	(run-hooks 'tisw-:before-hook))

    (setq charset (or set tisw-:word-boundary-set))
    (setq re-word-boundary  (concat  "[^" charset "]"))
    (setq re-word (concat  "[" charset "]"))	;considered single word

    ;; ...................................................... set word ...

    (setq word (tisw-grab-word charset))

    (if (null word)
	(message "Word not grabbed.")

      ;;   enable C-s and C-r to use the word, look isearch.el
      ;;   NOTE: this doesn't put the WORD regexp there...
      ;;
      (tisw-add-to-ring-isearch19 word)

      ;; post a message saying what we're looking for
      (message "searching for \`%s\`" word)
      (setq no-msg (concat "No more words: [" word "]" ))

      (setq re-word
	    (concat
	     "\\(^" word  "\\|"
	     re-word-boundary word "\\)" re-word-boundary
	     ))

      ;; ................................................... do search ...
      ;;
      (while loop
	;;  Record the point only if the word is accepted.
	;;
	(if accept
	    (setq prev-point (point)))

	(if backward			;choose backward
	    (progn
	      (setq found (re-search-backward re-word nil t))
	      (if (null found)
		  (message no-msg)
		(save-match-data	;highlight needs orig region
		  (re-search-forward re-word-boundary)) ;Goto first char
		))
	  ;;  - This a little hard to explain: the search
	  ;;    does not succeed, if the variable 'a' is at
	  ;;    the beginning of line due to backward-char 2 correction
	  ;;
	  (if (eq (current-column) 0)
	      (safe-do (forward-char 1)))

	  (setq found (re-search-forward re-word nil t))
	  (if found (backward-char 2))
	  )

;;;	(d! re-word loop (point))

	(cond
	 ((null found)
	  (message no-msg)
	  )
	 (t
	  ;;  - So that NEXT word will be grabbed, that's why 1 char words
	  ;;    can't be found
	  (setq mb (match-beginning 0)   me (match-end 0) )
	  ))

	;; ........................................................ done ...
	(setq tisw-:search-status found)		;save status


	;;  Should we continue searching ?
	;;
	(cond
	 ((and (null found)
	       wrap
	       )
	  (if (> loop 0)
	      (setq loop nil)		;No hits at all
	    (if backward		;start a new round
		(PMAX) (PMIN)))
	  )

	 ((and (null found)
	       (> loop 0))
	  ;;  Word accept function caused loop to run again, but
	  ;;  there were no more hits. Back to prev position
	  ;;
;;;	  (d! loop found prev-point)
	  (goto-char prev-point)
	  (setq loop nil)
	  )

	 ((or (null found)
	      (not (fboundp tisw-:accept-word-function)))
	  (setq loop nil))

	 ((and found
	       ;;  Is this found word accepted in the context
	       ;;  surrounding the text ?
	       (setq accept (funcall tisw-:accept-word-function word)))
	  (setq loop nil)
	  ;;  Restore prvious search point
	  ))

	;; .................................................... do hilit ...

	(if (and tisw-:overlay found (null loop))
	    (ti::xe-overlay-move tisw-:overlay  mb me nil 'highlight))

	(when tisw-:overlay		;Hide overlay
	  (sit-for 1)
	  (ti::xe-overlay-move tisw-:overlay 1 1))

	(if loop
	    (inc loop))

	))

    ;; ---------------------- grabbed
    (if tisw-:final-hook
	(run-hooks 'tisw-:final-hook))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tisw-charset-control ()
  "Dynamic character set change according to mode. This is example function."
  (let* ((type (symbol-name major-mode))
	 set
	 )
    (cond
     ((string-match  "^c-\\|^cc-\\|c[+]+" type)
      (setq set "A-Za-z0-9_"))

     ((string-match "lisp" type)
      ;;  Add ':' , which I use in variable names.
      (setq set "-:A-Za-z0-9_"))
     ((string-match "text\\|shell\\|perl" type)
      (setq set "A-Za-z0-9_")
      ))
    set
    ))

;;; ----------------------------------------------------------------------
;;; - Small functions to bind to keys
;;;
(defun tisw-search-word-forward ()
  (interactive)
  (tisw-search-word-main nil (tisw-charset-control)))

(defun tisw-search-word-backward ()
  (interactive)
  (tisw-search-word-main 'back (tisw-charset-control)))


;;}}}

(provide   'tinysword)
(run-hooks 'tisw-:load-hook)


;;; tinysword.el ends here
