;; @(#) tinytab.el -- Programmers TAB minor mode. Very flexible.

;; @(#) $Id: tinytab.el,v 1.40 1997/06/19 10:16:00 jaalto NotReleased $
;; @(#) $Keywords: tools $
;;
;; $KnownCompatibility: 19.28, XEmacs19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Oct 26 1995
;;
;; To get information on this program use ident(1) or do M-x tit-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinytab.el|Jari Aalto|jari.aalto@poboxes.com|
;; TAB handling minor mode, eg. step 4 chars bck/fwd , insert/del 4 spaces |
;; 19-Jun-1997|$Revision: 1.40 $|~/misc/tinytab.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;   Put this file on your Emacs-Lisp load path, add following into your
;;   ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;     (require 'tinytab)
;;
;;   or use this; your .emacs loads up a bit quicker
;;
;;* _
;;*  (autoload 'tit-mode		"tinytab" t t)
;;*  (autoload 'tit-return-key-toggle "tinytab" t t)
;;
;;   You also need keybingins to use this module
;;
;;* _
;;*  (global-set-key "\C-c\t"        'tit-mode)   ;; Non-X
;;*  (global-set-key [C-S-backtab]   'tit-mode)   ;; HP X
;;*  (global-set-key "\C-c\C-m"      'tit-return-key-toggle)
;;* _
;;
;; - It load this file only when you turn the mode on for the first time.
;;
;; - Here is how you use your ow setup
;;
;;*  (add-hook 'tit-:load-hook 'my-tit-:load-hook)
;;* _
;;*  (defun my-tit-:load-hook ()
;;*    "Add some things after install"
;;*    (add-hook 'tit-mode-define-keys-hook 'my-tit-keys))
;;* _
;;*  (defun my-tit-keys ()
;;*    "My tinytab key additions, override settings."
;;*    ;; ... code here ...
;;*    )
;;* _
;;
;; - If you have any questions, use this functions
;;
;;	M-x tit-submit-bug-report	,send bug report


;;}}}

;;{{{ Documentation


;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tit-" in front of
;;   every function & variable. It stands for '(ti)ny (t)ab minor mode.
;; - variable names contain letters 'tit-:', excecpt hook/mode vars.

;; Preface, oct 1995
;;
;;	I saw a post in gnu.emacs.sources (what an source of
;;	inspiration!), where someone asked:
;;
;;	    "Is there anyway to reset the number of spaces that TAB does?
;;	     Like, I want to make it jump four spaces instead of the
;;	     usual whatever.How can I set the tabs to 4?
;;	    "
;;
;;	And the typical answer was:
;;
;;	    "In .emacs, set the variable tab-stop-list, like so:
;;	     (setq tab-stop-list (list 4 8 12 ...))
;;	    "
;;
;;	Well, I didn't want to touch the original tab-stop-list, since I
;;	want to have 8 tabs normally. But I like to have 4 tabs when I'm
;;	doing my shell programming or whatever text formatting, I though it
;;	was time to turn on my think-and-create-mode.
;;
;;	The goal was very simple: write a minor mode, that you can turn on
;;	and off, which handles _only_ tab key. That shouldn't be hard at
;;	all.
;;
;;	Yes, If someone of you know, I have also written tinyindent.el,
;;	which is tab-alike mode too. But there is a subtle, but very
;;	important difference, it behaves according to previous lines, so
;;	its job is to "line up" with the previous text(code).
;;
;;	Instead, this mode was supposed to be plain rigid. The tab goes
;;	where I want it, and I can control the amount of movement to either
;;	direction, back or forward. Hm. Maybe I should merge these two
;;	packages in long run?
;;
;;  Overview of features
;;
;;	o   Programmable TAB. If you set the spacing(div factor) to 4,
;;	    you can virtually program "blindly" without any other modes.
;;	o   moving commands: tab-forward, tab-backward
;;	o   modify commands: tab-insert, tab-delete
;;	o   indent commands: indent-by-div-factor, forward , backward
;;	o   Positioning braces { } supported.
;;
;;	Extras
;;
;;	o   special auto-indent function offered for return key,
;;	    switch it on, and you can continue your shell,awk,perl comments...
;;	o   Press C-c TAB and your in constant indentation mode where
;;	    keys "wq" "as" "zx" indent region <--> by 1,2 and 4 until
;;	    esc pressed.
;;
;;  What this package does?
;;
;;	It mimics `tab-stop-list' with minor mode if some analogy can be
;;	drawn. You only set one variable, that controls the amount of
;;	movement, whereas you would have to put many values inside
;;	`tab-stop-list'. The variable for tab widths is:
;;
;;	    tit-:div-factor-table
;;
;;	The advantage is, than you don't have to alter the `tab-stop-list'
;;	at all. When the mode is off, the tabs are back again as they were
;;	(or as the other mode thinks how the tab should be)
;;
;;	But what I like most, is the movement and deletion functions, that
;;	really make day shine when you program. They all operate according
;;	to the variable `tit-:div-factor'. Try out the functions
;;
;;	    tit-indent-by-div-factor
;;	    tit-indent-by-div-factor-back
;;
;;	to format region of code forward and backward easily and you know
;;	what I mean. (this is different than using command indent-region)
;;	When you delete backward "one indentation level", you can do it
;;	when the division factor is 2/4/8.
;;
;;	The nearest column that satisfies the division factor is used when
;;	"indenting back". See the example:
;;
;;      Before:
;;                *		<< cursor here
;;      123 567 9012		<< columns, starting from 1
;;
;;      After:
;;             *		<< cursor here, at div-factor compliant point
;;      123 567 9012		<< columns, starting from 1
;;
;;	Don't forget to look at function
;;
;;	    tit-change-tab-width
;;
;;	Which allows you to switch between 2, 4, 8, you name it,
;;	tabs widths with one key press.
;;
;;  Major modes and this minor mode
;;
;;	When you use some programming mode, say C++ mode, it usually
;;	provides function to indent the line right when you press tab key.
;;	If you then turn on this mode, you loose the mode specific indenting,
;;	because turning on minor mode overrides the underlying major mode
;;	bindings.
;;
;;	I have included one simple function to deal with major modes: it
;;	preserves the original indenting style in some extent, so that you
;;	can use this minor mode and current major mode together. In
;;	variable `tit-:tab-insert-hook' there is function
;;	`tit-tab-mode-control' which looks at variable
;;
;;	    tit-:mode-table
;;
;;	and if the mode is listed in the table _and_ current point is at
;;	the beginning of line, then the line is handled by original major
;;	mode and not by this minor mode. This allows you to indent the
;;	line initially and then use normal tab indent within the current
;;	line according to this minor mode. I mostly use this tab mode to
;;	adjust variable placement inside code.
;;
;;	However, this minor mode is normally meant to be used as turn
;;	on/off basis in such programming modes that indent lines when you
;;	pressing tab key. Current compatibility function
;;	`tit-tab-mode-control' only allows you to get some flexibility when
;;	this mode is temporarily on.
;;
;;	Bind this mode to some fast key which you can use to toggle this
;;	mode on/off when you need tab for a moment in programming modes.
;;	If you don't want any support to major modes, put following into
;;	your ~/.emacs
;;
;;	    (setq tit-:mode-table nil)
;;
;;  Return key addition
;;
;;	In this package there is also this little function which will
;;	make itself a must in no time:
;;
;;	    tit-return-key-toggle   ;; I have bound this to C-c C-m
;;
;;	When the function is active, you can continue the ndentation from
;;	current position.
;;
;;				// Comment here. Call C-c C-m...and press RET
;;				// And it automatically indents here.
;;
;;	See variable
;;
;;	    tit-:auto-indent-regexp
;;
;;	what line prefixes are "copied" along with the indented spaces.
;;
;;  Development note
;;
;;	This package solely concerns TAB key. Nothing outside of it is
;;	counted. I have no intention to make this a full fledged
;;	programming mode, since there is `sh-mode', `ksh-mode', `perl-mode'
;;	and many more for other languages. I almost added
;;
;;	    (make-variable-buffer-local 'tit-:div-factor)
;;	    (make-variable-buffer-local 'tit-:div-factor-table)
;;
;;	But after thinking I decided not to add it to this package, since
;;	most of the time user is satisfied with the tab "4", at least
;;	that's what I use mostly. And changing the width usually means
;;	a little flick of my finger to reach out the function
;;
;;	    tit-change-tab-width
;;
;;	In very rare occasions one wants different tab widths in different
;;	buffers and in those cases you can add the lisp commands
;;	`make-variable-buffer-local' to mode hooks or after `tit-:load-hook'.

;;}}}
;;{{{ history


;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun	19	1997	[jari]		19.28	v1.38-1.40	NotReleased
;; - Checkdoc 1.42 clear. defcustom support.
;;
;; May	19	1997	[jari]		19.28	v1.34-1.37	NotReleased
;; - Internal changes.
;; - Changed all non standard boolean tests to use when and unless
;; - Corrected Mode install. There is no prefix key and no Menu.
;;
;; Mar	14	1997	[jari]		19.28	v1.33		NotReleased
;; - Rewrote modes to use fmacro llibrary functions. Also renamed all
;;   hooks and variables to use -: convention.
;;
;; Feb	27	1997	[jari]		19.28	v1.32		NotReleased
;; - rearranged library loading
;;
;; Feb	11	1997	[jari]		19.28	v1.28-1.31	NotReleased
;; - Changed the default keybindings a little. Added 2 more kp-tab bindings.
;; - Added mode specific tab control
;;   tit-:mode-table			:+ new user variable
;;   tit-tab-mode-control		:+
;;
;; Aug	25	1996	[jari]		19.28	v1.27		NotReleased
;; - Rearranged loading and cleared all XEmacs byteComp errors. Now uses
;;   autoloads as much as possible.
;;
;; Aug	20	1996	[jari]		19.28	v1.23-26	NotReleased
;; - Changed tit-indent-region-dynamically function to use user
;;   configurable keys in tit-:indent-region-key-list.
;; - Corrected C-h m tit-mode keymap display error.
;;
;; Aug	16	1996	[jari]		19.28	v1.22		NotReleased
;; - Rewrote the keymap define-key commands to work with XEmacs 19.14.
;; - Added new function tit-indent-region-dynamically to allow easy
;;   region adjusting.
;;
;; May	25	1996	[jari]		19.28	v1.21		NotReleased
;; - XEmacs 19.14 compatibility: used ti::k-add-minor-mode function.
;;
;; May	25	1996	[jari]		19.28	v1.20		NotReleased
;; - One extra defconst --> defvar.
;;
;; May	9	1996	[jari]		19.28	v1.17-19	NotReleased
;; - Got idea that I needed also handy indet functionality, the I went
;;   and rewrote the installation too.
;;   tit-:key-func			:-
;;   tit-mode-define-keys-hook		:+
;;   tit-indent-by-div-factor		:+
;;   tit-indent-by-div-factor-back	:+
;;   tit-tab-forward			:! bug corrected
;;   tit-mode-define-keys		:! rewrote
;;   tit-install			:! runs key hook now
;;
;; May	6	1996	[jari]		19.28	v1.16		NotReleased
;; - Rewrote the TAB insert handling. Now it relies on the
;;   tit-:tab-insert-hook functions, where is new tit-tab-brace-control
;;   function to line up braces.
;;   tit-tab-brace-control		:+
;;   tit-tab-key			:+
;;   tit-tab-del-key			:+
;;
;; May	3	1996	[jari]		19.28	v1.15		NotReleased
;; - Changed modeline update command, because XEmacs labels it as obsolete.
;; - Rewrote the tit-change-tab-width, which used to be tit-4-8-toggle.
;;   tit-:div-factor-table		:+
;;   tit-install			:+ New install/deinstall
;;   tit-4-8-toggle			-> tit-change-tab-width
;;
;; Apr	2	1996	[jari]		19.28	v1.13-1.14	NotReleased
;; - Added my private return key definition here, because it makes happy
;;   couple when the TIT mode is on.
;;   tit-:auto-indent-regexp		:+
;;   tit-return-key-toggle		:+
;;   tit-auto-indent			:+
;;
;; Mar	8	1996	[jari]		19.28	v1.12		NotReleased
;; - Changed version variables not to have ":".
;;
;; Jan	26	1996	[jari]		19.28	v1.11		NotReleased
;; - Now package has lisp-mnt.el 'section' markers. Also added 2-4-8
;;   tab selection in the tit-change-tab-width.
;; - Removed extra library code which should not resize here any more.
;; - Added bug and feedback functions
;;
;; Nov	2	1995	[jari]		19.28	v1.10		Release_2
;; - Renamed many function/var names. Made better key setting function
;;   for user in nonWindowed emacs. Now uses "add-minor-mode"
;;   function from my libs.
;;
;; Oct	28	1995	[jari]		19.28	v1.9		NotReleased
;; - Okay, I forgot. The analogy must be for the -del function too,
;;   so now there is new variable. That's it for tonaight, it's
;;   01:00 and I got to go to bed or i sleep right now...
;;   tit-:tab-special-del-funcs	    :+ func list
;;   tit-tab-backward-del	    :! modified
;;
;; Oct	28	1995	[jari]		19.28	v1.8		NotReleased
;; - It's past midnight and my inspiration still flows.. Now I wanted to
;;   have special indentation if previous line held comment, so I
;;   integrated the gap between this package and tinyindent.el
;;   NEXTP			    :+ from my libs..
;;   tit-:tab-special-insert-funcs	    :+ This is ist of funcs to call
;;   tit-tab-forward-insert	    :! modified to take advantage of it
;;
;; Oct	27	1995	[jari]		19.28	v1.7		Release_1
;; - I'm pretty satisfied now, so let's make this first alpha.
;;   OHIO, here I come...
;;
;; Oct	27	1995	[jari]		19.28	v1.6		NotReleased
;; - Cleaned compile errors away.
;;
;; Oct	27	1995	[jari]		19.28	v1.5		NotReleased
;; - Added missing ###autoload.
;; - Hmm, The tab-insert and tab-del gives me headache. This file turned
;;   into much more complex than I intended at the beginning.
;; - Now the problem was with moving "inside" the tabs char, that  caused
;;   "\t" code to be replaced with multiple spaces. And when going
;;   forward, we really don't want to leave "    " + "    ", instead we
;;   like to see nice "\t"'s while we advance..
;; - But I did all these, since this is _the_ tab mode.
;;   EOLP,BOLP..		:+ from my lib
;;   ti::s-tabify		:- deleted, not needed in this version
;;   tit-XXX			:! renamed all tab functions
;;   tit-tab-forward-insert	:! fully reprogrammed
;;   tit-tab-backward-del	:! fully reprogrammed
;;
;; Oct	27	1995	[jari]		19.28	v1.4		NotReleased
;; - Samll change to ky defining function.. C-tab goes forward.
;;
;; Oct	27	1995	[jari]		19.28	v1.3		NotReleased
;; - The sugar missed... The tab width should be shown in the modeline.
;;   Now it displays it
;;   tit-:mode-name		:+ mode name moved here
;;   tit-set-mode-name		:+ function to change the modename
;;   tit-:mode-name		:! no more user variable
;;
;; Oct	27	1995	[jari]		19.28	v1.2		NotReleased
;; - Bugs crawling out. Serious error happened when the *tmp* buffer
;;   was used for tabify. If it accidentally were tuned to read-only, the
;;   tab-inset couldn't use it.
;; - The backward tab deletion was also a flop. Corrected.
;; - Correected by adding couple of more lib funcs. Added "*" to
;;   appropriate interactive funcs.
;; - I had an exellent idea, new 4-8 function that allows swithing
;;   tabs real fast...oh boy this is getting good!
;;   use-tmp-buffer		:+ lib func
;;   ti::m-tmp-buffer		:+ lib func
;;   tit-change-tab-width	:+ fast 4-8-4 tab switching
;;   tit-tab-backward-del	:! deleted chars, error in regexp
;;   tit-tab-backward-del	:! totally rewritten
;;   tit-:mode-map-define-keys	:! more instructions
;;
;; Oct	26	1995	[jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyTab tit-: tools
  "TAB minor mode. The most basic 'programming mode' for any language.
  Overview of features

	o   Programmable TAB. If you set the spacing(div factor) to 4,
	    you can virtually program blindly without any other modes.
	o   moving commands: tab-forward, tab-backward
	o   modify commands: tab-insert, tab-delete
	o   indent commands: indent-by-div-factor, forward , backward
	o   Positioning braces { } supported.

	Extras

	o   special auto-indent function offered for return key,
	    switch it on, and you can continue your shell,awk,perl comments...
	o   Press C-c TAB and your in contant indentation mode where
	    keys \"wq\" \"as\" \"zx\" indent region <--> by 1,2 and 4 until
	    esc pressed."
  )

;;}}}

;;{{{ setup: variables

;;; .......................................................... &v-mode ...

;;;###autoload (autoload 'tit-mode	"tinytab" t t)
(eval-and-compile

(ti::m-fmacro-minor-mode-wizard
 "tit-" " +" nil nil 'TinyTab "tit-:"		;1-6


  "Tab movement minor mode. Adjustable movement division.
References:
  tit-:div-factor

Mode description:
\\{tit-:mode-map}
"
  "TinyTab"
  (progn
    (if tit-mode
	(tit-set-mode-name))
    )

  nil					;No Menu
  nil

  (progn
    (tinyck-maybe-activate 'emacs)

  ;; ... ... ... ... ... ... ... ... ... ... ... ... ... . Non Wwindow . .
  ;;  (define-key	tit-:mode-map "\M-\t"		'tit-tab-key)

  (define-key   tit-:mode-map "\t"		'tit-tab-key)
  (define-key   tit-:mode-map "\e\t"		'tit-indent-by-div-factor)
  (define-key   tit-:mode-map "\C-c\e\t"	'tit-indent-by-div-factor-back)

  ;;  In HP, upper left, near the TAB
  ;;  Please relocate these two as you like.
  ;;
  (define-key   tit-:mode-map "\e`"		'tit-tab-forward)
  (define-key   tit-:mode-map "\e~"		'tit-tab-backward)
  (define-key   tit-:mode-map "\C-c`"           'tit-change-tab-width)
  (define-key   tit-:mode-map "\C-c\t"          'tit-indent-region-dynamically)

  ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... .. X keys . .

  (define-key   tit-:mode-map "\t"		'tit-tab-key)

  (define-key tit-:mode-map [C-tab]		'tit-tab-forward)
  (define-key tit-:mode-map [(control ?\`)]	'tit-tab-backward)

  (define-key tit-:mode-map [S-backtab]		'tit-tab-del-key)
  (define-key tit-:mode-map [S-hpBackTab]	'tit-tab-del-key) ;; XEmacs
  (define-key tit-:mode-map [S-tab]	        'tit-tab-del-key)
  (define-key tit-:mode-map [S-kp-tab]		'tit-tab-del-key)

  (define-key tit-:mode-map [A-tab]		'tit-indent-by-div-factor)
  (define-key tit-:mode-map [A-kp-tab]		'tit-indent-by-div-factor)
  (define-key tit-:mode-map [A-kp-tab]		'tit-indent-by-div-factor)

  (define-key tit-:mode-map [A-S-backtab]	'tit-indent-by-div-factor-back)
  (define-key tit-:mode-map [A-S-kp-tab]    'tit-indent-by-div-factor-back)

  ;; XEmacs A-S-TAB
  ;;
  (define-key	tit-:mode-map [A-hpBackTab]	'tit-indent-by-div-factor-back)


  (tinyck-maybe-activate 'emacs 'dis)
  ))
)



;;; ......................................................... &v-hooks ...

(defcustom tit-:load-hook nil
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyTab
  )


(defcustom tit-:tab-insert-hook
  '(tit-tab-mode-control
    tit-tab-brace-control
    tit-tab-forward-insert
    tab-to-tab-stop
    )
  "*List of functions to call for inserting logical TAB.
If any of these functions return non-nil, it is assumed,
that the tab handling was performed."
  :type 'hook :group 'TinyTab
  )


(defcustom tit-:tab-delete-hook
  '(tit-tab-backward-del
    )
  "*List of functions to delete a logical TAB backward.
If any of these functions return non-nil, it is assumed,
that the tab handling was performed."
  :type 'hook :group 'TinyTab
  )



;;; ....................................................... &v-private ...

(defvar tit-:div-factor 4
  "Current tab division.")

;;; ........................................................ &v-public ...
;;; User configurable

;;   Simple name is enough. Think this as "Tab +" or "extended tab" -mode
;;
(defcustom tit-:mode-name-base " +"
  "*Minor mode base name."
  :type 'string :group 'TinyTab
  )

;;  If I accidentally press key I didn't meant to, I want to know
;;  about it. Like in empty line, where is no visual aids
;;
(defcustom tit-:verbose t
  "*Enable Insert/Delete messages."
  :type 'boolean :group 'TinyTab
  )

(defcustom tit-:div-factor-table '(2 4 8)
  "*When using `tit-change-tab-width', cycles through list of tab positions.
Default values are '(2 4 8)"
  :type '(repeat integer) :group 'TinyTab
  )


(defcustom tit-:mode-table  '(c++-mode cc-mode c-mode)
  "*List of mode name symbols where the TAB kay call mode's TAB function.
But, only if the point is at the beginnning of line."
  :type '(repeat (symbol :tag "Mode name symbol")) :group 'TinyTab
  )


(defcustom tit-:indent-region-key-message
  "Indenting...  z=4 a=2 q=1 < >  w=1 s=2 x=4 [Esc - exit]"
  "*Message displayed while in dynamic indent mode."
  :type 'string  :group 'TinyTab
  )

(defcustom tit-:indent-region-key-list '(?q ?w   ?a ?s   ?z ?x  ?\e)
  "*List of keys to control dynamic indenting. The keys go in pairs.
elt 0 1       ,left and right by 1
elt 2 3       ,left and right by 2
elt 4 5       ,left and right by 4
elt 6         ,exit key"
  :type '(list
	  character character
	  character character
	  character character
	  character
	  )
  :group 'TinyTab
  )

;;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... return key  ..

(defcustom tit-:auto-indent-regexp  "[#!;*/]\\|REM\\|//"
  "*If previous line match this regexp, it is copied when you hit RET.
This allows eg. continuing C++'s // comments.
See function `tit-return-key-toggle' to turn on this auto-indent feature."
  :type 'string  :group 'TinyTab
  )


;;}}}

;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinytab.el"
 "tit"
 tit-:version-id
 "$Id: tinytab.el,v 1.40 1997/06/19 10:16:00 jaalto NotReleased $"

 '(tit-:load-hook
   tit-:mode-hook
   tit-mode-define-keys-hook
   tit-:mode-name
   tit-:tab-insert-hook
   tit-:tab-delete-hook
   )))


;;}}}
;;{{{ code: misc

;;; ########################################################### &Funcs ###

;;}}}
;;{{{ extra functions

;;; ........................................................... &extra ...


;;; ----------------------------------------------------------------------
;;;
(defsubst tit-get-division  ()
  "Return TAB division."
  (if (not (integerp tit-:div-factor))
      (setq tit-:div-factor 4))
  tit-:div-factor
  )

;;; ----------------------------------------------------------------------
;;; - So that you can bind this to fast key
;;;
(defun tit-indent-by-div-factor-back (beg end)
  "Just shortcut to `tit-indent-by-div-factor'. Indent BEG END backward."
  (interactive "*r")
  (tit-indent-by-div-factor beg end 'back))


;;; ----------------------------------------------------------------------
;;;
(defun tit-indent-by-div-factor  (beg end &optional back)
  "Indent region BEG END by current tab division.
Optional arg BACK indents backward."
  (interactive "*r\nP")
  (tit-get-division)
  (let* ((div  (if back
		   (- 0 tit-:div-factor)
		 tit-:div-factor))
	 )
    (indent-rigidly beg end div)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tit-tab-mode-control  ()
  "If `mode-name' in in the list `tit-:mode-table', call mode's tab key.
But only if
o  point is at the beginning of line.
o  the line is empty

This way you can partly mix eg. C++ mode and this minor mode."
  (interactive)
  (let* ((sym  (intern (format "%s-map" (symbol-name major-mode))))
	 map
	 func
         )
    ;;  If we're ar the beginnning of line, see if there is keymap for
    ;;  this current mode. Then try to find function for "\t" key
    ;;  and call it
    ;;

    (when (and (or (bolp)
		   (save-excursion
		     (beginning-of-line)
		     (looking-at "^[ \t]+$")))
	       (memq major-mode tit-:mode-table)
	       (boundp sym)
	       (keymapp (setq map (eval sym)))
	       (setq func (lookup-key map "\t"))
	       )
      (call-interactively func)
      t
      )
    ))

;;; ----------------------------------------------------------------------
;;; - For a little more smarter TAB key to line up { } braces
;;;   in variaous programming modes I made this. It's simple,
;;;   but suffices for most common needs.
;;; - I don't know how the C-mode or cc-mode does this, but, hey,
;;;   this is one way :-)
;;;
;;;
(defun tit-tab-brace-control ()
  "When hitting TAB, line up {} braces, otw do nothing special.
Remember that opening brace, the {, follows previous line's indentation
and } follows the \"{\".


Return:
  t		,if TAB handled in this fuction.
  nil		,nothing done."
  (interactive)
  (let* (line
	 rest
	 indent
	 pindent			;previous
	 col
	 ret				;flag
	 handle
	 equal
	 )
    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... check brace ...
    (save-excursion
      (beginning-of-line)

      ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... { . .
      (cond
       ((looking-at "^\\([ \t]*\\)\\({.*\\)")
	(setq indent (or (read-match 1) "")
	      rest   (read-match 2)
	      handle '{
	      )
	(save-excursion
	  (forward-line -1)		; peek previous line indent
	  (setq pindent (or (bufmatget "^[ \t]+" 0) "")))
	)

       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... .. } ..
       ((looking-at "^\\([ \t]*\\)\\(}.*\\)")
	(setq indent (or (read-match 1) "")
	      rest   (read-match 2)
	      handle '{
	      )
	(save-excursion
	  (cond
	   ((re-search-backward "^\\([ \t]*\\){" nil t)
	    (setq pindent (or (read-match 1) ""))
	    )))
	)))

    ;; ... ... ... ... ... ... ... ... ... ... ... ... .. adjust brace ...
;;;    (d! "AFTER"  (string= indent pindent) pindent indent rest)

    (setq equal (and indent pindent (string= indent pindent))
          col   (and indent (length (subst-char-with-string indent))))
    (cond
     ((and indent pindent equal
	   (memq handle '({ }))
	   (< (current-column) col)
	   )
      ;;  Pressing TAB, before the {, puts cursor to brace.
      ;;
      (move-to-column col)
      (setq ret t)
      )

     ((and indent pindent
	   (not (string= indent pindent)))

      ;;  This is reindent case: { and } didn't line up.
      ;;
      (setq line (concat pindent rest)
	    col  (current-column)
	    ret  t
	    )
      (setq col (current-column))
      (delete-region (BOLPP) (EOLPP))
      (insert line)

      ;;  If user is in the LEFT side of brace, put cursor to brace
      ;;  If user if in the RIGHT side, then move-to-column will
      ;;  preserve position.
      ;;
      (move-to-column col)
      (cond
       ((< col (length (subst-char-with-string pindent)))
	(re-search-forward "{\\|}")
	(forward-char -1)
	))

      ))

    ret
    ))

;;}}}

;;{{{ code: return key

;;; ----------------------------------------------------------------------
;;; Replaces the RET key
;;; The arg is just due to: (newline &optional ARG1)
;;;
(defun tit-auto-indent (&optional arg)
  "Automatically indent according to previous line.
If optional ARG is given, behave exactly like 'newline' function."
  (interactive "P")
  ;;  The RE matches few common comments and empty whitespaces
  ;;  #     = shell
  ;;  ;	    = lisp comments
  ;;  *	    = C comments
  ;;  !	    = .Xdefaults comments
  ;;  //    =  C++ comments
  ;;  REM   = oracle SQL comments
  ;;
  (let ((re      (concat "^[ \t]*\\(" tit-:auto-indent-regexp "\\)*[ \t]*"))
	str
	)
    (if arg     ;;  We do not do anything special if user has given arg.
	(newline arg)
      (if (not (eolp))   ;; Now let's see if user wanted fresh line
	  (progn
	    ;;  No, he wanted to divide line.
	    ;;  Read the line up till cursor point
	    (setq str (buffer-substring (BOLP) (point)))

	    ;;  Ignore portion match --> nothing important matched
	    (if (not (and (string-match re str)
			  (equal (length str) (match-end 0))))
		(newline)		;something else than re, break line
	      (newline)
	      (insert str)
	      ))
        ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. else ...
	;;  Let's peek current line
	(save-excursion
	  (beginning-of-line)
	  (setq str (bufmatget re 0)))
	(newline)
	(if str
	    (insert str)))
      )))

;;}}}
;;{{{ code: tab


;;; ----------------------------------------------------------------------
;;;
(defun tit-set-mode-name ()
  "Set mode name according to tabs in effect."
  (interactive)
  (let* ((base  tit-:mode-name-base)
	 (val   (tit-get-division))
	 )
    (setq tit-:mode-name (concat base val))
    (ti::xe-modeline-update)
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tit-change-tab-width ()
  "Toggle tab width according to `tit-:div-factor-table'."
  (interactive)
  (let* ((verb  (interactive-p))
	 (val   (tit-get-division))
	 (table tit-:div-factor-table)
	 elt
	 )
    (cond
     ((not (integerp val))
      (setq val (car table)))		;default value

     ((setq elt (memq val table))
      (if (eq 1 (length elt))		;it's last item
	  (setq val (car table))	;pick first value
	(setq val (nth 1 elt))		;get next in the list
	))

     (t					;can't find value from table ?
      (setq val (car table))		;get first then.
      ))

    (setq tit-:div-factor val)		;update
    (tit-set-mode-name)
    (if verb				;this does no harm....
	(message (concat "Tab factor is now " val)))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tit-tab-backward-del ()
  "Move Tab backward.
Delete whitespace if all characters preceeding the point are whitespaces
_and_ the final position is in divideable by current div-factor.

Eg. If you factor is 4, and there is 2 spaces before your cursor \"*\",
    Thi func will not delete the extra spaces, because it can't reach
    position 8.

	 bar Geezy *
	 12345678901234
            ^   ^     ^

In this case, calling the function is no-op.

References:
  `tit-:tab-delete-hook'
  `tit-:div-factor'"
  (interactive "*")
  (let* ((div   (tit-get-division))
	 (verb  tit-:verbose)
	 (col   (current-column))
	 (dest  (- col (% col div)))
	 MARK
	 str
	 eob				;flag
	 p p2				;points
	 )


      ;; ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^^^ ^ normal deletion ^ ^
      (setq MARK  (save-excursion
		    (if (eobp)
			(setq eob t)
		      (forward-char 1))	;push marker one forward
		    (point-marker)))

      (if (= col dest )			; would be exact
	  (setq dest (- col div )))
      (if (< dest 0)
	  (setq dest 0))


      (if (= col 0)			;beg of line
	  nil
	(move-to-column dest 'force)	;converts tabs to spaces.

	;; consider following:
	;;    actual         seen
	;;    12345678       123456789
	;;    ----------------------------------
	;;    #\thello      "#       hello"	,suppose cursor is in "h"
	;;    |  |
	;;    |  point 3
	;;    point 1
	;;
	;;    Now you indent back by 4, this is what happens
	;;    12345678       12345678
	;;    #   hello	   "#   hello"
	;;    |   |
	;;    |	 point 5			, Geez!
	;;    point 1
	;;
	;;    The tab is converted and it caused all point to be altered.
	;;    That's why we have to use the marker, because it stays
	;;    releative to text, in this case just _behind_ the letter "h"
	;;

	(setq p  (if eob
		     (marker-position MARK)
		   (1- (marker-position MARK))))
	(setq p2 (point))
	(setq str (buffer-substring p2 p))


;;;       (d! p2 p str dest col (point) )

	(if (not (string-match "^[ \t]+$" str))
	    (progn
	      (if verb (message "Can't reach previous tab position"))
	      (goto-char p)		;do not move. Stay put.
	      )
	  (delete-region p2 p)

	  (if verb (message "Deleted"))
	  ))

      (setq MARK nil)			;kill the marker
      ))



;;
;;


;;; ----------------------------------------------------------------------
;;;
(defun tit-tab-backward ()
  "Logical tab movement backward, until bol."
   (interactive)
   (let* ((div   (tit-get-division))
	  (dest  (- (current-column) div))
	  )
     (if (< dest 0)
	 (setq dest 0))

     (move-to-column dest 'force)
     (if (and tit-:verbose
	      (looking-at "[ \t]+$"))	;no visible hints
	(message "Moved."))
     ))


;;; ----------------------------------------------------------------------
;;;
(defun tit-tab-forward-insert ()
  "Move tab forward, insert spaces or tabs, see variable `indent-tabs-mode'.

References:
  `tit-:div-factor'"
  (interactive "*")
  (let* ((verb  tit-:verbose)
	 (col   (current-column))
	 (div   (tit-get-division))
	 (nbr   (- div (% col div)))
	 div
	 eob				;flag
	 MARK				;marker
	 str
	 )
    (cond
     ((= 0 nbr)
      (setq str (make-string div ?\ )))
     (t
      (setq str (make-string nbr ?\ ))
      ))

    (insert str)

    (if (null indent-tabs-mode)
	nil
      ;; - When we insert non-tabs, like in mode "tab 4", what happens is
      ;;   that we insert "    " + "    " ie. 4 + 4 spaces.
      ;; - but, we really like them to be like one "\t" code in text,
      ;;   So, let's fix the line every time something is inserted.
      ;; - We have to use markers again due to tabify.
      ;; - The EOB is special case
      ;;
      (setq MARK (save-excursion
		   (if (eobp)
		       (setq eob t)
		     (forward-char 1))
		   (point-marker)))

      (tabify (BOLP) (point))
      (goto-char (if eob
		     (EOLPP)
		   (1- (marker-position MARK))))

      (setq MARK nil)			;kill it
      )
    (if verb (message "Insert"))
    t					;we handled this
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tit-tab-forward (&optional verb)
  "Step logical tab forward. Does not insert anything. Stops at EOL. VERB.
Tabs are converted to spaces when needed; because you can't step inside
'\t' character in the line otherwise.."
   (interactive)
   (let* ((div   (tit-get-division))
	  (col   (current-column))
	  (nbr   (- div (% col div)))
	  (ecol  (save-excursion (end-of-line) (current-column)))
	  (dest  (+ col nbr))
	  (verb  (or verb tit-:verbose))
	  )
     (cond
      ((> dest ecol)
       (end-of-line)
       (if verb (message "End of line.")))
      (t
       (move-to-column dest 'force)
       (if (and verb
		(looking-at "[ \t]+$"))	;no visible hints in this line...
	   (message "Moved."))
       ))
     ))



;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tit-tab-key  ()
  "Main function for TAB key. See variable `tit-:tab-insert-hook'."
  (interactive)
  (run-hooks-until 'tit-:tab-insert-hook)
  )


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tit-tab-del-key  ()
  "Main function for TAB key. See variable `tit-:tab-delete-hook'."
  (interactive)
  (run-hooks-until 'tit-:tab-delete-hook)
  )


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tit-return-key-toggle ()
  "Toggle auto indent mode / regular newline mode ."
  (interactive)
  (let* ((cmd 'tit-auto-indent)
	  )
    (local-set-key
     "\C-m"
     (if (equal (or (lookup-key  (current-local-map) "\C-m")
		    (lookup-key  (current-global-map) "\C-m"))
		cmd)
	 (progn
	   (message "Return key: normal.")
	   'newline)
       (progn
	 (message "Return key: auto indent on")
	 cmd
	 )))))



;;; ----------------------------------------------------------------------
;;;
(defun tit-indent-region-dynamically  (beg end)
  "Move region BEG END until exit key is pressed.
Default key setup, see `tit-:indent-region-key-list' where default
keys are:

  left  = q by 1, a by 2, z by 4
  right = w by 1, s by 2, x by 4"
  (interactive "*r")
  (let* ((i     1)
	 (k     tit-:indent-region-key-list)
	 (msg   tit-:indent-region-key-message)
	 ch EXIT
         )
    (if (not (eq (length k) 7))
	(error "Not enough members in tit-:indent-region-key-list."))
    (setq EXIT (nth 6 k))

    (while (not (eq EXIT (setq ch (read-char-safe-until msg))))

      (setq i nil)

      (cond
       ((eq ch  (nth 0 k))
	(setq i -1))
       ((eq ch  (nth 1 k))
	(setq i 1))

       ((eq ch  (nth 2 k))
	(setq i -2))
       ((eq ch  (nth 3 k))
	(setq i 2))

       ((eq ch  (nth 4 k))
	(setq i -4))
       ((eq ch  (nth 5 k))
	(setq i 4))
       )

      (if i
	  (indent-rigidly (region-beginning) (region-end) i))
      )))



;;}}}
;;{{{ example

;;; ......................................................... &example ...
;;; - Code can be easily extracted with function tinylib.el/ti::m-pkg-rip-magic
;;; - I find using the tinyindent.el as much as this mode, so
;;;   here is some integration of the two.
;;;
;;;   The code will do special indent if
;;;   o  you're at the beginning of line, and there is comment
;;;      above you.

;;* (add-hook 'tit-:tab-insert-hook 'my-tit-special-tab-insert)
;;* _
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-tit-special-tab-insert ()
;;*   "Special tab handling. Continues comment if needed"
;;*   (let* (fill
;;* 	 )
;;*     (require 'tinyindent)
;;*     (if (not (= 0 (current-column)))
;;* 	nil
;;*       (save-excursion
;;* 	(forward-line -1)		;peek previous line ?
;;* 	(setq fill (tii-special-handle)))
;;* _
;;*       (if (not (stringp fill))
;;* 	  nil
;;* 	(insert fill)
;;* 	t				;we handled it!
;;* 	))
;;*     ))





;;}}}

(add-hook 'tit-:mode-define-keys-hook 'tit-mode-define-keys)

(provide   'tinytab)
(run-hooks 'tit-:load-hook)

;;; tinytab.el ends here
