;; @(#) tinylib.el -- Main library of general functions

;; @(#) $Id: tinylib.el,v 1.123 1997/06/26 23:27:43 jaalto NotReleased $
;; @(#) $Keywords: extensions languages lisp $
;;
;; $KnownCompatibility: 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995, 96, 97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 6 1995
;;
;; To get information on this program use ident(1) or do M-x tilib-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el


;; LCD Archive Entry:
;; tinylib|Jari Aalto|jari.aalto@poboxes.com|
;; Main library of useful functions, all you ever dreamed of...|
;; 27-Jun-1997|$Revision: 1.123 $|~/misc/tinylib.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;     (require 'tinylib)
;;
;;
;; Setup note:
;;
;; See end of file for possible examples or hints. Eg. Better
;; UUDECODE program is included.
;;
;; If you have any questions or feedback, use this function
;;
;;      M-x tinylib-submit-feedback


;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "ti::" in front of
;;   every function, variables have "ti:-". It stands for '(ti)ny library
;;   The '::' notation is commonly used in C++ and perl 5.00


;;  Preface, 1995
;;
;;      I noticed, that I started using several functions in program
;;      modules I developed over the time, so it was time to move all common
;;      parts into general library.
;;
;;      Use M-x Occur 'defun ' to view all the functions at once
;;
;;      Briefly:
;;      o   This is library, so the package itself does nothing,
;;          there may be some interactive functions, so just skim through code.
;;      o   Collection of general functions. With these you can solve your
;;          problems within minutes, instead of spending days thinking
;;          "How do I do this with lisp? Does there exist a function...?"
;;
;;  Quick function help
;;
;;      Please use function
;;
;;          tinyliby.el / ti::y-get-file-documentation
;;
;;      which will build up a buffer containing all functions/macros
;;      and variables used in any lisp package file. You can print the results
;;      to get an overview of all functions in this file.
;;
;;      If you feel that some function description is not clear, please
;;      drop me a mail. There is no plans for separate info files; that's why
;;      I try to keep the function descriptions as good as possible.
;;
;;      Also look at the function
;;
;;          tinyliby.el / ti::y-describe-symbols
;;
;;      which will help you to find particular lisp information from the
;;      emacs. Eg. to search for function that has "file" in their name.
;;
;;  If you're pacakge developer, get these packages and read the source:
;;
;;      Here is standard toolset I use for writing/studying/lining up
;;      variables for lisp...
;;
;;          tinybm.el           ,drawing sections (bookmarks)
;;          tinytab.el          ,tab minor mode
;;          tinyindent.el       ,another "line up my vars" minor mode
;;          lisp-mnt.el         ,std emacs package use lm-verify
;;          eldoc.el            ,show func doc immediately (under cursor)
;;          find-error.el       ,find lisp error. put cursor on it
;;          fdb.el              ,filter non-important errors
;;
;;  What is this 'tiny' thing?
;;
;;      The term *tiny* does not refer to file size, it's the philosophy
;;      behind my lisp packages. The packages I developed, solved
;;      some specific task where I needed help at the time, hence the
;;      philosophy --> *tiny* *tools* for emacs users.
;;
;;      These libraries are also *tiny* in a sense, that they offer
;;      you ready functions, to create your own packages very easily
;;      Besides, it has started beeing my trademark :-/
;;
;;  Code policy
;;
;;      All functions are written by [jari], unless otherwise stated.
;;      If I have found interesting function from some other .el I have
;;      written a reference to that .el. If the code is directly taken
;;      'almost as is' all the credit belongs to the Author of the
;;      original code. Cheers for him, to write so recpectable code, that
;;      it can be used right from the box!
;;
;;      If someone contributes good, general purpose function, all the
;;      credit belongs to him automatically. Persons name and date and so
;;      on is included.
;;
;;  tinyXXX Libraries' intention
;;
;;      These libraries are by no means private. My wish is that all Lisp
;;      developers would contribute their general purpose functions and
;;      they would be merged into these libraries so that the functions would
;;      be commonly available.
;;
;;      Emacs has been around for quite a while, 10's of years, but still
;;      there were no decent library of functions. How can it be possible!
;;      Even Perl, which is in it's boyhood compared to Emacs, if years are
;;      counted, has respectable number of libraries around. I hope this
;;      serious lack is now corrected at least for some parts.
;;
;;      The biggest problem in the lisp world seems to be, that everybody has
;;      to start from "scratch". And when there is no ready modules that one
;;      could use, it's quite hard to develop new programs, especially if you
;;      first have to learn to do things.
;;
;;      I hope all people's efforts were found from one place instead of
;;      scattered emacs files, so that reinventing the wheel would
;;      stop. And code development would be much quicker when you have
;;      ready libs around that provide base for new programs.
;;
;;  Registration
;;
;;      If you're going to use these function, it may be good idea to register
;;      you as a "user" of these libraries. When the libs changes, you will
;;      be mailed a message and you can follow the development; what new funcs
;;      may help you better, which bugs are found and may affect you.
;;
;;      I do not have accees to mailing lists, nor can I maintain one,
;;      so the registration is used for simple message that inform about the
;;      lib development phases only.
;;
;;  Code reliability
;;
;;      Almost all of these functions are in actual use all the time and
;;      they have been found to be quite good behaving. This doesn't
;;      mean that they don't contain bugs. And if you feel that
;;      function should have more parameters or they should behave "more
;;      generally", please mail me. I'm willing to take suggestions of
;;      all kinds.
;;
;;      The functions are *not* tested systematically, one by one.
;;      Instead, I have done couple of quick tests after coding to
;;      verify that the func works expectedly. It may still have bugs,
;;      when fed with some other data or used in some other task. Please
;;      be patient and send me backtrace and the values used and I'll fix
;;      bugs real fast.
;;
;;      Report bugs immediately, since someone may be using that
;;      funtion without knowing the error.
;;
;;  Turn on you debug
;;
;;      If some function gives you error, immediately turn on debug
;;      and repeat your task
;;
;;          M-x set-variable RET debug-on-error RET t RET
;;
;;      Now when error happened, you get *Backtrace* buffer displayed.
;;      Send me the contenst of the buffer and all variable values that
;;      are relevant to call. See 'e' command in backtrace to get values
;;      displayed.
;;
;;      Btw, it's good to have debug permanently on. Gget fdb.el to ignore
;;      standard emacs errors. This exellent package was made by one of my
;;      friends, Anders Lindgren. You can get it from:
;;
;;          ftp://ftp.csd.uu.se/pub/users/andersl/emacs/fdb.el
;;
;;
;;  Function naming convention in this library
;;
;;      So why this kind of naming? Have you ever tried to find function in
;;      emacs? Then you know... If you want to find all functions related
;;      to file handling, you can't just say (in 19.28)
;;
;;          ESC ESC file-[TAB]          ,tab means lisp-complete-symbol
;;
;;      because the function naming in emacs is "traditional", meaning
;;      that they are like spoken language, where word "file" may exist
;;      in any part of the function name, like
;;
;;          xxx-file-xxx-xxx
;;
;;      So, to ease finding function while I'm programming in lisp, I chose
;;      to name functions so that they have some sort of prefix. If you
;;      want to list all functions related to file in tinylibXX you just do
;;
;;          ESC ESC ti:f-[TAB]
;;
;;      Same goes to any other category. I assure you , while it may not
;;      look nice, it certainly saves lot of programming time.
;;      Anyway the "nice look" of language is not important. Seen
;;      perl programs ever?
;;
;;      Following syntax is used, so that functions and variables
;;      can be found easily, when they have common beginning.
;;
;;          LIB-id-funcName
;;
;;          LIB = library where function belongs  'ti::'
;;                variables have only one ':' --> 'ti:'
;;
;;          id  = specifier that tell where the function applies to.
;;          a  = (a)dviced emacs functions
;;          b  = (b)uffer related
;;          c  = (c)onversions, convert in general
;;          d  = (d)ate tools
;;          e  = (e)vents
;;          f  = (f)ile handling
;;          k  = (k)eymap control
;;          l  = (l)ist manipulation
;;          m  = (m)isc category
;;          o  = (o)verlay handling
;;          p  = shell and (p)rocess handling
;;          q  = (q)uestions and user interaction in general
;;          s  = (s)string manipulation
;;          t  = (t)ext properties (faces)
;;          w  = (w)indow and frame handling
;;          x  = (x) and X-popup menu related
;;          y  = s(y)stem manipulation, information from internals.
;;
;;      Two char identifiers are for specific tasks/libs
;;
;;          mt  = (m)ail (t)ools, includes News handling tools too.
;;              See tinylibmt.el
;;          xe  = (Xe)macs compatibility functions. You should use
;;              these is you want to write portable code.
;;
;;  Local varibale naming in the tinylibXXX.el functions.
;;
;;      I have tried to use following names in functions' 'let' body.
;;      However, when I started coding these long time ago I hadn't
;;      thought about setting up any discipline in variable naming, so there
;;      is still functions which do not use standard naming.
;;
;;      I find it very handy, when I always use same names for same purposes,
;;      it makes the code more easy to read and understand when the variety
;;      of names is not so big.
;;
;;      If you think about submitting general code, I'd like to see the
;;      functions using these variables. Of cource it's up to you if
;;      you feel naming conventions are waste of time. Still, spend a moment
;;      thinking who might be reading your code.
;;
;;	    beg end	begin area, end area [points]
;;	    bf, bfn	buffer file name
;;	    bn		buffer name
;;	    bp		buffer pointer  , sometimes "beginning point"
;;	    buffer	buffer [pointer or name]
;;	    cmd		usually a shell or process command string
;;	    ep		end point -- see bp
;;	    func	variable to store a function symbol
;;	    fid		"function id", name which is used in debug trace
;;	    list	a list
;;	    len		length of object [string, list, array]
;;	    line	current line contents as string
;;	    level	matching subexpression level
;;	    mb me	match end/beginning point or Marker beg/end
;;	    nbr		holding integer
;;	    ob		original buffer [pointer or name]
;;	    p		general point
;;	    pb pe	point: beg, end
;;	    ptr		a list pointer that will be moved.
;;	    [-]re[-]	referring to regexp
;;	    reg		register [integer ie. char]
;;	    ret		return value of a function
;;	    str		string
;;	    verb	flag representing 'vebosity' usually (interactive-p)
;;	    win		a window pointer
;;
;;  Detailled explanation of some functions or macros in this library
;;  ----------------------------------------------------------------------
;;
;;  Defining minor mode fast
;;
;;	If you're eager to define minor mode in 1 minute, then here is
;;	one macro that does everything for you. After this you have all the
;;	minor mode variables ready and the mode works in bot Emacs
;;	and XEmacs including the menus.
;;
;;	    (eval-and-compile   ;; So that defvars and defuns are seen
;;          (ti::m-fmacro-minor-mode-wizard
;;           ;;
;;           ;; 1. prefix for variables and functions
;;           ;; 2. Modeline name
;;           ;; 3. prefix key for mode.
;;           ;; 4. Menu bar name
;;           ;; 5. <forget this>
;;           ;;
;;	     "xxx-" " xxxModeline" "\C-cx" "xxxMenubar" nil
;;
;;	     "XXX minor mode. Does fancy things."  ;; mode description
;;	      Defined keys:
;;	      \\{xxx-mode-map}
;;	      "
;;
;;            "XXX help"    ;; message displayed when user calls mode
;;            nil           ;; Forms When minor mode is called
;;
;;	     ;; This is used by easy-menu.el and defines menu items.
;;           (list
;;            xxx-mode-easymenu-name
;;            ["Eval whole buffer" xxx-eval-current-buffer    t]
;;            ..
;;            )
;;
;;	     ;;  this block defines keys to the mode. The mode minor map is
;;	     ;;  locally bound to 'map' symbol.
;;	     (progn
;;	       (define-key map (concat p "-") 'xxx-eval-current-buffer)
;;	       (define-key map (concat p "=") 'xxx-calculate)
;;	       )
;;           ))
;;
;;  Defining minor mode step by step
;;
;;	If you want to take more personal control over the minor mode
;;	creation, here I explain step by step what macros you need to include
;;	in your pacakge to get minor mode created, This takes only
;;	half an hour and you have basic minor mode ready. Put all
;;	following calls near top of your file. We suppose we're
;;	creating XXX-mode.
;;
;;	_1_ First, define standard variables for minor mode.
;;
;;	    (ti::m-vmacro-minor-mode "xxxModeline" "\C-cx" "xxxMenubar")
;;
;;	After that user has  following varibles for customization. (for
;;	complete list of created variables, see the macro's description)
;;
;;	    ;; Don't like default key combo C-c x
;;	    (setq xxx-mode-prefix-key "\C-cm")
;;
;;	    ;; The default mode string was too long, use shorter.
;;	    (setq xxx-mode-name "xxx")
;;
;;	    ;;  When mode runs, I want to do this.
;;	    (add-hook 'xxx-mode-hook 'my-xxx-settings)
;;
;;	    ;;  I want to add additional keys
;;	    (add-hook 'xxx-mode-define-keys-hook 'my-xxx-keys)
;;
;;	_2_ Next we need installation function, which installs our minor
;;	mode so that emacs is aware of it. The minor mode funcktions,
;;	xxx-mode, will call xxx-mode-define-keys-hook which takes care of
;;	defining keys to keymaps and creating menus with easy-menu.el. The
;;	separate installation function is used, because it takes care of
;;	emacs specific things and if called with additional argument, it
;;	also knows how to remove the mode gracefully.
;;
;;	    (ti::m-fmacro-minor-mode-install
;;	     xxx-install-mode
;;	     xxx-mode
;;	     xxx-mode-map
;;	     xxx-mode-name
;;	     xxx-mode-define-keys-hook
;;	     )
;;
;;	_3_ Do we have additional files attached to the end of package?
;;      If yes, then we need pgp-tar unpack function too.
;;
;;	    (ti::m-fmacro-install-pgp-tar "xxx-install-files"  "xxx.el")
;;
;;	_4_ Finally the user callable minor mode function is created.
;;
;;	    (ti::m-fmacro-minor-mode
;;	     xxx-mode
;;	     "XXX minor mode. Does fancy things."
;;	     Defined keys:
;;	     \\{xxx-:mode-map}
;;	     "
;;	     xxx-install-mode
;;	     xxx-mode
;;	     xxx-mode-name
;;	     xxx-mode-prefix-key
;;	     xxx-mode-easymenu
;;	     nil			;Yes, print turn on/off message
;;	     "XXX help"
;;	     xxx-mode-hook
;;	     )
;;
;;	That's it. when you execute all these statements you have basic core
;;	for emacs minor mode. The only things misisng is the actual
;;	functions that the minor mode commands uses and the function that
;;	defines keys and menus for the minor mode. You propably want to
;;	start from the function that defines keys and menus. Here is ready
;;	macro for that too.
;;
;;	    (add-hook' xxx-mode-define-keys-hook 'xxx-mode-define-keys)
;;
;;          (ti::m-fmacro-define-keys
;;           "xxx-mode-define-keys"
;;           'xxx-:mode-map
;;           'xxx-:mode-prefix-key
;;
;;           'xxx-:easymenu
;;           'xxx-:easymenu-name
;;           "Programming help menu."
;;           (list
;;            xxx-:easymenu-name
;;            ["Eval whole buffer" xxx-eval-current-buffer    t]
;;            ..
;;            )
;;           '(progn
;;	        (define-key map (concat p "-") 'xxx-eval-current-buffer)
;;	        (define-key map (concat p "=") 'xxx-calculate)
;;	        ..
;;	        )
;;	     )
;;

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun 6      1997	[jari]		19.28	v1.116-1.123	NotReleased
;;   ti::xe-activate-region		:+ show region
;;   ti::m-pkg-autoload-create-on-file  :! serious bug, 'macro autoload args.
;;
;; Jun 5      1997	[jari]		19.28	v1.116-1.119	NotReleased
;; - I removed some functions from 'm' lib and neede to make the
;;   modifications here too. Internal changes. Checkdoc 1.42 clear.
;; - moved around functions a lot and made many defsubst for speed.
;;   ti::s-verify-ends			:! bug; doesn't use ^$ any more.
;;   ti::f-days-old			:! internal change
;;   ti::d-standard-date		:! input parameter changed
;;
;; May 27      1997	[jari]		19.28	v1.112-1.115	NotReleased
;; - Checkdoc 1.29 docstring corrections to all of the code.
;; - minor mode create macros now support defcustom too.
;;
;; May 16      1997	[jari]		19.28	v1.109-1.111	NotReleased
;; - Rearranged byte compiler autoloads. Removed all nreverse calls
;;   (itimer code broke due to it : used list-find)
;;   ti::xe-set-mode-line-format	:+
;;   ti::s-get-blocks			:! push is faster (used append)
;;   ti::s-substitute			:! rewritten
;;
;; May 14      1997	[jari]		19.28	v1.107-1.108	NotReleased
;; - Rewrote minor mode usage instructions. Substituted nreverse with
;;   reverse, because nreverse is destructive and cannot be used to return
;;   list (changes original list passed to function)
;; - Changed all non standard boolean tests to use when and unless
;;   ti::m-fmacro-debug			:+ This is not ready!
;;
;; Apr 18	1997	[jari]		19.28	v1.106		NotReleased
;; - Still lot of errors corrected in the minor mode macros. Also removed
;;   all references to old lis-* add and prepend functions. The weird
;;   byte compiler erro was due to wrong call args to push (1 2 3), when
;;   it only accepts (1 2). No byte compiler errors present.
;; - Removed mouse drag test function.
;;
;; Apr 15	1997	[jari]		19.28	v1.105		NotReleased
;; - Finished the minor mode creating macros and functions. The wizard
;;   now works as expected. If I byte compile this file, I get
;;   some weird error. Don't know why yet.
;; - While compiling toplevel forms in file /users/jaalto/elisp/tinylib.el:
;;   !! Wrong number of arguments ((#<compiled-function (from "cl.elc")
;;   (x place) "...(18)" [place setq cons x callf2] 5 595974> 1)) Done
;;
;;   ti::m-passwd-build-alist		:! reordered body (a bit faster now)
;;   ti::m-passwd-read-entry		:! changed
;;
;; Mar 30	1997	[jari]		19.28	v1.103-1.104	NotReleased
;; - Rewrote the minor mode macros, thanks to Dewey M. Sasser.
;;   All the rest but wizard macro works.
;; - Now got wizard working, but it's in function format. I would like
;;   it to be in macro format...Hm.
;;   ti::s-sqz				:! accepts non-string now
;;
;; Mar 27	1997	[jari]		19.28	v1.102	       NotReleased
;; - Donald_Tang@avanticorp.com reported bug in function
;;   ti::b-copy-line-and-inc-numbers; which didn't handle expanding
;;   line end point. correxted with marker at the end of line.
;;   ti::f-complete-file-name		:! dies if no default-directory
;;
;; Mar 26	1997	[jari]		19.28	v1.101	       NotReleased
;; - new -fmacro- implementations,  DID LOT OF WORK FOR THESE!
;;   The WIZARD macro does not work yet.
;;
;; - Wrote section in the docs how to create minor modes easily
;;   ti::m-vmacro-minor-mode		:+
;;   ti::m-fmacro-define-keys		:+
;;   ti::m-fmacro-minor-mode-wizard	:+
;;   ti::m-pkg-autoload-create-on-file	:! now supports macros
;;
;; Mar 24	1997	[jari]		19.28	v1.98-1.100	NotReleased
;;   ti::f-complete-file-name-word	:! partial completion added
;;   ti::xe-set-frame-name		:! 'name symbol in 19.34
;;   ti::s-case-replace			:! error with puctuation chars.
;;
;; Mar 23	1997	[jari]		19.28	v1.95-1.97	NotReleased
;;   ti::xe-overlay-some		:+
;;   ti::xe-overlay-properties		:+
;;   ti::xe-overlays-at			:+
;;   ti::xe-overlay-put			:+
;;   ti::xe-overlay-move		:+
;;   ti::m-pkg-autoload-create-on-file  :! buffer create bug
;;   ti::xe-popup			:! EVENT set to nil in XEmacs
;;   ti::m-fmacro-install-pgp-tar	:! Added removing old install files.
;;
;; Mar 21	1997	[jari]		19.28	v1.90-1.94	NotReleased
;; - XE20 character handling corrections. Changed NEXTP calls to list-loop
;;   ti::xe-popup			:+ checks XEmacs event type now
;;   ti::m-fmacro-minor-mode		:! no longer hook required
;;   ti::format-time-string		:! bug in new implementation
;;
;; Mar 13	1997	[jari]		19.28	v1.88-1.89	NotReleased
;; - Changed input parameters in -fmacro- macros.
;;   ti::b-diff-type-p			:+ from tinydiff.el
;;
;; Mar 4	1997	[jari]		19.28	v1.86-1.87	NotReleased
;; - XEmacs 20and XE19.15  now reruire EFS, not ange-ftp. now prints
;;   ange-ftp warning message when compiled in XEmacs.
;;   ti::p-tar-zip-view-maybe-command	:+
;;   ti::xe-read-password		:+
;;   ti::m-move-paragraph-to-column	:! bug, didn't move "as is". Ok now
;;
;; Feb 25	1997	[jari]		19.28	v1.85		NotReleased
;;   renamed all ti::b-rcs to ti::m-rcs
;;   ti::m-pkg-autoload-create-on-file	:! rewritten.
;;   ti::k-function-bind-info		:! enhanaced
;;
;; Feb 16       1997    [jari]          19.28   v1.83-1.84      NotReleased
;;   ti::m-pkg-install-pgp-tar          :+ new installation tool
;;   ti::f-days-old                     :! serious bug
;;   ti::d-standard-date                :! removed extra code
;;   ti::p-comint-run-command           :! infinite loop prevent
;;   ti::m-move-paragraph-to-column     :! rewrote
;;
;; Feb 14       1997    [jari]          19.28   v1.82           NotReleased
;;   ti::w-get-buffer-window-other-frame:+
;;   ti::m-move-paragraph-to-column     :+
;;   ti::p-tar-read-listing-forward     :+
;;   ti::xe-mouse-call-original         :+
;;   ti::x-popup                        -> ti::xe-popup
;;   ti::w-frame-list                   :! rewritten
;;   ti::m-pkg-autoload-create-on-file  :! fix, no more ".el" to autoloads
;;
;; Dec 17       1996    [jari]          19.28   v1.81           NotReleased
;;   ti::w-frame-list                   :+
;;   ti::xe-display-depth               :+
;;   ti::xe-frame-window-config         :+
;;   ti::xe-timer-list-control          :+
;;   ti::xe-timer-elt                   :! 19.34 changes
;;   ti::w-list                         :! renamed name
;;   ti::p-http-request                 :! corrected
;;   ti::f-dir-files                    :! corrected
;;
;; Dec 3        1996    [jari]          19.28   v1.80           NotReleased
;;   ti::d-time-diff-days               :+ simple day diff
;;   ti::f-days-old                     :+
;;   ti::s-sqz                          :! bug, corrected
;;
;; Nov 25       1996    [jari]          19.28   v1.77-1.79      NotReleased
;;   ti::p-http-request                 :! now fully functional
;;   ti::b-grep-lines                   :! faster, now uses nconcc
;;   ti::m-mail-abbrevs-get-alist       :! in XEmacs caused infinite loop.fixed
;;
;; Nov 11       1996    [jari]          19.28   v1.76           NotReleased
;;   ti::p-finger-error                 :+
;;   ti::p-http-request                 :+ unfinished
;;   ti::p-finger                       :! better error handling
;;
;; Nov 8        1996    [jari]          19.28   v1.75           NotReleased
;; - Moved all the code from 'q' library to here. 'q' is now dead.
;; - Now all autoloads are in 'm' lib.
;;   ti::p-finger                       :+ finger command in Emacs
;;   ti::m-pkg-autoload-create-on-file  :+
;;
;; Nov 8        1996    [jari]          19.28   v1.74           NotReleased
;;   ti::b-replace-region-with          :+
;;   ti::m-rcs-guess-buffer-version     :! bug, now finds "Revision"
;;   ti::m-rcs-buffer-version           :! bug
;;   ti::m-find-function-name           :! looks at current line in lisp
;;   ti::d-get-std-time-now             :! 19.29+ detected
;;
;; Nov 5        1996    [jari]          19.28   v1.73           NotReleased
;;   ti::m-mail-abbrevs-get-alist       :! rewritten
;;   ti::xe-mail-abbrev-table           :! rewritten
;;
;; Nov 1        1996    [jari]          19.28   v1.72           NotReleased
;; - Moved rcs dleta file code prior the RCS code so that macros
;;   get defined (compiler error)
;;   ti::b-delete-until-non-empty-line  :+
;;
;; Oct 29       1996    [jari]          19.28   v1.68-71        NotReleased
;; - removed '(lambda quotations and used (function. Rearranged lib loads.
;; - Rearranged RCS function under same categories.
;;    ti::m-rcs-guess-buffer-version    :+
;;    ti::xe-set-frame-name             :+ depends on Emacs
;;    ti::m-ad-control                  :! corrected. safe-do in wrong place.
;;
;; [Log deleted]
;;
;; May  6       1995    [jari]          19.28   v1.1            Created

;;}}}

;;; Code:

;;{{{ require

;;; ......................................................... &require ...


(require 'tinylibm)                     ;macro package

(eval-and-compile
  (defvar mail-abbrevs nil "tinylib")   ;silence byteComp

  ;;  Can't autoload timer, because the calls are behind
  ;;  sfuncall --> ByteComp doesn't see them.
  (use-timer)				;XEmacs and Emacs differencies

  (cond
   ((xemacs-p)

    (message "\n\
tinylib.el: ** Library's ange ftp interface is Emacs dependent
            ** and wont' work in XEmacs, which has own Ange/EFS libs.
            **
            ** You can safely ignore byte compilation errors in XEmacs
            ** if they concern missing ange-ftp functions.")


    ;;  Ange-ftp function used in this package won't work in XEmacs.
    ;;  They are primarily used for backgrounf ftp tasks and low
    ;;  level and easy calling of ange functions. Currently used on
    ;;  in one pacakge tinydired.el; which lets you donwload/upload
    ;;  files at the bg in Emacs.
    ;;
    ;;  at least this statement shuts up ByteCompiler a little.
    ;;
    (cond
     ((xemacs-pp)			;XEmacs 19.15+, XEmacs20+
      (require 'efs-auto))		;no longer has ange-ftp
     (t
      (require 'ange-ftp)
      ))

    (defvar mail-aliases nil)
    (autoload 'build-mail-abbrevs                   "mail-abbrevs")
    (autoload 'build-mail-aliases                   "mail-abbrevs")

    (autoload 'read-passwd			    "passwd" t t)
    )
   ((emacs-p)
    (defvar ange-ftp-process-result                 nil)
    (defvar ange-ftp-ascii-hash-mark-size           1024)
    (defvar ange-ftp-binary-hash-mark-size          1024)
    (defvar ange-ftp-process-busy                   nil)

    (autoload 'ange-ftp-process-handle-line         "ange-ftp")
    (autoload 'ange-ftp-get-process                 "ange-ftp")
    (autoload 'ange-ftp-ftp-name                    "ange-ftp")
    (autoload 'ange-ftp-real-file-name-as-directory "ange-ftp")
    (autoload 'ange-ftp-expand-dir                  "ange-ftp")
    (autoload 'ange-ftp-ftp-process-buffer          "ange-ftp")
    (autoload 'ange-ftp-set-binary-mode             "ange-ftp")
    (autoload 'ange-ftp-send-cmd                    "ange-ftp")
    (autoload 'ange-ftp-cd                          "ange-ftp")
    (autoload 'ange-ftp-raw-send-cmd                "ange-ftp")

    (defvar mail-aliases nil)

    (autoload 'mail-abbrevs-setup "mailabbrev")
    (autoload 'build-mail-aliases "mailalias")
    (autoload 'build-mail-abbrevs "mailabbrev")
    ))

  (autoload 'lm-version                 "lisp-mnt")
  (autoload 'lm-summary                 "lisp-mnt")
  (autoload 'lm-maintainer              "lisp-mnt")
  (autoload 'lm-creation-date           "lisp-mnt")
  (autoload 'lm-last-modified-date      "lisp-mnt")
  (autoload 'lm-commentary              "lisp-mnt")
  (autoload 'lm-header                  "lisp-mnt")
  (autoload 'lm-report-bug              "lisp-mnt")

  (autoload 'comint-send-input          "comint")
  (autoload 'comint-check-proc          "comint")
  (autoload 'comint-mode                "comint")


  (autoload 'ad-disable-advice          "advice")
  (autoload 'ad-enable-advice           "advice")
  (autoload 'ad-activate                "advice")

  (autoload 'jka-compr-write-region     "jka-compr")
  (autoload 'word-at-point              "thingatpt")
  (autoload 'reporter-submit-bug-report "reporter")
  (autoload 'delete-rectangle           "rect")

  )


(eval-when-compile (use-dynamic-compilation))




;;}}}
;;{{{ setup: -- variables

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defvar ti:-load-hook nil
  "*Hook run when package is loaded.")

;;; ....................................................... &v-private ...


(defconst ti:-syntax-info
 '(
   (?\ "Whitespace")
   (?- "Whitespace")

   (?w "Word")
   (?_ "Symbol, variables and commands")
   (?. "Punctuation, separate symbols from one another")
   (?( "Open parenthesis")
   (?) "Close parenthesis")
   (?\"  "String quote, string as a single token")
   (?\\ "Escape")
   (?/ "Character quote, only the character immediately following.")
   (?$ "Paired delimiter, like string quote, chars between are not suppressed")
   (?< "Comment starter")
   (?> "Comment ender")
   (?@ "Inherit from standard syntax table")
   )
 "Short syntax definition table ((CLASS . DESC) ..).")


(defvar ti:-ret nil
  "Global return value of this package.")

;;; ........................................................ &v-public ...
;;; User configurable


(defvar ti:-x-coord 170
  "*Default X menu coordinate.")

(defvar ti:-y-coord 170
  "*Default X menu coordinate.")

(defvar ti:-tmp-buffer "*tmp*"
  "*Temporary buffer name.")


(defvar ti:-tmp-dir "/tmp/"
  "*Temporary directory for file operations.
You must have write permission to this")


;; Make this invisible by default, note leading space.
(defvar ti:-passwd-buffer " *passwd-entries*"
  "*Contents of password file.")

(defvar ti:-compression-ext ".gz"
  "*Compress extension used if any command deals with compressed file.
Note, this extension must be jka-compr compatible.")



(defvar ti:-uude-cmd 'ti::p-uude-cmd
  "*Shell command that should accept one argument, FILE, to unpack the UU data.
This variable is evaled, so it can be FORM ie. something that
returns appropriate command.")

(defun  ti::p-uude-cmd (file)
  "Return shell command  'uudecode FILE'."
  (concat "uudecode " file))



;;  At least in HP-UX this command does not work !!
;;  You should replace it with some shell script name that does the job right
;;

(defvar ti:-uu-cmd 'ti::p-uu-cmd
  "*Shell command that should accept one argument, FILE.
This variable is evaled, so it can be FORM
ie. something that returns appropriate command.")

(defun ti::p-uu-cmd (file)
  "Return shell command  'uuencode FILE'."
  (concat "uuencode " file " " file))


(defconst ti:-package "ti::"
  "Library's identifier. For error messages etc...")


;;}}}
;;{{{ setup: -- version

;;; ....................................................... &v-version ...
;;; These are not library funcs, so they have normal 'tinylib-' prefix

(defconst tinylib-version
  (substring "$Revision: 1.123 $" 11 16)
  "Latest version number.")

(defconst tinylib-version-id
  "$Id: tinylib.el,v 1.123 1997/06/26 23:27:43 jaalto NotReleased $"
  "Latest modification time and version number.")


;;; ----------------------------------------------------------------------
;;;
(defun tinylib-version (&optional arg)
  "Show version information. ARG will instruct to print message to echo area."
  (interactive "P")
  (ti::m-pkg-version-info "tinylib.el" arg))

;;; ----------------------------------------------------------------------
;;;
(defun tinylib-submit-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (ti::m-pkg-submit-feedback "tinylib.el"))

;;}}}

;;; ########################################################### &funcs ###

;;{{{ defsubst

;;; ........................................................ &defsubst ...
;;; inlined functions, they must be introduced before used

;;; ----------------------------------------------------------------------
;;;
;;; (ti::s-verify-ends "Z" "\\." "." 'beg)
;;; (ti::s-verify-ends "dir" "/")
;;;
(defsubst ti::s-verify-ends (str re &optional add-str beg)
  "Make sure STR match RE and add ADD-STR string to it when necessary.
if ADD-STR is not given, adds RE to the string.

Default is to check end of string, Optionally BEG of string.
The RE may not include anchors.

Examples:
   making sure directory has ending slash
   (ti::s-verify-ends \"dir\" \"/\")       --> \"dir/\"

   Making sure, time is zero based:
   (ti::s-verify-ends \"7\" \"0\" nil 'beg) --> \"07\"

   This does not give you the rsult you assume!
   because the second parameter, \"  \", is regexp that is tried.
   This function can't know that there is only \" \" space at front,
   since the regexp dind't match.
   Use `ti::s-del-re' if you want exactly 2 spaces to the beginning
   of string.

   (ti::s-verify-ends \" padd\" \"  \" nil 'beg)
   --> \"   padd\"


Return:
  str    ,possibly modified"
  (let* ((RE  (if beg
                  (concat "\\`" re)
                (concat re "\\'")))
         (add (or add-str re))          ;which one to add.
         )
    (if (string-match RE str)
        str
      (if beg
          (concat add str)
        (concat str add)
        ))
      ))



;;; ----------------------------------------------------------------------
;;; - Originally there was own function for this; but now
;;;   it uses general func verify...
;;; - The main purpose of this function is that when you cat words
;;;   together, you can be sure they have COUNT spaces.
;;; - kinda sprintf...
;;;
(defsubst ti::s-add-space (str &optional end count)
  "Add space to the beginning of STR if there isn't one.
Optionally adds to the END. COUNT is by default 1

If string length is 0, do nothing."
  (let* ((count  (or count 1))
         (padd   (make-string count ?\ ))
         )

    (ti::s-verify-ends str padd padd (not end))
    ))



;;; ----------------------------------------------------------------------
(defun ti::s-sqz (string)
  "Squeezes empty spaces around beginning and end of STRING.
If STRING is not stringp, then returns STRING as is."
  (when (stringp string)
    (if (string-match "^[ \t]+" string)
	(setq string (kill-match 0 nil string)))

    (if (string-match "[ \t]+\\'" string)
	(setq string
	      (substring string 0  (match-beginning 0))))
    )
  string
  )


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::s-del-re (re str &optional level)
  "Remove all that match RE from STR at subexpression LEVEL."
  (while (o-string-match re str)
    (setq str (kill-match (or level 0) nil str))
    )
  str
  )



;;; ----------------------------------------------------------------------
;;;
(defsubst ti::s-substitute (input re str2 &optional level count)
  "Make substitution in string.

Input:
  INPUT         ,string beeig processed
  RE            ,regexp
  STR2          ,the replacement
  LEVEL         ,if regexp contains subexpressions. Defaults to 0
  COUNT         ,how many matches to substitute. Default is all.

Return:
  string"
  (let* ((i  0)
	 )
    (or level (setq level 0))

    (while (string-match re input)
      (if (> (inc i) 5000)
	  (error "Substituted string causes circular match. Loop never ends.")
	(inline (setq input (kill-match level str2 input)))
	))
    input
    ))

;;}}}

;;{{{ package private funcs

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-tmp-buffer (&optional clear)
  "Return tinylib package's tmp buffer and optionally CLEAR it."
  (use-tmp-buffer ti:-tmp-buffer clear))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-tmp-dir (file)
  "Return temporary directory for FILE. Eg. for compressing files."
  (concat
   (ti::s-verify-ends ti:-tmp-dir "/")
   (file-name-nondirectory file)
   ))



;;}}}

;;{{{ Conversions

;;; ##################################################### &Conversions ###



;;; ----------------------------------------------------------------------
;;; Try
;;;     (setq str "%s")
;;;     (message str)          ;; suppose you don't know what's in there
;;;
;;; and you get error... but (message (ti::c-format-string str)) does the job
;;;
(defun ti::c-format-string (str)
  "Convert STR to message string, doubling diffucult charactes, like % and \\."
  (let* ((len  (length str))
         (i    0)
         (ret  str)
         ch-string
         extra
         ch
         )
    (cond
     ((string-match "[%\\]" str)        ;only now do
      (setq ret "")
      (while (< i len)
        (setq ch        (aref str i)
              ch-string (char-to-string ch)
              extra     ""
              )
        (if (char= ch ?%)
            (setq extra ch-string))
        (setq ret (concat ret ch-string extra))
        (inc i)
        )))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - I once saw extra ange like colon ":" after
;;;   the HOST, so I added support for it too
;;;
(defun ti::c-url-to-ange-ftp (str)
  "Converts URL STR into ange ftp address.

Eg:
   ftp://ftp.csd.uu.se
   ftp://ftp.csd.uu.se/pub/users/andersl/emacs/folding.el
   ftp://ftp.csd.uu.se:/pub/users/andersl/emacs/folding.el
   ftp://ftp@ftp.csd.uu.se/pub/users/andersl/emacs/folding.el
   ftp://ftp@ftp.csd.uu.se:/pub/users/andersl/emacs/folding.el

   -->
   /ftp@ftp.csd.uu.se:/
   /ftp@ftp.csd.uu.se:/pub/users/andersl/emacs/folding.el

Return:
  string
  nil"
  (let* (ref
         idx login host dir
         ret
         )
    (cond
     (;;
      (o-string-match "ftp:/\\(/.*@\\)\\([^/]+:\\)\\(/.*\\)" str)
      (setq login (read-match 1 str)       ;; case 3
            host  (read-match 2 str)
            dir   (read-match 3 str)
            ret   (concat login host dir))
      )

    (;;
     (and  (o-string-match "ftp:/\\(/.*@\\)\\(.*\\)" str)
           (setq login (read-match 1 str)       ;; case 4
                 ref   (read-match 2 str))
           )
     (setq idx  (ti::s-index ref ?/ ))
     (setq host (or host (substring ref 0 idx)))
     (setq dir  (substring ref idx))
     (setq ret (concat (or login "/ftp@") host ":" dir))
     )

    (;; ftp://ftp.csd.uu.se/pub/users/andersl/emacs/folding.el
     ;; ftp://ftp.csd.uu.se:/pub/users/andersl/emacs/folding.el
     (and (o-string-match "ftp://\\([^@/]+\\)\\(:?/.*\\)" str)
          (setq host (read-match 1 str)
                dir  (read-match 2 str)))
     (setq ret (concat
                "/ftp@" host
                (if (ti::s-index dir ?:) "" ":") ;add colon if needed
                dir))
     )

    (;; ftp://ftp.csd.uu.se
     (and (o-string-match "ftp://\\([^@:]+\\)$" str)
          (setq host (read-match 1 str)))
     (setq ret (concat "/ftp@" host ":/"))
     ))

    ret
    ))




;;; ----------------------------------------------------------------------
;;; - in many C/C++ styles the variables are names so that they start
;;;   with lowercase letters and following ones are catenated + first char
;;;   in upcase.
;;; - Function names traditionally start with uppercase.
;;;
;;;
(defun ti::c-upcase-words-to-variable-names (beg end &optional case-fold)
  "Does following conversion by searhing caps only words in region.

  THE_COLUMN_NAME  --> theColumnName

Input:
  BEG END       ,region bounds
  CASE-FOLD     ,the value of `case-fold-search'. nil means that  the
                 upcase \"words\" are counted only. Non-nil accepts
                 seearching mixed case words."
  (interactive "*r\nP")
  (let* ((case-fold-search      case-fold)          ;; case is significant..
         (ptable                (syntax-table))     ;; previous
         (table                 (make-syntax-table))
         f1 f2
         )

    (save-restriction
      (unwind-protect
          (progn
            (narrow-to-region beg end)
            (PMIN)
            ;;  let's make sure the _ is not in a word class, put it
            ;;  into some other class for now.
            (modify-syntax-entry ?_ "_" table)
            (set-syntax-table table)


            (while (re-search-forward "[A-Z][A-Z_]+" nil t)
              (setq beg (match-beginning 0)
                    end (match-end 0))

              (save-excursion
                (setq f1 (o-looking-at "[ \t]\\|$"))
                (goto-char (1- beg))
                (setq f2 (o-looking-at "[ \t]\\|$"))
                )

              (cond
               ((and f1 f2)
                ;; make first word "lowercase only"
                (goto-char beg)
                (downcase-word 1)

                ;; handle next words, until space/eol/eob is seen
                (while (and (not (eobp))
                            (not (o-looking-at "[ \t]\\|$")))
                  ;; Remove that underescore
                  ;; Capit. command moves forward while doing
                  (and (o-looking-at "_")
                       (delete-char 1))
                  (capitalize-word 1)
                  ))))
            ;; ... ... ... ... ... ... ... ... ... ... ... .. unwind end . .
            ;;  Now, make sure the old table is restored,
            ;;  the unwind protects against Ctrl-g
            ;;
            (set-syntax-table ptable)
            )))
    ;; let-defun end
    nil
    ))






;;; ----------------------------------------------------------------------
;;;
(defsubst ti::c-get-nth-str (nbr)
  "Return string representing NBR position: st, nd, th.

Input:
  string or number in digit form.

Return:
  \"st\", \"nd\", \"th\""
  (if (stringp nbr)
      (setq nbr (string-to-int nbr)))
  (cond
   ((eq nbr 1) "st")
   ((eq nbr 2) "nd")
   ((eq nbr 3) "rd")
   ((>  nbr 3) "th")
   (t (error "invalid ARG" nbr))
   ))


;;; ----------------------------------------------------------------------
;;; #todo
;;; - Did 19.29+ change the current-time function? Oh my...say no?
;;;   --> should handle it if the format changed.
;;;
(defun ti::c-time-list (&optional zero-form time-string)
  "Return list of elements derived from `current-time'.
Input:
  ZERO-FORM	make sure numbers have preceeding zeroes. Like 7 --> 07
  TIME-STRING	user supplied time string in `current-time' format.

Return list form: \( dd mm ...\)
  0 dd     nbr,  day                  ,if zero-form: 2str
  1 mm     nbr,  month                ,if zero-form: 2str
  2 yy     2nbr, year
  3 tt     2nbr, hh:mm
  4 wd     3str, week day, string like 'Mon'
  5 m      str,  month, full string
  6 yyyy   4str, whole year"
  (interactive)
  (let (time m mm dd yy tt wd yyyy)
    (setq time (or time-string
                   (current-time-string)))  ;"Wed Oct 14 22:21:05 1987"
    (setq wd (substring time 0 3))

    (setq m  (substring time 4 7))
    (setq mm (or (ti::d-s-month-n m) 0))

    ;;    we remove trailing space  "2 " --> 2 --> "2"
    (setq dd (string-to-int (substring time 8 10)))
    (setq tt (substring time -13 -8))

    (setq yy (substring time -2 nil))
    (setq yyyy (substring time -4 nil))

    (cond
     (zero-form                         ;convert "7" --> "07"
      (setq dd (int-to-string dd))
      (setq mm (int-to-string mm))

      (if (not (eq (length dd) 2))
          (setq dd (concat "0" dd)))
      (if (not (eq (length mm) 2))
          (setq mm (concat "0" mm)))
      ))

    (list dd mm yy tt wd m yyyy)
    ))



;;; ----------------------------------------------------------------------
;;; - This is mainly used, if you read the regexp from the buffer:
;;;   obviously you can't just pick it from there:
;;;
;;;             "find this\t+"
;;;                       ^^
;;;   and use it in re-search-XXX commands. See function  ti::b-get-re
;;;   which does the conversion automatically by calling these functions.
;;;
(defun ti::c-ch2escape (item)
  "Converts ITEM to escape sequence \"t\" --> \"\\t\".

Input:
  item       ,integer, character, or single string

Return:
  nil        ,if cannot identify ITEM.
  string     ,escape char"
  (let* (el ret
         (table
          '(("a" . 7)
            ("b" . 8)
            ("f" . 12)
            ("n" . 10)
            ("r" . 13)
            ("t" . 9)
            ("v" . 11)
            )
          ))
    (if (integerp item)
        (setq item (char-to-string item)))
    (if (setq el (assoc item table))
        (setq ret (char-to-string (cdr el))))
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::c-str2re (str)
  "Convert slashes in STR \\\ --> \.
If you read from buffer two some special characters, it can't be
used like that right a way for regexp. Eg in buffer \\\\ two slashes mean
one slash actually when assigned to string to form the regexp."
  (let* ((ret           "")
         (i             0)
         (len           (length str))
         (look-ch       ?\\)
         (prev-ch       ?d)                     ;just some dummy
         (count         0)
         chs

         ch
         )
    (while (< i len)
      (setq ch      (aref str i)
            chs     (char-to-string ch))


      (if (eq ch look-ch)       ;add counter when EQ
          (inc count))

      (cond
       ((eq count 2)                    ;two successive ?
        (if (eq prev-ch look-ch)
            (setq count 0)              ;delete second
          (setq ret (concat ret chs))
          (setq count 0)
          ))

       ((eq count 1)
        (if (eq ch look-ch)
            ;;  Right now it was found
            (setq ret (concat ret chs))
          ;; - Count is still 9, but we aren't looking at double \\ ?
          ;;   --> there is \t sequence
          ;; - we revove last char and put our sequence instead
          (setq ret (concat
                     (substring ret 0 (1- (length ret)))
                     (ti::c-ch2escape chs)
                     ))
          (setq count 0)))

       (t
        (setq ret (concat ret chs))
       ))

      (setq prev-ch ch )
      (inc i)
      )
    ret
    ))




;;; ----------------------------------------------------------------------
;;; arc.mode.el  -- This is direcly from 19.28 distrib.
;;;
(defun ti::c-int-to-access-mode-string (mode)
  "Turn an integer MODE, 0700 (i.e., 448) into a mode string like -rwx------."
  (let ((str (make-string 10 ?-)))
    (or (zerop (logand 16384 mode)) (aset str 0 ?d))
    (or (zerop (logand  8192 mode)) (aset str 0 ?c)) ; completeness
    (or (zerop (logand   256 mode)) (aset str 1 ?r))
    (or (zerop (logand   128 mode)) (aset str 2 ?w))
    (or (zerop (logand    64 mode)) (aset str 3 ?x))
    (or (zerop (logand    32 mode)) (aset str 4 ?r))
    (or (zerop (logand    16 mode)) (aset str 5 ?w))
    (or (zerop (logand     8 mode)) (aset str 6 ?x))
    (or (zerop (logand     4 mode)) (aset str 7 ?r))
    (or (zerop (logand     2 mode)) (aset str 8 ?w))
    (or (zerop (logand     1 mode)) (aset str 9 ?x))
    (or (zerop (logand  1024 mode)) (aset str 3 (if (zerop (logand 64 mode))
                                                    ?S ?s)))
    (or (zerop (logand  2048 mode)) (aset str 6 (if (zerop (logand  8 mode))
                                                    ?S ?s)))
    str))



;;}}}
;;{{{ Version control, RCS delta files

;;; ....................................................... &rcs-delta ...
;;; In general, do not use these function, but use the top-level ones
;;; that deal with filenames or buffers.

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-delta-get-revisions  (&optional buffer)
  "Parse all revision numbers from delta file BUFFER.

Return:
  '(nbr nbr ..)"
  (let* (list
         )
    (save-excursion
      (if buffer
          (set-buffer buffer))
      (PMIN)
      (while (re-search-forward "^[0-9.]+" nil t)
        (push (read-match 0) list)
	))
    (reverse list)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-delta-get-file (file &optional buffer)
  "Read delta FILE to temporary BUFFER.
The delta file is truncated to header info only.

Errors:
  Generates error if file is not vc registered.

Return:
  buffer"
  (let* ((buffer     (or buffer (ti::m-tmp-buffer)))
         (rcs-name   (vc-name file))
         )
    (if (or rcs-name
            (error "Not an RCS file. %s" file))
      (save-excursion
        (set-buffer buffer)
        (erase-buffer)

        (if (fboundp 'vc-insert-file)   ;19.30
            (sfuncall 'vc-insert-file rcs-name "^desc")

          (insert-file-contents rcs-name)

          (buffer-disable-undo)
          (set-buffer-modified-p nil)
          (auto-save-mode nil)

          (if (re-search-forward "^desc" nil t)
              (delete-region (point) (point-max)))
          )))
    buffer
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-delta-lock-status (&optional user-name)
  "Return lock status by reading the delta buffer.
If USER-NAME is non-nil return locks for that user.

Return:
 ((USER . (VER VER ..))  (U . (V V)) ..)
 nil"
  (let (
        user ver
        ret
        )
    ;; Must succeed, otherwise this is not delta buffer.
    ;;
    (save-excursion
      (PMIN)
      ;; locks
      ;;       jaalto:1.13; strict;
      ;; comment        @; @;

      (re-search-forward "^locks")
      (forward-line 1)
      (while (looking-at "[ \t]+\\([^:]+\\):\\([^;]+\\)")
        (setq user (read-match 1)
              ver  (read-match 2))

        (if (or (null user-name)
                (string-match-case (regexp-quote user-name) user))
            (cond
             ((assoc user ret)  ;already a user in list
              (list-c-add-cdr 'assoc user ret ver))
             (t
              (if (null ret)
                  (setq ret (list (cons user (list ver))))
                (push (cons user (list ver)) ret ))
;;;       (d! ">>" ret)
              )))

        (forward-line 1)
        ))
    (reverse ret)
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-delta-lock-status-user (user)
  "Return list of locks for USER.
This is toplevel function to `ti::m-rcs-delta-lock-status'.
Please use it directly if you want other users information too.
If you only need *one* users information, use this function, because
it hides the lock data structure.

Return:
 (VER VER ..)    ,list of version strings.
 nil"
  ;; this always parses the buffer.
  (cdr-safe (assoc user (ti::m-rcs-delta-lock-status))))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-delta-highest-version  ()"Return the highest version from delta buffer."
  (interactive)
  (save-excursion
    (PMIN)
    (if (re-search-forward "head[ \t]+\\([.0-9]+\\)" nil t)
        (read-match 1))
    ))

;;}}}
;;{{{ Version control, buffer's RCS strings

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-str-find (str &optional mode)
  "Try to find rcs string STR starting from the point forward.

Return:
    By default whole string is returned.
    If MODE is non-nil, the value of rcs identifier is returned."

  ;;  Keep in mindt that RCS keywords can be placed like this:
  ;;
  ;;	    $Revision: 1.123 $ / $Date: 1997/06/26 23:27:43 $
  ;;
  (let* ((re (concat "[$]" str ":[^$]+[$]"))
         ret
         )
    (if (null (re-search-forward re nil t))
        nil
      (setq ret (read-match 0))
      (if (null mode)
          ret
        (ti::s-rcs-read-val ret))
      )))

;;; ----------------------------------------------------------------------
;;; - In fact this should be macro, defsubst
;;;
(defsubst ti::m-rcs-str-find-buffer (str &optional mode)
  "Try to find rcs string STR starting from `point-min'.
Return:
    By default whole string is returned.
    If MODE is non-nil, the value of rcs identifier is returned.

Example call:
  (ti::m-rcs-str-find-buffer \"Id\" 'value)"
  (save-excursion
    (safe-widen
     (PMIN)
     (ti::m-rcs-str-find str mode)
     )))

;;}}}
;;{{{ Version control, string, RCS information
;;; ............................................................. &rcs ...
;;; Refer to GNU RCS ident(1) how to construct valid identifiers.
;;;


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::s-rcs-read-val (str)
  "Cleans the RCS identifiers from the STR and return the value."
  (let* ((re ".*[$][^ \t]+: \\(.*\\) [$]")
         )
    (if (and (stringp str)
             (string-match re str))
        (read-match 1 str)
      nil)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::s-rcs-look-id (str)
  "Return the RCS identifier in STR."
  (let* ((re ".*[$]\\([^ \t]+\\): .* [$]")
         )
    (if (string-match re str)
        (read-match 1 str)
      nil)
    ))


;;}}}
;;{{{ Version control, RCS

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-file-p (file)
  "Return t if FILE STRING is in RCS controlled form.
That is, if FILE has ,v at the end."
  (and (> (length file) 2)
       (string= (substring file -2) ",v")))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-file-exists-p (file)
  "Return t if equivalent RCS FILE can be found.
If the following condition is met, then such file exists:
  ~/dir1/dir2/file.cc     --> ~/dir1/dir2/RCS/file.cc,v"
  (let* ((rcs (ti::m-rcs-make-filename file))
         )
    (file-exists-p rcs)
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-make-filename (file )
  "Constructs RCS controlled FILE name."
  (let* (ret
         fn
         dir
         )
    (cond
     ((ti::m-rcs-file-p file)
      (setq ret file))
     (t
      (file-components file dir fn 'default)
      (setq ret (concat dir "RCS/" fn ",v"))
     ))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-normal-file (rcs-file)
  "Return normal file when version controlled RCS-FILE is given."
  (let* (( case-fold-search nil)
         )
    (cond
     ((ti::m-rcs-file-p rcs-file)
      (setq rcs-file (ti::s-del-re "RCS/"  rcs-file))
      (setq rcs-file (ti::s-del-re ",v"  rcs-file))
      ))
    rcs-file
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-sort-same-level-list (list)
  "Sort RCS revision LIST, which are at same level.
Ie. when only the last version number changes:
1.1 1.2 1.3, or 1.2.1.1 1.2.1.3 1.2.1.10"
  (let* ((max 0)
         ptr
         new-list
         len
         ret
         padd str
         elt
         )

    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. greatest ...
    ;;
    (setq ptr list)
    (while ptr                          ;find greatest. 1.xx
      (setq max (max (length (car ptr)) max))
      (NEXTP ptr))


    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... .. padd ...
    ;;
    (setq ptr list    new-list nil)

    (list-loop ptr elt			;padd 1.1 --> 1.01
      (setq len (length elt))

      (cond
       ((not (eq len max))
        (setq padd (make-string (- max len) ?0))
        (string-match "[0-9]+$" elt)

        (setq str (read-match 0 elt) )
        (setq elt (kill-match 0 (concat padd str) elt))

        ))
      (push elt new-list)
      )

;;;    (setq NL (copy-list new-list))

    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... .. sort ...
    ;;
    (setq new-list (sort new-list 'string<))

    ;; Check if the values are doubled, and only then fix the list.
    ;; Hmm, if this happens, then the error is not in the 'sort'
    ;; but somewhere else.

;;;    (cond
;;;     ((and new-list (string= (nth 0 new-list)
;;;                          (nth 1 new-list)))
;;;      (setq new-list (list-remove-successive new-list 'string=))
;;;      ))

    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... remove padd ...
    ;;
    (setq ptr new-list)
    (list-loop
        ptr elt                         ;fix 1.01 > 1.1
      (cond
       ((string-match "\\.\\(0+\\)[1-9][0-9]*$" elt)
        (setq elt (kill-match 1 "" elt))
        ))
      (push elt ret)
      )

    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-files-in-dir (&optional dir re)
  "Return all RCS controlled files in directory DIR.
It doesn't matter if the directory points to RCS itself or
one level up. Thus the two DIR parameters are identical:

    /mydir/             ,takes longer to execute.
    /mydir/RCS/

The DIR parameter can hold filename, but only the directory portion
is used. If no directory portion exists \"./\" will be used.

Filenames returned do not have any  \",v\" extensions or directories.

Optional RE tells to return files matching RE only.


Return:
 list           ,(file file ..)"
  (let* ((re (or re "."))               ;default to match all
         d
         fn
         fnn
         list
         ret
         )
    (if (null (file-directory-p dir))
        (error "Not a directory"))
    (setq d (or (and dir
                     (or (file-name-directory (expand-file-name dir))
                         "./"))
                "./"))
    (cond
     ((string-match-case "RCS/?" d)
      (setq list (directory-files d nil re))
      (while list
        (set fn (ti::s-del-re ",v$" (car list)))
        (push fn ret)
        (NEXTP list)))
     (t
      (setq list (directory-files d nil re))
      (list-loop list fn
        (setq fnn (concat d fn))        ;with directory

        (if (and (not (file-directory-p fnn))
                 (ti::m-rcs-file-exists-p (concat d fn)))
            (push fn ret))
	)
      ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - The vc. does not return the _version_ latest.
;;;   See vc-hook/ vc-fetch-properties
;;;
(defsubst ti::m-rcs-head-version  (file)
  "Get latest version, the head, for FILE.
Return:
  string    ,version string
  nil       ,not an rcs file"
  (let* (ver
         buffer
         )
    (save-excursion
      ;;  This may not be RCS file
      ;;
      (when (setq buffer (safe-do (ti::m-rcs-delta-get-file file)))
        (set-buffer buffer)
        (setq ver (ti::m-rcs-delta-highest-version))
        ))
    ver
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-guess-buffer-version  (file &optional user)
  "Try to guess right version number for buffer visiting FILE.
If file is locked, look at delta log to find out version, otherwise call
`ti::m-rcs-buffer-version' and consult vc if needed.

Input:

  FILE      ,file name
  USER      ,rcs user name, defaults to (user-login-name)

Return:
  string
  nil"
  (let* ((user (or user (user-login-name)))
         buffer
         list
         ver
         )
  (when (not buffer-read-only)          ;It's Checked Out
    ;; Never trust the ID string in the buffer, always look
    ;; at delta file --> this may be checked out with -k and
    ;; then RCS strings are not updated.
    ;;
    (setq buffer (ti::m-rcs-delta-get-file file))
    (job-buffer buffer
      ;; We're interested in current user's locks only
      ;;
      (setq list (ti::m-rcs-delta-lock-status user))
      ))

  (cond
   ((and list
	 (eq 1 (length list))
	 (setq list (cdr (car list)))
	 (eq 1 (length list)))
    ;; Okay, only 1 version locked, then we're safe
    (setq ver (car list))
    )
   (t
    (setq ver
	  (or (save-excursion
		(set-buffer (get-file-buffer file))
		(ti::m-rcs-buffer-version))
	      (vc-file-getprop file 'vc-workfile-version)
	      nil
	      ))
    ))
;;;  (d! ">>" ver done )
  ver
  ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-buffer-version (&optional buffer)
  "Return version number for optional BUFFER.
Supposes that RCS string 'Revision' 'Id' or 'Log' exist.
If they do not exist, then see if VC is loaded and look at the modeline.

Please use `ti::m-rcs-guess-buffer-version' and not this function."
  (let* (rev
         tmp
         )
    (save-excursion
      (if buffer
          (set-buffer buffer))
      (safe-widen
       (PMIN)
       (cond
        ((setq tmp (ti::m-rcs-str-find "Revision"))
         (setq rev (ti::s-rcs-read-val tmp)))

        ((ti::m-rcs-str-find "Log" )
         (forward-line)
         (setq rev (bufmatget ".*Revision +\\([0-9.]+\\)" 1))
         )

        ((setq tmp (ti::m-rcs-str-find "Id" 'value))
         (setq rev (nth 1 (ti::s-get-blocks tmp "[ ]")))
         ))))

    ;;  See if VC is installed and ask from it then.
    (if (and (null rev)
             (fboundp 'vc-mode-line))
        (setq rev (strmatget  "[^.0-9]*\\([.0-9]+\\)" 1
                              (or (symbol-value 'vc-mode) ""))))

    rev
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-rlog-get-revisions ()
  "REad all revision numbers from rcs rlog buffer.
The line searched looks like:

   revision 1.10   locked by: loginName;
   revision 1.9

Return:
  list    ,revision numbers
  nil"
  (let* ((re   "^revision[ \t]+\\([.0-9]+\\)$")
         ver
         list
         )
    (save-excursion
      (PMIN)
      (while (re-search-forward re nil t)
        (if (setq ver (read-match 1))
            (push ver list)))
      )
    (reverse list)
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-rcs-all-versions  (file)
  "Return string list of all version numbers for FILE."
  (let* (info-buffer
         )
    (setq info-buffer   (ti::m-rcs-delta-get-file file))
    (save-excursion
      (set-buffer info-buffer)
      (ti::m-rcs-delta-get-revisions))
    ))

;;; ----------------------------------------------------------------------
;;; For big files this is real slow, since building up lists and
;;; sort the revisions is hard
;;;
(defun ti::m-rcs-previous-version (version v-list)
  "Return previous version for FILE.
Do not call this function Often, since it may be quite time consuming.

Input:
  VERSION       ,lever as string, eg. \"1.5\"
  V-LIST	,all version numbers for file, order not significant.

Return:
  RCS tree      previous version
  1.5           1.4
  1.4           1.3
  1.3           1,2
    1.3.1.1     1.3
    1.3.1.2     1.3.1.1
  1.2           1.1
  1.1           nil"
  (let* (
         branch-list
         list tmp elt
         ret
         )

    (setq branch-list   (ti::m-rcs-get-all-branches version v-list))
;;;    (setq BL branch-list VL v-list V version)

    (cond
     ((null branch-list)
      ;; record the error to 19.29+ *message* buffer
      (message "tii:rcs This level does not have version? %s" version)
      )

     ;; after 1.1.1.1 we go up one level, to 1.1
     ((setq ret (strmatget"\\([.0-9]*\\).1.1$" 1  version)))

     (t
      (setq list branch-list    tmp nil)

      (list-loop list elt
        (if (string= elt version)
            (setq list nil   ret tmp) ;stop there
          (setq tmp elt)
	  ))
      ))
    ret
    ))




;;; ----------------------------------------------------------------------
;;;
(defun ti::m-rcs-get-all-branches (rev rev-list)
  "Return sorted braches, lowest first, at same revion level.

Input:
  REV           ,version number string
  REV-LIST      ,list of version numbver string

Example:
  if version is 1.2,     return all 1.x     branches
  if version is 1.2.1.1, return all 1.2.1.x branches"
  (let* (list
         val
         )


    (if (null val)                      ;Quiet XEmacs 19.14 ByteComp
        (setq val (strmatget ".*\\." 0 rev)))   ;remove last number

    (setq
     list
     (list-find rev-list
                rev
                ;;  - The count thing just makes sure we get
                ;;    1.1  and 1.2  , not 1.1.1.1
                ;;  - match makes sure that the start of the string is same
                ;;    1.  --> 1.2 1.3 1.4
                '(if (and (eq (count-char-in-string ?. arg )
                              (count-char-in-string ?. (car table)))
                          (string-match val (car table)))
                     (setq element (car table)))
                'all-matches
                ))

    (cond
     (list
      ;; Simple (setq list (sort list 'string<)) won't do the job,
      ;; since it claims 1.10 is before 1.9
      ;;
      ;; 1.1
      ;; 1.10           ;; see ?
      ;; 1.2
      ;; 1.9
      ;;
      (setq list (ti::m-rcs-sort-same-level-list list))
      ))

    list
    ))

;;}}}

;;{{{ Date

;;; ............................................................ &date ...

;;; ----------------------------------------------------------------------
;;; #defalias  (defalias 'time-now 'ti::d-standard-date)
;;;

(when (fboundp 'format-time-string)           ;19.29+
  (defun ti::d-standard-date (&optional short time)
  "Return international time 'Nov 07 1995 20:49' or in SHORT
Input:
  SHORT		return YYYY-MM-DD instead which conforms to ISO 8601.
  TIME-STRING	User supplied string in format `current-time-string'.
"
  (if short
      (format-time-string "%Y-%m-%d" (or  time (current-time)))
    (format-time-string "%b %d %Y %H:%M"
			(or  time (current-time)))
    )))

;;; ---
(unless (fboundp 'format-time-string)
(defun ti::d-standard-date (&optional short time)
  "Return international time 'Nov 10th 1995 20:49'.
Input:
  SHORT	return YYYY-MM-DD instead which conforms to ISO 8601.
  TIME	User supplied time in format `current-time'."
  (interactive "P")
  (let* ((l  (ti::c-time-list nil (current-time-string
				   (or time (current-time)))))
         nbr
         )
    (cond
     (short
      (setq nbr (cdr (assoc (nth 5 l) (month-mm-alist))))
      (concat (nth 6 l) "-"
              nbr "-"
              (int-to-string (nth 0 l))
              ))
     (t
      (concat (nth 5 l) " "
              (int-to-string (nth 0 l))
              (ti::c-get-nth-str  (nth 0 l)) " "
              (nth 6 l) " "
              (nth 3 l)))
     )))
)




;;; ----------------------------------------------------------------------
;;;
(defun ti::d-s-month-n(arg &optional mode)
  "Return month number for string or vice versa.

When MODE is nil
  Accepts Jan or January with any case     --> Return nbr or nil
When MODE is non-nil
  Accepts nbr or str-nbr                   --> return str or nil
"
  ;; (interactive)
  (let (
        (alist
         '(("jan" . 1)    ("feb" . 2)     ("mar" . 3)     ("apr" . 4)
          ("may" . 5)     ("jun" . 6)     ("jul" . 7)     ("aug" . 8)
          ("sep" . 9)     ("oct" . 10)    ("nov" . 11)    ("dec" . 12)))
        len idx el
        ret str
        )
    (cond
     ((eq nil mode)
      (setq len (length arg))
      (if (> len 3) (setq arg (substring str 0 3)))  ; cut to 3 chars
      (setq idx (downcase arg))

      (if (setq el (assoc idx alist))
        (setq ret (cdr el))))
     (t
      (if (stringp arg) (setq arg (string-to-int arg)))
      (setq idx arg)

      (if (setq el (rassq idx alist))
          (setq ret (car el))))
     )
    ret
    ))



;;; ----------------------------------------------------------------------
;;;  05 Jun 1995, mucit@cs.rochester.edu (Bulent Murtezaoglu), comp.emacs
;;;
(defsubst ti::d-time-difference (a b)
  "Calculate difference beween times A and B.
The input must be in form of '(current-time)'"
  (let ((hi (- (car a) (car b)))
        (lo (- (car (cdr a)) (car (cdr b)))))
    (+ (lsh hi 16) lo)))



;;; ----------------------------------------------------------------------
;;;
(defun ti::d-time-diff-days  (std1 std2)
  "Return approximation of time difference in days.
STD1 and STD2 are two standard times in short format YYYY-MM-DD.
In calculation each month is supposed to have 30 days and a year 356 days."
  (let ((re  "\\([0-9][0-9][0-9][0-9]\\)-\\([0-9]+\\)-\\([0-9]+\\)")
        y1 m1 d1
        y2 m2 d2
        ret
        )
    (string-match re std1)
    (setq y1 (string-to-int (read-match 1 std1))
          m1 (string-to-int (read-match 2 std1))
          d1 (string-to-int (read-match 3 std1))
          )

    (string-match re std2)
    (setq y2 (string-to-int (read-match 1 std2))
          m2 (string-to-int (read-match 2 std2))
          d2 (string-to-int (read-match 3 std2))
          )

    (if (>= (- d2 d1) 0)                ;day2 is smaller
        (setq ret (- d2 d1))
      (setq ret (- (+ 30 d2) d1))
      (dec m2)
      )

    (inci ret (* 30  (- m2 m1)))
    (inci ret (* 356 (- y2 y1)))
    ret
    ))


;;}}}
;;{{{ string(s), chars

;;; ########################################################## &string ###


;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'string-repeat 'ti::s-repeat)
;;;
(defun ti::s-repeat (count char-or-string)
  "Repeat COUNT times CHAR-OR-STRING."
  (let* ((i 0)
         ret
         )
    (if (integerp char-or-string)
        (setq ret (make-string count char-or-string))
      (setq ret "")
      (while (< i count)
        (setq ret (concat ret char-or-string))
        (inc i)))
    ret
    ))

;;; ----------------------------------------------------------------------
;;; #defalias  (defalias 'syntax-info 'ti::s-syntax-info)
;;;
(defun ti::s-syntax-info (char &optional verb)
  "Return brief syntax definition string for CHAR. VERB."
  (interactive "cShow syntax of char: ")
  (let* ((syntax (char-syntax char ))
         (elt    (assq syntax ti:-syntax-info))
         (verb   (or verb (interactive-p)))
         ret
         )
    (setq ret
          (concat
           (char-to-string syntax)
           " "
           (if elt  (nth 1 elt) "")))
    (if verb
        (message ret))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::s-tabify (string &optional mode)
  "Tabify STRING, or if MODE is non-nil, untabifies."
  (let* ((indent-tabs-mode t)           ;makes sure tabs are used.
         )
    (cond
     (string                            ;only if non-nil
      (save-excursion
        (set-buffer (ti::m-tmp-buffer))
        (erase-buffer)
        (insert string)
        (if (null mode)
            (tabify (point-min) (point-max))
          (untabify (point-min) (point-max)))

        (buffer-string)
        )
      ))
    ))



;;; ----------------------------------------------------------------------
;;; - This is slightly different than the next one. Use the one you need.
;;;
(defun ti:s-read-match-subs (level-list &optional string terminate)
  "Return matcg list according to subexpression list LEVEL-LIST.

Supposes that you have already done the matching. If STRING is not
given, the buffer will be used for reading.

If optional TERMINATE is non-nil, terminates if any of the matches return
nil. In this case the return list will be empty signifying that all matches
weren't satisfied.

Input:
  level-list    list    eg.   '(1 0 2)
  string        str     eg.   \"testThis\"

Return:
 ( \"str\" nil \"str\" .. )
 nil                    ,see TERMINATE"
  (let* (level
	 ret
         str
         )
    (list-loop level-list level
      (setq str (read-match level string))
      (if (and terminate (null str))
          (setq ret nil   level-list nil) ;that's it then...
        (push str ret)
        ))
    (reverse ret)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti:s-read-match-list (match-list level-list string &optional terminate)
  "Return match list list according to subexpressions.

Input:

  MATCH-LIST    list    eg.   '(\"\\(re1\\)\" \"re2\" \"\\(cash\\(re3\\)\\)\"
  LEVEL-LIST    list    eg.   '(1 0 2)
  STRING        str     eg.   \"re1 re2 cashre3\"
  TERMINATE     any     eg.   nil, 'terminate

Supposes that you have already done the matching.

If the match wasn't found in current level, it assign nil to the
corresponding position in return list

If optional TERMINATE is non-nil, terminates if any of the matches return
nil. In this case the return list will be empty signifying that all matches
weren't satisfied.

Return:
 ( \"str\" nil \"str\" .. )
 nil                    ,see TERMINATE"
   (let* (ret
          str
          )
     (if (not (eq (length match-list)
                 (length level-list)))
         (error "List length mismatch."))

     (while level-list
;;;       (d! (car match-list) (car level-list))
       (setq str (strmatget (car match-list) (car level-list) string))

       (if (and terminate (null str))
           (setq ret nil   level-list nil) ;that's it then...
         (push str ret)
         )
       (NEXTP level-list)
       (NEXTP match-list)
       )
     (reverse ret)
     ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::s-case-replace (model str &optional symmetry rest-case)
  "Use MODEL and change case of characters in STR.
Preserve case if SYMMETRY is non-nil.

Eg. If your input is:

        model:          BARMAN
        str  :          Foomanager

and the symmetry is non-nil, you get

        output:         FOOMANager


If the model is too short the variable REST-CASE instructs what to do
  nil      -->  the rest of the STR will be added \"as is\"
  'follow  -->  the rest of the STR are in the same case as last
                char in MODEL
  'lower   -->  insert rest as lowercase
  'upper   -->  insert rest as uppercase"
  (let* ((i	    0)
	 (part	    "")
	 case-fold-search		;case is important
	 last
         len
         ret
         ch ch-model
         )
    (if (null symmetry)
        str                             ;don't care
      (setq len (min (length str) (length model))
            ret "")


      ;; ............................................ MODEL characters ...
      (while (< i len)
	(setq ch-model	(char-to-string (aref model i))
	      ch	(char-to-string (aref str i)))

	(cond
	 ((string-match "[a-z]" ch-model)
	  (setq ch (downcase ch)   last 'downcase))
	 ((string-match "[A-Z]" ch-model)
	  (setq ch (upcase ch)   last 'upcase))
	 (t
	  ;; MODEL has punctuation, choose previous case
	  ;;
	  (if (eq last 'upcase)
	      (setq ch (upcase ch))
	    (setq ch (downcase ch)))
	  ))

	(setq ret (concat ret ch))
        (inc i)
        )

      ;; ............................................. REST characters ...
      ;;  if MODEL is too short, then determine what to do to the rest
      ;;  of the characters theat are left.
      ;;
      ;;
      (when (< (length model) (length str))	;Need to guess REST model?
	(setq part (substring str len))
        (cond
	 ((eq rest-case 'follow)
	  (setq ch (char-to-string (aref model (1- len)))) ;read last char

	  (cond
	   ((string-match "[a-z]" ch)
	    (setq part (downcase part)))
	   ((string-match "[A-Z]" ch)
	    (setq part (upcase part)))
	   (t
	    ;; kast char was punctuation, choose last type
	    ;;
	    (if (eq last 'upcase)
		(setq part (upcase part))
	      (setq part (downcase part)))

	    )))

         ((equal rest-case 'upper)
          (setq part (upcase part)))

         ((equal rest-case 'lower)
          (setq part (downcase part)))
         ))



      (setq ret (concat ret part))
      ret
      )))


;;; ----------------------------------------------------------------------
;;; 18 Dec 1994, gnu.emacs.help, nj104@amtp.cam.ac.uk (Neil Jerram)
;;; - slightly modified by [jari]
;;;
(defun ti::s-index (str char &optional reverse)
  "Check STR and first CHAR position 0..nbr.
If REVERSE is non-nil, start searching at the end of string."
  (let ((len (length str))
        (i   -1)
        )
    (cond
     (reverse
      (while (and (>= (dec len) 0)
                  (/= (aref str len) char))) ;check character in string
      (if (>= len 0)
          len
        nil))
     (t
      (while (and   (< (inc i) len)
                    (/= (aref str i) char)))
      (if (< i len)
          i
        nil))
     )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::s-index-substring (str char &optional include right seek-end)
  "Return left hand substring from STR maching CHAR.
Input:
  INCLUDE   The CHAR itself is included too.
  RIGHT	    Return right hand portion.
  SEEK-END  Search from the end.

Example:

    ;; To get only the file part, you'd say
    ;;
    (setq string \"user@site:~/bin/myfile\")
    (ti::s-index-substring string ?: nil 'right)

    ;; To get last item, separated by |
    ;;
    (setq string \"aa|bb|cc|dd\")
    (ti::s-index-substring string ?| nil 'right 'seek-end)

Input:
  str           ,string
  char          ,character to look in string
  include       ,flag, should char be included too?
  right         ,return right side of string
  seek-end      ,start looking the position from the end instead

Return:
  str   ,if ch found
  nil   ,no ch found, or impossible condition. Like if input STR is \":\"
         and don't want to include ?: character."

  (let (idx
        ret
        )
    ;;   common mistakes, prevent it immediately, because
    ;;   looking the cause in debuffer may be a bit hairy, due to
    ;;   breakout only in ti::s-index
    ;;
    (if (not (and str char))
        (error "parameter error %s %s" str char))

    (if (null (setq idx (ti::s-index str char seek-end)))
        nil
      (cond
       (right
        (setq ret (substring str
                             (if include
                                 idx
                               (1+ idx)))))
       (t
;;;     (d! str include idx)
        (setq ret (substring str
                             0
                             (if include        ;; the second parameter
                                 (1+ idx )
                               idx)))
        )))


    (if (var-nil-p ret)                 ;do not return empty strings
        nil
      ret)
    ))





;;; ----------------------------------------------------------------------
;;; - This is handy many times.
;;;
(defun ti::s-1-space (str)
  "Convers all spaces/tabs in STR into 1 space."
  (let* ((out "")
         beg end
         )
    (while (and (> (length str) 0)
                (o-string-match "[ \t]+\\|$" str))
      (setq beg (match-beginning 0) end (match-end 0))
      ;;  Take only 1 space (1+ ..
      ;;
      ;;  no more spaces ? , the "$" matched ...
      (if (eq beg (length str))
          (progn
            ;;  is the rest of it spaces ?
            (if (o-string-match "[ \t]+$" str) nil
              (setq out (concat out str)))
            (setq str ""))                      ;found empty space
        (setq out (concat out (substring str 0 (1+ beg))))
        (setq str (substring str end)))
      )
    out
    ))


;;; ----------------------------------------------------------- &_list ---
;;; - I first made ti::s-1-space , but I realized that it could be
;;;   generilized, so here is the results.
;;; - &&list
;;;
(defun ti::s-get-blocks (str sep &optional inc)
  "Separate STR into words separated by SEP.
If optional INC is given, the matched SEP is included. Eg.

      \"Mike M.Monroe\"   SEP = [ .]+ , INC = [.]
   -->\(\"Mike\" \"M.\" \"Monroe\"\)

All leading SEP chars are ignored and they _are_not included allthough
INC has such chars. Thus below Return effectively nil

       \"            \"   SEP = [ .]+ , INC = [ ]

There is also difference if you use parameter '[ .]' or '[ .]+'.
Arguments STR and SEP can be nil."
  (let (el
        re
        SEP beg end ch ret
        )
    (setq re (concat sep "\\|$"))

    ;; Ignore leading matches ...
    (if (string-match (concat "^" sep) str)
	(setq str (substring str (match-end 0))))

      (while (and (> (length str) 0)
                  (string-match re str))
        (setq ch  ""
	      SEP nil
	      beg (match-beginning 0)
	      end (match-end 0))

        (if beg
            (setq SEP (substring str beg end)))

        (if (and beg                    ;should we include that match
                 (null inc))
            nil
          (if (o-string-match inc SEP)  ;belongs to the INC ?
              (setq ch (substring SEP (match-beginning 0) (match-end 0))))

;;;     (d! SEP inc ch)
          )

        ;;  no more matches? , the "$" matched ...
        (if (eq beg (length str))
            (progn
              ;;  is the rest of it a re match?
              (unless (string-match (concat sep "$") str)
                (setq str (concat str ch))
                (push str ret)
		)

              (setq str ""))            ;found empty space

          (setq el (concat (substring str 0 beg) ch))
	  (push el ret)
          (setq str (substring str end))
	  )
        )
    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;; 17 Aug 1995, gnu.emacs.help, kevinr@ihs.com (Kevin Rodgers)
;;; - Slightly modified by jaalto
;;;
(defun ti::s-listify (string &optional sep)
  "Look STRING and search SEP [whitespace] and return list of substrings."
  (let ((start 0)
        (sep (or sep "[^ \f\t\n\r\v]+"))
        list
        )
    (while (o-string-match sep string start)
      (setq list
            (cons (substring string (match-beginning 0) (match-end 0))
                  list))
      (setq start (match-end 0)))
    (reverse list)
    ))

;;}}}

;;{{{ buffer: line, information

;;; ........................................................ &ange-ftp ...

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::b-get-ange-buffer-list ()
  "Return list of ange-ftp buffers name strings."
 (let* ((re   "^[*]ftp")
        )
   (if re (setq re re))                 ;Quiet XEmacs 19.14 ByteComp

   (ti::b-buffer-list                   ;this returns list
    (quote
     (and (o-string-match "internal.*ange" (symbol-name major-mode))
          (o-string-match re (buffer-name))))
    'temp-buffers)))



;;; ----------------------------------------------------------------------
;;;
(defun ti::b-find-ange-buffer (user host)
 "Find ange ftp buffer with login USER running under HOST.

Return:
  buffer
   nil"
 (let* ((re   (concat "^[*]ftp +" user "@" host "[*]"))
        )

   (if re (setq re re))                 ;Quiet XEmacs 19.14 ByteComp

   (car-safe                            ;may be nil list
    (ti::b-buffer-list                  ;this returns list
     (quote
      (and (o-string-match "internal.*ange" (symbol-name major-mode))
           (o-string-match re (buffer-name))))
     'temp-buffers))
   ))




;;; ----------------------------------------------------------------------
;;;
(defun ti::b-find-ange-to-dired-buffer ()
 "Find associated dired buffer for current ange-ftp buffer.

Return:
 list           ,list of possible buffers
 nil"
 (let* (;;      Check that we're in ange buffer "*ftp ..."
        (name   (strmatget "^[*]ftp +\\(.*\\)[*]" 1 (buffer-name)))
        ret
        )
   (if name
       (setq ret
             (ti::b-buffer-list
              (quote
               (and (eq major-mode 'dired-mode)
                    (o-string-match name (or dired-directory "")))))
             ))
   ret
   ))



;;; ........................................................ &uuencode ...
;; - I need these all the time when receiving mail. For interactive
;;   use, please look at functions
;;
;;      ti::p-uu-make   ti::p-uu-save-buffer
;;
;; - Suggested keybindings
;;
;;* (global-set-key "\C-cu"  'nil)      ;make "u" available
;;* (global-set-key "\C-cum"  'uu-make)
;;* (global-set-key "\C-cus"  'uu-save)


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-uu-area ()
  "Return uuencoded data area. The ready data is put into `ti:-tmp-buffer'.

Return
  (beg end)     ,list, the uu data area
  nil           ,no uu after point found"
  (let* (
         (case-fold-search  nil)                ;must use case sensitive
         (beg-re            "begin[ \t]+[0-9]+[ \t]+.")
         (end-re            "end[ \t]*$")
         (buffer            (ti::m-tmp-buffer))
         (ob                (current-buffer))

         beg end
         bol
         ret
         leading
         )

    (save-excursion
      (and (re-search-forward beg-re nil t)
           (setq bol (BOLP))
           (setq beg (match-beginning 0))
           (re-search-forward end-re nil t)
           (setq end (EOLP)))

      (if (not (and beg end))
          nil                           ;not found beg end
        (setq ret (list beg end))
        ;;  First get the data
        (set-buffer buffer) (erase-buffer)
        (insert-buffer-substring ob bol end)

        ;;  Remove possible leadings so that you can extract NEWS
        ;;  citated UUdata too
        ;;
        ;;  >  begin 0 cobol.el.gz
        ;;  >  M'XL("!?:;S```V-O8F]L+F5L`*P\:W/;1I*?Q;H?,4'MK@A%8"0GL9PH&Z\B

        (if (< (- beg bol) 1)           ;no leading
            nil
          (setq leading (concat "^" (make-string (- beg bol) ?.)))
          (PMIN)
          (ti::b-replace-regexp leading 0 ""))

        (PMAX) (insert "\n")
        )
    ret
    )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-uu-line-p (&optional string)
  "Determines if current line is UUencoded. Optionally check STRING.
The line is considered as an uu line if it has no lowercase chars and has
length more than 50 chars. Any leading spaces and tabs are skipped to find
the UU start [applies to buffer reading only].

Return length of line if it's UU, nil if not."
  ;; (interactive)
  (let* (
         (case-fold-search      nil)    ;case is important
         (at-least              50)
         line
         len
         ret
         )
    (cond
     ((setq line (or string (ti::b-read-if-solid)))
      (setq len  (length line))
      (if (and (not (o-string-match "[a-z]" line)) ;--> not UU line
               (> len  at-least))               ;must be longer than xx chars
          (setq ret len))))
      ret
      ))


;;; ----------------------------------------------------------------------
;;; - Original implementation in c-comment-edit.el/within-c-comment-p
;;;   by kyle_jones@wonderworks.com
;;;
(defun ti::b-area-bounds (beg end)
  "Search area bounds delimited by _strings_ BEG and END.
First searches backward, them forward.

Return:
  (beg-point . end-point)
  nil"
  (condition-case nil
      (let (p pp)
        (save-excursion
          (search-backward beg)
          (setq p (point))
          (search-forward end)
          (setq pp (point)))
        (if (< (point) pp) (cons p pp) nil))
    (search-failed
     nil))
  )

;;}}}


;;; ########################################################## &Buffer ###

;;{{{ buffer: reading lines, chars

;;; ----------------------------------------------------------------------
;;; - This is great for dtecting straight lines.
;;;
(defun ti::b-read-if-solid ()
  "Read from current point all the non-whitespace characters.
Ignores leading and trailing whitespace."
  (let* (beg
         (eol (EOLP))
         ret
         )
    (save-excursion
      (if (looking-at "[ \t]")
          (skip-syntax-forward " " eol))

      (setq beg (point))
      (unless  (eolp)
        (skip-chars-forward "^ \t" eol)
        (if (eq (point) beg)            ;not moved
            (end-of-line))              ;no trailing spaces


        (if (eq (point) beg)
            nil                         ;still no movement
          (setq ret (buffer-substring beg (point))))
        ))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::b-get-wspc (&optional pp)
  "Gets whitespace following the point or optional at PP point.
Return
  ''     ,if no whitespace
  str    ,whitespace string"
  (let* (mp op                          ;maximum point, end of line
         (re-w "[ \t]+")                ;whitespace
         )
    (save-excursion
      (if (null pp)
          (setq op (point))
        (setq op pp)
        (goto-char pp))
      (setq mp (EOLP))
      (if (or (null  (o-looking-at re-w))       ;not sitting on whitespace
              (null (re-search-forward re-w mp t)))
          ""
        (buffer-substring op (point)))
  )))



;;; ----------------------------------------------------------------------
;;; - if you use outline or folding, please open the buffer first
;;;   otw lines cannot be read correcly [the \n is missing if file
;;;   has closed folds]
;;;
(defun ti::b-read-line (&optional len skip)
  "Read whole line from buffer. Optionally LEN chars.
If LEN is more than line has characters then Return whole line.
Optionally ignores SKIP count chars at front.
If there is not that many to skip, Return full line."
  (let* ((line     (read-full-line))
         (len-full (length line))
         )
    (if (null skip) nil
      (cond
       ((and len (> len skip))
        (setq line (substring line skip)))
       ((eq len skip) (setq line ""))))

    (if (and len (< len len-full))
        (substring line 0 len)
      line)))




;;; ----------------------------------------------------------------------
;;;
(defun ti::b-grep-lines (re &optional beg end inc-prop)
  "Greps lines matching RE from buffer.

Optionals:
  BEG           ,default is `point-min'
  END           ,default is `point-max'
  INC-PROP      ,do not remove propties while reading lines.

Return:
  nil or  \(str str str ..\)"
  (let* ((beg           (or beg (point-min)))   ;point begin
         (end           (or end (point-max)))   ;point end
         list
         line
         )
    (save-excursion
      (goto-char beg)
      (while (re-search-forward re end t)
        (setq line (read-full-line))
        (if (null inc-prop)
            (setq line (remove-properties line)))

        (nconcc list line)
        (forward-line 1)
        ))
    (reverse list)
    ))

;;}}}
;;{{{ buffer: matching, reading words, chars

;;; ....................................................... &b-reading ...



;;; ----------------------------------------------------------------------
;;;
;;; - I found this little function very long time ago and it's
;;;   very usefull. The bad thing is that it is impossible slow, so
;;;   use it only when time is not critical (not in loops)
;;;
;;; Date:       22 Sep 94
;;; Newsgroups: gnu.emacs.help
;;; Subject:    Re: o-looking-at backwards?
;;; Author:     nickson@cs.uq.oz.au (Ray Nickson)
;;;
(defun ti::b-looking-back-at (re)
  "Return t if text immediately before point match RE.
This function modifies the match data that `match-beginning',
`match-end' and `match-data' access; save and restore the match
data if you want to preserve them.

Note:
  Use only if you need this badly. It's impossible slow."
  (let ((beg (point))
        ret
        )
    (while (and (null ret)
                (re-search-backward re nil t))
      (setq ret (eq (match-end 0) beg)))
    (goto-char beg)
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - originally developed for tinyeat.el
;;;
(defun ti::b-read-char (&optional direction distance)
  "Read character towards the DIRECTION from current point:
nil = forward, non-nil backward. DISTANCE 0/nil means reading from
current position.

Return:
  nbr   ,read char value
  nil   ,if the position is not within `point-min-marker' and
         `point-max-marker'."
  (let* (
         (beg  (point-min-marker))
         (end  (point-max-marker))
         (pos  (or distance 0))
         (dest (if direction (- (point) (1+ pos)) (+ (point) pos)))
         (read (if (or (< dest beg) (> dest end)) nil t))
         )
    (if (null read) nil                 ;allowed to read ?
      (char-after dest))
    ))


;;; ----------------------------------------------------------------------
;;; This wonderfull basic code was found from
;;; findlib.el  -- Bill Brodie <wbrodie@panix.com>
;;; Slightly rewritten by [jari]
;;;
;;; - You can define the "word" syntax here without changing syntax entries.
;;; - If you want to get word according to current mode's syntax table,
;;;   use following instead
;;;
;;;   (require 'thingatpt)                      ;19.29
;;;   (word-at-point)
;;;
(defun ti::b-read-word (&optional charset strict)
  "Return word specified by optional CHARSET at or after point.
If optional STRICT is non-nil, requires that point is sitting on
CHARSET before continuing. If thre is no CHARSET under point it would
normally search forward for word, but when STRICT is in effect it
Return nil.

Limitations:
  Cannot read word that starts at beginning of buffer

Return:
  str        ,word or nil
  `ti:-ret'   ,list of word boundary points \(beg end\) or nil"
  (let* ((charset       (or charset "-a-zA-Z0-9_"))
         (not           (concat "^" charset))
         pb pe
         ret
         )
  (save-excursion
    (setq
     ret
     (if (or (null strict)
             (and strict (o-looking-at charset)))
         (buffer-substring
          (progn
            (skip-chars-forward not)
            (skip-chars-backward charset)
            (setq pb (point))
            )
          (progn
            (skip-chars-forward charset)
            (setq pe (point))
            ))))
    (setq ti:-ret nil)                  ;<< GLOBAL
    (if (equal pe pb)
        nil
      (setq ti:-ret (list pb pe))
      ret)
    )))


;;; ----------------------------------------------------------------------
;;; - This is totally different from the other word reading funcs,
;;;   it gives you the word separated by spaces. For more finer control see,
;;;   CHARSET in ti::b-read-word
;;;
(defun ti::b-read-space-word ()
  "Return word separated by spaces or bol/eol.
If sitting on space or tab, read next word forward. If sitting in the
middle of word, find the word beginning until bol, and start reading from
that point. Point is moved to the beginning of word.

Return:
 str
 nil    ,empty line"
  (let* ((bol (BOLP))
         p                              ;point
         )
    (cond
     ((or (bobp)
          (char-in-list-case (preceding-char) '(?\t ?\ )))
      ;; At the beginning of word, first char
      )

     ((looking-at "[^ \t\n]")
      (setq p (point))
      (skip-chars-backward "^ \t\n" bol)
;;      (skip-syntax-backward " " bol)
      (if (eq p (point))                ;jump not done.
          (beginning-of-line))          ;text starts at bol
      )

     ((o-looking-at "[ \t\n]")
      (skip-chars-forward " \t\n"))

     ((save-excursion                   ;is the line end of buffer
        (end-of-line)                   ;--> eg. in minibuffer
        (eobp))
      (beginning-of-line)
      ))

    (ti::b-read-if-solid)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-read-syntax-word (syntax &optional mode)
  "Read block of characters from current point.
Blocks are separated by SYNTAX Normally the block is read
from current point forward.

Input:
 SYNTAX         ,class like \"w\" for words.
 MODE    'back  ,read backward
         'word  ,read full word, skip syntax forward, then backward.


Return:
 str
 nil    ,current point does not contain SYNTAX class char."
  (let* ((beg  (point))
         end
         ret
         )
    (save-excursion
      (cond
       ((eq mode 'back)
        (setq end (point))
        (skip-syntax-backward syntax)
        (setq beg (point))
        )

       ((eq mode 'word)
        (skip-syntax-forward syntax) (setq end (point))
        (skip-syntax-backward syntax) (setq beg (point))
        )

       (t
        (skip-syntax-forward syntax)
        (setq end (point))
        )))


    (if (not (eq beg end))
        (setq ret (buffer-substring beg end)))
    ret
    ))




;;; ----------------------------------------------------------------------
;;; #not fully tested
;;; - Why did I do this after all ?
;;; - This won't work if cursor it at SPACE and BOL and user wants
;;;   word BACK
;;;
;;;
(defun ti::b-read-nth-word (&optional count mode back charset)
  "Read COUNT nth word in line.

Input:
  COUNT                 ,defaults to 0 ,current word according to MODE.
  MODE   nil            ,count from the bol/eol.
         'end           ,count from the bol/eol, stop at eol/bol
         'this          ,start counting from this position
         'thisEnd       ,start counting from this position, stop at eol/bol
  BACK                  ,read backward. Affects the mode parameter.
  CHARSET               ,use charset as \"word\", otw defaults to mode's
                         word syntax.

Examples:
 (ti::b-read-nth-word)                  ,return first word in line
 (ti::b-read-nth-word 5 'end)           ,return 5th word, but stop at eol

 ;; return 5th word, counting backwards stopping at bol. Read the word
 ;; with charset a-zA-z.
 (ti::b-read-nth-word 5 'end 'back \"a-zA-Z\")

Caveats:
  You get different results, if point is already sitting at word, or
  if it's sitting at whitespace, when using 'this modes.
  Try yourself with `forward-word' command.

  REMEMBER THAT WORD IS MODE DEPENDENT (syntax tables)

Return:
  str   ,word
  nil   ,nth word does not exist."
  (let* ((next-func     (if back 'backward-word 'forward-word))
         (prev-func     (if back 'forward-word 'backward-word))
         (next-skip     (if back 'skip-chars-backward 'skip-chars-forward))
         (cmp-func      (if back '< '>))
         (count         (or count 0))
         limit
         ret
         )
  (save-excursion
    ;; ... ... ... ... ... ... ... ... ... ... ... ... ...  set limits ...
    (if (memq mode '(end nil))          ;starting position
        (if back (EOLPP) (BOLPP)))

;;;    (d! "point" (point) mode (memq mode '(end nil)))

    (if (memq mode '(end thisEnd))      ;setting the limit value
      (setq limit (if back (BOLP) (EOLP))))


    (if (eq 0 count)
        ;; Skip over spaces, stay put ...
        (if (char-in-list-case (following-char) '(?\t ?\ ))
            (funcall next-skip " \t"))

      (funcall next-func count)
;;;      (d! "before" (ti::b-read-word) (point))
      (if (char-in-list-case (following-char) '(?\t ?\ ))
          (funcall prev-func 1))
;;;      (d! "after" (ti::b-read-word) (point))
      )

;;;    (d! count (point))

    (if (and limit
             (funcall cmp-func (point) limit))
        nil                             ;limit exceeded
      (cond
       (charset
        (setq ret (ti::b-read-word charset)))
       (t
        (require 'thingatpt)
;;;     (d! (point))
        (setq ret (sfuncall 'word-at-point)) ;silence Bytecomp.
        )))
    ret
    )))






;;}}}
;;{{{ buffer: replacing, modifying lines

;;; ..................................................... &b-replacing ...



;;; ----------------------------------------------------------------------
;;;
(defsubst ti::b-replace-region-with  (beg end string &optional keep-point)
  "Replace region BEG END with STRING.
Point is after the inserted string or if KEEP-POINT is non-nil
then point is at BEG."
  ;;  Prevent accidental delete
  ;;
  (if (not (stringp string)) (error "Input error."))

  ;;  mimic "r" tag region, do not kill that extra char.
  ;;
  (delete-region beg end)
  (goto-char beg)
  (insert string)
  (if keep-point
      (goto-char beg))
  )


;;; ----------------------------------------------------------------------
;;; Tue May 7, 1996 - feren@ctron.com (Andrew C. Feren), gnu.emacs.sources
;;; The basic code for this was borrowed from zap-to-char in simple.el
;;;
;;; (define-key esc-map "Z" 'zap-to-regexp) ; originally 'zap-to-char
;;;
(defun ti::b-zap-to-regexp (arg regexp)
  "Kill up to and including ARG'th occurrence of REGEXP.
Goes backward if ARG is negative; error if REGEXP not found."
  (interactive "p\nsZap to regexp: ")
  (kill-region (point) (progn
                         (search-forward-regexp regexp nil nil arg)
                         ;; This line makes zap-to-regexp behave like
                         ;; d/ and d? in vi (ie with forward deletion
                         ;; the regexp is left intact).  Is this
                         ;; really the right thing?  zap-to-char
                         ;; dropped this behavior.  Was there a good
                         ;; reason?  I like this behavior since I use
                         ;; vi frequently enough to get some benefit
                         ;; from the orthogonality.
                         (if (>= arg 0) (search-backward-regexp regexp 1))
                         ;; p.s.  Yes I know the '=' doesn't really do
                         ;; much.
                         (point)))
  )

;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'leave-nth-word 'ti::b-leave-nth-word)
;;; - This is great function if you have some column output generated
;;;   by SQL call or shell call, and you just want THOSE words left...
;;;
;;;
(defun ti::b-leave-nth-word (beg end &optional nbr strict)
  "Delete all between BEG and END except nth word NBR.
Default word nbr is 1, ie. the first word in the line.
The word is considered as space separated entity.

REMEMBER that word is mode dependent !

Input
  NBR           ,which word top leave on line, range 1..x
  STRICT        ,if non-nil then if word NBR is not found delete whole line"
  (interactive "*r\nP")
  (let* ((nbr   (or nbr 1))
         word
         )
    (save-restriction
      (narrow-to-region beg end) (PMIN)
      (while (not (eobp))
        (beginning-of-line)
        (setq word (ti::b-read-nth-word nbr 'end))

;;;     (d! word)

        (cond
         (word
          (delete-region (BOLPP) (EOLPP))
          (insert word)
          (forward-line 1)
          )
         ((and (null word) strict)
           (ti::b-kill-line))           ;already does fwd-line
         (t
          (forward-line 1)))
        ))))


;;; ----------------------------------------------------------------------
;;; - Easiest would have been using zap-to-char, but
;;;   it's not same in 18.xx and 19.xx
;;; #todo: detect 19.xx and use zap, it's much quicker
;;;
;;;
(defun ti::b-kill-line (&optional delete count)
  "Kill line and move next line up.
If cursor is sitting at the end of buffer, nothing happens.


Input:
  DELETE    use `delete-region', which doesn't manipulate `kill-ring',
	    thus the execution is faster.
  COUNT	    how many lines to wipe.

Portable:
  Between any emacs versions 18.xx - 19.xx

Errors:
  Never signalled.

Return:
  t             ,line killed
  nil           ,sitting at eob, cannot kill line"
  (interactive "*P")
  (let* ((null-line-re "^$")
         (count        (or count 1))
         (i            0)
         )
    ;;  emacs kill-line is little awkward, because if you're at the
    ;;  end of buffer it signals an error...

    (while (< i count)
      (inc i)
      (cond
       ((eobp)                          ;nothing to kill
        nil)
       ((and (null (eobp)) (o-looking-at null-line-re))
        (if delete
            (delete-char 1)
          (kill-line))
        t)
       (t                               ;shift line up
        (beginning-of-line)
        (if delete
            (delete-region (point) (EOLPP))
          (kill-line))
        (if (null (eobp))
            (if delete
                (delete-char 1)
              (kill-line)))
        t)))
      ))



;;; ----------------------------------------------------------------------
;;; #defalias   (defalias 'u2d 'ti::b-u2d-crlf)
;;;
(defun ti::b-u2d-crlf (&optional arg force)
  "Simple Unix to Dos converter. If ARG is non-nil -->  Dos to Unix.
Strips or inserts ^M (return) marker _only_ at the end of line.

If optional FORCE is given, ignores possible write protection.

Example:
  (if (dos-file-p)
      (ti::b-u2d-crlf 'Dos2unix 'doReadOnly))"
   (interactive "P")
   (let* ((stat   buffer-read-only)
          )
     (cond
      ((or (not stat)
           (prog1 force (setq buffer-read-only nil))) ;turn it off
       ;;  - We use unwind, because the buffer read only status must be
       ;;    restored. User may get anxious and press C-g for large buffers...
       ;;  - I wonder if we can clear the buffer-modified flag too?
       ;;    we leave it untouched for now...
       ;;
       (unwind-protect
           (save-excursion
             (goto-char (point-min))  ; start at the beg. of file
             (if arg
                 ;; ..................................... Dos --> unix ...
                 (progn
                   (while (search-forward "\015\n" nil t)
                     (replace-match "\n"))
                   (PMAX) (beginning-of-line)

                   ;; Maybe last line does not have newline?
                   (when (looking-at ".*\015$")
                     (end-of-line) (delete-backward-char 1)
                     ))
               ;; ....................................... unix --> dos ...
               (end-of-line)
               (if (not (char= (preceding-char) ?\015))
                   (insert "\015"))

               (while (not (eobp))
                 (forward-line)
                 (end-of-line)
                 (if (not (char= (preceding-char) ?\015))
                     (insert "\015"))
                 )))
         (setq buffer-read-only stat)   ;restore value
         )))))




;;; ----------------------------------------------------------------------
;;;
(defun ti::b-arrow-control (buffer &optional mode str pos)
  "Controls showing the arrow glyph.

Input:
  BUFFER        Where to put the arrow, must be visible.
  MODE  'show   show the arrow with optional STRING
        'hide   remove the arrow. If STR is given, change the value
                of `overlay-arrow-position'. This is usually for restoring
                the original content.
        'move   move to current bol position or to POS. STR argument is
                ignored.
        any     same as 'hide

  STR           arrow string to use, defaults to \"=>\"
  POS           any position, converted to beginning of line
                [Emacs docs say the arrow must be at bol]"
  (cond
   ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ show ^^^
   ((or (eq mode 'show)
        (eq mode 'move))

    ;;  We do not touch the arrow definition, if 'move is the mode
    (if (eq mode 'show)
        (setq overlay-arrow-string
              (if (stringp str) str "=>")))

    (or overlay-arrow-position
        (setq overlay-arrow-position (make-marker)))
    (set-marker overlay-arrow-position
                (if pos
                    (progn (goto-char pos) (BOLP))
                  (BOLP))
                buffer)

    )
   ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ hide ^^^
   (t
    (if overlay-arrow-position          ;Kill the marker
        (set-marker overlay-arrow-position nil))
    (if (stringp str)
        (setq overlay-arrow-string str))
    ))

  ;; - Here should be some kind of buffer refresh, since
  ;;   the markes isn't hidden, if you're using read-char,
  ;;   instead of read-from-minibuffer. See [tinyreply.el] for hack.
  ;; - Anybody knows how to refresh the view, please MAIL ME!!

;;; Not working, I thought moving the cursor would refresh arrow state
;;;  (save-excursion
;;;    (select-window (get-buffer-window buffer))
;;;    (set-buffer buffer)
;;;    (goto-char (BOLPP))
;;;    )

  )


;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'nl 'ti::b-insert-line-numbers), see unix nl(1)
;;;
;;; -- or is this better ?
;;; #defalias (defalias 'insert-lines-numbers 'ti::b-insert-line-numbers)
;;;
(defun ti::b-insert-line-numbers (beg end &optional line grow format)
  "Insert line numbers to buffer.
Mark the region where to insert the line numbers.

The default line format is '%02d:%s' for values lower that 100.
For bigger values the format is dynamical (digit len derived from
start value)

Input:

  BEG END       point  ,area bounds
  LINE          nbr    ,starting line number is 1 by default
  GROW          nbr    ,grow by 1 is default
  FORMAT        str    ,how line is formatted, see above

Return:
  --"
  ;;  We input number as string so that user may press return
  ;;
  (interactive "*r\nsStart line[1]: \nsInterval[1]: ")
  (let* (
         ;;  convert strings to sensible value
         (count         (cond
                         ((integerp line)               ;; calling lisp
                          line)
                         (t                             ;; interactive
                          (if (eq 0 (length line))
                              1
                            (string-to-int line)))))

         (factor        (cond
                         ((integerp grow)
                          grow)
                         (t
                          (if (eq 0 (length grow))
                              1
                            (string-to-int grow)))))

         (digits        (digit-len count))

         ;;  Select "02d" when numbers < 100
         ;;  Otw, select "digits" len.
         (fmt           (or format
                            (concat
                             "%0"
                             (int-to-string
                              (if (or (= digits 1) (eq digits 2))
                                  2 digits))
                             "d:%s")))
         line
         )
    (save-restriction
      (narrow-to-region beg end)
      (PMIN)

      (while (not (eobp))
        (setq line (read-full-line))
        (if (not (string-equal "" line))
            (delete-region (point) (EOLP)))

        (insert (format fmt count line))
        (setq count (+ count factor))
;;;     (d! count)
        (forward-line 1)
        ))
    ))


;;; ----------------------------------------------------------------------
;;; - There must be removing function too.. :-)
;;; #defalias (defalias 'remove-line-numbers 'ti::b-remove-line-numbers)
;;;
(defsubst ti::b-remove-line-numbers (beg end &optional re level)
  "Remove line numbers from region BEG END.
The Default line numbers are sticked to the left and have form

   xxx: text txt txt

where xxx represent some numbers.

You can supply optional RE and regexp LEVEL that should be
removed. Eg. in normal, above case the

  RE    = \"^[0-9]+:\"
  LEVEL = 0            ,match whole regexp"
  (interactive "*r")

  (ti::b-replace-regexp
   (or re "^[0-9]+:")
   (or level 0)
   "" nil
   beg end
   ))



;;; ----------------------------------------------------------------------
;;; - Original Idea by ericb@col.bsf.alcatel.fr (Eric Blanchard)
;;;   18 Nov 94 in comp.emacs
;;; - slightly rewritten by [jari] <jari.aalto@poboxes.com>
;;;
(defun ti::b-randomize-lines (beg end)
  "Scramble all the lines in region BEG END.
If region contains less than 2 lines, lines are left untouched."
  (interactive "*r")
  (catch 'cancel
    (save-restriction
      (narrow-to-region beg end)
      ;;   Exit when there is not enough lines in region
      (if (< (- (point-max) (point-min)) 3)
             (throw 'cancel t))

      ;;    Prefix lines with a random number and a space
      (goto-char (point-min))
      (while (not (eobp))
        (insert (int-to-string (random 32000)) " ")
        (forward-line 1))

      ;;  Sort lines according to first field (random number)
      (sort-numeric-fields 1 (point-min) (point-max))

      (goto-char (point-min))           ;Remove the prefix fields
      (while (not (eobp))
        (delete-region (point) (progn (forward-word 1) (+ (point) 1)))
        (forward-line 1))
      )))


;;; ----------------------------------------------------------------------
;;; Tue, 11 Apr 95 , ralf@prl.philips.nl (Ralf Fassel), comp.emacs
;;;
(defun ti::b-make-dup-line (&optional count)
  "Copy the current line COUNT times (default is 1) below the current line."
  (interactive "*p")
  (setq count (or count 1))
  (save-excursion
    (beginning-of-line)
    (let ((line (buffer-substring
                 (point)
                 (progn (forward-line 1) (point)))))
      (while (> count 0)
        (insert line)
        (setq count (1- count))))))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-inc-string-nbr (re inc-val increment &optional level)
  "Search string and increment integers.

Input:
  RE	    regexp to match integer. Subexpr 1 assumed in interactive call
  INC-VAL   start value.
  INCREMENT Step how much to increment every time.
  LEVEL     Subexpression in regexp to match the integer portion.

Eg. I you have just paste same variable on the lines multiple times

         tablevar10[10]
         tablevar10[10]
         tablevar10[10]
         tablevar10[10]
         tablevar10[10]


And now you want to make them unique:


         tablevar01[10]
         tablevar02[10]
         tablevar03[10]
         tablevar04[10]
         tablevar05[10]


You just give RE \"r\\([0-9]+\\)\" and start value 1, increment 1"
  (interactive "sRE: \nnstart value: \nnIncrement: ")
  (let* ((level (or level 1))
         len
         beg end fmt
         )

    (while (re-search-forward re nil t) ;search whole buffer
      (when (match-end level)
        (setq beg (match-beginning level)
              end (match-end level)
              len (- end beg)
              fmt (concat "%0" (int-to-string len) "d"))

        (delete-region beg end)
        (goto-char beg)
        (insert (format fmt inc-val))
        (inci inc-val increment)
        )
      )))


;;; ----------------------------------------------------------------------
;;; - Here is slightly different version. this increments every number
;;;   whereas the previous would increment only SUBMATCH by STEP
;;;
;;; 28 Jan 1996, ttn@netcom.com (Thien-Thi Nguyen), comp.emacs
;;;
;;; - Eg. copying the first line produces:
;;;
;;;     assign pi0_vld = (opc_i0 === alu0);
;;;     assign pi1_vld = (opc_i1 === alu1);
;;;
;;; - Slightly modified by [jari]
;;;
(defun ti::b-copy-line-and-inc-numbers (&optional increment)
  "Copy line, preserving cursor column, and INCREMENT any numbers found.
Prefix ARG is the increment value. Defaults to 1."
  (interactive "p")
  (let* ((col           (current-column))
         (line          (read-full-line))
	 (increment	(if (integerp increment) increment  1))
	 mark
         num
         )
    (end-of-line)
    ;;  We have to use markers, because the line is modified.
    ;;
    (setq mark (point-marker))

    (beginning-of-line)
    (while (re-search-forward "[0-9]+" (marker-position mark) 1)
      (setq num (string-to-int (read-match 0)))
      (replace-match (int-to-string (+ increment num)))
      )
    (beginning-of-line)
    (insert line "\n")
    (move-to-column col 'force)
    (setq mark nil)			;kill marker
    ))


;;; ----------------------------------------------------------------------
;;; date unknown, Masanobu UMEDA, umerin@mse.kyutech.ac.jp
;;;
(defun ti::b-copy-word (n)
  "Copy N words above the current line.
If there is no words above the line, then do nothing."
  (interactive "p")
  (let ((column (current-column))
        copy
        )
    (save-excursion
      (beginning-of-line)
      (if (bobp)
          nil
        (forward-line -1)
        (move-to-column column)

        (setq copy (buffer-substring
                    (point)
                    (min (save-excursion (end-of-line) (point))
                         (save-excursion (forward-word n) (point)))))
        ))
    (if copy
        (insert copy))
    ))

;;; ----------------------------------------------------------------------
;;; #defalias   (defalias 'double-space-region 'ti::b-newlines-to-region)
;;; 07 Feb 1996, Kai Grossjohann <grossjoh@ls6.informatik.uni-dortmund.de>
;;; Slightly generalized by [jari]
;;;
(defun ti::b-add-newlines-to-region (beg end &optional arg)
  "Insert to to the end of each line in region BEG END ARG newlines.
Default is to inser one which makes lines make double spaced."
  (interactive "*r\np")
  (save-restriction
    (narrow-to-region beg end)
    (PMIN)
    (while (search-forward "\n" nil t)
      (replace-match
       (concat "\n" (make-string arg ?\n))
       nil t))))


;;; ----------------------------------------------------------------------
;;; - STRICT parameter can be used from lisp call
;;; #defalias   (defalias 'remove-blank-lines 'ti::b-cnv-empty-lines)
;;;
(defun ti::b-cnv-empty-lines (beg end &optional nbr strict)
  "Convert empty lines in region BEG END to zero empty lines.
Optionally leaves NBR empty lines. If STRICT is non-nil, all lines
must have NBR amount of empty lines, no more or less.

Point is not preserved."
  (interactive "*r\nP")
  (let* ((empty-line-re  "^[ \t]+$\\|\n")
         (nbr            (or nbr 0))    ;default is to leave no empty lines
         pb pe                          ;points beg, end
         count
         do-it
         )
    (save-restriction
      (narrow-to-region beg end)
      (PMIN)

      (while (not (eobp))
        (if (null (o-looking-at empty-line-re))
            (forward-line 1)
          (setq pb (point))   (skip-chars-forward " \t\n")
          (beginning-of-line) (setq pe (point))

          ;;  There is a bug in count-lines, that's why we use EOLPP,
          ;;  not 'pe' to count the lines in region
          (setq count (count-lines pb (EOLPP)))


          (setq do-it nil)
          ;; ...................................................... cond ...
          (cond
           ((null strict)
            (if (> nbr count)
                nil                     ;not that many lines here
              (setq do-it t)))
           (t
            (setq do-it t)
            ))

;;;       (d! "~count " do-it count nbr pb pe)
          ;; .................................................... action ...
          (cond
           ((null do-it)
            (forward-line 1))           ;skip
           ((> count 0)
            (delete-region pb pe)
            (setq count nbr)

            (while (> count 0)          ;leave that many
              (dec count) (insert "\n"))

            (if (> count 1)
                (beginning-of-line)
              (forward-line))           ;nothing done, next line
            ))
          )
;;;     (d! (point) (read-full-line))
        )


      )))



;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'delete-duplicate-lines 'ti::b-del-dup-lines)
;;;
;;;  - Letting shell to do the job is the fastest, cleanest
;;;    way. Sometimes lisp just isn't the right tool...
;;;
;;; A. Want to do it fast?
;;;    Camel book has ready code for this. Pg 228
;;;    $ perl -ne 'print unless $seen{$_}++' file.in > file.out
;;;
;;; B. How about running a shell command over the region/buffer
;;;    with command "uniq"? This filters successive lines.
;;;    C-x h , ESC-| uniq RET
;;;
;;;
(defun ti::b-del-dup-lines(beg end &optional len white-lines)
  "Deletes duplicate lines in buffer. Optionally compares first LEN
characters to determine line equality.

Input:
  BEG,END       ,area bounds
  LEN           ,portion of line: chars to compare
  WHITE         ,if non-nil, don't touch whitespace only lines.

Requirements:
  Calls shell with small PERL program. Make sure PERL is along the path.
"
  (interactive "*r\nP")
  (let* (cmd
         )
    (save-restriction
      (narrow-to-region beg end)
      (PMIN)
      (setq
       cmd

       (concat
        "perl -ne '"
        (if len
            (concat "$line = substring($_,0, "
                    (int-to-string len)
                    ");")
          "$line = $_;")

        (if white-lines
            "/^\\s*$/ && do{print; next;};")

        "print unless $seen{$line}++;"
        "'"
        ))

      (shell-command-on-region
       (point-min) (point-max)
       cmd
       t)                               ;replace

      )))



;;; ----------------------------------------------------------------------
;;; #todo: not tested
;;;
;;; Subject: Re: How to replace a string in a rectangle?
;;; 20 Sep 1995, gnu.emacs.help, Erik Naggum <erik@naggum.no>
;;;
(defun ti::b-replace-in-region-arguments (&optional iact)
  "Ask arguments for `ti::b-replace-in-rectangle'. IACT is interactive flag."
  (let ((match (if iact                 ;query-replace-interactive
                  (car regexp-search-ring)
                (read-from-minibuffer "Replace " nil nil nil
                                      'query-replace-history))))
    (list match
          (read-from-minibuffer (format "Replace %s with " match)
                                nil nil nil 'query-replace-history)
          (region-beginning)
          (region-end))))


;;; ----------------------------------------------------------------------
;;; 20 Sep 1995, Erik Naggum <erik@naggum.no>
;;; #todo: not tested
;;;
(defun ti::b-replace-in-rectangle (match replace corner1 corner2)
  "Search for MATCH (regexp) and replace with REPLACE in rectangle.
Rectangle is defined by two diagonally opposite CORNER1 and CORNER2,
each given as a buffer position (number) or marker."
  (interactive (ti::b-replace-in-region-arguments))
  (let ((top (min corner1 corner2))
        (bot (max corner1 corner2))
        (left (progn (goto-char corner1) (current-column)))
        (right (progn (goto-char corner2) (current-column))))
    (goto-char top)
    (while (< (point) bot)
      (ti::b-replace-in-rectangle match replace
                               (progn (move-to-column left) (point))
                               (progn (move-to-column right) (point)))
      (forward-line 1))))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-delete-until-non-empty-line (&optional backward point)
  "Delete all lines starting from current point.
Stop on [be]obp or non-empty line. Optionally delete BACKWARD
and start at POINT or current position.

Moves point to the beginning of non-empty line."
  (interactive "P")
  (let* (end
         )
    (when point
      (goto-char point))

    (beginning-of-line)
    (setq point (point))
    (cond
     (backward
      (while (and (not (bobp))
                  (looking-at "^[ \t]*$"))
        (setq end (point))
        (forward-line -1)))
     (t
      (while (and (not (eobp))
                  (looking-at "^[ \t]*$"))
        (forward-line 1)
        (setq end (point))
        )
      ))
    (if end     (delete-region point end))
    ))

;;; ----------------------------------------------------------------------
;;; - The delete-region, according to emacs C-developers,
;;;   is _lighting_ fast way to do deletions in emacs.
;;; #defalias (defalias 'trim-blanks 'ti::b-trim-blanks)
;;;
(defun ti::b-trim-blanks (beg end)
  "Delete trailing blanks in region BEG END."
  (interactive "*r")
  (save-restriction
    (save-excursion
      (narrow-to-region beg end)
      ;;  _much slower would be:       (replace-regexp "[ \t]+$" "")
      (goto-char (point-min))
      (while (not (eobp))
        (end-of-line) (delete-horizontal-space) (forward-line 1)
        )))
  nil
  )                                     ;for possible hook



;;; ----------------------------------------------------------------------
;;;
(defun ti::b-replace-regexp (re level str &optional back beg end)
  "Like `replace-regexp' but for lisp programs.
Lisp infopage says in \"Node: Style Tip\", that lisp programs shouldn't
use `replace-regexp', so here is identical function that doesn't touch
the mark. The point is left after last match.

Input:

  RE	    regexp
  LEVEL	    subexpression
  STR	    string used in replacing.
  BACK	    replace backward
  BEG END   region. If both BEG and END is given, the
	    BACK parameter is ignored."
  (let* (bp                             ;begin position
         ep
         (func (if back 're-search-backward 're-search-forward))
         )
    (if (not (integerp level))          ;common error
        (error "Level is not integer."))

    (cond
     ((and beg end)
      (setq bp beg  ep end  func 're-search-forward))

     ((and back end)
      (setq bp (point)  ep (point-min)))

     ((and back beg)
      (setq bp beg  ep (point-min)))

     ((and (not back) beg)
      (setq bp beg ep (point-max)))

     ((and (not back) end)
      (setq bp (point) end))
     (t                                 ;fall thru case
      (setq bp (point)  ep (point-max))))

    (save-restriction
      (narrow-to-region bp ep)
      (PMIN)
      (while (and (funcall func re nil t)
                  (not (eobp)))
        (if (null (match-end level)) nil ;not matched
          (kill-match level)
          (insert str)))                ;point is at the end of STR inserted
        )))

;;}}}
;;{{{ buffer: misc

;;; ..................................................... &buffer-misc ...



;;; ----------------------------------------------------------------------
;;;
(defun ti::b-diff-type-p ()
  "Check the diff type in buffer.
Assumes that whole buffer contains diff. Searches for traces.
Lines must be left flushed.

   *** /tmp/T.11  Fri Oct 20 12:22:51 1995
   --- /tmp/T.1   Fri Oct 20 12:24:29 1995
   ***************

Normal diff shows:

   20,21d19
   < clrFamily;
   < clrInfo;

Gnu diff -n (or --rcs, Output an RCS format diff)

    d696 1
    a696 1
            (tdi-goto-kbd 'verb)
    d704 2
    a705 2

Gnu diff -u (unified diff)

    @@ -17,6 +17,8 @@
     bAnsTime[16+1];
     clearCode;
     endChargeTime[16+1];
    +clrFamily;
    +clrInfo;
     statClrTime[16+1];
     clearPart;
     aDirNbrType;


Returns:
  cons cell
    (TYPE . POS)        ,POS is the diff start position
    nil                 ,no diff found

  TYPE can be
    'context            ,context diff -c
    'gnu-n              ,gnu diff -n
    'gnu-u              ,gnu diff -u
    'normal             ,normal diff

 POS
    character position where the first diff was found"
  (let* ((re-c1     "^[ \t]*[*][*][*] [^ \t\n]") ;context diff regexps

         ;;   The normal diff line is following, but PGP breaks it.
         ;;   That's why we have those ? ? in thge regexp
         ;;   --- 1.2.1.1
         ;;   - --- 1.2.1.1
         ;;
         (re-c2     "^-? ?--- .")
         (re-c3     (concat  "^" (regexp-quote "***************")))
         (re-n1     "^[0-9]+[dca][0-9]+$\\|^[0-9]+,[0-9]+[dca][0-9]")
         (re-n2     "^[<>]")
         ;;  Gnu types
         (re-gn1            "^[dac][0-9]+ [0-9]+$")
         (re-gu1            "^@@ [-+][0-9]+,[0-9]+[ \t]+[-+]+")
         type
         pos
         ret
         )
    (save-excursion
      (PMIN)
      (cond
       ;; ... ... ... ... ... ... ... ... ... ... ... ... ...  context ..
       ((and (re-search-forward re-c1 nil t)
             (setq pos (BOLP))
             (progn
               (forward-line 1)
               (looking-at re-c2))
             (progn
               (forward-line 1)
               (looking-at re-c3))
             )
        (setq type 'context))

       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... . normal ..
       ((and (re-search-forward re-n1 nil t)
             (setq pos (BOLP))
             (progn
               (forward-line 1)
               (looking-at re-n2)))
        (setq type 'normal))

       ((re-search-forward re-gu1 nil t)
        ;;  There is filename information above the diff start.
        ;;  --- file.xx
        ;;  +++ file.xx
        ;;
        (forward-line -2)
        (setq pos (point))
        (setq type 'gnu-u)
        )

       ((and (re-search-forward re-gn1 nil t) ;require two same lines
             (setq pos (BOLP))
             (progn
               (forward-line 1)
               (looking-at re-gn1)))
        (setq type 'gnu-n)
        ))

      (if (and type pos)
          (setq ret (cons type pos)))

      ret
      )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-xtra-open-outline ()
  "Open folded/outlined buffer if some of the modes is active.
You have to call this function if you want to do something for
the whole buffer."
  (interactive)
  (let* (
	 )
    ;;  Unfold the buffer, so that we can see all.
    ;;  We must also preserve point
    (save-with-marker
      (and (boundp 'folding-mode)
	   ;;  No autoloads allowed, this makes sure the fboundp
	   ;;  is converted to real function. The sfuncall command
	   ;;  cannot use autoload function.
	   ;;
	   (progn (require 'folding) t)
	   (if (symbol-value 'folding-mode)	;ByteComp silencer
	       (save-line-col nil nil
		 (sfuncall 'fold-open-buffer)) ;ByteComp silencer
	     ))

      (and (eq major-mode 'outline-mode)
	   (fboundp 'show-all)
	   (progn (require 'outline) t)
	   (save-line-col nil nil
	     (sfuncall 'show-all)
	     ))

      (and (boundp 'outline-minor-mode)
	   (fboundp 'show-all)
	   (progn (require 'outline) t)
	   (save-line-col nil nil
	     (sfuncall 'show-all)
	     ))
      )))

;;; ----------------------------------------------------------------------
;;; - If user has given regexp in a file '\b\\(something\\)\b' you
;;;   just can't go and read that directly, because eg. \b is in file
;;;   as separate characters '\' and 'b'. So we have to do little
;;;   coverting to get real regexp...
;;;
;;;
(defun ti::b-get-re (re &optional level)
  "Serach RE in subexpression LEVEL and read regexp format string from buffer.
Look at the documentation of the `ti::c-str2re' for more details."
  (interactive)
  (setq level (or level 0))
  (save-excursion
    (PMIN)
    (when (and (re-search-forward re nil t)
	       (match-end level))
      ;;   We have to take care of those esapes .. '\t' --> '\\t'
      (ti::c-str2re (read-match level)))))

;;; ----------------------------------------------------------------------
;;;
(defun ti::b-buffer-list (test &optional temp-buf exclude action-form)
  "Return list of buffer names matching TEST.

If optional TEMP-BUF is non-nil, every buffer is searched.
Normally following buffers are ignored.
-  Temporary buffers which start with character asterisk '*'
-  Invisible buffers which start with space ' '

Optional EXCLUDE can also be given, which excludes buffers from
matched ones.

If optional ACTION-FORM is given executes forms for every matched buffer.
At the moment of eval the `set-buffer' is already done.

Examples:
  ;;  Get all buffers matching \"cc\"
  (ti::b-buffer-list \"cc\")

  ;;  Get all buffers in `dired-mode'
  (ti::b-buffer-list '(eq major-mode 'dired-mode))

  ;;  Get all ange-ftp dired buffers
  (ti::b-buffer-list
   (quote
    (and (eq major-mode 'dired-mode)
         (o-string-match \"@\" (or dired-directory \"\")))))

Input:
  test          ,regexp or form to get matching buffers.
  temp-buf      ,flag. Non-nil allows scanning temp buffers too
  exclude       ,regexp or form  -- against matched ones
  action-form   ,if exist, eval this for every buffer.

Return:
  list          (buffer buffer ..)"
  (let* ((ptr (buffer-list))
         elt
         bn                             ;buffer name
         ret
         ok
         )
    (save-excursion
      (list-loop ptr elt
;;      (d! ptr )
        (setq bn (buffer-name elt))     ;it's killed if no name

        (when (stringp bn)
          (set-buffer bn)

          (setq ok                      ;include
                (cond
                 ((stringp test)
                  (o-string-match test bn))
                 (t
                  (eval test))))

;;;     (d! "-" ok bn)

          (if (and ok exclude)
              (setq ok                  ;exclude
                    (and ok
                         (cond
                          ((stringp exclude)
                           (not (o-string-match exclude bn)))
                          (t
                           (eval exclude))))))


          ;; ... ... ... ... ... ... ... ... ... ... only mached buffers ...
          (cond
           (ok
            (cond
             ((and (null temp-buf)              ;drop temp bufs
                   (o-string-match "^[* ]" bn))
              nil)
             (t
              (push bn ret)
;;;         (d! "->" ret)
              (if action-form
                  (eval action-form))
              )))) ;; this cond
          ))) ;; the loop and when buffer name
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::b-buffer-list-files (&optional re str)
  "Return all filenames loaded into emacs.

If optional RE and STR are given, then a file name substitution
takes place:

 args           re = \"/usr43/users/john/\"   ,str = \"~/\"
 buffer file    \"/usr43/users/john/t.txt\"
 substituted    \"~/t.txt\"

Example:
 (ti::b-buffer-list-files \"/usr43/users/john\" \"~\")

Return:
 (file ..)      ,list of filenames"
  (let* ((ptr   (buffer-list))
         elt
         list
         file
         )
    (list-loop
        ptr elt
      (setq file  (buffer-file-name elt))
      (when (stringp file)              ;might be nil if buffer has no file
        (if (and re str
                 (o-string-match re file))
            (setq file (kill-match 0 str file)))
        (push file  list)
        ))
    list
    ))

;;}}}

;;{{{ Window, frames

;;; .......................................................... &window ...


;;; ----------------------------------------------------------------------
;;;
(defun ti::w-frame-list  (&optional all exclude-current win)
  "Return only frames that are not dedicated.
Input:
  ALL                   ,if non-nil, return all frames.
  EXCLUDE-CURRENT       ,if non-nil, exclude current active frame.
  WIN                   ;Use this is as a current window when searching
                         current frame."
  (let* ((oframe  (if win
                      (window-frame win)
                    (current-frame)))
         flist
         frame
         ret
         )
    (if exclude-current
        (setq flist (delete oframe (frame-list)))
      (setq flist (frame-list)))

    (list-loop flist frame
      (select-frame frame)
      (if (or all (not (window-dedicated-p (selected-window))))
          (nconcc ret frame))
      )

    (if (framep oframe)
        (select-frame oframe))               ;Return back to original
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::w-list (&optional buffers)
  "Gather all visible windows or BUFFERS visible in current frame."
  (let* ((s     (selected-window))      ;start window
         (loop  t)
         (w     s)                      ;current cycle
         l
         ww
         )

    ;;  Start list
    ;;
    (if buffers
        (setq l (list (window-buffer s)))
      (setq l (list s)))

    (while loop
      (setq ww (next-window w))
      (setq w ww)                       ;change
      (other-window 1)                  ;move fwd
      (if (eq ww s)                     ;back to beginning ?
          (setq loop nil)

        (if buffers                     ;list of buffers instead
            (setq ww (window-buffer ww)))
        (setq l (cons ww l))
        ))

    (reverse l)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::w-get-buffer-window-other-frame  (buffer)
  "Return (frame . win). If BUFFER is visible..
in some other frame window than in the current frame."
  (interactive)
  (let* ((flist     (delete (current-frame) (frame-list)))
         frame
         win
         ret
         )
    (list-loop flist frame
      ;;  maybe in other frame...
      (if (setq win (get-buffer-window buffer frame))
          (setq flist nil               ;stop loop
                ret   (cons frame win))))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - don't know good way how to generalize this to return either top/bottom
;;;   window. I guess we just copy this and make small changes...
;;; - Does anyone have good suggestions to do therwise?
;;;
(defun ti::w-find-bottom  (win-list)
  "Find bottom window from WIN-LIST.
Any non-visible window in list is skipped.
If there are adjacent windows, return all of them.

        -------------
        |           |   <- top window
        -------------
        |  |   |    |   < three splitted windows at the bottom
        | A| B |  C |
        -------------

Return:
  list          ,single or many windows. In any order."
  (let* (data
         top
         top-cmp
         bot
         bot-cmp
         win-val
         init
         )

    (window-loop
     win-list

     (setq data (window-edges win))

     (if (null init)                    ;init vars
         (setq init    t                ;initalized ok
               win-val (list win)       ;win comes from 'window-loop'
               top     (nth 1 data)
               bot     (nth 3 data)))

     (setq top-cmp     (nth 1 data)
           bot-cmp     (nth 3 data))

     (cond
      ((> bot-cmp bot)                  ;this is more lower
       (setq win-val (list win)
             top     top-cmp
             bot     bot-cmp))

      ((or (eq bot-cmp bot)                     ;hmm, same horizontal top row..
           (eq top-cmp top))                    ;split sideways...
       (push win win-val))

      ((eq bot-cmp bot)                 ;  .........
       (> top-cmp  top)                 ;  ....    .
                                        ;  .........  < pick lowest in left
       (setq win-val (list win)
             top     top-cmp
             bot     bot-cmp)
       ))
     )
    win-val
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::w-match-buffers (buffer-name-list)
  "Check all windows that match BUFFER-LIST.

Input:
  BUFFER-NAME-LIST      ,strings, list of buffer names.

Return:
  '((BUFFER-NAME WIN-PTR WIN-PTR ..)
    (BUFFER-NAME ..)
    ..
    )"
  (let* ((win-list (ti::w-list))
         alist
         buffer
         ptr p                          ;pointers
         win
         )
    (list-loop win-list win
      ;;  last walue will tell the BOTTOM
      ;;
      (setq buffer      (buffer-name (window-buffer win)))

      ;;  Create alist
      ;;  '((BUFFER-NAME WIN-PTR WIN-PTR ..)
      ;;    (BUFFER-NAME ..)
      ;;    )
      (cond
       ((member buffer buffer-name-list ) ;does it interest us ?

        (cond
         ((not (setq ptr (assoc buffer alist)))  ;; create initial element
          (push (list buffer win) alist))
         (t                             ;; add element
          (setq p  (cdr ptr))           ;drop 1st element away
          (nconcc p win)		;add new element
          (setcdr ptr p)                ;replace with new list
          ))))
      )
    (reverse alist)
    ))

;;}}}
;;{{{ Keymaps, translations


;;; ----------------------------------------------------------------------
;;; rpotter@halibut.cis.upenn.edu, (Robert), in rp-describe-function.el
;;;
(defun ti::k-function-bind-info  (function-sym &optional map)
  "Return binding information for FUNCTION-SYM from MAP as string or nil."
  (let* ((gm  (current-global-map))
         global-bindings
         local-bindings
         bind-info
         )
    (setq global-bindings (where-is-internal function-sym)
          local-bindings
          (prog2
	      ;;  We have to set this to nil because where-is-internal
	      ;;  searches global map too. We don't want that to happen
	      ;;
              (use-global-map (make-keymap))
              (where-is-internal
               function-sym
               (or map (current-local-map)))
	    (use-global-map gm)))

  (setq
   bind-info
   (if (or global-bindings local-bindings)
       (format "%s%s%s"
               (if global-bindings
                   (format "global %s"
                           (mapcar 'key-description
                                   global-bindings))
                 "")
               (if (and global-bindings local-bindings)
                   " and "
                 "")
               (if local-bindings
                   (format "local to %s"
                           (mapcar 'key-description
                                   local-bindings))
                 ""))))
  bind-info
  ))


;;; ----------------------------------------------------------------------
;;;
;;;  because of the nature of minor modes, changes in the maps
;;;  are not reflected unless, the minor mode is installed again
;;;
;;;  The following removes minor keymap, if it exists,
;;;  and reinstalls it with new added bindings.
;;;
(defun ti::k-reinstall-minor-mode (mode-name-symbol)
  "Reinstall minor mode MODE-NAME-SYMBOL.
This is needed if you have made changes to minor modes keymaps.
They don't take in effect until you reinstall the minor mode.

Return:
 t      ,minor mode found and reinstalled
 nil    ,no susch minor mode."
  (let* (mode-string
         elt
         map-sym
         map
         )
      (when (setq elt (assq mode-name-symbol minor-mode-alist))
        (setq mode-string (nth 1 elt))
        (setq elt (assq mode-name-symbol minor-mode-map-alist))

        (unless elt
            (error "No map for minor mode %s"  mode-name-symbol))
        (setq map-sym (intern-soft
                       (concat
                        (symbol-name mode-name-symbol)
                        "-map")))

        (if (or (null map-sym)
                (not (keymapp (setq map (eval map-sym)))))
            (error "The keymap was not found %s" map-sym))

        (ti::k-add-minor-mode mode-name-symbol nil nil 'remove)
        (ti::k-add-minor-mode mode-name-symbol mode-string map)
        )
      ))



;;; ----------------------------------------------------------------------
;;; - Why doesn't emacs offer this simple interface by default ?
;;;
(defun ti::k-add-minor-mode
  (mode-func-sym mode-name-sym  mode-map &optional remove)
  "Add the minor mode into emacs. If mode exists, do nothing.

Input:
  MODE-FUNC-SYM         ,func symbol, mode to turn on
  MODE-NAME-SYM         ,var  symbol to hold mode name string
  MODE-MAP              ,keymap
  REMOVE                ,OPTIONALLY removes mode with mode-name-sym

Examples:
   ;;  to add mode
   (ti::k-add-minor-mode 'foo-mode 'foo-mode-name  foo-mode-map)

   ;;  to remove mode
   (ti::k-add-minor-mode 'foo-mode nil nil 'remove)"

  (let* (elt                            ;element
         )
    (cond
     ((null remove)
      (or (assq mode-func-sym minor-mode-map-alist)
          (setq minor-mode-map-alist
                (cons (cons mode-func-sym  mode-map)
                      minor-mode-map-alist)))

      ;;  Update minor-mode-alist
      (or (assq  mode-func-sym minor-mode-alist)
          (setq minor-mode-alist
                (cons (list mode-func-sym mode-name-sym)
                      minor-mode-alist)))
      )
     (t
      (require 'cl)

      (and (setq elt (assq mode-func-sym minor-mode-map-alist))
           (setq minor-mode-map-alist (delq elt minor-mode-map-alist)))

      (and (setq elt (assq mode-func-sym minor-mode-alist))
           (setq minor-mode-alist (delq elt minor-mode-alist)))

      ))

    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::k-map-bind-control (map-symbol get-set prop key)
  "Get or set the stored property binding in map.
This is a good function to use if you modify the original
bindings in the map. You can then call the original
function behind the binding in your modified function.

Input:

  MAP-SYMBOL    map name
  GET-SET       operation 'get 'set 'sett. Where 'set puts the
                functions name behind the PROP only if is there
                is no previous value. 'sett replaces the PROPERTY
                contents.
  PROP          property name
  KEY           string -- keybinding.

Examples:

  (ti::k-map-bind-control 'mail-mode-map 'set 'my \"\\C-c\\C-c\")
  --> mail-send-and-exit, saved to property 'my

  (ti::k-map-bind-control 'mail-mode-map 'set 'my \"\\C-c\\C-c\")
  --> nil, could only be done once.

  (ti::k-map-bind-control 'mail-mode-map 'get 'my \"\\C-c\\C-c\")
  --> mail-send-and-exit, get the saved property 'my.

Live example:

  ;; - first save original, then use our function. Use property
  ;;   'my, because The C-cC-c can already be occupied by
  ;;   some other package...
  ;; - it calls the original afterwards
  ;;
  (ti::k-map-bind-control 'mail-mode-map 'set 'my \"\\C-c\\C-c\")
  (define-key      mail-mode-map \"\\C-c\\C-c\" 'my-mail-func-CcCc)

  (defun my-mail-func-CcCc (arg)
    ...
    (funcall  ;; Call the original.
      (ti::k-map-bind-control 'mail-mode-map 'get 'my \"\C-c\C-c\")
      arg)
    ;; Function ends here.
    )"
  (let* (map
         map-key
         sym
         val
         func
         )

    (unless (boundp map-symbol)
      (error "No variable bound %s" map))

    (setq map           (eval map-symbol))

    (unless (keymapp map)
      (error "Not a keymap %s" map-symbol))

    (if (or (var-nil-p key)                     ;must be valid string
            (not (stringp key)))
        (error "Invalid KEY %s" key))



    (setq map-key       (concat (symbol-name map-symbol) key))
    (setq func          (lookup-key map key))

    (when func                  ;does function exist?
      (setq sym (intern map-key))
      (setq val (get sym prop))

      (cond
       ((eq get-set 'get)
        val)

       ((and (eq get-set 'set)
             (null val))                ;set only if PROP not exist
        (put sym prop func))

       ((eq get-set 'sett)              ;replace value
        (put sym prop func)
        )))
    ))



;;; ----------------------------------------------------------------------
;;; - What is an translate table?
;;; - Well; it says "if you press this key I give you this character back"
;;; - It is used for remapping the keys, but beware! In X envinronment,
;;;   where you can paste data between emacs, the translation gives
;;;   unpleasant results. Try pasting the _normal_ \ char from other
;;;   window to emacs that uses transltion presented in example below.
;;;   --> you get the | character pasted into emacs !
;;;
;;; - In general, you should map only keys that are not usually
;;;   "copied" by pasting... such as !; are quite rare..
;;;
;;; - See live translate map example in my guide
;;;   "GUIDE: Emacs key bindings + Xterm and remote emacs -nw"
;;;
(defun ti::k-xlat-table (&optional arg)
  "Make new translate table.

Input ARG

  'use      starts using the new table if the
            `keyboard-translate-table' if nil. Otherwise does nothing.
  'use-new  replaces current table with fresh one
  nil       returns new, default translate table.

Examples:

    Switch these keys. Let's assume the \\ key is on top after this,
    since it is used more often in emacs.

    (ti::k-xlat-table 'use)
    (aset keyboard-translate-table ?\\| ?\\\\ )
    (aset keyboard-translate-table ?\\\\ ?\\| )

Returns.
  new xlat table"
  (let ((index 0)
        (xlat-table (make-string 128 0))
        )
    (while (< index 128)                ;Generate the identity map.
      (aset xlat-table index index)
      (setq index (1+ index) ))

    (cond
     ((eq arg 'use-new)
      (setq keyboard-translate-table xlat-table))
     ((eq arg 'use)
      (and (null keyboard-translate-table)
           (setq keyboard-translate-table xlat-table))
      )
     (t
      ))

    xlat-table
    ))



;;; ----------------------------------------------------------------------
;;; - For preventing Emacs to beep and disabling the normal keys
;;;   (for mail, gnus, ... )
;;;
(defun ti::k-put-abc-map (map &optional func)
  "Put function `ignore' to abc key MAP, optionally put FUNC."
  (let* ((i             0)
         (func          (or func 'ignore))
         low up
         )
    (while (< i 27 )
    ;;  Set lowercase/upcase keys to nil
      (setq low (char-to-string (+ 65 i))
            up  (char-to-string (+ 97 i)))
      (define-key map low func)
      (define-key map  up func)
      (inc i)
      )
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::k-put-map (map &optional func)
  "Put function `ignore' to a0 > x <128 key MAP, optionally put FUNC."
  (let* ((i             20)
         (func          (or func 'ignore))
         )
    (while (< i 128 )
      (define-key map (char-to-string i) func)
      (inc i))))



;;; ----------------------------------------------------------------------
;;; - Mapping keysto functions easily.
;;;
(defun ti::k-mapkeys (map-key-fun args)
  "Maps MAP-KEY-FUN to list of keys in ARGS.

Example:
  (mapkeys
   'global-set-key
   '(
     [f1] 'hilit-rehighlight-buffer
     [f2] 'eval-defun
     [f3] 'repeat-complex-command
     ))"
  (let (key
        func
        (i 0)
        (len (length args))
        )
    (if (eq 0 (% len 2)) nil
      (error "args not paired"))

    (while (< i len )
      (setq key (nth i args)   func (nth (1+ i) args)   i (+ 2 i) )
      (funcall map-key-fun key func)
      )
    ))

;;}}}

;;{{{ (T)ext properties, faces

;;; ........................................................... &faces ...

;;; ----------------------------------------------------------------------
;;;
(defun ti::t-wipe-properties  (beg end)
  "Remove all, ie. literally all, text properten between BEG and END.
Doesn't care about read only status of buffer."
  (interactive "r")
  (let* (buffer-read-only
         (inhibit-read-only t)  ;allow read-only prop wipe out
         )
    (set-text-properties beg end nil)
    ))

;;; ----------------------------------------------------------------------
;;; - Many times I want to set different color for face;
;;;   but then the damn color is occupied and emacs halts with message
;;;
;;;     (error "X server cannot allocate color" "DarkSeaGreen3")
;;;
;;; - This function allows you to give several "try" choices,
;;;   and it doesn't halt you down...
;;;
(defun ti::t-try-set-colors (list face &optional attribute)
  "Try to assign colors to face.
The input is list of color names which are tried one by one.
First one that succeeds is assigned. If color is occupied, tries
next one. Doesn't signal any errors.

Input:
  LIST          ,(\"color1\" \"color2\" ..) or single color string
  FACE          ,symbol.  Eg. 'region
  ATTRIBUTE     ,symbol.  Choises are 'fg and 'bg. Default is 'fg

Return
  color         ,color that was assigned
  nil           ,all tries failed"
  (let* ((ptr           list)
         (attribute     (or attribute 'fg))
         color ok
         )
    (if (stringp list)                  ;single item only
        (setq ptr (list list)))

    (list-loop ptr color
      (if (condition-case nil
              (progn
                (cond
                 ((eq attribute 'fg)
                  (set-face-foreground face color))
                 (t
                  (set-face-background face color)
                  ))
                (setq ok color)
                t
                )
            (error
             nil                        ;cannot set
             ))
          ;; succesfull; stop the loop
          (setq ptr nil))               ;if end
      )
    ok
    ))

;;}}}

;;{{{ misc: movement

;;; ############################################################ &Misc ###


;;; ----------------------------------------------------------------------
;;; - See EXAMPLE section !!
;;;
(defun ti::m-word-move (set &optional back)
  "Move to next word defined in SET, optionally BACK.
SET must be string, that can be turned into regexp and that can
be used with skip-chars functions.

Eg. \"-[]$%@#&*\":;{}()<>/\\ \t\n\""
  (interactive)
  (let* ((nset          (concat "^" set))       ;not-set
         (set-re        (concat "[" (regexp-quote set) "]"))
         (char          (char-to-string
                         (if back
                             (preceding-char)
                           (following-char))))
         (point (point))
         )
    (cond
     (back
      (if (string-match set-re char)
          (progn
            (skip-chars-backward set)
            (skip-chars-backward nset)
            )

        ;;  If we're over word already, this moves. But if we're
        ;;  at the beginning of word this doesn't move.
        ;;
        (skip-chars-backward nset)
        (when (eq (point) point)
          (skip-chars-backward set)
          (skip-chars-backward nset))
        ))
     (t
      (if (string-match set-re char)
          (progn
            (skip-chars-forward set)
            (skip-chars-forward nset)
            )
        (skip-chars-forward nset)
        (skip-chars-forward set)
        )
      ))
    ))


;;; ----------------------------------------------------------------------
;;; 25 Oct 1995, DvdS@eden.com  (David Vanderschel), gnu.emacs.help
;;; - Some such occurrences are valid, so it desn't fix them
;;;
;;; 27 Oct 1995, mjm@borneo.gmd.de (Matthew Morley), comp.emacs
;;; - Suggested following regexp too:
;;;   (re-search-forward "\\b\\(\\w+\\)\\W+\\1\\b")
;;;
;;; 28 Nov 1995, kevinr@ihs.com (Kevin Rodgers), gnu.emacs.help
;;; - posted another regexp...
;;;   "\\(\\b\\sw+\\b\\)\\s +\\1"
;;;
;;;
(defun ti::m-find-duplicate-same-word (&optional back)
  "Find consecutive occurrences of same word, optionally search BACK."
  (interactive "P")
  (let* ((func  (if back 're-search-back 're-search-forward))
         )
    (if (funcall func "\\(\\<\\w*\\>\\)[ \t\n]*\\1" nil t)
        (isearch-highlight (match-beginning 0) (match-end 0))
      nil
      )))



;;; ----------------------------------------------------------------------
;;; Kai Grossjohann <grossjoh@ls6.informatik.uni-dortmund.de>
;;; 05 Dec 1995, gnu.emacs.help
;;;
(defun ti::m-count-words (beg end)
  "Count words in region BEG END.
Caveat:
  Region can only start at the beginning of line and end at the end of line."
  (interactive "r")
  (save-excursion
    (narrow-to-region beg end)
    (beginning-of-buffer)
    (count-matches "\\w*")))

;;; ----------------------------------------------------------------------
;;; - This is quite a handy function when you're programming eg
;;;   in C++ and want to know how many chars are in the string.
;;;
(defun ti::m-count-chars-in-delimited-area (arg &optional verb)
  "Counts characters within quotes. ARG C - u to search single quotes.
Other argument invokes asking the beginning delimiter: if you give
\"(\"  the end delimiter is automatically set to \")\".
This function is mainly for interactive use. VERB.

Return:
  nbr   ,count of chars
  nil   ,beg or end delimiter was not found"
  (interactive "P")
  (let* ((alist '(( ?\(  ?\) )
                  ( ?\{  ?\} )
                  ( ?\[  ?\] )
                  ( ?\`  ?\' )
                  ( ?\<  ?\> )
                  ))
         (verb   (or verb (interactive-p)))
         beg-ch end-ch
         beg-re end-re
         re

         elt
         point
         ret
         )
    ;; ... ... ... ... ... ... ... ... ... ... ... ... . preliminaries ...
    (setq
     re   (cond
           ((equal arg nil)
            "\"")
           ((equal arg '(4))
            "'")
           (t
            (message "Begin delimiter char: ")
            (setq beg-ch (read-char))
            (setq end-ch
                  (if (setq elt (assq beg-ch alist))
                      (nth 1 elt)
                    ;;   Can't find match for it, so use same char
                    ;;   for both delimiters
                    beg-ch
                    ))
            nil
            )))

    (if re                              ;now, what we got?
        (setq beg-re (regexp-quote re)   end-re beg-re)
      (setq beg-re (regexp-quote (char-to-string beg-ch))
            end-re (regexp-quote (char-to-string end-ch))))

    ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... . do it ...
    (save-excursion
      (if (null (re-search-forward end-re nil t))
          (and verb
               (message (concat "Can't find end mark: " end-re)))

        (setq point (point))
        ;;  the re-search-forward leaves point after the char,
        ;;  we have to go small step back before we change the direction.
        (forward-char -1)

        (if (null (re-search-backward beg-re nil t))
            (and verb
                 (message (concat "Can't find beginning mark: " beg-re)))

          ;; the -2 exludes the markers itself.
          ;;
          (setq ret (- (length
                        (buffer-substring point (point)))
                       2))

          (and verb
               (message (concat (int-to-string ret) " characters.")))
          )))
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-move-paragraph-to-column (beg end col)
  "Move text BEG END to column position COL.

The indent is done in the following way:
o  untabify region
o  Search first non-whitespace line starting from the beginning
   of region.
o  count how much the line is indented: remove that indentation
   from all the rest of the lines.
o  Now when lines have no indet; reindent to COL

The procedure described preserves the actual paragraph style, so that
if text inside paragraph is more indented that the previous line the
relative indent is preserved.

    txt txt txt txt txt txt
    txt txt txt txt txt txt
      inner indent txt txt txt
      inner indent txt txt txt
    txt txt txt txt txt txt
    txt txt txt txt txt txt

Input:
  beg   ,always calculates to bol
  end   ,always calculates to eol"
  (interactive "*r\np")
  (let (min
        marker
        len
        )
    (goto-char (min beg end))		;Setting MIN
    (setq min (BOLPP))

    (goto-char (max beg end))		;setting MAX
    (end-of-line)
    (setq marker (point-marker))        ;Because untabify moves end

    (untabify min (marker-position marker))


    ;;  Is there non whitespace line?
    ;;
    (goto-char min)
    (cond
     ((re-search-forward "^[^ \n]" (marker-position marker)  t)
      ;;  non whitespace line found.
      nil				;Do nothing -- indent directly
      )
     ((re-search-forward "^\\( +\\)[^ \n]" (marker-position marker)  t)
      ;;  Remove this indentation.
      ;;
      (when (> (setq len (length (or (read-match 1) ""))) 0)
        (indent-rigidly min (marker-position marker) (- 0 len)  ))
      ))
    ;;  Now reindent the region
    ;;
    (indent-rigidly min (marker-position marker) col) ;new
    (setq marker nil)                   ;Kill marker
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-move-to-col (col)
  "Doesn't care about line length. Insert spaces to get to COL.
Converts tabs to spaces to get to exact COL."
  (interactive "Nto col: ")
  (move-to-column col 'force)
  (if (not (eq (current-column) col))
      (while (< (current-column) col)
        (insert " "))))

 ;;;;; Experimental
 ;;;(defun space-to-column (target)
 ;;;  "Insert spaces as necessary to move pt to TARGET column."
 ;;;  (interactive "p")
 ;;;  (let ((cur (current-column)))
 ;;;    (if (< cur target)
 ;;;        (insert (make-string (- target cur) ? )))))


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::m-fwd-line (&optional count)
  "Moves vertically lines down. If COUNT is negative, then up.

`forward-line' moves point always to the beginning
of next line, and the elisp manual says not to use `next-line' in
programs.

This function behaves exactly as `next-line'. If the next line is shorter
it moves to the end of line."
  ;; (interactive "P")
  (let* ((col (current-column)))
    (and (null count) (setq count 1))   ;No arg given
    (forward-line count)
    (move-to-column col)
    ))

;;}}}
;;{{{ misc: line handling , addings strings

;;; ......................................................... &m-lines ...



;;; ----------------------------------------------------------------------
;;; 11 Dec 1995, kfogel@floss.cyclic.com (Karl Fogel), gnu.emacs.sources
;;;
;;; - This exellent function by Karl allows user to surround word with
;;;   anything very easily: *emphasize* _unserline_ +smile+ ...
;;;
(defun ti::m-surround-with-char (char)
  "Insert two same CHAR around a string near point.
String is delimited by whitespace, although the function will do
the right thing at beginning or end of a line, or of the buffer.
If the char is one of a matching pair, do the right thing.
Also makes a great gift."
  (interactive "cSurround with char: ")
  ;; hmm, ought to be able to do this with syntax tables?
  (let
      ((begchar char)
       (endchar char))
    (cond
     ((or (char= char ?{) (char= char ?}))
          (setq begchar ?{)
          (setq endchar ?}))
     ((or (char= char ?\() (char= char ?\)))
          (setq begchar ?\()
          (setq endchar ?\)))
     ((or (char= char ?<) (char= char ?>))
          (setq begchar ?<)
          (setq endchar ?>))
     ((or (char= char ?`) (char= char ?'))
          (setq begchar ?`)
          (setq endchar ?'))
     ((or (char= char ?[) (char= char ?]))
          (setq begchar ?[)
          (setq endchar ?])))
    (re-search-backward "^\\|\\s-" (point-min))
    (if (not (bolp))
        (re-search-forward "\\s-")
      (if (o-looking-at "\\s-") (re-search-forward "\\s-")))
    (insert-char begchar 1)
    (let ((opoint (point)))
      (if (re-search-forward "\\s-\\|\n" (point-max) t)
          (forward-char -1)
        (goto-char (point-max)))
      (insert-char endchar 1)
      (if (eq (point) (+ opoint 1))
          (forward-char -1)))))




;;; ----------------------------------------------------------------------
;;;
;;; ;; Mark region (eg. C-x h) and call this function in picture mode
;;;
;;; (add-hook 'picture-mode-hook 'my-picture-mode-hook)
;;; (defun my-picture-mode-hook ()
;;;   "My settings."
;;;   (define-key picture-mode-map "\C-c8" 'ti::m-add-spaces-80)
;;;   )
;;;
(defun ti::m-add-spaces-80 (beg end &optional column)
  "This is a bit exotic function. Fill spaces until column 80.
When you're in picture mode and you want to paste/copy rectangles,
it's most convenient if the area has spaces in every row up till
column \"80\", so that the paste operations work correctly.

To return to 'ragged' text, use function `ti::b-trim-blanks'

Input:
  BEG           ,beginning of area, always line beginning
  END           ,end of area, always line end.
  COLUMN        ,the fill column. Defaults to 79, because 80 would
                 add annoying \\ marks at the end of line."
  (interactive "*r")
  (let* ((column   (or column 79))
         (spaces   (make-string (+ 2 column) ?\ ))
         line
         len
         add
         )
    (save-restriction
      (narrow-to-region beg end)
      (untabify (point-min) (point-max)) ;very important !!

      (PMIN)
      (while (not (eobp))
        (setq line (read-full-line)
              len  (length line)
              add  (- column len)
              )
        (if (<= add 0)
            nil                         ;we can't touch this
          (end-of-line)
          (insert (substring spaces 1 add))
          )
        (forward-line 1)
        )

      )))




;;; ----------------------------------------------------------------------
;;; 22 Nov 1995, Bruce Montrose <montrose@itd.nrl.navy.mil>, gnu.emacs.help
;;; - This nice and elegant solution to get quotes around the words,
;;;   but someday it should be generalized to put ANYTHING around the word.
;;;
(defsubst ti::m-quote-words-in-region (beg end)
  "This function quotes words in selected region BEG END."
  (interactive "r")
  (goto-char beg)
  (while (< (point) end)
    (kill-word 1)
    (insert (prin1-to-string (current-kill 0)))
    (setq end (+ end 2))
    (forward-word 1)
    (forward-word -1)))


;;; ----------------------------------------------------------------------
;;; - Eg. if you want to decide "fast filling", you could check if any line
;;    is longer that fill-column.
;;;
(defun ti::m-long-line (beg end len)
  "Check BEG END if there exist line longer than LEN.

Return:
  point    ,beginning of line
  nil"
  (let* (pos
        )
    (save-excursion
      (goto-char (min beg end))
      (while (and (null pos)
                  (not(eobp))
                  (< (point) (max beg end))
                  )
        (end-of-line)
        (if (<= (current-column) len)  nil
          (beginning-of-line) (setq pos (point)) )
        (forward-line 1)))
    pos
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-scramble-reg (beg end &optional char)
  "Scrables text BEG END with char so that it's not readable any more.
Preserves words by substituting every [a-zA-Z] with optional CHAR."
  (interactive "r")
  (let* ((ch (if char                   ;pick the scramble char
                 (char-to-string char)
               "o"))
         )
    (save-excursion
      (save-restriction                 ;preserve prev narrowing
        (narrow-to-region beg end)
        (PMIN)
        (ti::b-replace-regexp "[a-zA-Z]" 0 ch)
        ))))


;;; ----------------------------------------------------------------------
;;; - This function requires user input when RE-LOOK is given
;;; - This is aimed for lisp programs
;;;
(defun ti::m-add-str-reg (beg end str &optional re-look)
  "Add to region BEG END STR and optionally to lines matching RE-LOOK.
You might use this as intend-region by adding more spaces to any
vertical position, but most likely this is best function for
commenting arbitrary blocks of code.

1) set mark to _exact_column_ where to add string
2) move cursor to destination line, column does not matter.

If you want to add string to specific lines only, supply
rex when you are asked for 'look for rex'. Remember that this
rex will be used from that mark column to the end of line, so whole line
is not looked. Here is one example:

      *mark here
    ;;; triple comment
    ; single comment

    ;;; another triplet
    *cursor here


    ;;#; triple comment
    ; single comment

    ;;#; another triplet
      ^^^^^^^^^^^^^^^^^^^^ --> the REX match area, note not incl. leading!

Note that the single ';' isn't matched, because the mark's column position
is further away.

References:

  Emacs 19.28 has almost similar function. Look
  `string-rectangle'. It does not overwrite existing text."

  (interactive "r\nsString to region :\nsLook for re :")
  (let* (col
         look
         )

    (if (var-nil-p re-look)            ;reset
        (setq re-look nil))

    (if (var-nil-p str)
        nil                             ;pass, nothing given
      (save-excursion
        ;;  Get true boundaries.
        ;;
        (goto-char (min beg end)) (setq col (current-column))
        (setq beg (BOLPP))
        (goto-char (max beg end)) (setq end (EOLPP))

        (save-restriction
          (narrow-to-region beg end) (PMIN)

          (while (not (eobp))
            (move-to-column col 'force)
            (setq
             look   (if (and re-look
                             (eq (current-column) col))
                        (o-looking-at re-look)
                      t))

            (if look
                (insert str))
            (forward-line 1)))))
    ))

;;}}}

;;{{{ misc: lists handling, sorting

;;; ----------------------------------------------------------------------
;;; - The default sort-regexp-fields is too limited and wakward to use.
;;; - This one offers easy interface to 'sort'
;;;
(defun ti::m-sort-regexp-fields (list level re &optional numeric reverse case)
  "Sort LIST of strings at subexpression LEVEL of RE.
Sort can optionally be NUMERIC, REVERSE or CASE sensitive.

Return:
  sorted list."
  (let* ((clist (copy-list list))       ;sort modifies it otw.
         )
    (sort clist
          (function
           (lambda (l r &optional ret elt1 elt2)
             (cond
              ((not case)               ;not sensitive
               (setq l  (downcase l)    ;ignore case
                     r (downcase r))))

             ;;  read the maches from strings
             (setq elt1 (strmatget re level l)
                   elt2 (strmatget re level r))

             (cond
              ((not (and elt1 elt2)) ;make sure match happened
               (setq ret nil))
              (numeric
               (setq ret
                     (if reverse
                         (< (string-to-int elt2)  (string-to-int elt1))
                       (< (string-to-int elt1)  (string-to-int elt2)))))
              (t
               (setq ret
                     (if reverse
                         (string< elt2 elt1)
                       (string< elt1 elt2)))
               ))
             ret
;;;          (d! ret elt1 elt2)
             )))
    clist
    ))

;;}}}

;;{{{ misc: shell, exec, process

;;; ......................................................... &process ...
;;; - Current "misc" category



;;; ----------------------------------------------------------------------
;;; - This is great function to build up completions for login names...
;;; - I have 400 entries in passwd file and it's not very  fast.
;;; - You Should call this only once with regexp "." and put all the entries
;;;   into some variable. Use that variable for lookup.
;;;
(defun ti::m-passwd-grep-user-alist (re &optional not-re passwd-alist)
  "Grep all login names, where person name match RE.
The matches are gathered from `ti:-passwd-buffer' and  if it does not
exist, error is signalled.

If optional NOT-RE is string, it will be used after the RE match is done.
It is used to exclude items.

If PASSWD-ALIST is given it will be used instead to gather needed
information. It should be alist int he form returned by function
`ti::m-passwd-build-alist'

Return:
  ((login  . user-name-entry) ..)"
  (let* ((passwd-buffer   ti:-passwd-buffer)
         ;;  The name is 5th entry
         ;;  neva:I5KJd2C33dtMg:418:200:Max Neva,Houston Texas ...
         (passwd-re   "^\\([^:]+\\):[^:]+:[^:]+:[^:]+:\\([^:,]+\\)")
         alist
         line
         login person
         )
    (cond
     (passwd-alist
      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ list ^^^
      ;; Hmm, the loops are almost identical, but what the heck...
      (while passwd-alist
        (setq line (cdr (car passwd-alist)))

        ;; It's possible, that there is no "person" field, eg.
        ;;     "lp:*:9:7::/usr/spool/lp:/bin/sh"
        ;;              |
        ;;              empty field
        ;;
        ;;  It's quicker to test 2 at the same time, and only then decode
        ;;  the field intyo parts

        (if (not (and  (o-string-match re line)
                       (o-string-match passwd-re line)
                       ))
            nil
          (setq login  (read-match 1 line))
          (setq person (read-match 2 line))

          (if (null (and login person))
                nil
            (if (or (not (stringp not-re))
                    (and (stringp not-re)
                         (not (o-string-match not-re person))))
                (push (cons login person) alist)
                )))
        (NEXTP passwd-alist)
        ))

     (t
      ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ buffer ^^^
      (if (null (set-buffer-safe passwd-buffer))
          (error "Passwd buffer does not exist"))
      (if (eq (point-min) (point-max))
          (error "Passwd buffer is empty."))
      (PMIN)

      (while (not (eobp))
        (setq line (read-full-line))
        (if (not (and (o-string-match  re  line)
                      (o-looking-at    passwd-re)))

            nil
          (setq login  (read-match 1))
          (setq person (read-match 2))

          (if (null (and login person))
              nil
            (if (or (not (stringp not-re))
                    (and (stringp not-re)
                         (not (o-string-match not-re person))))
                (push (cons login person) alist)
              )))
        (forward-line 1)
        )))
    alist
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-passwd-build-alist (cmd)
  "Build assoc list out of passwd table using CMD.
Please note, that building a list takes some time, so call this
only once per program. The CMD must be valid Unix command to get
the password contents.

Note:

    The performance of this function is not very good. Expect
    parsing 1000 users/15 seconds.

Return:

    ((login . full-passwd-entry) ..)"
  (let* (
         (passwd-buffer   ti:-passwd-buffer)
         alist
         line
         login
         )
    ;;  force loading passwd entries
    ;;
    (ti::m-passwd-read-entry "SomeUser" cmd)

    (save-excursion
      (set-buffer passwd-buffer)
      (PMIN)
      (while (not (eobp))
	(beginning-of-line)
        (setq line (buffer-substring
		    (point) (progn (end-of-line) (point))))
	;; passwd entry looks like this, sometimes there may be garbage
	;; after shell command like these two grep notes.
	;;
	;;   grep: can't open a
	;;   grep: can't open tty
	;;
	;;   lm58817:x:23193:23193:Leena M{ki|:/home3/li7/lm58817:/bin/tcsh
	;;
        (when (setq login (strmatget "^[^:]+" 0 line))
	  (setq alist (cons (cons login line) alist))
          )
        (forward-line 1)
        ))
    alist
    ))


;;; ----------------------------------------------------------------------
;;; Eg. in HP-UX the command is this
;;; (my-read-passwd-entry "jaalto" "ypcat passwd")
;;;
(defun ti::m-passwd-read-entry (&optional user cmd)
  "Return USER's password entry using Shell CMD.

If the password buffer's content is not empty, the CMD isn't called, instead
the entry is searched from the buffer. This reduces overhead of calling
processes every time function is invoked.

References:
  `ti:-passwd-buffer'"
  (let* (
         ;;  Permanent buffer, since launching process is expensive
         (user (or user (user-login-name)))
         (re   (concat "^" user ":"))
         (b    (get-buffer-create ti:-passwd-buffer))
         ret
         )
    (save-excursion
      (set-buffer b)
      (unwind-protect
          (progn
            (when (eq (point-min) (point-max)) ;No entries yet
	      (if (null cmd)
                  (error "Need command to get the passwd file")
                (erase-buffer)
                (call-process shell-file-name nil b nil "-c" cmd)))
            (PMIN)
            (if (re-search-forward re nil t)
                (setq ret (read-full-line)))
            )))
    ret
    ))

;;}}}
;;{{{ misc: rmail, mail, news

;;; ............................................................ &mail ...

;;; ----------------------------------------------------------------------
;;; See mailabbrev.el how to build your abbrevs.
;;;
(defun ti::m-mail-abbrevs-get-alist  (&optional expand-until)
  "Return alist of all `mail-abbrevs'.
Build the abbrev table from your ~/.mailrc with command
\\[build-mail-abbrevs]. The following parameter is _not_ yet functional.

Input:
  EXPAND-UNTIL        ,expand until the RH elt is pure email.

Return:
  '((ABBREV-STRING . EXPANDED-STRING) (A . E) ..)"
  (let* (tmp
         (pre-abbrev-expand-hook        nil) ;; prevent recursion
         (mail-abbrev-aliases-need-to-be-resolved t)
         table
         exp-list
         elt
         )

    ;; XEmacs 19.14 no-op for ByteCompiler
    (if (null mail-abbrev-aliases-need-to-be-resolved)
        (setq mail-abbrev-aliases-need-to-be-resolved nil))


    (cond
     ((emacs-p)
      ;; mail-aliases is already in (A . S) form
      (when (not (listpp mail-aliases))
        (setq mail-aliases nil)
        (build-mail-aliases))
      (setq exp-list mail-aliases)
      )
     (t
      ;;  We have to expand abbrevs by hand because XEmacs doesn't
      ;;  parse them like emacs mail-alias
      ;;
      ;;  If the abbrevs package is notloaded then this
      ;;  returns nil.
      ;;
      (setq table (ti::xe-mail-abbrev-table))

      (when table
        (setq tmp       (ti::m-tmp-buffer 'clear))
        (job-buffer tmp
          (setq local-abbrev-table table)
          (mapatoms
           (function
            (lambda (x)
              (setq elt (prin1-to-string (identity x)))
              (when (not (string= "0" elt)) ;abbrev in this slot?
                (insert elt)
                (end-of-line)
                (expand-abbrev)
                (push (cons (symbol-name x) (read-full-line)) exp-list)
                (end-of-line)
                (insert "\n")
                )))
           table)
           ))
      ))
    (reverse exp-list)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-mail-abbrevs-email-list  (&optional abbrev-alist)
  "Build email list of abbrevs; optionally use ABBREV-ALIST.
Only entries in this format in ~/.mailrc are returned. There must be
no \",\" chained lists in the line.

  alias[spaces]some[spaces]user@address.xx

Input:
  '((\"abbrev\" . \"expansion\") (A . E) ..)

Return:
  '(email email ...)"
  (interactive)
  (let* ((alist (or abbrev-alist (ti::m-mail-abbrevs-get-alist)))
         elt str email
         list
         )
    (list-loop alist elt
      (setq str (cdr elt))
      (when (null (string-match "," str)) ;filter out multiple mail lists
        (setq email (strmatget "\\([^< \t]+@[^> \t\n]+\\)" 0 str))
        (if email
            (push str list))
        ))
    (reverse list)
    ))


;;}}}
;;{{{ misc: multidimensional arrays

;;; .......................................................... &arrays ...

;;; ----------------------------------------------------------------------
;;;  11 Jan 1996, kevinr@ihs.com (Kevin Rodgers), gnu.emacs.help
;;;
(defun ti::m-make-2D-array (n m &optional value)
  "Create vector array N M with VALUE.

(setq a-2x2-array
      (make-2D-array 2 2 0))
[[0 0] [0 0]]

(aset (aref a-2x2-array 1) 0 12)
12

a-2x2-array
[[0 0] [12 0]]"
  (apply 'vector
         (let ((i 0)
               (subarrays nil))
           (while (< i n)
             (setq subarrays
                   (cons (make-vector m value) subarrays))
             (setq i (1+ i)))
           subarrays)))




;;; ----------------------------------------------------------------------
;;;  11 Jan 1996, kevinr@ihs.com (Kevin Rodgers), gnu.emacs.help
;;;
(defun make-ND-array (dimensions &optional value)
  "Make multidimensional array: DIMENSIONS with VALUE..


(setq ti:m-make-2x3x4-array (make-ND-array '(2 3 4) 0))
[[[0 0 0 0] [0 0 0 0] [0 0 0 0]] [[0 0 0 0] [0 0 0 0] [0 0 0 0]]]

(aset (aref (aref a-2x3x4-array 1) 1) 1 12)
12

a-2x3x4-array
[[[0 0 0 0] [0 0 0 0] [0 0 0 0]] [[0 0 0 0] [0 12 0 0] [0 0 0 0]]]"
  (cond ((null dimensions) nil)
        ((null (cdr dimensions)) (make-vector (car dimensions) value))
        (t (apply 'vector
                  (let ((i 0)
                        (subarrays nil))
                    (while (< i (car dimensions))
                      (setq subarrays
                            (cons (make-ND-array (cdr dimensions) value)
                                  subarrays))
                      (setq i (1+ i)))
                    subarrays)))))

;;}}}
;;{{{ misc:

;;; ----------------------------------------------------------------------
;;; 20 Nov 91, rp-describe-function.el, rpotter@halibut.cis.upenn.edu
;;; derived from momentary-string-display
;;;
;;; #todo: not tested
;;;
(defun ti::m-momentary-string-flash (STRING &optional wait-for-key)
  "Momentarily display STRING in the buffer; erase it on the next keystroke.
The string is displayed starting on the line after point.  The window is
recentered if necessary to make the whole string visible.  If the window isn't
large enough, at least you get to read the beginning.

Input:
  STRING                ,string to display
  WAIT-FOR-KEY          ,if we should wait user. Normally display it
                         for 2 seconds."
  (let (
        (modified       (buffer-modified-p))
        (name           buffer-file-name)
        buffer-read-only
        insert-start
        insert-end
        )
    (unwind-protect
        (progn
          (save-excursion
            ;; defeat file locking... don't try this at home, kids!
            ;;
            (setq buffer-file-name nil)
            (setq insert-start (point))
            (insert STRING)
            (setq insert-end (point)))
          ;;  make sure the whole string is visible
          ;;
          (when (not (pos-visible-in-window-p insert-end))
            (recenter (max 0
                           (- (window-height)
                              (count-lines insert-start insert-end)
                              2))))
          (cond
           ((null wait-for-key)         ;DOES NOT WORK. Why not?
            (sleep-for 2))

           (t
            (message "Type %s to continue editing..."
                     (single-key-description ?\ ))
            (let ((char (read-char)))
              (or (char= char ?\ )
                  (setq unread-command-events char)))
            )))
      ;; .................................................. unwind end ...
      ;;
      (if insert-end
          (save-excursion
            (delete-region insert-start insert-end)))
      (setq buffer-file-name name)
      (set-buffer-modified-p modified)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-find-function-name (&optional point)
  "Return possible function name.
Starts searching backward form current or optional POINT.
Be sure to be in right mode, so that right `beginning-of-defun' is used.

In Lisp, the current function can be found only if it is left flushed.

In C++, this will simply returns line portion, which it thinks
contains function name.

In Perl, it is supposed that word following \"sub\" is function name.

Input:
  point	    ,where to look

Return:
  nil
  string"
  (let* ((name      (symbol-name major-mode))
         (lisp-re   (concat
                     "def\\(un\\|subst\\|macro\\|advice\\|var\\|const\\)"
                     "[ \t]+\\([^ \t]+\\)"))

         line
         ret
         )

    (setq line (read-full-line))
    (save-excursion
      (safe-do
       ;;  Now comes fun part...Ugh!
       (cond
        ((or (setq ret (strmatget lisp-re 2 line))
	     (string-match "lisp" name))

         ;;  This beginning-of-defun only finds only left
	 ;;  flushed FORMS
	 ;;

         (or ret
             (progn
               (beginning-of-defun) (setq line (read-full-line))
               (setq ret (strmatget lisp-re 2 line))
               ))
         )

        ((or (o-string-match "CC" name)
             (o-string-match "C++" name))
         (beginning-of-defun)
         ;; A nightmare...
         ;;
         ;; perAtom_c *
         ;; pMscCg_c::DecodeV7
         ;; ()
         ;; {
         ;;
         ;; perAtom_c *pMscCg_c::DecodeV7
         ;; ()
         ;; {

         ;; Try our best...
         ;;
         (search-backward "(")
         (beginning-of-line)

         (or (setq ret (bufmatget "^[ \t]*\\([^ \t(]+\\)[ \t]*(" 1))
             (progn             ;skip one line up
               (forward-line -1)
               (setq ret (bufmatget "^[ \t]*\\([^\n(]+\\)" 1))
               ))
         )

        ((and (o-string-match "perl" name)
              (re-search-backward "^[ \t]*sub[ \t]+\\([^ \t]+\\)" nil t))
         (setq ret (read-match 1))
         )

        ))
      ret
      )))


;;; ----------------------------------------------------------------------
;;; #todo ** under development
;;;
(defun ti::m-reindent (beg end)
  "Indent BEG END code block. Remove previous indentation."
  (interactive "r")
  ;;  Remove previous indent and then indent again
  (indent-rigidly beg end -80)
  (indent-region beg end nil))

;;}}}

;;{{{ file

;;; ############################################################ &File ###


;;; ----------------------------------------------------------------------
;;;
(defsubst ti::f-days-old   (file)
  "Calculate how many days old the FILE is. This is rough approximation."
  (let ((now  (ti::d-standard-date 'short))
        time
        )
    (setq
     time
     (ti::d-standard-date
      ;;  Modification time
      ;;
      'short
      (nth 5 (file-attributes file))))

    (ti::d-time-diff-days time now)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::f-ange-completed-message (&rest args)
  "Default message after file has been loaded. Ignore ARGS."
  (message "Ange-ftp bg completed"))


;;; ----------------------------------------------------------------------
;;; (ti::f-ange-status "/ssjaaa@cs.uta.fi:/")
;;; #todo  Not qute what I want...
;;;
(defun ti::f-ange-status (ange-ref)
  "Return status on ANGE-REF ftp buffer.
Return:
 'no-ange        ,if no ange buffer exists
 (..)            ,some ange status values"
  (let* ((ret   'no-ange)
         ange
         buffer
         host user proc
         line stat busy
         )
    (require 'ange-ftp)
    (setq ange  (ange-ftp-ftp-name ange-ref) ;crack addr
          host  (nth 0 ange)
          user  (nth 1 ange))

    (cond
     ((setq buffer (ti::b-find-ange-buffer user host))
      (if (null buffer) (setq buffer buffer)) ;XEmacs 19.14 Bytecomp silencer

      ;;  Create a new process if needed
      (setq proc (ange-ftp-get-process host user))

      ;;  The status value is valid only when process finishes.
      ;;
      (save-excursion
        (set-buffer (process-buffer proc))
        (PMAX)
        (setq ret   ange-ftp-process-result
              line  (read-full-line)
              stat  (ange-ftp-process-handle-line line proc)
              busy  ange-ftp-process-busy
              )
;;;;    (d! ret stat)

        ;; STAT
        ;; t     = skip message
        ;; ange-ftp-process-result-line = good
        ;; fatal, deletes process.

        (setq ret (list ret stat busy))
        )))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - an easy interface to ange ftp to get dingle file in bg.
;;; - this actually is a "macro" or toplevel func to the
;;;   ti::f-ange-file-handle
;;;
(defun ti::f-ange-download-file (ange-ref download-dir &optional not-bg)
  "Download single file pointed by ANGE-REF in background to the DOWNLOAD-DIR.

Input:
  ANGE-REF      ,/login@site:/dir/dir/file.xx
  DOWNLOAD-DIR  ,valid directory where to put the file.
  NOT-BG        ,if non-nil the ftp is done in foregroung.

Return
  nil           ,if job is done in background
  status        ,if in fg. Nil means failure."

  (let* (ange
         host user dir file to-dir
         )
    (require 'ange-ftp)
    (setq ange          (ange-ftp-ftp-name ange-ref) ;crack addr
          host  (nth 0 ange)
          user  (nth 1 ange)
          dir   (file-name-directory (nth 2 ange))
          file  (file-name-nondirectory (nth 2 ange))
          to-dir (expand-file-name download-dir)
          )
    (ti::f-ange-file-handle 'get user host dir to-dir (list file) not-bg)
    ))



;;; ----------------------------------------------------------------------
;;; - an easy interface to ange ftp to get/put wanted files
;;; #todo: sometimes ange hangs, rarely but... should check if
;;;        process is live somehow?
;;; #todo: check that no process is going in the buffer, so that it's
;;;        not called many times (overlapping).
;;;
(defun ti::f-ange-file-handle
  (mode user host dir lcd file-list &optional not-bg msg-func)
  "Fetch or put files to remote site.

Important:
  All directory names must be absolute

Input:
  MODE          'put or 'get
  USER          ,login name when logging to site
  HOST          ,site name
  DIR           ,remote site directory
  LCD           ,download local dir
  FILE-LIST     ,files to get from/put to remote site
  NOT-BG        ,should we wait until ange is done?
                 nil = run on bg, non-nil = wait until done.
  MSG-FUNC      ,function to call after download completes. Should
                 contain &rest args parameter. See more in ange-ftp.el
Return:
  nil           ,always if NOT-BG is nil
  status        ,if NOT-BG is non-nil. Value nil means that session
                 failed."
  (let* ((func          (or msg-func 'ti::f-ange-completed-message))
         (max-try       5)
         (try           0)

         proc
         point
         ret
         )
    (require 'ange-ftp)



    (cond                               ;get commands
     ((eq mode 'get)
      (setq mode "mget"))
     ((eq mode 'put)
      (setq mode "mput")
      (setq func 'ignore)               ;can't use any function for this...
      )
     (t
      (error "What mode?")
      ))

    (if (not (listpp file-list))
        (error "file-list must be LIST and _not_ empty"))

    ;;  We need absolute directory names, because the FTP process
    ;;  running does not understand anything else.
    (setq lcd (expand-file-name lcd))

    ;;  Start FTP session if it does not exist
    ;;
    (setq proc (ange-ftp-get-process host user))

;;;    (setq M mode U user H host D dir L lcd F file-list P proc)

    ;;  - Expand remote site's directory reference
    ;;
    (setq dir (ange-ftp-real-file-name-as-directory
               (ange-ftp-expand-dir host user dir)))

    ;;  Without this, the next command dies. This is already called in function
    ;;  ange-ftp-get-process, but for some unknown reason it must be called
    ;;  again to be sure: the hash mark size was sometimes nil
    ;;
    (save-excursion
      (set-buffer (ange-ftp-ftp-process-buffer host user))
      (if (null ange-ftp-ascii-hash-mark-size)
          (setq ange-ftp-ascii-hash-mark-size 1024))
      (if (null ange-ftp-binary-hash-mark-size)
          (setq ange-ftp-binary-hash-mark-size 1024))
      )


    (ange-ftp-set-binary-mode host user)

    ;;  - After this commands ANGE hangs quite often and never executes
    ;;    the "raw" commands
    ;;  - That's why we loop MAX-TRY times to start the
    ;;    command.
    (ange-ftp-send-cmd host user (list 'lcd lcd) "Setting lcd...")
    (message "")

    ;;  CD command dies if it the directory is wrong
    ;;
    ;;  The socond command just makes sure the command was successfull.
    ;;  I added this, because when connection was cloased and ange
    ;;  opened the connection again, the CWD command didn't succeed
    ;;  right away. We must wait here until it succeeds and only then
    ;;  send the real put or get request.
    ;;
    (ange-ftp-cd host user dir)
    (save-excursion
      (set-buffer (process-buffer proc))
      (setq try 0)

      (while
          (and (progn
                 (PMAX) (forward-line -1)
                 ;;  ftp> 250 CWD command successful.
                 (not (string-match "success" (read-full-line)))
                 )
               (< try max-try)
               )
        (inc try)
        ))


    (push mode file-list)       ;command for ange

    (save-excursion
      (set-buffer (process-buffer proc))
      (PMAX)

      ;;  Try sending untill the point moves... => process started
      ;;
      (setq point (point)   try 0)
      (while (and (eq point (point))
                  (< try max-try))

;;;     (d! (eq point (point)) point (point))

        (ange-ftp-raw-send-cmd
         proc
         (list-to-string file-list)
         "ftp ..."                      ;message displayed during 0%..100%
         (list func)                    ;called after completion ?
         (not not-bg)                   ;continue without wait
         )
        (PMAX) (inc try)
        ))


    ;;  The status value is valid only when process finishes.
    ;;
    (if not-bg
        (save-excursion
          (set-buffer (process-buffer proc))
          (setq ret ange-ftp-process-result)
          ))

    ret
    ))



;;; ----------------------------------------------------------------------
;;; - See tinyfh.el / tifh-ro-chmod which you can bind to a key
;;;
(defun ti::f-chmod-w-toggle (file)
  "Toggle read-only flag for FILE.
If file does not exist, or is not owned by user this function does nothing.

Return
  'w+    ,file made writable
  'w-    ,file made read-only.
  nil    ,file not processed."
  (let* ((file (expand-file-name file))
         mode
         )
    (if (null (file-modify-p file))
        nil
      (setq mode (file-toggle-rw (file-modes file)))
      (set-file-modes file mode)

      ;;  return value , -r--r--r-- , 600 oct= 384 dec
      (if (= 0 (logand mode 128)) 'w- 'w+)
      )))


;;; ----------------------------------------------------------------------
;;; - this will find duplicate lisp files in your path.
;;;
;;; 18 Dec 1995, pot@cnuce.cnr.it (Francesco Potorti`)     ,gnu.emacs.help
;;; 19 Dec 1995, Sudish Joseph <joseph@cis.ohio-state.edu> ,gnu.emacs.help
;;; 13 Feb 1996, Jari Aalto <jaalto@tre.tele.nokia.fi>, small correction.
;;;
(defun ti::f-find-shadows (&optional path)
  "Find duplicate files along optional PATH, which defaults to `load-path'."
  (interactive)
  (or path (setq path load-path))
  (save-excursion
    (let ((true-names   (mapcar 'file-truename path))
          (reduds       0)
          files dir
          out-buffer
          curr-files
          orig-dir
          files-seen-this-dir
          file
          d1 d2                         ;directories
          )

      (while path
        (if (member (car true-names) (cdr true-names))
            (setq reduds (1+ reduds))
          (setq dir (car path))
          (setq curr-files (if (file-accessible-directory-p dir)
                               (directory-files dir nil ".\\.elc?$" t)))
          (and curr-files
               (interactive-p)
               (message "Checking %d files in %s..." (length curr-files) dir))
          (setq files-seen-this-dir nil)
          (while curr-files
            (setq file (car curr-files))
            (setq file (substring
                        file 0 (if (string= (substring file -1) "c") -4 -3)))
            (if (member file files-seen-this-dir)
                nil
              (setq files-seen-this-dir (cons file files-seen-this-dir))
              (if (not (setq orig-dir (assoc file files)))
                  (setq files (cons (list file dir) files))
                (if (null out-buffer)
                    (progn
                      (setq out-buffer (get-buffer-create "*Shadows*"))
                      (display-buffer out-buffer)
                      (set-buffer out-buffer)
                      (erase-buffer)))

                ;; Do not print if directories are the same
                ;; ++ [jari]
                (setq d1 (file-name-as-directory (car (cdr orig-dir)))
                      d2 (file-name-as-directory dir))

                (cond
                 ((not (string= d1 d2))
                  (insert
                   (format "%s%s shadows\n%s%s\n\n" d1 file d2 file))
                  ))
                ))

            (setq curr-files (cdr curr-files))))
        (setq path (cdr path)
              true-names (cdr true-names)))
      (if (interactive-p)
          (let ((msg
                 (if out-buffer
                     (let ((n (/ (count-lines (point-min) (point-max)) 3)))
                       (format "%d shadowing%s found" n (if (eq n 1) "" "s")))
                   "No shadowings found")))
            (message "%s%s" msg
                     (if (zerop reduds) ""
                       (format " (skipped %d redundant entr%s in path)"
                               reduds (if (eq reduds 1) "y" "ies"))))))
      out-buffer)))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-dir-files (dir re &optional absolute form not-re-form)
  "Return files from DIR.

Input:

  DIR		directory name
  RE		regexp for files to match
  ABSOLUTE	flag, Return files as absolute names?
  FORM		eval form, test each file with FORM instead of RE
  NOT-RE-FORM	eval form, drop file if this evaluates to t

Example:

  ;;  Get all filenames that aren't zipped, backups or objects.
  ;;  The 'arg' will hold the filename
  ;;
  (ti::f-dir-files dir re t nil '(string-match \"g?[Z~#o]$\" arg)))


  ;; Return only directory names
  ;;
  (ti::f-dir-files dir \".\" 'absolute
                   '(file-directory-p arg)
                   '(string-match \"\\\\.\\\\.?$\" arg)
                   )

Return:
  list          ,(file file file ..)"
  (let* ((files   (directory-files dir absolute re))
         arg
         ret
         )
    (when (or form not-re-form)
      (list-loop files arg
        (when (or (null form)           ;accept all
                  (eval form))          ;accept only these
          (if not-re-form
              (if (null (eval not-re-form))
                  (push arg ret ))
            (push arg ret)
            ))))
      (reverse ret)
      ))


;;; ----------------------------------------------------------------------
;;;
;;;
(defun ti::f-files-only (list &optional eval-form)
  "Picks only existing files. Drops directories from LIST of strings.
Note: 200 files takes about 2-3 secs. If you supply EVAL-FORM, the item
will be included if the form Return t. You can refer to current item
with symbol 'arg'.

Input:
  list          ,list of strings
  `eval-form'     ,optional eval statement

Return:
 (file ..)     ,list"
  (let* (ret
         arg
         )
    (list-loop list arg
      (if (if eval-form
              (eval eval-form)
            (and (file-exists-p arg)
                 (not (file-directory-p arg))))
          (push arg ret))
      )
    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-newer-exist (f1 f2)
  "Return file F1 or F2 which is newer. If only one of them exist, return it.

Return:
  str   ,file
  nil   ,none of them exist"
  (cond
   ((and (file-exists-p f1)
         (file-exists-p f2))
    (if  (file-newer-than-file-p f1 f2)
        f1 f2))
   ((file-exists-p f1)
    f1)
   ((file-exists-p f2)
    f1)
   (t
    nil)
  ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-get-ext (file &optional mode)
  "Return FILE extension.
If MODE is nil, then return nil if none exist,
if MODE is non-nil, return empty string instead."
  (let* (list
         ext len
         )
;;;    (d! (null file) (null (o-string-match "\\." file)))

    (if (or (null file)
            (null (o-string-match "\\." file)))
        nil
      (setq list  (ti::s-get-blocks file "[\.]"))
      (setq len   (length list))

      (if (eq 1 len)
          (setq ext (car list))         ; first element
        (setq ext (nth (1- len) list))) ; last element
      )
    (if ext ext                         ;what to return?
      (if mode "" nil))
    ))



;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'which 'ti::f-get-load-path)
;;;
(defun ti::f-get-load-path (fn paths &optional all-paths verb)
  "Return full path name for FN accross the PATHS.
Input:
  FN            filename to search
  PATHS         list of path names
  ALL-PATHS     return all matches.
  VERB          verbose flag. Allows printing values in echo area

Return:
  nil           no matches
  str           first match if all-paths is nil
  list          list of matches along paths."
  (interactive
   (let* ((map (copy-keymap minibuffer-local-map))
          var1
          var2
          )
     (define-key map "\t"   'lisp-complete-symbol)
     (define-key map "\C-m" 'exit-minibuffer)

     (setq var1 (read-from-minibuffer "sFile: "))
     (setq var2 (read-from-minibuffer "Lisp var: " "exec-path" map))

     (list var1 (eval (intern-soft var2)))
     ))


  (let ((verb (or verb (interactive-p)))
        file found elt
        )
    (list-loop paths elt
      (when (stringp elt)               ;you never know what's in there...
        (setq file (concat (ti::s-verify-ends (car paths) "/") fn))

        (when (file-exists-p file)
          (if (null all-paths)
              (setq  found file  paths  nil) ;stop right there
            (push file found)
            ))
        ))

    (if (and found verb)
        (message (prin1-to-string found)))

    found
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-user-home ()
  "Try to guess user's home directory.

Return:
   /PATH/PATH/USER/    ,users home
   nil                 ,not found"
  (let* ((usr       (or (getenv "USER") (getenv "LOGNAME") ))
         (home      (or (getenv "HOME") (getenv "home") ))
         (tmp-buf   (ti::m-tmp-buffer))
         path
         )

    (cond
     ((> (length home) 0)               ;$HOME exist
      (setq path home))
     ((> (length usr) 0)                ;users name exist
      (save-excursion
        ;;   Okay try to get via 'pwd' process then.
        (call-process "pwd" nil tmp-buf nil)
        (set-buffer tmp-buf) (PMIN)
        (if (re-search-forward usr nil t)
            (setq path (buffer-substring (point-min) (match-end 0))))

        ;;  Failed ? try ls then...
        (if path t
          (erase-buffer)
          (call-process "ls" nil tmp-buf nil)
          (if (re-search-forward usr nil t)
              (setq path (buffer-substring (point-min) (match-end 0)))))

        (kill-buffer (current-buffer))
        )))

    ;;  make sure it has  trailing "/"
    (and (stringp path)
         (setq path (ti::s-verify-ends path "/")))
    path
    ))


;;; ----------------------------------------------------------------------
;;; Original idea by dcw@gcm.com (David Worenklein)
;;; in gnu.emacs.help Mon, 12 Jun 1995
;;;
;;; You can use this in interactive command to build up a completion list:
;;; like this:
;;;
;;;  (interactive
;;;     (list (completing-read
;;;             "Visit file: " (ti::f-file-list load-path "\\.el$"))))
;;;   (let ((pair (assoc emacs-file (ti::f-file-list load-path "\\.el$"))))
;;;     (if pair
;;;             (find-file (cdr pair))
;;;       (find-file (expand-file-name emacs-file "~/emacs")))))
;;;
(defun ti::f-file-list (dirs re)
  "Read DIRS and return assoc of files matching RE. (FILE FULL-PATH-FILE)."
  (let ((files nil)
        )
    (and (stringp dirs)                 ;only one entry given ?
         (setq dirs (list dirs)))
    (while dirs
      (setq files
            (append files (directory-files (car dirs) t re)))
      (setq dirs (cdr dirs)))
    (mapcar
     (function
      (lambda (file)
        (cons (file-name-nondirectory file) file))
      files)
     )))



;;; ----------------------------------------------------------------------
;;;
(defun ti::f-complete-file-name (file-name &optional dir flist)
  "Given a FILE-NAME string return the completed file name.

Input:
  If FILE-NAME is invalid entry, signal no error and return nil
  If no DIR is not given, use FILE-NAME's directory.
  If no DIR nor FILE-NAME dir, use `default-directory'
  if non-nil flag FLIST, then return completed filename list

Return:
  str           ,full completion
  list          ,list of completions if FLIST is set.
  nil           ,not unique"
  (let* ((file         (substitute-in-file-name file-name))
         (uncomplete   (file-name-nondirectory file))
         dir odir
         completed
         )

    ;;  Save the original directory.
    (setq odir
          (substring file-name 0 (- (length file-name) (length uncomplete))))

    ;;  expand-file-name dies if default-directory is nil
    ;;
    (setq dir
	  (expand-file-name
	   (or dir
	       (file-name-directory file-name)
	       default-directory
	       "~")))

    (setq completed
          (safe-do
            ;;   if given impossible entry like "!@#!#" this breaks
            ;;   without safe-do
            (file-name-all-completions uncomplete dir)))

;;;    (d! completed)

    ;; Only one match in the list? voila!
    ;;
    (if (and completed
             (eq 1 (length completed)))
        (setq completed (car completed)))

    (cond
     ((and (stringp completed)
           (not (string= completed uncomplete)))
      (concat odir completed))

     ((and flist completed)
      completed
      )

     (t
      nil
      ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-complete-file-name-word  (&optional no-msg)
  "Complete filename at point.
`default-directory' is used if no directory part in filename.
See `ti::f-complete-file-name'.

You can use this feature easily in lisp interactive call.
See macro `ti::f-complete-filename-minibuffer' for more.

NO-MSG  if non-nil, do not flash possible choices at current point
        The `sit-for' command is used for displaying, so you can
	interrupt it by pressing any key."
  (interactive "P")
  (let* ((word  (save-excursion
                  (forward-char -1)
		  (ti::b-read-space-word)))
	 (oword  word)
         (enable-recursive-minibuffers t)
         point
         msg
         )
    ;;  expand-file-name dies if default-directory is nil
    ;;
    (or default-directory
	(error "default-directory is nil !!"))

;;;    (d!! word default-directory "\n" )
    (unless (var-nil-p word)
      (setq word (ti::f-complete-file-name word nil 'list))
;;;      (d!! (null no-msg) (listpp word) word "\n" )

      (when (and (null no-msg) (listpp word))
        (setq msg
              (format "%d: %s"
                      (length word)
                      (list-to-string word)))

        (message msg)
        (sit-for 1)
        )

      (when (or (stringp word)          ;Cursor is at echo-area
		;;  It was list, common substring of all completions
		;;
		(progn
		  (setq word (try-completion
			      oword
			      (list-to-assoc-menu word)))
		  ;;  Try completion returs t if exact match, but
		  ;;  it shouldn't happen here any more. Still,
		  ;;  be cautious.
		  ;;
		  (stringp word)
		  ))
	(skip-chars-backward "^\n\t ")
	(setq point (point))
	(skip-chars-forward "^\n\t ")
	(delete-region point (point))
	(insert word)
	)

      )
    ))

;;; ----------------------------------------------------------------------
;;;
(put 'ti::f-complete-filename-minibuffer 'lisp-indent-hook 0)
(defmacro ti::f-complete-filename-minibuffer  (&rest body)
  "Complete filename in minibuffer prompt and do BODY.
Use variable 'map' to pass map to `read-from-minibuffer' function.

Example call:

  (ti::f-complete-filename-minibuffer
    (read-from-minibuffer \"test\" nil map))

Example 2:

    (defun my-example (string file-list)
      \"Giving file list as sring while allowing completion on words\"
      (interactive
       (list

        (read-from-minibuffer \"Gimme string: \")

        (ti::s-get-blocks
         (ti::f-complete-filename-minibuffer
           (read-from-minibuffer \"Gimme file-list: \" nil map))
         \"[ ]+\")
        ))
      (list string file-list)
      )

    (setq result (call-interactively 'my-example)) \"test\" RET <files> RET
    result
    --> (\"test\" (\"~/\" \"~/bin\" \"~/exe/\"))"
  (`
   (let* ((map (copy-keymap minibuffer-local-map))
          )
     ;;  this X-event also exists for tab
     ;;
     (define-key map [kp-tab]	'ti::f-complete-file-name-word)
     (define-key map [tab]	'ti::f-complete-file-name-word)
     (define-key map "\t"	'ti::f-complete-file-name-word)
     (,@ body)
     )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::f-read-file-list (&optional message)
  "Read file or directory list as one string, and return it as LIST.
Display optional MESSAGE, otherwise use default message.

Filesnames can be completed with tab. `default-directory' is used for
files that do not have directory part. Make sure default dir has ending
slash.

Example:

  (setq files (mapcar 'expand-file-name (ti::f-read-file-list)))

Return:
  (ELT ELT ..)          ,with `default-directory'
  nil                   ,no input"
  (let* (list
         files
         str
         )
     (setq str
           (ti::f-complete-filename-minibuffer
             (read-from-minibuffer
              (or message (format
                           "...%s: "
                           ;; limit the directory name
                           (str-right default-directory 10)))
              nil map)))
     (unless (var-nil-p str)		;not empty?
       (setq files (ti::s-get-blocks str "[ \t]+"))
       (list-loop
           files str
         (if (not (string-match "/" str))
             (setq str (concat default-directory str)))
         (push str list)
         ))

     (reverse list)
     ))

;;}}}

;;{{{ Network streams

;;; ......................................................... &network ...


;;; ----------------------------------------------------------------------
;;;
(defun ti::p-finger-error (&optional buffer)
  "Read BUFFER containing a finger response after `ti::p-finger'.
If there is an error, then return possible error cause string.

Return:
 string     ,cause of error
 nil        ,no error"
  (let* (ret
         )
    (job-buffer (or buffer (current-buffer))
      (PMIN)
      (when (re-search-forward "unknown host:" nil t)
        (setq ret (read-full-line))
        ))
    ret
    ))

;;; ----------------------------------------------------------------------
;;; Original function in   mc-pgp.el:mc-pgp-fetch-from-finger
;;;
(defun ti::p-finger (email &optional port timeout buffer verb)
  "Finger EMAIL on PORT with TIMEOUT.
The output is clered from possible ^M characters.

Input:
  EMAIL             ,email address foo@site.com
  PORT              ,default is 79
  TIME              ,default is 25
  BUFFER            ,where to store result, default is *finger tmp*
  VERB              ,print verbose messages

Return:
  string            ,error while doing opening network stream
  buffer-pointer"
  (interactive "sFiger email: ")
  (let (connection
        user
        host
        ret
        )
    (setq verb      (or verb (interactive-p))
          port      (or port 79)
          timeout   (or timeout 25)
          )

    (if (not (string-match "^\\([^ \t]+\\)@\\([^[ \t]+\\)" email))
        (error "Need email address foo@site.com '%s'" email)
      (setq user (read-match 1 email)
            host (read-match 2 email))

      (save-excursion
        (unwind-protect

            (progn
              (if verb     (message "Fingering %s ..." email))
              (setq buffer (or buffer (use-tmp-buffer "*finger tmp*" 'clear)))

;;;           (pop-to-buffer buffer) (d! "going finger....")
              (condition-case error
                  (progn
                    (setq
                     connection
                     (open-network-stream "*finger*" buffer host port))

                    (process-send-string
                     connection (concat "/W " user "\r\n"))

                    (while (and (memq  (process-status connection) '(open))
                                (accept-process-output connection timeout))
                      ))
                (file-error
                 ;; '(file-error "connection refused "connection failed" ..)
                 (setq ret (list-to-string (cdr error)))
                 )
                (error
                 (setq ret (list-to-string (cdr error)))
                 ))
              (if connection (delete-process connection))

              ;;  Strip Ctrl-M marks
              (job-buffer buffer (ti::b-u2d-crlf 'dos2unix))
              )))

      (when verb
        (message "Fingering %s ...done" email))

      (if (interactive-p)
          (pop-to-buffer buffer))

;;;    (d! "fing" connection buffer ret)

      (if connection
          buffer ret)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::p-http-request  (command &optional port timeout buffer verb)
  "Send http COMMAND.

If COMMAND includes port number, eg:

  http://www-swiss.ai.mit.edu:80/htbin/pks-extract-key.pl

This is actually intepreted as

  http    = www-swiss.ai.mit.edu
  port    = 80
  command = /htbin/pks-extract-key.pl

Input:

  COMMAND       http command string
  PORT          default is 80
  TIMEOUT       default is 60
  BUFFER        where to store result, default is *finger tmp*
  VERB          print verbose messages

Return:

  string        error while doing opening network stream
  buffer-pointer"
  (interactive "sHttp request: ")
  (let (connection
        host
        ret
        )
    (setq verb      (or verb (interactive-p))
          port      (or port 80)
          timeout   (or timeout 90)
          )

    (if (not (string-match "^http://\\([^/]+\\)\\(/.*\\)" command))
        (error "Must be _http_ request '%s'" command)
      (setq host    (read-match 1 command)
            command (read-match 2 command)
            )
      (if (string-match "\\(.*\\):\\([0-9]+\\)" host)
          (setq port (read-match 2 host)
                host (read-match 1 host)))
      )



    (save-excursion
      (unwind-protect
          (progn
            (if verb     (message "Http %s ..." host))
            (setq buffer (or buffer (use-tmp-buffer "*http tmp*" 'clear)))

;;;         (d! host port command "sending http....")
            (condition-case error
                (progn
                  (setq
                   connection
                   (open-network-stream "*http*" buffer host port))

                  (process-send-string
                   connection (concat "GET " command "\r\n"))

                  (while (and (eq 'open (process-status connection))
                              (accept-process-output connection timeout)))
                  )
              (file-error
               ;; '(file-error "connection refused "connection failed" ..)
               (setq ret (list-to-string (cdr error)))
               )
              (error
               (setq ret (list-to-string (cdr error)))
               ))
            (if connection (delete-process connection))

            ;;  Strip Ctrl-M marks
            (job-buffer buffer (ti::b-u2d-crlf 'dos2unix))
            )
          ))

    (when verb
      (message "Http %s ...done" host))

    (if (interactive-p)
        (pop-to-buffer buffer))

    (if connection
        buffer ret)
    ))




;;}}}
;;{{{ shell: uu/de/en/code

;;; .............................................................. &uu ...

;;; ----------------------------------------------------------------------
;;; #defalias    (defalias 'uu-save 'ti::p-uu-save-buffer)
;;;
(defun ti::p-uu-save-buffer (file &optional do-it verb)
  "Save uu data in buffer to FILE and uuencode it if DO-IT is non-nil. VERB.
Starts searching from current point. Leaves point after The UUdata if
it is found.

References:

  `ti:-uude-cmd'          ,command to unpack data
  `ti:-tmp-buffer'        ,output of shell command

Return:
  status                ,shell unpack status if unpack requested"
  (interactive "Fsave UU data to file: \nP")
  (let* ((verb    (or verb (interactive-p)))
         (buffer  (ti::m-tmp-buffer))
         status                         ;shell status
         dir
         ret
         )
    (setq ret (ti::b-uu-area))

    (if (file-directory-p file)
        (error "Need filename: FILE arg was directory"))

    (if (null ret)
        (if verb
            (message "Cannot detect UU data..."))
      (save-excursion
        (set-buffer buffer)
        (write-region (point-min) (point-max) file))

      (file-components file dir file)

      (if (or (and (interactive-p) (null (y-or-n-p "Unpack UU ? ")))
              (and (null verb) (null do-it)))
          nil

        (set-buffer buffer)
        (erase-buffer)
        (if verb
            (display-buffer buffer))

        (if verb  (message "Shell unpacking uu..."))
        (setq status
              (shell-execute
               (concat "cd " dir "; " (funcall ti:-uude-cmd file))
               buffer
               ))
        (if (null verb)
            nil
          (message
           (concat
            "Shell unpacking uu...done. Saved uu file was " dir file))

          ;;   Show the shell output to user if there was an error
          (cond
           ((shell-exec-nok-p status)
            (display-buffer buffer)
            (message (concat "Shell returned error code: " status))))
          )

        ))
    status
    ))



;;; ----------------------------------------------------------------------
;;; #defalias    (defalias 'uu-make 'ti::p-uu-make)
;;;
(defun ti::p-uu-make (file &optional zip-ext insert buffer verb)
  "Uuencodes FILE.

Input:

  ZIP-EXT       String, if \".Z\" or \".gz\" file is compressed.
                Empty string is ignored.
  INSERT        If non-nil file is inserted into current point.
                point is after the UU data.
  BUFFER        Buffer to insert.
  VERB          Enable verbose messages. Turned on automatically
                when user calls this function. The VERB also
                causes `save-excursion'; so that the point does not
                move if INSERT is in effect.

Prefix arg:
  interactive call      ,file is inserted automatically

References:
  `ti:-uu-cmd'            ,command to pack data
  `ti:-tmp-buffer'        ,output of shell command; ready UU is here by default"
  (interactive "*Ffile to uu: \nsCompression ext: \n")
  (let* (
         (tmp-buffer    (ti::m-tmp-buffer 'clear))
         (uu-cmd        ti:-uu-cmd)
         (insert        (or insert
                            (interactive-p)))
         (verb          (or verb (interactive-p)))
         (buffer        (or buffer
                            (if (or insert
                                    (interactive-p))
                            (current-buffer))))
         cmd
         fn
         dir
         )

    (save-excursion
      (set-buffer tmp-buffer)

      (if (var-nil-p zip-ext)
          nil
        (require 'jka-compr)
        (insert-file-contents file)
        (setq fn  (concat (ti::m-tmp-dir file)
                          (ti::s-verify-ends    ;; " Z " --> ".Z"
                           (ti::s-sqz zip-ext)
                           "\\." "." 'beg)
                          ))
        (use-tmp-file fn)               ;aborts with error if nok...


        (set-buffer-modified-p nil)
        (and verb (message "Compressing..."))
        (jka-compr-write-region (point-min) (point-max) fn)
        (and verb (message "Compressing...done"))
        (erase-buffer)

        (setq file fn)

        ;;  Write file gives messages....
        (message "")
        ))


    (file-components file dir fn)
    (if insert
        (setq tmp-buffer (or buffer tmp-buffer)))

    ;;  It's funny that standard UU requires filename twice....
    ;;
    ;;  When in interactive mode, user does *not* want to
    ;;  end up after the UU, instead, he wants to stay at the same
    ;;  position. We use save excursion in that case.
    ;;
    ;;  Shell commands need buffer name, not buffer pointer


    (setq tmp-buffer (buffer-name tmp-buffer))
    (setq cmd (concat "cd " dir "; " (funcall uu-cmd fn)))
    (if verb
        (save-excursion (shell-execute cmd tmp-buffer ))
      (shell-execute cmd tmp-buffer))

    (if (not (var-nil-p zip-ext))       ;remove temporary zip file
        (file-delete-safe file))

    (if (and verb (not insert))         ;show to user ?
        (pop-to-buffer buffer))


    ))

;;}}}
;;{{{ shell: zipping

;;; ....................................................... &shell-zip ...



;;; ----------------------------------------------------------------------
;;; - Many times I want to send zipped packages over the mail, they are more
;;;   convenienty than tar.gz files...
;;;
;;; #todo
;;; #not tested
;;;
(defun ti::p-zip (zip-file files &optional zip-cmd clear)
  "Achive to ZIP-FILE. FILES is list (file file ..).
The ZIP-CMD defaults to \"zip -9 -q\",

When CLEAR is given, output buffer is cleared before command is
thrown to shell.

Command will not return until the process has finished.

References:
  `ti:-tmp-buffer'   ,all output the process transmits."
  (let* ((zcmd          (or zip-cmd "zip -9 -q "))
         (buffer        (ti::m-tmp-buffer))
         (flist         (list-join files))
         (cmd           (concat zcmd " " zip-file " " flist))
         )
    (if clear
        (clear-buffer buffer))
    (and zip-file files
         (shell-execute cmd buffer))
    ))



;;; ----------------------------------------------------------------------
;;; #todo
;;; #not tested
;;;
(defun ti::p-zip-dir-files (zip-file dir re &optional zip-cmd)
  "Achive to ZIP-FILE from DIR matching RE with ZIP-CMD."
  (let* ((list          (directory-files t (or zip-cmd "zip -9 -q ")))
         (buffer        (ti::m-tmp-buffer))
         (flist         (list-join list))
         (cmd           (concat zip-cmd " " zip-file " " flist))
         clear
         )
    (error "do not use this.")
    (if clear
        (clear-buffer buffer))

    (shell-execute cmd buffer)
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::p-zip-view-command (file &optional buffer nice zip-cmd verb)
  "Insert zip file listing to point.

Input:
  FILE      ,tar file
  BUFFER    ,defaults to current buffer
  NICE      ,if non-nil, insert file name and empty lines around listing.
  ZIP-CMD   ;defaults to 'unzip -v %s'
  VERB      ,verbose mode

Return:
  nil       ,no action [file not exist ...]
  nbr       ,shell return code"
  (interactive "fTar file: ")
  (let* ((cmd   (or zip-cmd "unzip -v %s"))
         )
    (VERB)
    (if (not (and (stringp file)
		  (file-exists-p file)))
	(error "Invalid file argument")
      (if nice
	  (insert "file " (file-name-nondirectory file) ":\n"))

      (shell-execute (format cmd (expand-file-name file))
                     (or buffer (current-buffer)))
      (if nice (insert "\n"))
      )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::p-tar-zip-view-maybe-command (file)
  "If FILE is zip/tar then insert listing to current point."
  (cond
   ((string-match "\\.tar$\\|\\.tar.gz$\\|\\.tgz$" file)
    (ti::p-tar-view-command file nil 'nice))
   ((string-match "\\.zip$" file)
    (ti::p-zip-view-command file nil 'nice))
   ))



;;}}}
;;{{{ shell: tar


;;; ----------------------------------------------------------------------
;;;
(defun ti::p-tar-view-command (file &optional buffer nice verb)
  "Insert tar file listing to point.

Input:
  FILE      ,tar file
  BUFFER    ,default to current buffer
  NICE      ,if non-nil, insert file name and empty lines around listing.
  VERB      ,verbose mode

Return:
  nil       ,no action [file not exist ...]
  nbr       ,shell return code"
  (interactive "fTar file: ")
  (let* ((def  (cond
                ((string-match "\\.tar$" file)
                 "tar -tvf %s")
                ((string-match "\\.tar\\.gz$" file)
                 "gzip -d -c %s |tar -tvf -")
                ;;  don't know this currently ...
                ((string-match "\\.tgz$" file)
                 nil)
                ))
         cmd
         )
    ;; Default tar switches:
    ;; -t       ,List the name
    ;; -v       ,verbose
    ;; -f       ,next arg argument as the name of the archive (file)
    ;;

    (VERB)
    (when (and
           (stringp file)
           (file-exists-p file)
           (progn
             (or (file-exists-p "/hp-ux/")
                 (file-exists-p "/vol/")
                 (and verb
                      (y-or-n-p
                       (format "\
Can't guess tar command, try using default %s ? " def)
                      )))
             (setq cmd def)
             ))
      (if nice (insert "file " (file-name-nondirectory file) ":\n"))
      (shell-execute (format cmd (expand-file-name file))
                     (or buffer (current-buffer)))
      (if nice (insert "\n"))
      )))



;;; ----------------------------------------------------------------------
;;;
(defun ti::p-tar-read-listing-forward  ()
  "Read all tar filenames from current point forward.
The tar listing looks like this:

r-xr-xr-x 240/222   4269 Feb  3 09:25 1997 aa.cc
r-xr-xr-x 240/222  41515 Feb  3 09:40 1997 bb.cc
r-xr-xr-x 240/222   3013 Feb  3 09:40 1997 dd.cc


Return:
 '((FILE SIZE PERMISSIONS)  (F S P) ..)"
  (let* ((re (concat
              "^\\([drwx-]+\\)[ \t]+[0-9]+/[0-9]+"
              "[ \t]+\\([0-9]+\\).*[0-9]+:[0-9]+ +[0-9]+"
              " +\\(.*\\)"
              ))
         list
         )
    (when (or (looking-at re)
              (re-search-forward re nil t))
      (beginning-of-line)
      (while (and (looking-at re)
                  (not (eobp)))
        (push (list (read-match 3) (read-match 2) (read-match 1)) list)
        (forward-line 1)
        ))
    (reverse list)
    ))





;;}}}
;;{{{ shell: grep

;;; ----------------------------------------------------------------------
;;;
(defsubst ti::p-run-grep (grep-pat absolute-path)
  "Use Unix 'grep' command to find GREP-PAT along files in ABSOLUTE-PATH.

Return:
 string
 nil"
  (let* ((cmd (concat
               "grep '" grep-pat "' "
               (expand-file-name absolute-path)
	       ))
         ret
         )
    (if (var-nil-p (setq ret (exec-to-string cmd)))
        nil
      ret
      )))

;;}}}
;;{{{ shell: comint handling

;;; ----------------------------------------------------------------------
;;;
(defun ti::p-comint-run-command
  (buffer-name clear form wait-re string-arg-list)
  "Run command string inside shell comint buffer.
Every item in string-arg-list is feed after the previous
arg doesn't produce any output.

Input:

  BUFFER-NAME      defaults to \"ti::comint\"
  CLEAR            should we clear the buffer before feeding args
  FORM             form to eval in comint buffer, eg. you can give
                   '(comint-interrupt-subjob) in this arg.
  WAIT-RE          if given, then wait this regexp to appear
                   at the end of input before feeding the args
                   to the process.
  STRING-ARG-LIST  list of string to feed to comint shell.

Return:
  comint buffer pointer

Example:
  ;; this runs 3 commands
  (ti::p-comint-run-command nil 'clear nil nil '(\"pwd\" \"cd .\" \"date\"))"

  (interactive)
  (let* ((prog          (or explicit-shell-file-name "/bin/sh"))
         (buffer-comint (or buffer-name "ti::comint"))
         (buffer        (concat "*" buffer-comint "*"))

         ;;  Gives '(run stop) if process is idle.
         (stat           (comint-check-proc buffer))
         (count          0)
         line
         proc
         string
         point

         )
    (save-excursion
      (if (null stat)
          (progn
            (kill-buffer-safe buffer)
            (set-buffer (apply 'make-comint buffer-comint prog nil nil))
            (comint-mode)
            (sleep-for 2)               ;Wait shell, #todo
            ))
      (set-buffer buffer)

      (setq proc (get-buffer-process (current-buffer)))




      ;;  Just silence ByteComp. I may use this later
      ;;
      (if (null proc) (setq proc nil))


      (if clear
          (erase-buffer))

      (if form
          (eval form))

      (goto-char (PROC-MARK))

      (list-loop string-arg-list string
        (goto-char (or (PROC-MARK) (point-max)))
        (insert string)
        (comint-send-input)

        (while
            (progn                      ;Wait until process gives something
              (cond
               (wait-re
                (PMAX) (setq line (read-full-line))
                (inc count)
                (or (> count 100)       ;prevent infinite loop
                    (string-match wait-re line))
                )
               (t
                (setq point (buffer-size))
                ;; #todo, should be better way to check input end.
                ;;
                (sit-for 1)
                (inc count)
                (or (> count 500)
                    (not (eq point (buffer-size))))
                ))))

        )

      (current-buffer)
      )))

;;}}}
;;{{{ Reading lines, passwords

;;; ----------------------------------------------------------------------
;;;
(defun ti::q-read-input-invisible ()
  "Read keyboard input. If user presses ESC, the asking is interrupted.
Return:
  nil
  string"
  (let* ((echo-keystrokes 0)            ;prevent showing
         str ch
         )
    (while (not (char-in-list-case ch '(?\n ?\C-m ?\e)))
      (cond
       ((char-in-list-case ch '(?\b ?\177))
        (if (eq 0 (length str))
            (beep)
          (setq str (substring str 0 (1- (length str))))
          ))

       ((print-p ch)
        (setq str (concat str (char-to-string ch)))
        ))

      (setq ch (read-char-safe-until))
      )

    (if (char= ch ?\e)
        (setq str nil))

    str
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::q-read-input-as-password (&optional prompt max echo-char)
  "Return read password using PROMPT, MAX chacters with ECHO-CHAR.
If user presses ESC, return nil."
  (let* (
        (prompt                 (or prompt ""))
        (cursor-in-echo-area    nil)
        (max                    (or max 80));maximum string
        (bar (if echo-char
                 (make-string (+ max 2) echo-char )
             (make-string (+ max 2) ?* )))
        str
        ch
        len
        )
    (message prompt)
    (while (not (char-in-list-case ch '(?\n ?\C-m ?\e)))
      (cond
       ((or (char-in-list-case ch '(?\b ?\177)))
        (setq len (length str))
        (unless (= len 0 )
          (setq str (substring str 0 (1- len))))
        )

       ((print-p ch)
        (if (>= (length str) max)
            (beep)                      ;signal error
          (setq str (concat str (char-to-string ch)))
          (message (substring bar 0 (length str))))
        ))

      (setq ch (read-char-safe-until
                (concat prompt (substring bar 0 (length str)))))
      )
    (message "")

    (if (char= ch ?\e)
        (setq str nil))

    str
    ))

;;}}}

;;{{{ misc: selective display

;;; ................................................... &misc-packages ...


;;; ----------------------------------------------------------------------
;;; - Separating the "efective dispalay" is easy with this...
;;;
;;;
(defun ti::m-selective-display-copy-to  (beg end buffer &optional verb)
  "Copy region BEG END selective display to BUFFER. VERB.
Eg. folding.el and outline based modes use selective display."
  (interactive
   (let* ()
     (if (not (ti::xe-mark-active-p))
         (error "Region not selected."))

     (list
      (region-beginning)
      (region-end)
      (read-from-minibuffer "To buffer: " "*selective display*")
      )))

  (let* ((bp    (get-buffer-create buffer)) ;barfs if invalid...
         (bp    (use-tmp-buffer bp 'clear)) ;ok, use it
         (verb  (or verb (interactive-p)))
         line
         )
    (save-excursion
      (save-restriction
        (narrow-to-region beg end) (PMIN)
        (while (not (eobp))
          ;;  - Reset for normal lines.
          ;;  - Or reads until \r. Ie. the hidden part is not read


          (setq line (or (and (o-looking-at ".*\r")
                              (concat
                               (bufmatget  "\\([^\r]+\\)+\r" 1)
                               "..."))
                         (read-full-line)))
          (strcat line "\n")

          (forward-line 1)
          (append-to-buffer-string bp line)
          )))
    (if verb
        (pop-to-buffer bp))
    ))


;;; ----------------------------------------------------------------------
;;; - Print folding.el and outline based buffer with this...
;;;
(defun ti::m-selective-display-print  (beg end)
  "Print selective display region BEG END."
  (interactive "r")
  (let* ((buffer  (ti::m-tmp-buffer))
         )
    (ti::m-selective-display-copy-to beg end buffer)
    (save-excursion
      (set-buffer buffer)
      (print-buffer)
      )))

;;}}}
;;{{{ misc: advice control

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-ad-control
  (single-or-list regexp &optional disable verb msg)
  "Enables/disables SINGLE-OR-LIST of adviced functions that match REGEXP.
Signals no errors, even if function in LIST is not adviced.
All advice classes ['any] are ena/disabled for REGEXP.

Input:

  SINGLE-OR-LIST        function of list of functions.
  REGEXP                advice name regexp. Should normally have ^ anchor
  DISABLE               flag, of non-nil then disable
  VERB                  enable verbose messages
  MSG                   display this message + on/off indication"
  (let* ((list (list-make single-or-list))
         func
         )
    (list-loop list func
      (safe-do
        (if disable
            (ad-disable-advice  func 'any regexp)
          (ad-enable-advice     func 'any regexp))
        (ad-activate func)                              ;;change state
        ))

     (if verb
        (message
         (concat
          (or msg "advice(s): ")
          (if disable "off" "on"))))
    ))

;;}}}

;;{{{ misc: -- packaging, install, reports

;;; ..................................................... &bug-reports ...
;;; - I'd advice you to take a look at lisp-mnt.el if you're writing
;;;   your own packages.

;;; ----------------------------------------------------------------------
;;; #defalias (defalias 'package-feedback 'ti::m-pkg-feedback)
;;;
(defun ti::m-pkg-submit-feedback (lib)
  "Composes feedback report with lisp-mnt.el conmoncerning lisp file LIB.
Make sure the file beeing reported is valid according to
lisp-mnt's command `lm-verify'."
  (interactive "sSend mail regarding file: ")
    (let (file
          version
          buffer
          )
      (cond
       ((setq file
              (or (ti::f-get-load-path lib load-path)
                  (progn
                    (setq lib (concat lib ".gz"))
                    (ti::f-get-load-path lib load-path)
                    )))
        (require 'lisp-mnt)
        (set-buffer (setq buffer (file-pure-find file)))
        (setq version (ti::m-rcs-buffer-version))
        (lm-report-bug
         (format "%s %s Feedback"
                 (or version "" )
                 (file-name-nondirectory file)))
        (kill-buffer buffer)
        )
       (t
        (error (concat "No such file in load path: " lib))
        ))
      ))



;;; ----------------------------------------------------------------------
;;; - See package tinydiff.el and function tdi-feedback there if you
;;;   are still curious how to use this function
;;;
(defun ti::m-pkg-submit-bug-report (lib id var-list)
  "Submit bug report with reporter.

PRECONDITIONS before using this function

1. You file must be in RCS control and it must have the \"\$ Id \$\" identifier
   stored into variable. Like the following:

   (defconst tinylib-version-id
     \"\$ Id: tinylib.el,v 1.18 1996/01/24 09:44:48 jaalto Exp jaalto \$\"
     \"Latest modification time and version number.\")

2. Your lisp code package must be valid according to lisp-mnt.el's command
   `lm-verify' so that the \"maintainer\" information can be extracted.
   This means that you file must have header like this:

  ;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>

Input:
  LIB           ,filename without path. Eg \"tinylib.el\"
  ID            ,the RCS Id string
  VAR-LIST      ,list of variables to get from package. Like '(var1 var2)"
  (interactive)
  (let* (maintainer
         subj
         list
         )
   (require 'reporter)

   (setq maintainer
         (or (car-safe (ti::m-pkg-get-header lib  "maintainer")) ""))
   (setq list (ti::s-get-blocks id "[ ]+"))
   (setq subj (concat (nth 2 list) " " (nth 1 list)))  ;; name && version

   (cond
    ((y-or-n-p "Do you really want to submit a report? ")
      (reporter-submit-bug-report
       maintainer
       (nth 1 list)
       var-list
       nil nil
       (concat "Hi,\n")
       )

      (PMIN)
      (if (re-search-forward "Subject: *" nil t)
          (insert subj))
      (re-search-forward "Hi,\n")
      ))
   ))



;;; ----------------------------------------------------------------------
;;; - So that you don't need to type long function name all the time,
;;;   you could add this into your ~/.emacs
;;;
;;; #defalias  (defalias 'tver 'ti::m-pkg-version-info)
;;;
;;;
(defun ti::m-pkg-version-info (lib &optional arg)
  "Gets package information and prints it to another buffer.
The LIB is searched along 'load-path'.

Preconditions:
 The file must be valid according to lisp-mnt.el::lm-verify

Interactive call:
  You can complete the filename with TAB key

Input:
  LIB   filename with .el added
  ARG   prefix arg, print the versionin info in mode-line
        instead of creating full version buffer."
  (interactive
   (let* (file
          )
     (setq
      file
      (ti::f-complete-filename-minibuffer
        (read-from-minibuffer
         (format "[%s] Version info for library: " default-directory)
         nil map
         )))
     (if (null file) (setq file file))  ;XEmacs 19.14 bytecompiler silencer

     ;; Make sure there is .el
     (list
      (ti::s-verify-ends file ".el")
      current-prefix-arg
      )))

  (let (out
        file
        buffer

        lm-version
        lm-summary
        lm-maintainer
        lm-creation-date
        lm-last-modified-date
        lm-commentary
        rcs-id
        maintainer-name
        maintainer-email
        )
    (cond
     ((setq file
            (or (ti::f-get-load-path lib load-path)
                (progn
                  (setq lib (concat lib ".gz"))
                  (ti::f-get-load-path lib load-path)
                  )))
      (require 'lisp-mnt)

      (cond
       ((not (null arg))
        (set-buffer (setq buffer (file-pure-find file)))
        (setq rcs-id (or (ti::m-rcs-str-find-buffer "Id") "<no rcs id>"))
        (kill-buffer buffer)
        (read-char-safe-until rcs-id)
        )

       (t
        (setq out (use-tmp-buffer "*version*" 'clear))


        ;;  Now get the information from file with lisp-mnt.el
        ;;
        (set-buffer (setq buffer (file-pure-find file)))
        (setq
         lm-version             (lm-version)
         lm-summary             (lm-summary)
         lm-maintainer          (lm-maintainer)
         lm-creation-date               (lm-creation-date)
         lm-last-modified-date  (lm-last-modified-date)
         lm-commentary          (lm-commentary)
         rcs-id                 (ti::m-rcs-str-find-buffer "Id")
         )
        (kill-buffer buffer)

        (setq maintainer-name
              (if  (not (null lm-maintainer))
                  (or (car-safe lm-maintainer) "<name not known>")
                "<name not known>"))

        (setq maintainer-email
              (if  (not (null lm-maintainer))
                  (or (cdr-safe lm-maintainer) "no email info")
                "no email info"))


        (switch-to-buffer-other-window out)
        (insert
         lib " -- "        (or lm-summary            "<no info>")        "\n\n"
         "Created      : " (or lm-creation-date      "<no info>")        "\n"
         "Last modified: " (or lm-last-modified-date "<no info>")        "\n"
         "Maintainer   : " maintainer-name " <" (or maintainer-email "") ">\n"
         "Version      : " (or lm-version            "<no info>")        "\n"
         "\n\n"
         (or lm-commentary "<no commentary found>")
         )
        (pop-to-buffer  out)
        (PMIN)
        )))
     (t
      (error (concat "No such file in load path: " lib))
      ))
    ))


;;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... ..



;;; ----------------------------------------------------------------------
;;;
;;;
(defun ti::m-pkg-get-header (lib header-list)
  "Get's standard header information: eg. maintainer, version, author.
The valid syntax of these headers is defined in lisp-mnt.el.
Make sure the file beeing visited can be run with  lisp-mnt's
command `lm-verify'.

Input:
  LIB           ,the filename of the package, including \".el\"
  HEADER-LIST   ,string or list of strings. Eg '(\"maintainer\")

Return:
  list          ,notice that empty hits are stored: '(nil nil ..)
  nil"
  (let ((header-list  (list-make header-list))
        hit elt
        file
        buffer
        ret
        )
    (cond
     ((setq file (ti::f-get-load-path lib load-path))
      (require 'lisp-mnt)
      (unwind-protect                   ;make sure file is removed
          (progn
            (set-buffer (setq buffer (file-pure-find file)))

            (mapcar
             (function
              (lambda (header)
                (setq elt (lm-header header))
                (if elt                 ;did we find any ?
                    (setq hit t))               ;raise flag
                (push elt ret)
                ))
             header-list))

        ;; Kill the file no matter what happens.
        (kill-buffer buffer))
      )

     (t
      (error (concat "No such file in load path: " lib))
      ))

    (if (null hit)                      ;if no hits, clear the ret value
        (setq ret nil))
    ret
    ))



;;; ......................................................... &package ...
;;; - Here is some special functions. When you insert some example to
;;;   your package, you can convert functions and text directly to
;;;   "poor man's shar" format :-)
;;; - With function ti::m-pkg-make-mode-magic, you just
;;;
;;;   1. Be in lisp mode
;;;   2. Select example area to be inserted into somewhere
;;;   3. call the functions --> The result is inserted into registed
;;;   4. Go to package buffer and insert the register contents there.
;;;
;;; - Likewise the user can rip these "shar" examples with function
;;;   ti::m-pkg-rip-magic
;;;
;;;   1. Select area and call the function. --> examples in register
;;;   2. Put them into your .emacs or another favourite file.
;;;
;;; - Use similar bindings
;;;   (global-set-key   "\C-cp" 'ti::m-pkg-make-mode-magic)
;;;   (global-set-key   "\C-cP" 'ti::m-pkg-rip-magic)


;;; ----------------------------------------------------------------------
;;; - Below is example of how to use this function. Taken from tinyreply.el
;;
;;* (defun tir-example-install ()
;;*   "Retrieve all example code"
;;*   (interactive)
;;*   (ti::m-pkg-ins-example "tinyreply.el")
;;*   (pop-to-buffer ti:-tmp-buffer)
;;*   (lisp-mode)
;;*   (message "Remember to confige! ")
;;*   )
;;
(defun ti::m-pkg-ins-example (lib &optional re)
  "Install example setup for you from LIB.
The LIB must be normal source file name ending in '.el'.
Function tries to find $PackageInstallRe: ' ' $
line which has the installation code chars in the surrounding
quotes. The common practise is to have '^[ \t]*;;+[*]' for lisp.
If that regexp is followed by char '_' it means that the line is left empty.

If you supply RE, it must have match in LEVEL 1.

Return:
  buffer pointer"
  (interactive "sLibrary: ")
  (let* ((tmp  "*ti::pkg*")
         (file (ti::f-get-load-path lib load-path))
         (verb (interactive-p))
         ;;    There has to be " " after the :, otw it's not
         ;;    rcs ident(1) compatible. Also before the last $ ,
         ;;    there must be space.
         (re   (or re "[$]PackageInstallRe: [ \t]*'\\(.*\\)' [$]"))
         (empty-line-ch   "_")
         bp                             ;buffer pointer
         id
         comment-re
         )
    (if (or (null file)
            (null (file-readable-p file)))
        (error (concat "Cannot locate/read " lib " in load-path: " file))
      (setq bp (use-tmp-buffer tmp 'clear))
      (job-buffer bp
        (insert-file-contents file)
        (PMIN)

        (if (or (null (re-search-forward re nil t))
                (null (match-end 1)))
            (progn
              (pop-to-buffer bp)
              (error (concat "Cannot satisfy install re: " re)))

          (setq comment-re (read-match 1)) ;read match in level 1

          (if (var-nil-p comment-re)
              (error (concat "Level 1 mismatch_" (read-match 0) "_" re)))
          (save-excursion (setq id (ti::m-rcs-str-find "Id" )))

          (ti::m-pkg-rip comment-re empty-line-ch)
          (PMIN)

          ;;  And final touch, add version id if it existed.
          (if (null id )
              (insert (concat ";; @(#) No rcs id found.\n\n"))
            (insert (concat ";; @(#) " id "\n\n")))

          ;;  Show contents if user called interactively.
          (when verb
            (pop-to-buffer bp)
            (message "Automatic install done."))
          )))
    bp
    ))


;;; ----------------------------------------------------------------------
;;
(defun ti::m-pkg-rip (re ch &optional beg end)
  "Delete section of commented text, so that only _code_ remains.
The installed code partion should have RE at front of each line.

RE must have anchor ^ and CH must have some magic char to
mean empty line. like RE = '^;;+[*]' and CH = '_':
    ;;*  ;;This belongs to automatic install, below is empty line code
    ;;*  _

Input:
 RE             ,regexp matching the examples
 CH             ,character signifying empty lines
 BEG END        ,area bounds

Return:
  t or nil"
  (interactive)
  (let* (
         (beg   (or beg (point-min)))
         (p     beg)
         (end   (or end (point-max)))
         ret
         )
    (save-restriction
      (narrow-to-region beg end)
      (PMIN)
;;;      (d! "~pkg-rip search " re)
      (if (null (re-search-forward re nil t))
          nil
        (goto-char p)
        (save-excursion (delete-non-matching-lines re))

        ;; Now we have only RE lines
        (while (not (eobp))
          (if (null (o-looking-at re))
              nil
            (delete-region (match-beginning 0) (match-end 0))
            (if (o-looking-at ch)               ;remove that char
                (delete-char 1))
            )
          (forward-line)
          (setq ret t)
          )))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::m-pkg-rip-magic (beg end)
  "As `ti::m-pkg-rip' BEG END, except the area is pasted to temporary buffer.
Tthe lines are prepared AND the result is inserted to register.

Make sure your are viewing the piece of code in the same mode that it is
supposed to be used. Otherwise the magic syntax isn't regognized.

Return:
  t or nil"
  (interactive "r")
  (let* ((ob      (current-buffer))
         (tbuf    (ti::m-tmp-buffer))
         (verb    (interactive-p))
         (str     (ti::m-pkg-make-var))
         (empty   "_")
         (reg     ?p)                   ; "p" as "package"
         (reg-n   (char-to-string reg)) ;register name
         ret
         re
         )
    (if (var-nil-p str)
        (error "No syntax found.")
      (set-buffer tbuf) (erase-buffer)
      (insert-buffer-substring ob beg end) ;get the area

      (setq re (concat "^" (regexp-quote str)))
      (setq ret (ti::m-pkg-rip re empty (point-min) (point-max)))
      (if (null (and ret verb))
          nil
        (set-register reg (buffer-substring (point-min) (point-max)))
        (message (concat "Example ripped into register '" reg-n "'."))
        ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-pkg-make-mode-magic (beg end)
  "As `ti::m-pkg-make-mode', except BEG END is pasted to temporary buffer.
The lines are prepared AND the result is inserted to register.

Return:
  t or nil according to success."
  (interactive "r")
  (let* ((d     (ti::m-tmp-buffer))     ;dest. buf
         (s     (current-buffer))       ;source buf
         (m     major-mode)             ;we must use same mode
         (verb  (interactive-p))
         (reg   ?p)
         (reg-n (char-to-string reg))   ;register name
         )
    (save-excursion
      (set-buffer d) (erase-buffer)
      (insert-buffer-substring s beg end)
      ;;  turning mode on may have effects, since it runs hooks...
      (funcall m)                       ;turn on same mode
      (if (null (ti::m-pkg-make-mode (point-min) (point-max))) nil
        ;; Succeeded
        (set-register reg (buffer-substring (point-min) (point-max)))
        (if verb (message "example in register:" reg-n ))
        ))
    ))


;;; ----------------------------------------------------------------------
;;;
;;;
(defun ti::m-pkg-make-mode (beg end)
  "Make embedded package around BEG END according to mode.
** DOES NOT WORK FOR MODES WITH `comment-end' ***

Return:
  nil or t if successfull."
  (interactive "*r")
  (let* ((str     (ti::m-pkg-make-var))
         (empty   "_")
         ret
         )
    (if (not (var-nil-p comment-end))
        (message "tinylib: Comment end found, cannot proceed.")
      (ti::m-pkg-make beg end str empty)
      (setq ret t)
      )
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
;;;
(defun ti::m-pkg-make-var ()
  "Return Packaging variable 'str' according to mode.
If mode has no comment syntax default ';;* ' is used."
  (let* (
         (cs comment-start)
         (cs (cond                      ;set up something special
              ((memq major-mode
                     '(lisp-mode emacs-lisp-mode lisp-interaction-mode))
               (setq cs ";;"))          ;default ';' isn't enough
              (t cs)))                  ;do not change it
         (str (if (null cs)
                ";;* "
                (concat cs "* ") ;make sure there is space
                ))
         )
    str
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-pkg-make (beg end str ch)
  "Format area for automatic install.

Input:
  BEG END       area
  STR           string to be added at front
  CH            additional character for empty lines."
  (let* ((empty (concat str
                        (cond
                         ((integerp ch)
                          (char-to-string ch))
                         (
                          ch))))
         )
    (save-restriction
      (narrow-to-region beg end)
      (goto-char (min beg end))
      (while (not (eobp))
        (if (o-looking-at "^[ \t]*$")
            (insert empty)
          (insert str))
        (forward-line 1)
        ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-pkg-autoload-create-on-file (file &optional buffer no-show)
  "Very simple autoload function generator out of FILE.
Optionally put results to BUFFER. NO-SHOW does not show buffer.

Notes:
  Doesn't care about ###autoload tags; reads all functions."
  (interactive "fConstruct lisp autoloads from file: ")
  (let* ((tmp (if buffer
		  (use-tmp-buffer buffer 'clear)
		buffer (ti::m-tmp-buffer 'clear)))
         (fn  (file-name-nondirectory file))
         list elt
         args
         func type
         str
         iact
         point
         )

    ;;   We want to say (autoload 'func "pacakge" t t)
    ;;   and not        (autoload 'func "pacakge.el" t t)
    ;;   so that .elc files can be used.
    ;;
    (if (string-match "\\(.*\\).el" fn)
        (setq fn (read-match 1 fn)))


    (save-excursion
      (set-buffer (find-file-noselect file))

      ;;  Make sure the selective display is not on.
      ;;
      (if (featurep 'folding)
          (sfuncall 'fold-open-buffer))

      (PMIN)
      (while (re-search-forward
	      "^(\\(defun\\|defmacro\\)[ \t]+\\([^ \t\n(]+\\)[ \t]*" nil t)
        (setq iact          nil
              args          nil
              )

	(setq type (read-match 1) func (read-match 2))
;;;     (d! type func)

        (when (and func
                   (progn
                     (goto-char (goto-char (match-end 0)))
;;;                  (d! (point) (buffer-substring (point) (EOLP)))

                     (when (search-forward "(" nil t)
                       (setq point (point))
                       (backward-char 1)
                       (forward-sexp 1)
                       (backward-char 1)

                       (setq
                        args
                        (subst-char
                         ;;  Convert multiline args to one line.
                         (buffer-substring point (point))
                         ?\n ?\
                         ))
                       )))

          (if (re-search-forward
               "[ \t\n]+([ \t]*interactive"
               (save-excursion (end-of-defun) (point))
               t)
              (setq iact "t"))

          (if (null args)
              (setq args (format ";; %-36s <args not known>\n" func))

            (if (> (length args) 32)
                (setq args (format ";; %-15s %s\n" func args))
              (setq args (format ";; %-36s %s\n" func args))
              ))

          (push args list)
          ;; (autoload FUNCTION FILE &optional DOCSTRING INTERACTIVE TYPE)
          ;;
          (setq str (format "(autoload '%-36s %s nil %s%s)\n"
			    func
			    (format "\"%s\"" fn)
			    (or iact "nil")
                            (if (string= "defmacro" type)
				" 'macro" "")
			    ))

;;;        (d! func iact str)

          (append-to-buffer-string tmp str)
          (setq iact "t")
          ))

      (set-buffer tmp)
      (insert "\n")                     ;list arguments for functions.
      (list-loop list elt
        (insert elt))
      )

    (unless no-show
      (pop-to-buffer tmp)
      (PMIN)
      )

    buffer
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-pkg-install-pgp-tar  (dir &optional log-buffer source)
  "Install PGP signed tar block using DIR from the end of current buffer.
The 'BEGIN PGP MESSAGE' is searched from the end of buffer backward.

The TAR block in the buffer looks like this and it is base64 pgp
signed (clearsig is off) with Author's public key.

;; -----BEGIN PGP MESSAGE-----
;; Version: 2.6.3ia
;;
;; owHsWc1vG0l2n0GwwYjA3pJLgEXKlNaSDJLilySblrWWLXrMrCQrpOydzcxA02wW
;; ...
;; ...
;; -----END PGP MESSAGE-----


This function
o   Asks to what directory the tar files are installed.
o   shows the log buffer and echoes commads used.
o   Calls pgp to unpack the signed block
o   Calls tar to unpack the files
o   temporary files are stored to TMP, TMPDIR or /tmp

Error conditions:
o   if 'pgp' executable is not found, function aborts.
o   if 'tar' executable is not found, function aborts.
o   if previously installed files exists, function aborts.

Input:
  DIR		where to unpack the files
  LOG-BUFFER	where to print log messages.
  SOURCE	instead of using current buffer, read this file"

  (interactive "DSave programs to directory: ")
  (let* (
         (pgp     (or (ti::f-get-load-path "pgp" exec-path)
                      (error "Can't find 'pgp' executable; exec-path.")))
         (tar     (or (ti::f-get-load-path "tar" exec-path)
                      (error "Can't find 'tar' executable; exec-path.")))
         (tmp     (or (ti::s-verify-ends (getenv "TMP") "/")
                      (ti::s-verify-ends (getenv "TMPDIR") "/")
                      "/tmp/"
                      ))

         ;;  This may be system dependent somedday..
         ;;
         (tar-opt-show "tvf")
         (tar-opt-x    "xvf")

         (obuffer  (current-buffer))
         (in-file  (concat tmp "t.in"))
         (out-file (concat tmp "t.out"))
         source
         cmd
         in
         buffer
         beg end
         file-list
         list
         elt
         )

    ;; We need to expand this for shell calls
    ;;
    (setq dir (expand-file-name (ti::s-verify-ends dir "/")))

    (cond
     ((and source
           (not (file-exists-p source)))
      (error "Can't find '%s'" source))
     ((not (file-directory-p tmp))
      (error "Can't use directory '%s', set env variable TMP." tmp))
     ((not (file-exists-p dir))
      (error "No such directory.")
      ))

    (setq buffer (use-tmp-buffer
                  (or log-buffer "*ti::install*")
                  'clear))


    (job-buffer buffer
      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. find base64 portion  ..
      (buffer-disable-undo)

      (if source
          (insert-file-contents source)
        (insert-buffer obuffer))

      (PMAX)
      (re-search-backward
       (concat "^;;+[ \t]*\\(" (ti::mt-pgp-msg-end-line) "\\)"))
      (setq end (match-beginning 1))

      (re-search-backward
       (concat "^;;+[ \t]*" (ti::mt-pgp-msg-begin-line)))
      (beginning-of-line)

      ;;  remove comments
      (delete-rectangle (point) end)

      ;;  Leave only the signed region, remove rest
      (delete-region (point-min) (point))

      (buffer-enable-undo)
      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . call pgp  ..
      ;;
      (file-delete-safe (list in-file out-file))
      (write-region (point-max) (point-min) in-file)
      (erase-buffer)

      ;; Okay, start showing the log to user
      ;;
      (pop-to-buffer buffer)

      (setq cmd  (format "%s -o %s %s" pgp out-file in-file))
      (insert "% " cmd "\n")
      (shell-execute cmd buffer)

      (PMIN)
      (if (null (re-search-forward "Plaintext filename:" nil t))
          (error "Can't proceed, PGP didn't verify signature."))

      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. show tar content  ..
      ;;
      (PMAX)
      (setq cmd  (format "%s %s %s" tar tar-opt-show out-file))
      (insert "% "cmd "\n") (setq beg (point))
      (shell-execute cmd buffer)

      (goto-char beg)
      (if (null (setq file-list (ti::p-tar-read-listing-forward)))
          (error "Can't find tar listing."))

      ;; .. .. .. .. .. .. .. .. .. .. .. .. ..  previously installed?  ..
      ;;
      (setq list file-list)
      (list-loop list elt
        (setq in (concat dir (car elt)))
        (when (file-exists-p in)
	  (if (y-or-n-p
	       (format
		"Previously installed file '%s' exists. Overwrite it? "
		in
		))
	      (delete-file in)
	    (error "Abort.")))
	)

      (setq cmd  (format "cd %s; tar %s %s"
                         (expand-file-name dir)
                         tar-opt-x
                         out-file))
      (insert "% "cmd "\n")
      (shell-execute cmd buffer)

      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . clean  ..
      ;;
      (when (y-or-n-p "Clean up tmp files? ")
        (push in-file file-list)
        (push out-file file-list)
        (list-loop file-list elt
          (file-delete-safe (car elt))
          ))
      (message "installation to %s complete" dir)
      )))

;;}}}
;;{{{ misc: XEmacs compatibility

;;; .............................................................. &xe ...

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-overlay-some ()
  "Return some existing overlay that is used in Emacs.
Usually the primary mouse selection. You can use this function to get an
overlay that you can move in text if you don't want to create
new overlay.

Return:
  overlay symbol"
  (cond
   ((and (xemacs-p)
	 (boundp 'primary-selection-extent))
    'primary-selection-extent)
   ((and (emacs-p)
	 (boundp 'mouse-drag-overlay))
    'mouse-drag-overlay
    )))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-overlay-properties  (overlay)
  "Return properties of OVERLAY."
  (cond
   ((overlay-supported-p)
    (sfuncall 'overlay-properties overlay))
   ((xemacs-p)
    (sfuncall 'extent-properties overlay)
    )))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-overlays-at (point)
  "Return overlays at POINT."
  (cond
   ((overlay-supported-p)
    (sfuncall 'overlays-at point))
   ((xemacs-p)
    (let* (list
	   )
      (sfuncall
       'map-extents
       (function (lambda (ov maparg) (push ov list)))
       (current-buffer) point point)
      list
      ))))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-overlay-put (ov-sym prop val)
  "Set properties to overlay OV-SYM. Put PROP VAL pair to OV-SYM."
  (cond
   ((overlay-supported-p)
    (sfuncall 'overlay-put (symbol-value ov-sym) prop val))
    ((xemacs-p)
     (sfuncall 'set-extent-property (symbol-value ov-sym) prop val))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-overlay-move (ov-sym beg end &optional make-local face)
  "Move overlay OV-SYM to BEG END. Overlay is created if it does not exist.
MAKE-LOCAL localizes the overlay. If the overlay is created,
then FACE is assigned to it (default 'highlight)"
  (let* (
         )
    (cond
     ((overlay-supported-p)
      ;;  later XEmacs may have overlay emulation
      ;;
      (or (symbol-value ov-sym) ;Exist?
	  (progn
	    (if make-local (make-local-variable ov-sym))
	    (set ov-sym
		 (sfuncall 'make-overlay (point) (point)))
	    (sfuncall 'overlay-put
		      (symbol-value ov-sym)
		      'face (or face 'highlight))
	    ))

      (sfuncall 'move-overlay (symbol-value ov-sym)
		beg end (current-buffer))
      )

     ((xemacs-p)
      (or (symbol-value ov-sym) ;Exist?
	  (progn
	    (if make-local (make-local-variable ov-sym))
	    (set ov-sym
		 (sfuncall 'make-extent (point) (point)))
	    (sfuncall 'set-extent-property
		      (symbol-value ov-sym)
		      'face (or face 'highlight))
	    ))
      (sfuncall 'set-extent-endpoints
		(symbol-value ov-sym)
		beg end (current-buffer))
      ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-activate-region  (&optional off)
  "Activate region or turn the region OFF."
  (if (emacs-p)
      (sfuncall 'transient-mark-mode (if off 0 1)) ;From Simple.el
    (if off
       (sfuncall 'zmacs-deactivate-region)
      (set 'zmacs-regions (if off nil t)) ;Avoid bute compile mesage in Emacs
      (activate-region)
      )))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-read-password  (&optional prompt)
  "Read password with PROMPT which defaults to 'Password: '."
  (setq prompt (or prompt "Password: "))
  (cond
   ((xemacs-p)
    (require 'passwd)			;utils/passwd.el
    (sfuncall 'read-passwd prompt))
   (t
    (ti::q-read-input-as-password prompt)
    )))

;;; ----------------------------------------------------------------------
;;; This code is originally from folding.el. Function by Anders Lindgren
;;;
(defvar ti:-xe-mouse-call-original nil "See ti::k-mouse-call-original.")

(defun ti::xe-mouse-call-original (minor-mode-symbol &optional event)
  "Execute original mouse function by turning of MINOR-MODE-SYMBOL.
EVENT is mouse event. You use this function to to handle 'hot spots' in the
buffer and in other places you call the original function.

Do nothing if original function does not exist.
Does nothing when called by a function which has earlier been called
by us.

Example for some minor mode implementation:

  (defun folding-mode-context-sensitive  (event)
    (interactive \"e\")
    ;; If test.. if test..no, then call original function!
    (ti::xe-mouse-call-original 'folding-mode event)
    )

Note:
  Works in XEmacs and Emacs

Sets global:
  `ti:-xe-mouse-call-original'"
  (interactive "@e")

  ;; Without the following test we could easily end up in a endelss
  ;; loop in case we would call a function which would call us.
  ;;
  (if ti:-xe-mouse-call-original
      nil
    (setq ti:-xe-mouse-call-original t)
    (unwind-protect
        (let (mouse-key
              orig-buf
              orig-func
              )
          (or event
              (setq event last-input-event))

          (cond
           ((emacs-p)
            (setq mouse-key (make-vector 1 (car event))))
           ((xemacs-p)
            (setq mouse-key
                  (vector
                   (append (event-modifiers event)
                           (list (intern
				  (concat
				   "button"
				   (sfuncall 'event-button event)
				   ))))))
            ))

          ;;  Turn off minor mode, so that we can see the real
          ;;  fuction behind it.
          (put minor-mode-symbol 'ti::orig-value
               (symbol-value minor-mode-symbol))
          (set minor-mode-symbol nil)

          (setq orig-buf  (current-buffer)
                orig-func (key-binding mouse-key))

          (when orig-func
            ;;  Only if existed
            ;;  call it with the event as argument.
            ;;  We have to restore the current buffer too, because
            ;;  the minor mode is there.
            ;;
            (unwind-protect
                (call-interactively orig-func)
              (set-buffer orig-buf)
              (set minor-mode-symbol (get minor-mode-symbol 'ti::orig-value))
              ))
          )
      ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..  unwind  ..
      ;; This is always executed, even if the above generates an error.
      ;;
      (setq ti:-xe-mouse-call-original nil)
      )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-popup (string-list &optional event mode title)
  "Show STRING-LIST popup. If EVENT is nil, use default tinylib coordinates.
Works in XEmacs and Emacs.

Input:
  STRING-LIST   '(str str ..)
  EVENT         mouse-event or nil
  MODE          if non-nil, return selection NBR [0..n]. Normally
                returns the selection itself.
  TITLE         title of popup

Return:
  selection     ,member or nbr
  nil           ,nothing selected"
  (interactive "e")
  (let* (
         (title  (or title ""))
         (count  0)
         ;;  Allow calling from keypress also.
         (event  (or event
                     (ti::xe-make-x-popup-event ti:-x-coord  ti:-y-coord)))
         menu
         item-list
         alist
         arg
         ret
         )

    (when (listpp string-list)
      (setq alist  (list-to-assoc-menu string-list))
      (cond
       ((emacs-p)
        (setq item-list  alist)
        (setq menu
              (cons title
                    (list (cons title item-list))))
        (setq ret (sfuncall 'x-popup-menu  event menu))
        (if ret
            (if (null mode)
                (setq ret (nth ret string-list))
              )))
       (t
	;; Scenario: User selects item from menu-bar-menu which calls
	;; function that should be called from mouse press --> selecting
	;; from pull-down-menu, is not a mouse event!
	;;
	;; First one is real mouse call for function; the other one
	;; is called from popup selection
	;;
	;;	#<buttondown-event button1>
	;;	#<misc-user-event (call-interactively tig-index-x-popup)>
	;;
	;; get-popup-menu-response call breaks if EVENT is something
	;; else than mouse-event. Check it immediately and set EVENT
	;; to nil, because the parameter is optional.

	(if (and event (null (sfuncall 'mouse-event-p event)))
	    (setq event nil))

        ;;  Menu format is like this in XEmacs
        ;;
        ;; '("title" ["A" ("A") t] ["B" ("B") t] ["C" ("C") t]

        (setq item-list string-list)
        (setq menu
              (mapcar
               (function
                (lambda (x &optional vec)
                  (setq vec (make-vector 3 nil))
                  (aset vec 0 x)
                  (aset vec 1 (list x))
                  (aset vec 2 t)
                  vec
                  ))
               item-list))

        (setq menu (push title menu))

        ;; #todo, I don't know why there is nothing in the RET
        ;; after the selection has been done...

        ;;  See menubar.el
        ;;
        (setq ret (sfuncall 'get-popup-menu-response  menu event ))

        (if (sfuncall 'misc-user-event-p ret)
            (setq ret (car-safe (sfuncall 'event-object  ret))))

        (when (and ret mode)            ;find position in list
          (list-loop menu arg
            (if (and (vectorp arg)
                     (string= ret (elt arg 0)))
                (setq menu nil   ret  (1- count)))
            (inc count)
            ))

        )))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-display-depth  ()
  "Return how many colors display can show."
  (cond
   ((emacs-p)
    (sfuncall 'x-display-planes (symbol-value 'x-display-name)))
   (t
    (sfuncall 'device-bitplanes (sfuncall 'default-x-device))
    )))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-read-event ()
  "Read X event."
  (cond
   ((emacs-p)
    (if (fboundp 'event-to-character)
	(sfuncall 'read-event)
      (error "Cannot read events.")))
   (t
    (sfuncall 'next-command-event)
    )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-executing-macro ()
  "Check if executing macro."
  (cond
   ((boundp 'executing-macro)
    (symbol-value 'executing-macro))    ;Emacs and old XEmacs
   ((boundp 'executing-kbd-macro)       ;New XEmacs
    (symbol-value 'executing-kbd-macro)
    )))


;; briefly: events in 19.28, see subr.el
;; -------------------------------------------
;; event       :(mouse-3 (#<window 34 on *scratch*> 128 (20 . 104) -23723628))
;;
;; (setq event-start event)
;; event-start :(#<window 34 on *scratch*> 128 (20 . 104) -23723628))
;;                                         |   |          time
;;                               mouse point   coordinates
;;
;; (setq posn-col-row event-start) --> turn (20 . 104) into (col row)
;;
(defun ti::xe-make-x-popup-event (x y)
  "Make fake EVENT using X and Y coordinates.
Very handy if you call from kbd a function that requires mouse event."
  (cond
   ((emacs-p)
    (list (list x  y) (selected-window)))
   (t
    ;;; (message "ti::xe-make-x-popup-event, XEmacs implementation not known.")
    nil
    )))



;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-make-fake-event (x y &optional mouse-sym)
  "Make fake event using X and Y coordinates and MOUSE-SYM[mouse - 1].

Remeber: this is not full blown fake, just sufficent one, if
receiver uses any of 'posn-' function, this doesn't fool it."

  ;; (mouse-1 (#<window 42 on tinylib.el> 271088 (92 . 138) -492011))
  (cond
   ((emacs-p)
    (list
     (or mouse-sym 'mouse-1 )
     (list
      (selected-window)
      1                                 ;<just some calue>
      (cons x y )
      -23723628
      )))
   (t
    ;; (message "ti::xe-make-fake-event, XEmacs implementation not known.")
    ;; <kyle_jones@wonderworks.com> (Kyle Jones)
    ;;
    ;; You can't create fake events in XEmacs.  The object data is
    ;; hidden behind an abstraction layer and there are no functions to
    ;; build or modify event objects.  You can only allocate and copy
    ;; them.
    ;;
    nil
    )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-modeline-update ()
  "XEmacs and Emacs Compatibility. Update modeline."
  (cond
   ((and (xemacs-p)
         (fboundp 'redraw-modeline))
    ;; Xe 19.14
    ;; force-mode-line-update is an obsolete function; use redraw-modeline
    (sfuncall 'redraw-modeline))

   ((fboundp 'force-mode-line-update)
    (sfuncall 'force-mode-line-update))

   (t
    (set-buffer-modified-p (buffer-modified-p))
    )))


;;; .......................................................... &frames ...


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-set-frame-name (string &optional frame get)
  "Change the frame display STRING in FRAME.
The implementation works differently in various emacs versions.

If GET is non-nil return frame name."
  (let* ((symbol 'name)
         )
    (when (emacs-p)
      ;; somewhere along the line the symbol was renamed to 'title
      ;; ..at least my records say so, but I don't know for sure,
      ;; because I don't have 31-33 versions.
      ;;
      ;; #todo: 19.31 - 33, frame, Would someone confirm this?
      ;;
      (when (and (> emacs-minor-version 31)
		 (< emacs-minor-version 34))
	(setq symbol 'title)))

    (if get
        (ti::xe-get-frame-parameter symbol frame)
      (ti::xe-set-frame-parameter symbol string frame))
    ))


;;; ----------------------------------------------------------------------
;;; Idea by, 23 Mar 1995,  mueller@cs.unc.edu (Carl Mueller), comp.emacs
;;; - Changing the frame label is same as changing the icon label
;;;
;;;
;;;
(defun ti::xe-set-frame-parameter (prop-or-list value &optional frame)
  "Use PROP-OR-LIST and VALUE to set FRAME's parameters.
When called interactively, set name of the frame.

Input:
 PROP-OR-LIST       ,alist of parameters or single property name
                     ((param . val) ..)
 VALUE              ,only used if single property given.
 FRAME              ,defaults to current frame."
  (interactive
   (list
    'name
    (read-from-minibuffer "frame label name: ")
    ))

  (let* ((frame (or frame (selected-frame)))
         )
    (cond
     ((and (xemacs-p)
           (fboundp 'set-frame-properties))

      ;; #todo:  Why don't these work in XEmacs 19.14 ?
      (if (listpp prop-or-list)
          (sfuncall 'set-frame-properties frame prop-or-list)
        (sfuncall 'set-frame-property frame prop-or-list value))
      )

     (t
      (if (not (listpp prop-or-list))
          (setq prop-or-list (list (cons prop-or-list value))))
      (sfuncall 'modify-frame-parameters frame prop-or-list)
      ))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-get-frame-parameter (parameter &optional frame)
  "Return _one_ PARAMETER value from FRAME."
  (let* ((frame  (or frame
                     (selected-frame)))
         (func   (if (emacs-p)
                     'frame-parameters
                   'frame-properties))
         list
         )
    (setq list (funcall func frame))

    (if (null parameter)
        list
      (cond
       ((emacs-p)                       ;((arg1 . val) (arg2 . val) ..)
        (cdr (assq parameter list)))
       (t
        (car-safe (cdr-safe (memq parameter list))) ;; (arg val arg2 val)
        )))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-frame-window-config ()
  "Return list '((FRAME WINDOW-CONFIGURATION) (F W) ..)."
  (let ((list (cdr (current-frame-configuration)))
        elt
        ret
        )
    (list-loop list elt
      (push (list (nth 0 elt) (nth 2 elt))  ret))

    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;; XEmacs 19.14 "window-system is an obsolete variable; use (console-type)"
;;;
(defun ti::xe-window-system  ()
  "XEmacs and Emacs Compatibility, Return window system."
  (cond
   ((and (xemacs-p)
         (when (boundp 'console-type)))
    (symbol-value 'console-type))
   ((when (boundp 'window-system)
      (symbol-value 'window-system))
    )))

;;; ....................................................... &xe-timers ...

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-timer-list-control (&optional mode)
  "Timer handling: MODE can be 'save 'restore or 'kill.

Example:
  ;; Turn off all processes for a while...
  (ti::xe-timer-list-control 'save)
  (ti::xe-timer-list-control 'kill)

  ;; ... do somtehing
  ;;

  ;;  Now restore the prosesses
  (ti::xe-timer-list-control 'restore)"
  (let* ((sym
          (cond
           ((boundp 'timer-alist)  'timer-alist)
           ((boundp 'timer-list)   'timer-alist)
           ((boundp 'itimer-list)  'itimer-list)
           ))
         )
    ;;  We store/restore the list into the timer variable symbol
    ;;  properties.

    (cond
     ((eq 'kill mode)
      (set sym nil))

     ((eq 'save mode)
      (put sym 'ti::saved (symbol-value sym)))

     ((eq 'restore mode)
      (set sym (get sym 'ti::saved)))
     )))



;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-timer-elt  (function)
  "Search FUNCTION and return the timer ELT.
You can use this function to check if some function is currently
in timer list. (ie. active)"
  (let* (pos
         list
         vec
         )
     (cond
      ((boundp 'timer-alist)
       ;; (("Mon Dec  9 10:01:47 1996-0" 10 tipgp-process nil))
       ;;
       (setq list (symbol-value 'timer-alist) ;quiet ByteComp
             pos  2))

      ((boundp 'timer-list)             ;19.34
       ;; ([nil 12971 57604 0 60 display-time-event-handler nil nil])
       ;;
       (setq list (symbol-value 'timer-list)
             pos  5
             vec  t
             ))

      ((boundp 'itimer-list)
       (setq list (symbol-value 'itimer-list)
             pos  3)
       ))

     (if pos (setq pos pos))            ;Quiet XEmacs 19.14 compiler

     (if list
         (list-find
          list
          function
          (if vec
              (quote (eq (aref element pos) arg))
            (quote (eq (nth pos element)   arg)))
          ))
     ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-timer-process-status ()
  "XEmacs and Emacs Compatibility, Return timer process status:
t if active."
  (cond
   ((boundp 'timer-alist)               ;Emacs
    (symbol-value 'timer-process))

   ((boundp 'timer-list)                ;Emacs 19.34
    (ti::xe-timer-elt  'display-time-event-handler))

   ((boundp 'itimer-list)               ;
    t                                   ;it is built in in XEmacs
    )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-timer-key-delete  (key)
  "Delete timer KEY entry, where KEY is full element in (i)`timer-alist'."
  (let (var
        )
    (cond
     ((boundp 'timer-alist)             ;Emacs
      (setq var 'timer-alist)
      (set var (delete key (symbol-value 'timer-alist))))

     ((boundp 'timer-list)              ;Emacs 19.34
      (setq var 'timer-list)
      (set var (delete key (symbol-value 'timer-list))))

     ((boundp 'itimer-list)             ;XEmacs

      (setq var 'itimer-list)
      (set var (delete key (symbol-value 'itimer-list)))
      ))
    ))



;;; ----------------------------------------------------------------------
;;; XEmacs also has command region-active-p
;;;
(defun ti::xe-mark-active-p  ()
  "XEmacs and Emacs Compatibility, Return t if mark (region) is active."
  (cond
  ((and (not (emacs-p))
        (boundp 'zmacs-regions))        ;XEmacs
   (let* ((zmacs-regions t))
     (mark)))
  (t
   (symbol-value 'mark-active)          ;Emacs
   )))



;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-mail-abbrev-table  ()
  "XEmacs and Emacs Compatibility, Return mail abbrev hash table."
  (cond
   ((emacs-p)
    (if (null mail-abbrevs)
        (sfuncall 'mail-abbrevs-setup))
    (symbol-value 'mail-abbrevs)
    )
   (t
    (require 'mail-abbrevs)
    ;;  in Emacs this is a list, in XEmacs this is a HASH
    (symbol-value 'mail-aliases)
    )))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-expand-mail-aliases  (beg end &optional alias-alist)
  "Expand aliases in region BEG END.
Please Cache results from `ti::m-mail-abbrevs-get-alist' and
use the result as argument ALIAS-ALIST. Otherwise aliases are always
reuild from scratch."
  (interactive "*r")
  (let* (mb me
         word exp
         )
    (cond
     ((and nil (feature-load 'mailalias)        ;Emacs Feature only
           (fboundp 'expand-mail-aliases))
      (expand-mail-aliases beg end)
      )

     (t                                 ;Too bad, this is much slower
      (unless alias-alist
        (setq alias-alist (ti::m-mail-abbrevs-get-alist)))

      (save-restriction
        (narrow-to-region beg end) (PMIN)
        (while (re-search-forward
                "^[ \t]+\\|^[ \t]*[\n,][ \t]*\\|:[ \t]*" nil t)
          (when (setq word (bufmatget"[^ \t,\n]+" 0))

            (setq mb (match-beginning 0)
                  me (match-end 0))

            ;;  Do not count field names, like  "CC:" words
            (when (and (not (string-match ":$" word))
                       ;;  Is this abbrev ?
                       (setq exp (assoc word alias-alist)))
              (setq exp (cdr exp))        ; Change alias to expansion
              (delete-region mb me)
              (insert exp)


              ;;  This isn't very smart formatting, the layout
              ;;  is so that each expansion is on it's own line,
              ;;  no fancy lining up things -- Mail me back
              ;;  with diff to this code if you code nicer one.
              ;;
              (when (looking-at "[ \t]*,") ;put on separate lines
                (goto-char (match-end 0))
                (when (not (looking-at "[ \t]*$"))
                  (insert "\n\t")
                  (beginning-of-line))  ;search continue point
                )
              ))))
      ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::xe-set-mode-line-format  (fmt)
  "Set modeline format using FMT."
  (let* ((sym
	  (if (emacs-p)
	      'mode-line-format
	    'modeline-format))
	 )
    ;; XEmacs 19.14 says:
    ;; ** mode-line-format is an obsolete var; use modeline-format instead.
    (set sym fmt)
    ))

;;}}}
;;{{{ misc: create standard functions, variables

;;; .......................................................... &fmacro ...

;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-vmacro-minor-mode
  (pfx
   mode-Name
   mode-Name-prefix-key
   easymenu-Name
   custom-group

   &optional style
   )
  "Return standard minor mode variables.
See below how to call this function  from the top of your minor mode pacakge.

Input:

  PFX			string, the package prefix, usually one or two
			words. Eg. \"xxx\" or \"xxx-mode\"
  MODE-NAME		string; which is displayed in modeline, should have
		        leading space. Eg \" Lisp\"
  MODE-NAME-PREFIX-KEY	string, Key sequences to access the minor mode
			functions.
  EASYMENU-NAME		string, the Menu bar name string.
  CUSTOM-GROUP		symbol, the defcustom.el group name.
  PREFIX-STYLE		string, How the characters should be named.
			if nil then uses standard emacs naming.


Example, when:

  PFX		    is \"xxx-\"
  STYLE		    is nil		;; Standard emacs style

  (defvar xxx-mode			nil)
  (make-variable-buffer-local		'xxx-mode)

  (defvar xxx-mode-name			MODE-NAME)
  (defvar xxx-mode-prefix-key		MODE-NAME-PREFIX-KEY)
  (defvar xxx-mode-map			nil)
  (defvar xxx-mode-define-keys-hook	nil)
  (defvar xxx-mode-hook			nil)
  (defvar xxx-mode-easymenu		nil)
  (defvar xxx-mode-easymenu-name	nil)

Example, when:

  PFX		    is \"xxx\"
  STYLE		    is 'xxx-:

  (defvar xxx-mode			nil)
  (make-variable-buffer-local		'xxx-mode)

  (defvar xxx-:mode-name		MODE-NAME)
  (defvar xxx-:mode-prefix-key		MODE-NAME-PREFIX-KEY)
  (defvar xxx-:mode-map			nil)
  (defvar xxx-:mode-define-keys-hook	nil)
  (defvar xxx-:mode-hook		nil)
  (defvar xxx-:mode-easymenu		nil)
  (defvar xxx-:mode-easymenu-name	nil)

How to call this function:

  (ti::m-vmacro-minor-mode \"xxx\" \" Xmode\" \"C-cx\" \"Xmenubar\" nil)"
  (` (, (ti::m-vmacro-minor-mode-1
	 pfx
	 mode-Name
	 mode-Name-prefix-key
	 easymenu-Name
	 custom-group

	 style
	 ))))

(defun ti::m-vmacro-minor-mode-1
  (pfx
   mode-Name
   mode-Name-prefix-key
   easymenu-Name
   custom-group

   &optional prefix-style
   )
  "Use `ti::m-vmacro-minor-mode' and see call arguments there.
PFX MODE-NAME MODE-NAME-PREFIX-KEY
EASYMENU-NAME CUSTOM-GROUP PREFIX-STYLE"
  (let* ((x "-")
	 sym
	 ret
	 )
    (if prefix-style
	(if (not (stringp prefix-style))
	    (error "style must be string")
	  (setq x prefix-style))
      (setq x pfx)
      )


    (push 'progn ret)

    (setq sym (intern (format "%smode" pfx)))

    (push (list 'defvar (` (, sym)) nil
		"mode on off variable."
		)
	  ret)
    (push (list 'make-variable-buffer-local (` (quote (, sym)))) ret)

    (setq sym (intern (format "%smode-name" x)))
    (push (list 'defcustom (` (, sym))
		(` (, mode-Name))
		"*Minor mode name."
		':type ''string
		':group (` (, custom-group))
		)
	  ret)

    (setq sym (intern (format "%smode-prefix-key" x)))
    (push (list 'defcustom (` (, sym))
		(` (, mode-Name-prefix-key))
		"*Prefix key to access mode."
		':type ''(string :tag "Key sequence")
		':group (` (, custom-group))
		)
	  ret)

    (setq sym (intern (format "%smode-map" x)))
    (push (list (quote defvar) (` (, sym))
		nil
		"Minor mode map.")
	  ret)

    (setq sym (intern (format "%smode-easymenu" x)))
    (push (list (quote defvar) (` (, sym))
		nil
		"Easymenu variable.")
	  ret)

    (setq sym (intern (format "%smode-easymenu-name" x)))
    (push (list 'defcustom  (` (, sym))
		(` (, easymenu-Name))
		"*Easymenu name that appears in menu-bar."
		':type ''string
		':group (` (, custom-group))
		)
	  ret)

    (setq sym (intern (format "%smode-define-keys-hook" x)))
    (push (list (quote defvar) (` (, sym))
		nil
		"*Hook that defines all keys and menus.")
	  ret)

    (setq sym (intern (format "%smode-hook" x)))
    (push (list 'defcustom (` (, sym))
		nil
		"*Hook that runs when mode function is called."
		':type ''hook
		':group (` (, custom-group))
		)
	  ret)


    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-minor-mode
  (
   func-min-sym
   doc-str

   install-func				;3
   mode-var
   mode-Name				;5
   prefix-var
   menu-var				;7

   no-mode-msg
   mode-desc				;9

   hook
   &optional body
   )
  "Create standard functions for minor mode.

Input:

  FUNC-MIN-SYM	symbol, the name of the function that is created.
                Eg. 'xxx-mode

  DOC-STR       string, the function documentation string

  INSTALL-FUNC  symbol, if func-min-sym isn't in `minor-mode-alist', this
                function is called to install the minor mode.

  MODE-VAR      symbol, a variable which turns minor mode on or off
  MODE-NAME     symbol, a variable, contains mode name.
  [PREFIX-VAR]  symbol, a variable, mode's prefix key. Can be nil
  [MENU-VAR]    symbol, a variable, mode's menu definitions. The menu must be
                in format of easy-menu.el so that it is Emacs and
	        XEmacs compatible

  [NO-MODE-MSG] if non-nil, then default mode turn on or off message
		is not displayed. The default message is
		'MODE-DESC mode minor mode is ON. Prefix key is XXX'
  MODE-DESC     string, used in the default turn on message, see above.

  [HOOK]        symbol, hook that is run when mode is called.

  [BODY]        lisp code to be added inside middle body. Can be nil.

Created function's arguments:

  (&optional arg verb)
  ARG           is mode on off variable. nil toggles mode.
  VERB          is set in interactive call and controlls printing mode
                turn on or off message. If nil, then no messages are
                displayed.

Example how to use this macro:

  ;;; We have to inform autoload that function exist after macro
  ;;;###autoload (autoload 'xxx-mode          \"package-file\" t t)

  (ti::m-fmacro-minor-mode
   xxx-mode
   \"XXX minor mode. This helps you to do ....
  Defined keys:
  \\\\ {xxx-mode-map}
  \"
    xxx-install-mode
    xxx-mode
    xxx-:mode-name
    xxx-:mode-prefix-key
    nil                     ;; no menu variables
    nil
    \"XXX\"
    xxx-:mode-hook
    ;; The forms
    ;;
    (progn
      (message \"Hey!\")
     ))

Example how to call created functions:

  (xxx-mode)		;; toggles
  (xxx-mode 1)		;; on
  (xxx-mode 0)		;; off, could also be -1
  (turn-on-xxx-mode)	;; function can be put to hook
  (turn-off-xxx-mode)"
  (` (,
      (ti::m-fmacro-minor-mode-1
       func-min-sym
       doc-str

       install-func
       mode-var
       mode-Name
       prefix-var
       menu-var

       no-mode-msg
       mode-desc

       hook
       body
       ))))

(defun ti::m-fmacro-minor-mode-1
  (func-min-sym
    doc-str				;1

    install-func			;2
    mode-var				;3
    mode-Name				;4
    prefix-var				;5
    menu-var				;6

    no-mode-msg				;7
    mode-desc				;8

    hook				;9
    &rest body				;10
    )
  "Use macro `ti::m-fmacro-minor-mode'. And see arguments there.
FUNC-MIN-SYM DOC-STR INSTALL-FUNC MODE-VAR
MODE-NAME PREFIX-VAR MENU-VAR NO-MODE-MSG MODE-DESC
HOOK BODY"

;;;  (d!! "\n\n" body)
    (let* ((sym (intern (symbol-name (` (, func-min-sym)))))
	   )
      (`
	(defun (, sym)
	  (&optional arg verb)
	  (, doc-str)

	  (interactive "P")
	  (VERB)
	  (if (null (assq (quote (, func-min-sym)) minor-mode-alist))
	      ((, install-func)))

;;;	  (let* ((val (symbol-value  (, mode-var)))
;;;		 )
;;;	    (setq  (, mode-var) (var-toggle val arg)))

	    (var-toggle (, mode-var) arg)


	  ;;  XEmacs needs this call, in emacs turning on the minor
	  ;;  mode automatically adds the menu too.
	  ;;
;;;	  (if (symbol-value (, mode-var))
;;;	      (easy-menu-add (symbol-value (, menu-var)))
;;;	    (easy-menu-remove (symbol-value (, menu-var))))

	  (if (and (, mode-var)
		   (, menu-var))
	      ;;  easy-menu-add dies if menu-var is nil
	      (easy-menu-add (, menu-var))
	    (easy-menu-remove (, menu-var)))



	  (,@ body)

	  (ti::xe-modeline-update)
	  (if (and verb (null (, no-mode-msg)))
	      (message
	       "%s minor mode is %s %s"
	       (, mode-desc)
	       (if  (, mode-var) "on." "off.")
	       (if  (null (, mode-var))
		   ""
		 (if (, prefix-var)
		     (format "Prefix key is %s" (, prefix-var))
		   "")
		 )))

	  (run-hooks (quote (, hook)))
	  ))

      ))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-fmacro-minor-mode-on (mode-func-sym)
  "Create standard function to turn on the minor mode MODE-FUNC-SYM."
  (let* ((sym
	  (intern (concat "turn-on-" (symbol-name (` (, mode-func-sym))))))
	 )
    (`
     (defun (, sym) ()
       "Turn minor mode on"
       (interactive)
       ((, mode-func-sym) 1)
       ))

    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::m-fmacro-minor-mode-off (mode-func-sym)
  "Create standard function to turn off the minor mode MODE-FUNC-SYM."
  (let* ((sym
	  (intern (concat "turn-off-" (symbol-name (` (, mode-func-sym))))))
	 )
    (`
     (defun (, sym) ()
       "Turn minor mode on"
       (interactive)
       ((, mode-func-sym) 1)
       ))

    ))


;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-minor-mode-install
  (func-ins-sym
   mode-sym
   map-sym
   mode-name-sym
   hook-sym
   &rest body
   )
  "Return standard function form.
Returned funcktion will install and remove minor mode.

Input:

  FUNC-INS-SYM	symbol, the name of the function that is created.
                Eg. 'xxx-install-mode

  MODE-SYM      function symbol to call to run the mode eg. 'xxx-mode

  MAP-SYM       mode's keymap symbol. Eg. 'xxx-mode-map

  MODE-NAME-SYM mode's name symbol. Eg. 'xxx-mode-name

  HOOK-SYM      hook symbol to call when mode has been installed.
                eg. 'xxx-key-define-hook, which calls necessary
	        functions to install keys and menus.

  BODY          lisp forms executed in the beginning of function.

Created function's arguments:

  (&optional remove verb)
  REMOVE        uninstalls minor mode
  VERB          is set for interactive calls: non-nil allows
	        displaying messages.

How to call this function:

   (ti::m-fmacro-minor-mode-install
    xxx-install-mode
    xxx-mode
    xxx-mode-map
    xxx-:mode-name
    xxx-:mode-define-keys-hook
    (progn
     ;; lisp forms here
     ))

Example how to call created function:

  M -x xxx-install-mode      ;; this calls created function and installs mode
  (xxx-install-mode)         ;; Same
  (xxx-install-mode 'remove) ;; Or prefix ARG, removes the minor mode"
  (` (, (ti::m-fmacro-minor-mode-install-1
	 func-ins-sym
	 mode-sym
	 map-sym
	 mode-name-sym
	 hook-sym
	 body
	 ))))


(defun ti::m-fmacro-minor-mode-install-1
  (func-ins-sym
   mode-sym
   map-sym
   mode-name-sym
   hook-sym
   &rest body
   )
  "Use macro `ti::m-fmacro-minor-mode-install'. See arguments there.
FUNC-INS-SYM MODE-SYM MAP-SYM MODE-NAME-SYM HOOK-SYM BODY"
  (let* ((sym (intern (symbol-name (` (, func-ins-sym)))))
	 )
    (`
     (defun (, sym) (&optional remove verb)
       "Install or optionally REMOVE minor mode. Calling this always
removes old mode and does reintall."
       (interactive "P")
       (VERB)
       (,@ body)
       (cond
	(remove
	 (ti::k-add-minor-mode '(, mode-sym) nil nil 'remove)
	 (if verb
	     (message "minor mode removed"))
	 )
	(t
	 (setq (, map-sym)  (make-sparse-keymap));; always refresh
	 (run-hooks '(, hook-sym))

	 ;;  Always do reinstall; because keymaps stored permanently and
	 ;;  making changed later is impossible.
	 ;;
	 (ti::k-add-minor-mode '(, mode-sym) nil nil 'remove)
	 (ti::k-add-minor-mode '(, mode-sym)
			       '(, mode-name-sym)
			       (, map-sym)
			       )
	 (if verb
	     (message "minor mode installed"))

	 ));; cond
       ))
    ))



;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-define-keys
  (func-def-sym
   keymap-sym
   prefix-key-sym

   easymenu-sym
   easymenu-Name-sym
   easymenu-doc-str
   easy-menu-forms
   eval-body
   )
  "Return standard function form.
The returned function will install keymaps and menu-bar menu for minor mode.

Inside the function you can refer to variables

 'map'			refers to keymap
 'p'			holds the prefix key.

Input:

 FUNC-DEF-SYM		symbol, function name which is created
 KEYMAP-SYM		symbol, keymap where to define keys, must exist
 PREFIX-KEY-SYM		symbol, variable holding the prefix key.
 [EASYMENU-SYM]		symbol, easy menu variable or nil.
 [EASYMENU-NAME-SYM]	symbol, easy menu's menu-bar name variable or nil
 [EASYMENU-DOC-STR]	string, Describe string for menu.
 [EASY-MENU-FORMS]      forms to define menus
 EVAL-BODY		forms executed at the end of function.

Created function's arguments:

  ()

How to call this function:

   (ti::m-fmacro-define-keys
     xxx-mode-define-keys
     xxx-:mode-map
     xxx-:mode-prefix-key

     xxx-:mode-easymenu
     xxx-:mode-easymenu-name
     (list
       xxx-:mode-easymenu-name
       [\"menu item1\"  xxx-function1 t]
       [\"menu item2\"  xxx-function2 t]
       \"----\"
       [\"menu item3\"  xxx-function3 t]
       )
     (progn
	(define-key map (concat p \"a\")   'xxx-function1)
	(define-key map (concat p \"b\")   'xxx-function2)
	(define-key map (concat p \"c\")   'xxx-function3)
	)
     )

Example how to call created function:

  (xxx-mode-define-keys)"
  (` (, (ti::m-fmacro-define-keys-1
	 func-def-sym
	 keymap-sym
	 prefix-key-sym

	 easymenu-sym
	 easymenu-Name-sym
	 easymenu-doc-str
	 easy-menu-forms
	 eval-body
	 ))))

(defun ti::m-fmacro-define-keys-1
  (func-def-sym
   keymap-sym
   prefix-key-sym

   easymenu-sym
   easymenu-Name-sym
   easymenu-doc-str
   easy-menu-forms
   body
   )
  "Use macro `ti::m-fmacro-define-keys' and see arguments there.
FUNC-DEF-SYM KEYMAP-SYM PREFIX-KEY-SYM
EASYMENU-SYM EASYMENU-NAME-SYM EASYMENU-DOC-STR EASY-MENU-FORMS
BODY"
  (let* (sym
	 )

    (setq sym (intern (symbol-name (` (, func-def-sym)))))


    (`
     (defun (, sym) ()
       (let* ((map (, keymap-sym))
	      (p   (, prefix-key-sym))
	      )

	 (when (, easymenu-doc-str)	;This could be nil (no menus)
	   (easy-menu-define
	    (, easymenu-sym)
	    (if (xemacs-p) nil (list map))
	    (, easymenu-doc-str)
	    (, easy-menu-forms)
	    ))

	 ;;  This is no-op, ByteComp silencer.
	 ;;  ** variable p bound but not referenced
	 (if (null p)   (setq p nil))
	 (if (null map) (setq map nil))

	 (, body)
	 )))
    ))


;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-version-bug-report
  (filename
   prefix
   version-variable version-value

   bug-var-list
   &optional bug-body
   )
  "Return standard function form.
One variable and two functions are created.

Input:

  FILENAME	    string eg. xxx.el
  PREFIX	    pacakge prefix for functions eg. xxx
  VERSION-VARIABLE  symbol varible holding the version information.
  VERSION-VALUE	    value for the variable. Should be RCS Id string or the
		    like.
  BUG-VAR-LIST	    variable list to send with bug report
  BUG-BODY	    lisp forms for the bug function.


How to call this macro:

    (ti::m-fmacro-version-bug-report
     \"xxx.el\"
     \"xxx\"
     xxx-:version-id
     \"...version Id string here, RCS controlled.\"

     '(xxx-:load-hook
       xxx-:mode-hook
       xxx-mode-define-keys-hook
       xxx-:mode-name
       ))

Example how to call created functions:

  M - x xxx-submit-bug-report
  M - x xxx-version"
  (`(, (ti::m-fmacro-version-bug-report-1
	filename prefix version-variable version-value bug-var-list bug-body
	))))

(defun ti::m-fmacro-version-bug-report-1
  (filename
   prefix version-variable version-value

   bug-var-list
   &optional bug-body
   )
  "Use macro `ti::m-fmacro-version-bug-report' and see arguments there.
FILENAME PREFIX VERSION-VARIABLE VERSION-VALUE BUG-VAR-LIST BUG-BODY."
  (let* (sym
	 ret
	 elt
	 )
    (push 'progn ret)

    (setq elt
	  (list
	   'defvar (` (, version-variable))
	   (` (, version-value))
	   "Package's version information."
	   ))
    (push elt ret)

    (setq sym (intern (format "%s-version" prefix)))

    (setq
     elt
     (`
      (defun (, sym) (&optional arg)
	"Version information."
	(interactive "P")
	(ti::m-pkg-version-info (, filename) arg))))
    (push elt ret)


    (setq sym (intern (format "%s-submit-bug-report" prefix)))

    (setq
     elt
     (`
      (defun (, sym) ()
	"Send bug report or feedback."
	(interactive)
	(ti::m-pkg-submit-bug-report
	 (, filename)
	 (, version-variable)
	 (, bug-var-list)
	 )
	(, bug-body)
	)))
    (push elt ret)

    (reverse ret)
    ))


;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-debug
  (debug-function debug-toggle-function debug-variable debug-buffer
   )
  "Return standard function forms for debug interface.
One variable, one function and one macro will be created.

Input:

  DEBUG-FUNCTION	symbol, function name to generate debug
  DEBUG-TOGGLE-FUNCTION symbol, function name to turn on/off debug
  DEBUG-VARIABLE	symbol, variable to control debug
  DEBUG-BUFFER		string, buffer name where to write debug.

How to call this macro:

  (ti::m-fmacro-debug xxx-debug xxx-debug-toggle xxx-debug \"*xxx-debug*\")

Example how to call created functions:

  M - x xxx-debug-toggle  ;; To turn on or off debug package debug
  (xxx-debug-toggle 0)    ;; off
  (xxx-debug-toggle 1)    ;; on

  ;;  To generate debug from inside code, you call:
  (xxx-debug ... anything frame-pointer buffer-pointer ...)"
  (`(, (ti::m-fmacro-debug-1
	debug-function debug-toggle-function debug-variable debug-buffer
	))))

(defun ti::m-fmacro-debug-1
  (debug-function debug-toggle-function debug-variable debug-buffer)
  "Use macro `ti::m-fmacro-debug' and see argument there.
DEBUG-FUNCTION DEBUG-TOGGLE-FUNCTION DEBUG-VARIABLE DEBUG-BUFFER."
  (let* (str
	 ret
	 elt
	 )
    (push 'progn ret)

    (setq elt
	  (list
	   'defvar (` (, debug-variable))
	   nil
	   "Debug control: on or off."
	   ))
    (push elt ret)


    (setq str
	  (concat
	   "Generate debug to buffer "
	   (` (, debug-buffer )) ".\n"
	   "Prefix ARG: nil = toggle, 0 = off, 1 = on."
	   ))
    (setq
     elt
     (`
      (defun (, debug-toggle-function) (&optional arg)
	(, str)
	(interactive "P")
	(var-toggle (, debug-variable) arg)
	(if (interactive-p)
	    (message (concat "Debug is %s"
		     (if (, debug-variable) "on" "off"))))
	)))
    (push elt ret)

    (setq str
	  (concat "Write debug log to " (` (, debug-buffer ))
		   " if "
;;;		   (symbol-name (quote (` (, debug-variable)) ))
		   "is non-nil."
		   ))
    (setq
     elt
     (list
      'defmacro (` (, debug-function)) (list '&optional 'arg)
      (` (, str))
      (list 'if (` (, debug-variable))
	    (list 'let*
		  (list (list 'ti:m-debug-buffer (` (, debug-buffer ))))
		  )
	    (list '`
	     (list 'd!! (list ',@ 'args) "\\n" ))
	    )))
    (push elt ret)

    (reverse ret)
    ))



;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-install-pgp-tar
  (func-ins-sym elisp-file &optional log-buffer)
  "Return standard pgp tar install function.
It handles installing pgp base 64 signed tar block from the end of file.

  1.   Create tar file (it sould not have directory names, but ...)
  2.   pgp base64 sign the tar file (clearsig off)
  3.   paste pgp data to to end of your lisp package


  ;; -----BEGIN PGP MESSAGE-----
  ;; Version: 2.6.3ia
  ;;
  ;; owHsWc1vG0l2n0GwwYjA3pJLgEXKlNaSDJLilySblrWWLXrMrCQrpOydzcxA02wW
  ...
  ;; -----END PGP MESSAGE-----

And nothing more is needed to get your programs untarred nicely.
The drop directory is asked from the user when he calls this function.

Input:

  FUNC-INS-SYM	,symbol, the created install function name
  ELISP-FILE    ,your lisp package name (with .el)
  [LOG-BUFFER]  ,where to print the install log. Can be nil.

Created function's arguments:

  (dir)
  DIR           Where to untar the included file, asked interactively

How to call this function:

  ;;;###autoload (autoload 'xxx-install-programs  \"package-file\" t t)

   (ti::m-fmacro-install-pgp-tar
    xxx-install-programs
   \"xxx.el\"
   \"*xxx-install-log*\"
   ))

Example how to call created function:

  M - x xxx-install-programs"
  (` (, (ti::m-fmacro-install-pgp-tar-1
	 func-ins-sym elisp-file log-buffer
	 ))))

(defun ti::m-fmacro-install-pgp-tar-1
  (func-ins-sym elisp-file &optional log-buffer)
  "Use macro `ti::m-fmacro-install-pgp-tar' and see arguments there.
FUNC-INS-SYM ELISP-FILE LOG-BUFFER."
  (let* (sym
	 )

    (setq sym (intern (symbol-name (` (, func-ins-sym)))))


    (`
     (defun (, sym) (dir)
       "Install additional programs from the end of package."
       (interactive "DSave programs to directory: ")
       (let* ((file    (, elisp-file))
	      (source  (or (ti::f-get-load-path file load-path)
			   (error "can't find %s along load-path." file)))
	      )
	 (ti::m-pkg-install-pgp-tar
	  dir (or (, log-buffer) "*install-log*") source)
	 )))
    ))



;;; ----------------------------------------------------------------------
;;;
(defmacro ti::m-fmacro-minor-mode-wizard
  (pfx					;1
   mode-Name				;
   mode-Name-prefix-key			;
   easymenu-Name			;
   custom-group				;
   variable-style			;6

   doc-str				;7
   mode-desc				;
   minor-mode-body			;

   easymenu-doc				;10
   easymenu-body			;
   define-key-body			;12
   )
  "Do all the necessary things to create minor mode.
We call here following macros on by one. If you want personalized
minor mode control, call each of these individually and don't use
this wizard.

    `ti::m-vmacro-minor-mode'
    `ti::m-fmacro-minor-mode-install'
    `ti::m-fmacro-minor-mode'
    `ti::m-fmacro-minor-mode-on'
    `ti::m-fmacro-minor-mode-off'
    `ti::m-fmacro-define-keys'

Input:

    PFX			    See -vmacro-
    MODE-NAME		    See -vmacro-
    MODE-NAME-PREFIX-KEY    See -vmacro-
    EASYMENU-NAME	    See -vmacro-
    CUSTOM-GROUP	    See -vmacro-
    VARIABLE-STYLE	    See -vmacro-

    DOC-STR		    See -fmacro-minor-mode
    MODE-DESC		    See -fmacro-minor-mode
    MINOR-MODE-BODY	    See -fmacro-minor-mode must be in format ((BOBY))

    EASYMENU-DOC	    See -fmacro-define-keys must be in format ((BOBY))
    EASYMENU-BODY	    See -fmacro-define-keys must be in format ((BOBY))
    DEFINE-KEY-BODY	    See -fmacro-define-keys

How to call this function:

   See example tinylisp.el package which uses this function to create
   minor mode.

If you want to see what this macro produces, use

  (macroexpand '(ti::m-fmacro-minor-mode-wizard ...))C - x C - e"
  (` (,
	(ti::m-fmacro-minor-mode-wizard-1
	 pfx				;1
	 mode-Name			;
	 mode-Name-prefix-key		;
	 easymenu-Name			;
	 custom-group			;
	 variable-style			;6

	 doc-str			;7
	 mode-desc			;
	 minor-mode-body		;9

	 easymenu-doc			;10
	 easymenu-body			;
	 define-key-body		;12
	 ))))

;;; ----------------------------------------------------------------------
;;;
(defun ti::m-fmacro-minor-mode-wizard-1
  (pfx					;1
   mode-Name				;2
   mode-Name-prefix-key			;3
   easymenu-Name			;4
   custom-group				;5
   variable-style			;6

   doc-str				;7
   mode-desc				;8
   minor-mode-body			;9

   easymenu-doc				;10
   easymenu-body			;11
   define-key-body			;12
   )
  "Use macro `ti::m-fmacro-minor-mode-wizard' and see parameters there.
PFX
   MODE-NAME
   MODE-NAME-PREFIX-KEY
   EASYMENU-NAME
   CUSTOM-GROUP
   VARIABLE-STYLE
   DOC-STR
   MODE-DESC
   MINOR-MODE-BODY

   EASYMENU-DOC
   EASYMENU-BODY
   DEFINE-KEY-BODY"

  (let* (sym1
	 sym2
	 sym3
	 sym4
	 sym5
	 sym6
	 sym7
	 ret
	 elt
	 vs
	 )
    (nconcc ret 'eval-and-compile)

    ;; ........................................... create variables ...
    (setq elt
	  (ti::m-vmacro-minor-mode-1
	   pfx
	   mode-Name
	   mode-Name-prefix-key
	   easymenu-Name
	   custom-group
	   variable-style
	   ))

    (setq vs  (if variable-style
		  variable-style
		pfx))

;;;    (d!! "\n\n>>" elt)
    (nconcc ret elt)
    ;; .................................... create install function ...

    (setq sym1 (intern (concat pfx "install-mode"))
	  sym2 (intern (concat pfx "mode"))
	  sym3 (intern (concat vs "mode-map"))
	  sym4 (intern (concat vs "mode-name"))
	  sym5 (intern (concat vs "mode-define-keys-hook"))
	  )
;;;    (d!! "\n\n>>minor-mode-install" sym1 sym2 sym3 sym4 sym5  "\n")
    (setq elt (ti::m-fmacro-minor-mode-install-1
	       sym1 sym2 sym3 sym4 sym5
	       ))

    (nconcc ret elt)
    ;; ....................................... define keys function ...
    (setq sym1 (intern (concat pfx "mode-define-keys"))
	  sym2 (intern (concat vs "mode-map"))
	  sym3 (intern (concat vs "mode-prefix-key"))
	  sym4 (intern (concat vs "mode-easymenu"))
	  sym5 (intern (concat vs "mode-easymenu-name"))
	 )

;;;   (d!! "\n\n>>define-keys"  sym1 sym2 sym3 sym4 sym5)

   (setq elt
	 (ti::m-fmacro-define-keys-1
	  sym1 sym2 sym3 sym4 sym5
	  easymenu-doc
	  easymenu-body
	  define-key-body
	  ))

    (nconcc ret elt)

   ;; ................................. create minor mode function ...

   (setq sym1 (intern (concat pfx "mode"))
	 sym2 (intern (concat pfx "install-mode"))
	 sym3 (intern (concat pfx "mode"))
	 sym4 (intern (concat vs "mode-name"))
	 sym5 (intern (concat vs "mode-prefix-key"))
	 sym6 (intern (concat vs "mode-easymenu"))
	 sym7 (intern (concat vs "mode-hook"))
	 )

;;;   (d!! "\n\n>>minor-mode" sym1 sym2 sym3 sym4 sym5 sym6 sym7 "\n")

   (setq elt
	 (ti::m-fmacro-minor-mode-1
	  sym1 doc-str  sym2
	  sym3 sym4 sym5
	  sym6 nil  mode-desc
	  sym7
	  minor-mode-body
	  ))
    (nconcc ret elt)

   (setq elt (ti::m-fmacro-minor-mode-on sym1)) (nconcc ret elt)
   (setq elt (ti::m-fmacro-minor-mode-off sym1)) (nconcc ret elt)

   ret
   ))


;;}}}

;;{{{ examples, hints

;;; ........................................................ &examples ...
;;; - This section contains examples: how you can utilize these libs easily
;;; - Code can be easily extracted with function ti::m-pkg-rip-magic


;;; .................................................... updating libs ...
;;; - You you ftp libs often here is example command how to update
;;;   libs function to your emacs quicly.
;;; - Bind it to some fast key like,
;;;
;;;     (global-set-key "\C-x\eu" 'my-lib)

;;* (defun my-lib ()
;;*   (interactive)
;;*   (load-library "tinylibm.el")
;;*   (load-library "tinylib.el")
;;*   )
;;* _

;;; ..................................................... moving words ...
;;; - If you don't like the default forward-word/backward-word function
;;;   in your default emacs, for movement keys, you can try this:

;;* _
;;* (global-set-key [C-right] 'my-forward-word)
;;* (global-set-key [C-left]  'my-backward-word)
;;* _
;;* _
;;* (defconst my-:move-word-set  "-[]$%@#&*\":;{}()<>/\\ \t\n"
;;*   "*How to move forward/backward word. This is character set.")
;;* _
;;* (defconst my-:move-word-case-set  "-[]$%@#&*\":;{}()<>/\\ \t\na-z"
;;*   "*How to move forward/backward word. This is character set, for
;;* Up/downcase words.")
;;* _
;;* (defsubst my-word-move-1 (&optional back)
;;*   "Low level word movement control."
;;*   (let* ((up-case (memq major-mode '(c-mode c++-mode cc-mode)))
;;* _
;;*      (case-fold-search (not up-case))
;;* _
;;*      (charset
;;*       (cond
;;*        ((and up-case
;;*              (looking-at "[a-z0-9]*[A-Z]+[a-z0-9]+[A-Z]+"))
;;*         ;;  This Upcase charset is only used if the cursor is
;;*         ;;  within AnUpCaseWord.
;;*         my-:move-word-case-set)
;;*        (t
;;*         my-:move-word-set
;;*         )))
;;* _
;;*      (point   (point))
;;*      )
;;*     (cond
;;*      ((and (null back)
;;*        (looking-at "\\([A-Za-z]+\\)[^a-zA-Z \t]*$"))
;;*       ;;  Skip to the end of word if at EOL
;;*       ;;  (this-he-is-word sse-it-now?)
;;*       ;;                          *     cursor here
;;*       ;;                            *   after
;;*       ;;  otherwise it would skip to next line's word begin.
;;*       (goto-char (match-end 1))
;;*       )
;;*      (t
;;*       (ti::m-word-move charset back)
;;*       ))))
;;* _
;;* (defun my-backward-word ()
;;*   (interactive) "Word backward"  (my-word-move-1 'back))
;;* _
;;* (defun my-forward-word ()
;;*   (interactive) "Word forward" (my-word-move-1))
;;* _





;;; ..................................................... line-numbers ...
;;; - This is how you print your code with line numbers easily.
;;; - See also variable lpr-switches (C-h v VAR-NAME)
;;;

;;* ;;  This runs shell first, then given command
;;* (setq lpr-command "sh")
;;* (setq lpr-switches '("-c" "lp -d03c3"))


;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-print-with-line-numbers  ()
;;*   "Adds line numbers to buffer and prints it. After printing,
;;* removes line numbers."
;;*   (interactive)
;;*   (let* (buffer-read-only
;;*      fmt
;;*      len
;;*      )
;;*     (save-excursion
;;*       (PMAX)
;;*       ;;  Set dynamic format according to biggest line number
;;*       (setq len  (digit-len (LINE))
;;*         fmt  (concat "%0" len "d: %s"))
;;* _
;;*       (unwind-protect
;;*       (progn
;;*         (ti::b-insert-line-numbers (point-min) (point-max) 1 1 fmt)
;;*         (print-buffer)
;;*         )
;;*     ;;  Make sure these are removed
;;*     ;;
;;*     (set-buffer-modified-p   nil)
;;*     (ti::b-remove-line-numbers
;;*      (point-min) (point-max)
;;*      "^[0-9]+: " 0))
;;*       )))


;;; ......................................................... ange-ftp ...
;;; - Here is the code how I update the files to my tiny tools site
;;;   cs.uta.fi
;;; - Iinvestigate it and you can do your own batch download easily.

;;* (defun my-mput-tiny-site-batch  ()
;;*   "Updates all files in cs.uta.fi."
;;*   (interactive)
;;*   (let* ((sdir   "~/elisp/ftp/")    ;source
;;*      (files (list-find
;;*              (directory-files sdir)
;;*              "\\.$\\|\\.\\.$"       ;drop these
;;*              '(not (o-string-match arg element ))
;;*              'list-all
;;*              ))
;;*      )
;;* _
;;*     (ti::f-ange-file-handle
;;*      'put
;;*      "ssjaaa" "cs.uta.fi"
;;*      "~ftp/pub/ssjaaa/" sdir
;;*      files)
;;*     ))




;;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... passwd  ..
;;; - If you ever need to complete password login names, you can take a
;;;   look at these functions
;;; - For example I have defined my own TAB key when I'm writing
;;;   mail to fellow workers.
;;; - Try putting this inside mail-setup-hook
;;;
;;;   (define-key mail-mode-map [tab]           'my-mail-key-tab)
;;;
;;;


;;* ;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-mail-key-tab ()
;;*   "Completes passwd entry's login names in TO, CC,BCC fields, otw calls
;;* tab-to-tab-stop.
;;* "
;;*   (interactive)
;;*   (let* ((re     "^\\(To\\|b?cc\\):")
;;*      table
;;*      word
;;*      completions
;;*      stat
;;*      )
;;*     (setq stat (save-excursion
;;*              (beginning-of-line)
;;*              (o-looking-at re)))
;;* _
;;*     (if (null stat)
;;*     (tab-to-tab-stop)
;;*       (my-define-passwd-vars)
;;*       (setq table my-:passwd-cmpl-table)
;;* _
;;*       (require 'thingatpt)
;;*       (save-excursion
;;*     (forward-char -1)               ;move ovr some char
;;*     (setq word (word-at-point)))
;;* _
;;*       (if (null word)
;;*       (tab-to-tab-stop)
;;*     (setq completions (all-completions word table))
;;* _
;;*     (cond
;;*      ((eq 1 (length completions))
;;*       (insert (substring (car completions) (length word)))
;;*       (message-quick (cdr completions))
;;*       )
;;*      (t
;;*       ;;   Flash possible completions
;;*       (message (list-to-string completions))
;;*       ))
;;*     ))
;;*     ))
;;* _


;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;; - We do not load these immediately, because it's slow to build..
;;* ;;;
;;* (defun my-define-passwd-vars ()
;;*   "Defines few passwd variables."
;;* _
;;* (defvar my-:passwd-table (ti::m-passwd-build-alist "ypcat passwd")
;;*   "Contents of passwd file in assoc form.")
;;* _
;;* (defvar my-:passwd-cmpl-table
;;*   (let* ((list    my-:passwd-table)
;;*      (i       0)
;;*      completions
;;*      )
;;*     (mapcar
;;*      (function
;;*       (lambda (x)
;;*         (setq i (1+ i))
;;*         (setq completions (cons  (cons (car x) i) completions))))
;;*      list)
;;*     completions
;;*     )
;;*   "Completion table for login names")
;;* _
;;* ;;; End of defun
;;* )
;;* _
;;* ;;}}}
;;* _
;;* _
;;* (defvar my-:mode-line-info nil
;;*   "*My mode line string.")
;;* _
;;* _
;;* _
;;* ;;  Convert DOS ^M char to softer choice
;;* ;;
;;* (defconst my-:display-table
;;*   (let ((table (if (boundp 'my-:display-table) ;reuse table if it exists
;;*                my-:display-table
;;*              (make-display-table))))
;;*     (aset table ?\C-m (vector ?_))
;;*     table)
;;*     "Char table for visible tab display.")
;;* _

;;}}}
;;{{{ UU "kiss" C-program

;;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... . uudecode  ..

;;; - You may want to use this uudecoding program, which can ignore
;;;   headers easily.
;;l
;;; - Just replace the original command with
;;;
;;;          (require   'tinylib)
;;;          (setq      ti:-uude-cmd 'my-uude-cmd)
;;;          (defun     my-uude-cmd  (file) (concat "kiss " file))
;;;
;;;   and save the C-file + compile it as told in the documentation
;;;           "kiss.c"
;;;
;;; - At the time of writing this, Oct 2 1995, the Author doesn't seem to be
;;;   reachable any more.


;;; ..................................................... kiss-doc.txt ...
;;* From: yang@sanantonio.cs.tcu.edu (FrogMan--II)
;;* Subject: Kiss-1.0.c for uudecode
;;* Date: 25 Jul 92 07:26:27 GMT
;;* Sender: news@riogrande.cs.tcu.edu
;;* _
;;* KISS-1.0.C
;;* _
;;* This is a C program that trims headers/footers and does
;;* uudecoding at the same time.  This application has some
;;* advantages over the programs that have appeared on this
;;* net.  For example, uuconvert.c , uudecode.c & eek.c , etc.
;;* Also, it can handle an unlimited number of input files
;;* including standard input and multiple parts in a single
;;* file.
;;* _
;;* _
;;* Further, it handles special cases such as those that
;;* happen to USKT-76.JPG These net programs uuconvert.c,
;;* eek.c, and others will crash because this special case is
;;* not taken into consideration.  This program has undergone
;;* extensive testing without failure to date.  All you need
;;* to do is to make sure all articals used are in order.
;;* Say, artical#1, artical#2, ...
;;* _
;;* This program is being written for your convenience.  Feel
;;* free to distribute copies.  If you find any bugs or have
;;* any comments please respond accordingly.  I spent an
;;* entire sunday night to write it, without going on a date.
;;* Oh, boy, I miss kissing those hot lips in the summer
;;* night.  :-) :-)
;;* _
;;* FLAMES -->> GAMMA.IS.TCU.EDU
;;* _
;;* KEVIN YANG
;;* SANANTONIO.CS.TCU.EDU
;;* _
;;* _

;;; ........................................................... kiss.c ...
;;* _
;;* /*
;;*  * kiss-1.0.c
;;*  * AUTHOR: Kevin Yang
;;*  *
;;*  * This C program (UNIX) takes an unlimited number of uuencoded files,
;;*  * and trims headers/footers.  The uuencoded lines are decoded and written
;;*  * to specific output files.
;;*  *
;;*  * To compile: "cc -O -o kiss kiss-1.0.c".
;;*  *
;;*  */
;;* _
;;* _
;;* #include <stdio.h>
;;* #include <ctype.h>
;;* _
;;* #define Author "Kevin Yang"
;;* #define PName "KiSS"
;;* #define DEC(C)      (((C) - ' ') & 077)
;;* #define LEN 128
;;* _
;;* /* most variables are declared globally, in order to
;;*  * speed up function calls.
;;*  */
;;* _
;;* static char input_buf[LEN], a[LEN], b[LEN], dest[LEN];
;;* static char *buf = input_buf, *ap = a, *bp = b, *tmp, *p;
;;* static int len, mode, newdata, endmark, hasdata, bquote, i, x, y, z;
;;* static unsigned siz = sizeof(input_buf);
;;* static FILE *ofp;
;;* _
;;* _
;;* _
;;* main(argc, argv)
;;* int argc;
;;* char *argv[];
;;* {
;;*     FILE *ifp;
;;* _
;;*     if (argc < 2)  {
;;*             KissFile(stdin);                                /* standard input */
;;*     } else  {
;;*             for (;--argc && ++argv; ) {
;;*                     if (! (ifp  = fopen(*argv,"r"))) {
;;*                             fprintf(stderr, "%s: %s: ", PName, *argv);
;;*                             perror(ifp);
;;*                             continue;
;;*                     }
;;*                     KissFile(ifp);
;;* /*                  remove(*argv);  */              /* uncomment this when you want to save spaces,*/
;;*                                                     /* it simply delete the input file             */
;;*                     fclose(ifp);
;;*             }
;;*     }
;;*     printf("\tKiss me, horny ...\n");
;;* }  /* main */
;;* _
;;* _
;;* _
;;* /* This function "kiss" the input file until all articals
;;*  * are decoded.
;;*  */
;;* _
;;* int KissFile(ifp)
;;* FILE *ifp;
;;* {
;;*     hasdata = bquote = 0;
;;*     while (fgets(buf, siz, ifp))  {
;;*             while(1) {
;;*                     if (!strncmp(buf,"begin ",6) && isdigit(buf[6]))  {
;;*                             hasdata = 1;                    /* yes, we found valid begin line */
;;*                             break;
;;*                     }
;;*                     if (!fgets(buf, siz, ifp)) {
;;*                             if (!hasdata) fprintf(stderr, "\t%s: No begin line\n", PName);
;;*                             return (0);
;;*                     }
;;*             }
;;* _
;;*             if (sscanf(buf, "begin %o %s", &mode, dest) != 2)  {            /* extract filename and file-mode       */
;;*                     fprintf(stderr, "%s: %s: Missing filename or mode\n", PName, buf);
;;*                     continue;                                               /* try again when error occur           */
;;*             }
;;* _
;;*             if ((ofp = fopen(dest, "w")) == NULL) {                         /* prepare output file                  */
;;*                     fprintf(stderr, "%s: %s: ", PName, dest);
;;*                     perror(ifp);
;;*                     continue;
;;*             }
;;* _
;;*             chmod(dest, mode);                                              /* set file-mode                        */
;;*             fprintf(stdout, "Extracting  <%s>\t\t-- ", dest);
;;* _
;;*             while (fgets(buf,siz,ifp)) {
;;*                     if ( (endmark = !strncmp(buf, "end", 3))
;;*                             || (newdata = !strncmp(buf, "begin ", 6)
;;*                             && isdigit(buf[6])) )  {
;;*                             if (newdata)  {                         /* no end marker*/
;;*                                     fseek(ifp, -strlen(buf), 1);    /* reset input stream */
;;*                             }
;;*                             KissLin(ap,ofp);
;;*                             break;
;;*                     }
;;*                     len = strlen(buf);
;;*                     if (*buf != 'M' || len < 60 || len > 70
;;*                             || !strncmp(buf, "Message-ID:", 11))  {
;;*                             if (*buf == '`')  {                     /* "`" char exist */
;;*                                     bquote = 1;
;;*                                     KissLin(bp, ofp);               /* or end marker comes  */
;;*                                     break;                          /* from another file.   */
;;*                             }                                       /* This is a tricky case*/
;;*                             tmp = ap;
;;*                             ap = bp;
;;*                             bp = buf;
;;*                             buf = tmp;
;;*                             continue;
;;*                     }
;;*                     KissLin(buf, ofp);
;;*             }
;;*             fclose(ofp);
;;* _
;;*             fprintf(stdout, "Done\n");
;;*             if ((newdata || !endmark) && !bquote) fprintf(stderr, "\t%s: No end line\n", PName);
;;*     }
;;* }
;;* _
;;* _
;;* _
;;* /* This function decodes one line of information.
;;*  * C codes are slightly different from those of uuconvert.c
;;*  * or uudecode.c.  I use pointers instead of array indices
;;*  * to avoid time consuming address multiplications.  The
;;*  * condition tests are modified and eliminates 2 tests
;;*  * for every 4 bytes.  Characters are decoded only when
;;*  * one of the three conditions matches, in which complicated
;;*  * mathmatical computation are eliminated for the last few
;;*  * bytes.
;;*  * I hope this can speed up the process!
;;*  */
;;* KissLin(str)
;;* char *str;
;;* {
;;*     i = DEC(*str);
;;*     p = ++str;
;;*     while (i > 0)  {
;;*             if (i >= 3)  {
;;*                     x  = (DEC(*p) << 2);    p++;
;;*                     x |= (DEC(*p) >> 4);
;;*                     y  = (DEC(*p) << 4);    p++;
;;*                     y |= (DEC(*p) >> 2);
;;*                     z  = (DEC(*p) << 6);    p++;
;;*                     z |= (DEC(*p));
;;*                     putc(x, ofp);
;;*                     putc(y, ofp);
;;*                     putc(z, ofp);
;;*             } else {
;;*                     if (i >= 2)  {
;;*                             x  = (DEC(*p) << 2);    p++;
;;*                             x |= (DEC(*p) >> 4);
;;*                             y  = (DEC(*p) << 4);    p++;
;;*                             y |= (DEC(*p) >> 2);
;;*                             putc(x, ofp);
;;*                             putc(y, ofp);
;;*                     } else {
;;*                             if (i >= 1)  {
;;*                                     x  = (DEC(*p) << 2);    p++;
;;*                                     x |= (DEC(*p) >> 4);
;;*                                     putc(x, ofp);
;;*                             }  /* if  */
;;*                     }  /* else  */
;;*             }  /* else  */
;;* _
;;*             str += 4;
;;*             p = str;
;;*             i -= 3;
;;*     }  /* while  */
;;* }  /* KissLin */
;;* _
;;* _

;;}}}


;;{{{ provide


(provide   'tinylib)
(run-hooks 'ti:-load-hook)

;;}}}

;;; tinylib.el ends here
